namespace mm {


// one-word synonyms for unsigned integral types
typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef unsigned long ulong;

// TIME definition
typedef double TIME;
typedef TIME TIME_t;

// real definition
typedef float real;

// counter definition
typedef ushort counter;

// integer definition
typedef short integer;

// index definition
typedef ulong index;

// logical definition
typedef uint logical;

// time constants
CONSTANT( TIME, ANY_AGE, (TIME) -1 );
CONSTANT( TIME, TIME_UNDEF, (TIME) -1 );
CONSTANT( TIME, TIME_INFINITE, (TIME) SHRT_MAX );

extern Thread EventQueue *gpoEventQueue;

void SetCaseWeight( double dCaseWeight, double dCaseSubsampleWeight = UNUSED_VALUE );

// classifications
enum CHILD_MORTALITY_RISKS {
CMR_AGE14,
CMR_AGE16,
CMR_NOPRIM,
CMR_PRIMDROP };
typedef CHILD_MORTALITY_RISKS CHILD_MORTALITY_RISKS_t;
CONSTANT( int, rc_size_CHILD_MORTALITY_RISKS, 4 );
enum HIGHER_BIRTHS_PARA {
HBP_PERIOD1,
HBP_PERIOD2,
HBP_PERIOD3,
HBP_PERIOD4,
HBP_PERIOD5,
HBP_PERIOD6,
HBP_AGE35,
HBP_AGE40,
HBP_AGE45,
HBP_EDUC1,
HBP_EDUC2 };
typedef HIGHER_BIRTHS_PARA HIGHER_BIRTHS_PARA_t;
CONSTANT( int, rc_size_HIGHER_BIRTHS_PARA, 11 );
enum MOTHER_AGE_CLASS {
MAC_14,
MAC_16,
MAC_17P };
typedef MOTHER_AGE_CLASS MOTHER_AGE_CLASS_t;
CONSTANT( int, rc_size_MOTHER_AGE_CLASS, 3 );
enum OUTPUT_TIMES {
OT_FIRST,
OT_LAST,
OT_INTERVAL };
typedef OUTPUT_TIMES OUTPUT_TIMES_t;
CONSTANT( int, rc_size_OUTPUT_TIMES, 3 );
enum OUTPUT_VARIABLES {
OV_TIME,
OV_WEIGHT,
OV_BIRTH,
OV_SEX,
OV_PROVINCE,
OV_EDUC,
OV_POB,
OV_UNION,
OV_PARITY,
OV_LASTBIR };
typedef OUTPUT_VARIABLES OUTPUT_VARIABLES_t;
CONSTANT( int, rc_size_OUTPUT_VARIABLES, 10 );
enum PERSON_MICRODATA_COLUMNS {
PMC_WEIGHT,
PMC_BIRTH,
PMC_SEX,
PMC_PROVINCE,
PMC_EDUC,
PMC_POB,
PMC_UNION,
PMC_PARITY,
PMC_LASTBIR };
typedef PERSON_MICRODATA_COLUMNS PERSON_MICRODATA_COLUMNS_t;
CONSTANT( int, rc_size_PERSON_MICRODATA_COLUMNS, 9 );
enum PERSON_TYPE {
PT_START,
PT_CHILD,
PT_IMMIGRANT };
typedef PERSON_TYPE PERSON_TYPE_t;
CONSTANT( int, rc_size_PERSON_TYPE, 3 );
enum PRIMARY_LEVEL {
PL_NO,
PL_ENTER,
PL_GRAD };
typedef PRIMARY_LEVEL PRIMARY_LEVEL_t;
CONSTANT( int, rc_size_PRIMARY_LEVEL, 3 );
enum PROVINCE_INT {
PI_PROV00,
PI_PROV01,
PI_PROV02,
PI_PROV03,
PI_PROV04,
PI_PROV05,
PI_PROV06,
PI_PROV07,
PI_PROV08,
PI_PROV09,
PI_PROV10,
PI_PROV11,
PI_PROV12,
PI_PROV13 };
typedef PROVINCE_INT PROVINCE_INT_t;
CONSTANT( int, rc_size_PROVINCE_INT, 14 );
enum PROVINCE_NAT {
PN_PROV00,
PN_PROV01,
PN_PROV02,
PN_PROV03,
PN_PROV04,
PN_PROV05,
PN_PROV06,
PN_PROV07,
PN_PROV08,
PN_PROV09,
PN_PROV10,
PN_PROV11,
PN_PROV12 };
typedef PROVINCE_NAT PROVINCE_NAT_t;
CONSTANT( int, rc_size_PROVINCE_NAT, 13 );
enum RTAB_COL {
RC_COLUMN };
typedef RTAB_COL RTAB_COL_t;
CONSTANT( int, rc_size_RTAB_COL, 1 );
enum RTAB_REG {
RR_NOUAK,
RR_REST };
typedef RTAB_REG RTAB_REG_t;
CONSTANT( int, rc_size_RTAB_REG, 2 );
enum RTAB_SEL_YEARS {
SY_2017,
SY_2030,
SY_2050 };
typedef RTAB_SEL_YEARS RTAB_SEL_YEARS_t;
CONSTANT( int, rc_size_RTAB_SEL_YEARS, 3 );
enum SELECTED_CHILD_MORTALITY_MODEL {
SCMM_MACRO,
SCMM_MICRO_NOT_ALIGNED,
SCMM_MICRO_ALIGNED_MACRO_TRENDS,
SCMM_MICRO_ALIGNED_MICRO_TRENDS };
typedef SELECTED_CHILD_MORTALITY_MODEL SELECTED_CHILD_MORTALITY_MODEL_t;
CONSTANT( int, rc_size_SELECTED_CHILD_MORTALITY_MODEL, 4 );
enum SELECTED_FERTILITY_MODEL {
SFM_MACRO,
SFM_MICRO,
SFM_ALIGNE_BIRTHS,
SFM_ALIGNE_AGE };
typedef SELECTED_FERTILITY_MODEL SELECTED_FERTILITY_MODEL_t;
CONSTANT( int, rc_size_SELECTED_FERTILITY_MODEL, 4 );
enum SEX {
FEMALE,
MALE };
typedef SEX SEX_t;
CONSTANT( int, rc_size_SEX, 2 );
enum UNION_MODEL_SELECTION {
UMS_COALE,
UMS_RATES };
typedef UNION_MODEL_SELECTION UNION_MODEL_SELECTION_t;
CONSTANT( int, rc_size_UNION_MODEL_SELECTION, 2 );
enum UNION_PARA {
UP_MINAGE,
UP_AVERAGE,
UP_EVER };
typedef UNION_PARA UNION_PARA_t;
CONSTANT( int, rc_size_UNION_PARA, 3 );
enum UNION_STATUS {
US_NO,
US_YES };
typedef UNION_STATUS UNION_STATUS_t;
CONSTANT( int, rc_size_UNION_STATUS, 2 );

// ranges
typedef uint AGE_RANGE;
CONSTANT( int, r_min_AGE_RANGE, 0 );
CONSTANT( int, r_max_AGE_RANGE, 100 );
CONSTANT( int, rc_size_AGE_RANGE, 101 );
typedef AGE_RANGE AGE_RANGE_t;
typedef uint AGE_UNION;
CONSTANT( int, r_min_AGE_UNION, 10 );
CONSTANT( int, r_max_AGE_UNION, 60 );
CONSTANT( int, rc_size_AGE_UNION, 51 );
typedef AGE_UNION AGE_UNION_t;
typedef uint ALL_YEAR_RANGE;
CONSTANT( int, r_min_ALL_YEAR_RANGE, 1912 );
CONSTANT( int, r_max_ALL_YEAR_RANGE, 2113 );
CONSTANT( int, rc_size_ALL_YEAR_RANGE, 202 );
typedef ALL_YEAR_RANGE ALL_YEAR_RANGE_t;
typedef uint CHILD_MORT_RANGE;
CONSTANT( int, r_min_CHILD_MORT_RANGE, 0 );
CONSTANT( int, r_max_CHILD_MORT_RANGE, 14 );
CONSTANT( int, rc_size_CHILD_MORT_RANGE, 15 );
typedef CHILD_MORT_RANGE CHILD_MORT_RANGE_t;
typedef uint CHILD_MORTALITY_AGE;
CONSTANT( int, r_min_CHILD_MORTALITY_AGE, 0 );
CONSTANT( int, r_max_CHILD_MORTALITY_AGE, 4 );
CONSTANT( int, rc_size_CHILD_MORTALITY_AGE, 5 );
typedef CHILD_MORTALITY_AGE CHILD_MORTALITY_AGE_t;
typedef uint CHILD_MORTALITY_YEARS;
CONSTANT( int, r_min_CHILD_MORTALITY_YEARS, 2018 );
CONSTANT( int, r_max_CHILD_MORTALITY_YEARS, 2113 );
CONSTANT( int, rc_size_CHILD_MORTALITY_YEARS, 96 );
typedef CHILD_MORTALITY_YEARS CHILD_MORTALITY_YEARS_t;
typedef uint FERTILE_AGE_RANGE;
CONSTANT( int, r_min_FERTILE_AGE_RANGE, 10 );
CONSTANT( int, r_max_FERTILE_AGE_RANGE, 49 );
CONSTANT( int, rc_size_FERTILE_AGE_RANGE, 40 );
typedef FERTILE_AGE_RANGE FERTILE_AGE_RANGE_t;
typedef uint PARITY_RANGE;
CONSTANT( int, r_min_PARITY_RANGE, 0 );
CONSTANT( int, r_max_PARITY_RANGE, 15 );
CONSTANT( int, rc_size_PARITY_RANGE, 16 );
typedef PARITY_RANGE PARITY_RANGE_t;
typedef uint PARITY_RANGE1;
CONSTANT( int, r_min_PARITY_RANGE1, 1 );
CONSTANT( int, r_max_PARITY_RANGE1, 15 );
CONSTANT( int, rc_size_PARITY_RANGE1, 15 );
typedef PARITY_RANGE1 PARITY_RANGE1_t;
typedef uint PARITY_RANGE2;
CONSTANT( int, r_min_PARITY_RANGE2, 2 );
CONSTANT( int, r_max_PARITY_RANGE2, 15 );
CONSTANT( int, rc_size_PARITY_RANGE2, 14 );
typedef PARITY_RANGE2 PARITY_RANGE2_t;
typedef uint RTAB_YEARS;
CONSTANT( int, r_min_RTAB_YEARS, 2017 );
CONSTANT( int, r_max_RTAB_YEARS, 2050 );
CONSTANT( int, rc_size_RTAB_YEARS, 34 );
typedef RTAB_YEARS RTAB_YEARS_t;
typedef uint SIM_YEAR_RANGE;
CONSTANT( int, r_min_SIM_YEAR_RANGE, 2013 );
CONSTANT( int, r_max_SIM_YEAR_RANGE, 2113 );
CONSTANT( int, rc_size_SIM_YEAR_RANGE, 101 );
typedef SIM_YEAR_RANGE SIM_YEAR_RANGE_t;
typedef uint SIM_YOB_RANGE;
CONSTANT( int, r_min_SIM_YOB_RANGE, 2013 );
CONSTANT( int, r_max_SIM_YOB_RANGE, 2060 );
CONSTANT( int, rc_size_SIM_YOB_RANGE, 48 );
typedef SIM_YOB_RANGE SIM_YOB_RANGE_t;
typedef uint YOB_GRAD_PRIMARY;
CONSTANT( int, r_min_YOB_GRAD_PRIMARY, 1997 );
CONSTANT( int, r_max_YOB_GRAD_PRIMARY, 2063 );
CONSTANT( int, rc_size_YOB_GRAD_PRIMARY, 67 );
typedef YOB_GRAD_PRIMARY YOB_GRAD_PRIMARY_t;
typedef uint YOB_START_PRIMARY;
CONSTANT( int, r_min_YOB_START_PRIMARY, 2001 );
CONSTANT( int, r_max_YOB_START_PRIMARY, 2063 );
CONSTANT( int, rc_size_YOB_START_PRIMARY, 63 );
typedef YOB_START_PRIMARY YOB_START_PRIMARY_t;
typedef uint YOB_UNION;
CONSTANT( int, r_min_YOB_UNION, 1964 );
CONSTANT( int, r_max_YOB_UNION, 2050 );
CONSTANT( int, rc_size_YOB_UNION, 87 );
typedef YOB_UNION YOB_UNION_t;

// partitions
typedef uint AGE5_PART;
CONSTANT( int, rc_size_AGE5_PART, 13 );
extern real p_points_AGE5_PART[];
typedef AGE5_PART AGE5_PART_t;
typedef uint AGE_15_16_18;
CONSTANT( int, rc_size_AGE_15_16_18, 4 );
extern real p_points_AGE_15_16_18[];
typedef AGE_15_16_18 AGE_15_16_18_t;
typedef uint AGE_15_17;
CONSTANT( int, rc_size_AGE_15_17, 3 );
extern real p_points_AGE_15_17[];
typedef AGE_15_17 AGE_15_17_t;
typedef uint AGE_FIVE;
CONSTANT( int, rc_size_AGE_FIVE, 17 );
extern real p_points_AGE_FIVE[];
typedef AGE_FIVE AGE_FIVE_t;
typedef uint AGE_MORT;
CONSTANT( int, rc_size_AGE_MORT, 5 );
extern real p_points_AGE_MORT[];
typedef AGE_MORT AGE_MORT_t;
typedef uint AGE_RTAB_MORT;
CONSTANT( int, rc_size_AGE_RTAB_MORT, 2 );
extern real p_points_AGE_RTAB_MORT[];
typedef AGE_RTAB_MORT AGE_RTAB_MORT_t;
typedef uint BIRTH_AGE_PART;
CONSTANT( int, rc_size_BIRTH_AGE_PART, 4 );
extern real p_points_BIRTH_AGE_PART[];
typedef BIRTH_AGE_PART BIRTH_AGE_PART_t;
typedef uint DUR_TIME_IN_PARITY;
CONSTANT( int, rc_size_DUR_TIME_IN_PARITY, 6 );
extern real p_points_DUR_TIME_IN_PARITY[];
typedef DUR_TIME_IN_PARITY DUR_TIME_IN_PARITY_t;
typedef uint LITTLE_TIME;
CONSTANT( int, rc_size_LITTLE_TIME, 2 );
extern real p_points_LITTLE_TIME[];
typedef LITTLE_TIME LITTLE_TIME_t;

class Parameters {
	public:
		Parameters();

		double	ActorWeight;
		int	AgeEnterPrimary;
		int	AgeGradPrimary;
		long	AgeImmigrants
[rc_size_SEX][CUM_RATES_ROWS_CNT]
[rc_size_AGE_RANGE];
		double	AgeSpecificFertility
[rc_size_FERTILE_AGE_RANGE]
[rc_size_SIM_YEAR_RANGE];
		double	AgeSpecificFertilityRate
[rc_size_FERTILE_AGE_RANGE]
[rc_size_SIM_YEAR_RANGE];
		double	BirthTrends
[rc_size_PARITY_RANGE1]
[rc_size_SIM_YEAR_RANGE];
		int	CalibratedYear;
		double	ChildMortalityBaseRisk
[rc_size_CHILD_MORTALITY_AGE]
[rc_size_SEX];
		double	ChildMortalityRelativeRisks
[rc_size_CHILD_MORTALITY_AGE]
[rc_size_CHILD_MORTALITY_RISKS];
		double	ChildMortalityTrend
[rc_size_CHILD_MORTALITY_AGE]
[rc_size_CHILD_MORTALITY_YEARS];
		long	DestinationImmigrants
[rc_size_SEX]
[rc_size_AGE5_PART][CUM_RATES_ROWS_CNT]
[rc_size_PROVINCE_NAT];
		double	EmigrationRates
[rc_size_SEX]
[rc_size_AGE5_PART]
[rc_size_PROVINCE_NAT];
		double	FirstBirthRates
[rc_size_PRIMARY_LEVEL]
[rc_size_UNION_STATUS]
[rc_size_FERTILE_AGE_RANGE]
[rc_size_PROVINCE_NAT];
		double	GradPrimary
[rc_size_SEX]
[rc_size_YOB_GRAD_PRIMARY]
[rc_size_PROVINCE_INT];
		double	HigherOrderBirthsPara
[rc_size_HIGHER_BIRTHS_PARA]
[rc_size_PARITY_RANGE2];
		long	ImmiPopSampleSize;
		double	ImmiPopSize;
		double	LifeExpectancy
[rc_size_SIM_YEAR_RANGE]
[rc_size_SEX];
		CString	MicroDataInputFile;
		long	MicroDataInputFileSize;
		CString	MicroRecordFileName;
		long	MigrationDestination
[rc_size_SEX]
[rc_size_PROVINCE_NAT]
[rc_size_AGE5_PART][CUM_RATES_ROWS_CNT]
[rc_size_PROVINCE_NAT];
		double	MigrationProbability
[rc_size_SEX]
[rc_size_AGE5_PART]
[rc_size_PROVINCE_NAT];
		double	MigrationProbabilityEduc
[rc_size_PRIMARY_LEVEL]
[rc_size_SEX]
[rc_size_AGE5_PART]
[rc_size_PROVINCE_NAT];
		logical	model_is_case_based;
		logical	ModelByEducation;
		logical	ModelEmigration;
		logical	ModelImmigration;
		logical	ModelMigration;
		double	MortalityTable
[rc_size_AGE_RANGE]
[rc_size_SEX];
		double	MortalityTrend
[rc_size_SIM_YEAR_RANGE]
[rc_size_SEX];
		double	NumberImmigrants
[rc_size_SIM_YEAR_RANGE]
[rc_size_SEX];
		double	NumberImmigrantsTable
[rc_size_SIM_YEAR_RANGE]
[rc_size_SEX];
		long 	NumberImmigrantsTable_lookup[CUM_RATES_ROWS_CNT]
[rc_size_SIM_YEAR_RANGE]
[rc_size_SEX];
		double	OddsMotherEnterPrimary
[rc_size_SEX]
[rc_size_PRIMARY_LEVEL];
		double	OddsMotherGradPrimary
[rc_size_SEX]
[rc_size_PRIMARY_LEVEL];
		logical	ScalePopulation;
		SELECTED_FERTILITY_MODEL	selected_fertility_model;
		SELECTED_CHILD_MORTALITY_MODEL	SelectedChildMortalityModel;
		logical	SelectedOutput
[rc_size_OUTPUT_VARIABLES];
		double	SexRatio
[rc_size_SIM_YEAR_RANGE];
		double	StartOfSchoolYear;
		long	StartPopSampleSize;
		double	StartPopSize;
		double	StartPrimary
[rc_size_SEX]
[rc_size_YOB_START_PRIMARY]
[rc_size_PROVINCE_INT];
		double	TimeMicroOutput
[rc_size_OUTPUT_TIMES];
		double	TotalFertilityRate
[rc_size_SIM_YEAR_RANGE];
		long	TotalPopSampleSize;
		double	TotalPopSize;
		double	UnionFormationHazard
[rc_size_PRIMARY_LEVEL]
[rc_size_SIM_YEAR_RANGE]
[rc_size_AGE_UNION];
		double	UnionFormationHazardParameter
[rc_size_PRIMARY_LEVEL]
[rc_size_AGE_UNION]
[rc_size_SIM_YEAR_RANGE];
		UNION_MODEL_SELECTION	UnionModelSelection;
		double	UnionParameters
[rc_size_PRIMARY_LEVEL]
[rc_size_YOB_UNION]
[rc_size_UNION_PARA];
		logical	UseMothersEducation;
		logical	UseWeightsInFile;
		logical	WriteMicrodata;
};

extern Parameters	*gprParam;

bool Lookup_AgeImmigrants( long lNumber, int nInd0, int *pnDest0 );
bool Lookup_AgeImmigrants( double dNumber, int nInd0, int *pnDest0 );
int EmptyDistributions_AgeImmigrants( void );
bool Lookup_DestinationImmigrants( long lNumber, int nInd0, int nInd1, int *pnDest0 );
bool Lookup_DestinationImmigrants( double dNumber, int nInd0, int nInd1, int *pnDest0 );
int EmptyDistributions_DestinationImmigrants( void );
bool Lookup_MigrationDestination( long lNumber, int nInd0, int nInd1, int nInd2, int *pnDest0 );
bool Lookup_MigrationDestination( double dNumber, int nInd0, int nInd1, int nInd2, int *pnDest0 );
int EmptyDistributions_MigrationDestination( void );
bool Lookup_NumberImmigrantsTable( long lNumber, int *pnDest0, int *pnDest1 );
bool Lookup_NumberImmigrantsTable( double dNumber, int *pnDest0, int *pnDest1 );
int EmptyDistributions_NumberImmigrantsTable( void );
bool IntIs_CHILD_MORTALITY_RISKS( int nNumber );
CHILD_MORTALITY_RISKS IntTo_CHILD_MORTALITY_RISKS( int nNumber );
int IntFrom_CHILD_MORTALITY_RISKS( CHILD_MORTALITY_RISKS nLevel );
bool IntIs_HIGHER_BIRTHS_PARA( int nNumber );
HIGHER_BIRTHS_PARA IntTo_HIGHER_BIRTHS_PARA( int nNumber );
int IntFrom_HIGHER_BIRTHS_PARA( HIGHER_BIRTHS_PARA nLevel );
bool IntIs_MOTHER_AGE_CLASS( int nNumber );
MOTHER_AGE_CLASS IntTo_MOTHER_AGE_CLASS( int nNumber );
int IntFrom_MOTHER_AGE_CLASS( MOTHER_AGE_CLASS nLevel );
bool IntIs_PROVINCE_INT( int nNumber );
PROVINCE_INT IntTo_PROVINCE_INT( int nNumber );
int IntFrom_PROVINCE_INT( PROVINCE_INT nLevel );
bool IntIs_PROVINCE_NAT( int nNumber );
PROVINCE_NAT IntTo_PROVINCE_NAT( int nNumber );
int IntFrom_PROVINCE_NAT( PROVINCE_NAT nLevel );
bool IntIs_RTAB_SEL_YEARS( int nNumber );
RTAB_SEL_YEARS IntTo_RTAB_SEL_YEARS( int nNumber );
int IntFrom_RTAB_SEL_YEARS( RTAB_SEL_YEARS nLevel );

class ChildMortalityCalibratorTable {
	public:
		ChildMortalityCalibratorTable() { };

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		double	tab23events;
		//   group=float
		real	tab23open0;
		real	tab23close0;
		real	tab23open1;
		real	tab23close1;
		//   group=long
		//   group=ulong
		index	tab23cell;
		index	tab23total_cell;
		//   group=int
		//   group=uint
		logical	tab23dim_filter : 1;
		//   group=short
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

class ChildMortalityCalibrator : public ActorClass {
	public:
		TIME	__time;
		double	__events;
		class ChildMortalityCalibratorStateFunctions	*poStateFunctions;
		class ChildMortalityCalibratorDerivedStates	*poDerivedStates;
		class ChildMortalityCalibratorTable	tableinfo;
		void *operator new( size_t count );

		// links to other actors
		class ChildMortalityCalibrator_mlBabies	*mlBabies;

		// user-supplied data members

		// user-supplied and required functions
		ChildMortalityCalibrator() { };
		void	Finish();
		void Start ( ) ; 

		// special functions generated by the pre-compiler
		ChildMortalityCalibrator( BOOL bInit ) { InitActor(); };
		~ChildMortalityCalibrator() { DeleteActor(); };
		void	InitActor();
		void	DeleteActor();
		void	InitializeStates();
		void	UpdateLinkedStates();
		void	StartSpecial1();
		void	StartSpecial2();
		void	FinishSpecial();
		void	WaitUntil( double target_time );
		ChildMortalityCalibrator	*UpdateTime( );
		void	WaitUntilThisActor(double target_time);
		void	WaitSpecial( TIME wait_time );

		// actor set nodes
		ActorSetNode	*poasTheChildMortalityCalibrator;

		// event functions and variables
		void __fastcall	ScheduleEvent();
		TIME __fastcall	timeScheduleEvent();
		Event	*poScheduleEvent;
		TIME schedule_event_time;
		void __fastcall	ChildMortalityCalibrationEvent();
		TIME __fastcall	timeChildMortalityCalibrationEvent();
		Event	*poChildMortalityCalibrationEvent;

		// event processing functions
		double	EventTime( int nEventNum, int *pnEventInfo );
		void	Implement( int nEventNum, int nEventInfo );

		// state variables and their special functions
		long	Get_actor_id() { return actor_id; };
		double	Set_actor_subsample_weight( double value );
		double	Set_actor_weight( double value );

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		double	actor_subsample_weight;
		double	actor_weight;
		TIME	age;
		double	case_seed;
		double	mort_female_0;
		double	mort_female_1;
		double	mort_female_2;
		double	mort_female_3;
		double	mort_female_4;
		double	mort_male_0;
		double	mort_male_1;
		double	mort_male_2;
		double	mort_male_3;
		double	mort_male_4;
		TIME	time;
		double	FemaleMortalityTrend;
		double	FemaleMortalityTrend_max_value_out;
		double	MaleMortalityTrend;
		double	MaleMortalityTrend_value_out;
		//   group=float
		//   group=long
		long	actor_id;
		long	case_id;
		//   group=ulong
		//   group=int
		//   group=uint
		logical	calibration_is_done : 1;
		logical	actorset_filter_asTheChildMortalityCalibrator : 1;
		SIM_YEAR_RANGE	c_year : 12;
		logical	table_filter_TabMortalityTrendFactor : 1;
		logical	__finished : 1;
		//   group=short
		integer	int_age;
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

class ChildMortalityCalibratorStateFunctions {
	public:
		ChildMortalityCalibratorStateFunctions() { };
		class ChildMortalityCalibrator	*poParent;
		class ChildMortalityCalibratorDerivedStates	*poDerivedStates;

		// state special functions
		double	Set_actor_subsample_weight( double value ){ return poParent->actor_subsample_weight = value; };
		double	Set_actor_weight( double value ){ return poParent->actor_weight = value; };
		TIME	Set_age( TIME value, bool bStartFunction = false );
		logical	Set_calibration_is_done( logical value );
		double	Set_mort_female_0( double value );
		double	Set_mort_female_1( double value );
		double	Set_mort_female_2( double value );
		double	Set_mort_female_3( double value );
		double	Set_mort_female_4( double value );
		double	Set_mort_male_0( double value );
		double	Set_mort_male_1( double value );
		double	Set_mort_male_2( double value );
		double	Set_mort_male_3( double value );
		double	Set_mort_male_4( double value );
		TIME	Set_time( TIME value, bool bStartFunction = false );
		logical	Set_actorset_filter_asTheChildMortalityCalibrator( logical value );
		SIM_YEAR_RANGE	Set_c_year( SIM_YEAR_RANGE value );
		double	Set_FemaleMortalityTrend( double value );
		double	Set_FemaleMortalityTrend_max_value_out( double value );
		double	Set_MaleMortalityTrend( double value );
		double	Set_MaleMortalityTrend_value_out( double value );
		logical	Set_table_filter_TabMortalityTrendFactor( logical value );
};

class ChildMortalityCalibratorDerivedStates {
	public:
		ChildMortalityCalibratorDerivedStates() { };
		class ChildMortalityCalibrator	*poParent;
		class ChildMortalityCalibratorStateFunctions	*poStateFunctions;

		// derived state variables and their special functions
		short	Set_ssint_age( short value );
		short	Set_ssint_time( short value );

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		TIME	ssint_age_time;
		TIME	ssint_time_time;
		//   group=float
		//   group=long
		//   group=ulong
		//   group=int
		//   group=uint
		//   group=short
		short	ssint_age;
		short	ssint_time;
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

class Observation : public ActorClass {
	public:
		TIME	__time;
		double	__events;
		class ObservationStateFunctions	*poStateFunctions;
		class ObservationDerivedStates	*poDerivedStates;
		void *operator new( size_t count );

		// links to other actors

		// user-supplied data members
		double	pmc[rc_size_PERSON_MICRODATA_COLUMNS];

		// user-supplied and required functions
		Observation() { };
		void Finish ( ) ; 
		void Start ( const input_csv & input ) ; 
		void WeightOrKill ( ) ; 

		// special functions generated by the pre-compiler
		Observation( BOOL bInit ) { InitActor(); };
		~Observation() { DeleteActor(); };
		void	InitActor();
		void	DeleteActor();
		void	InitializeStates();
		void	UpdateLinkedStates();
		void	StartSpecial1();
		void	StartSpecial2();
		void	FinishSpecial();
		void	WaitUntil( double target_time );
		Observation	*UpdateTime( );
		void	WaitUntilThisActor(double target_time);
		void	WaitSpecial( TIME wait_time );

		// actor set nodes
		ActorSetNode	*poasAllObservations;

		// event functions and variables

		// event processing functions
		double	EventTime( int nEventNum, int *pnEventInfo );
		void	Implement( int nEventNum, int nEventInfo );

		// state variables and their special functions
		long	Get_actor_id() { return actor_id; };
		double	Set_actor_subsample_weight( double value );
		double	Set_actor_weight( double value );

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		double	actor_subsample_weight;
		double	actor_weight;
		TIME	age;
		double	case_seed;
		TIME	time;
		//   group=float
		//   group=long
		long	actor_id;
		long	case_id;
		//   group=ulong
		//   group=int
		//   group=uint
		logical	actorset_filter_asAllObservations : 1;
		logical	__finished : 1;
		//   group=short
		integer	int_weight;
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

class ObservationStateFunctions {
	public:
		ObservationStateFunctions() { };
		class Observation	*poParent;
		class ObservationDerivedStates	*poDerivedStates;

		// state special functions
		double	Set_actor_subsample_weight( double value ){ return poParent->actor_subsample_weight = value; };
		double	Set_actor_weight( double value ){ return poParent->actor_weight = value; };
		TIME	Set_age( TIME value, bool bStartFunction = false ){ return poParent->age = value; };
		TIME	Set_time( TIME value, bool bStartFunction = false ){ return poParent->time = value; };
		logical	Set_actorset_filter_asAllObservations( logical value );
};

class ObservationDerivedStates {
	public:
		ObservationDerivedStates() { };
		class Observation	*poParent;
		class ObservationStateFunctions	*poStateFunctions;

		// derived state variables and their special functions

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		//   group=float
		//   group=long
		//   group=ulong
		//   group=int
		//   group=uint
		//   group=short
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

class PersonTable {
	public:
		PersonTable() { };

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		double	tab0events;
		double	tab1events;
		double	tab2events;
		double	tab3events;
		double	tab4events;
		double	tab5events;
		double	tab6events;
		double	tab7events;
		double	tab8events;
		double	tab9events;
		double	tab10events;
		double	tab11events;
		double	tab12events;
		double	tab13events;
		double	tab14events;
		double	tab15events;
		double	tab16events;
		double	tab17events;
		double	tab18events;
		double	tab19events;
		double	tab20events;
		double	tab21events;
		double	tab22events;
		double	tab24events;
		double	tab25events;
		double	tab26events;
		double	tab27events;
		double	tab28events;
		double	tab29events;
		double	tab30events;
		double	tab31events;
		double	tab32events;
		double	tab33events;
		double	tab34events;
		double	tab35events;
		//   group=float
		real	tab0open0;
		real	tab0close0;
		real	tab1open0;
		real	tab1close0;
		real	tab2open0;
		real	tab2close0;
		real	tab2open1;
		real	tab2close1;
		real	tab2open2;
		real	tab2close2;
		real	tab3open0;
		real	tab3close0;
		real	tab4open0;
		real	tab4close0;
		real	tab5open0;
		real	tab5close0;
		real	tab6open0;
		real	tab6close0;
		real	tab7open0;
		real	tab7close0;
		real	tab8open0;
		real	tab8close0;
		real	tab8open1;
		real	tab8close1;
		real	tab9open0;
		real	tab9close0;
		real	tab10open0;
		real	tab10close0;
		real	tab10open1;
		real	tab10close1;
		real	tab10open2;
		real	tab10close2;
		real	tab10open3;
		real	tab10close3;
		real	tab11open0;
		real	tab11close0;
		real	tab12open0;
		real	tab12close0;
		real	tab13open0;
		real	tab13close0;
		real	tab14open0;
		real	tab14close0;
		real	tab15open0;
		real	tab15close0;
		real	tab16open0;
		real	tab16close0;
		real	tab16open1;
		real	tab16close1;
		real	tab17open0;
		real	tab17close0;
		real	tab18open0;
		real	tab18close0;
		real	tab18open1;
		real	tab18close1;
		real	tab19open0;
		real	tab19close0;
		real	tab19open1;
		real	tab19close1;
		real	tab19open2;
		real	tab19close2;
		real	tab19open3;
		real	tab19close3;
		real	tab20open0;
		real	tab20close0;
		real	tab20open1;
		real	tab20close1;
		real	tab21open0;
		real	tab21close0;
		real	tab21open1;
		real	tab21close1;
		real	tab22open0;
		real	tab22close0;
		real	tab22open1;
		real	tab22close1;
		real	tab24open0;
		real	tab24close0;
		real	tab24open1;
		real	tab24close1;
		real	tab25open0;
		real	tab25close0;
		real	tab26open0;
		real	tab26close0;
		real	tab26open1;
		real	tab26close1;
		real	tab27open0;
		real	tab27close0;
		real	tab27open1;
		real	tab27close1;
		real	tab27open2;
		real	tab27close2;
		real	tab27open3;
		real	tab27close3;
		real	tab27open4;
		real	tab27close4;
		real	tab28open0;
		real	tab28close0;
		real	tab28open1;
		real	tab28close1;
		real	tab28open2;
		real	tab28close2;
		real	tab28open3;
		real	tab28close3;
		real	tab29open0;
		real	tab29close0;
		real	tab29open1;
		real	tab29close1;
		real	tab29open2;
		real	tab29close2;
		real	tab29open3;
		real	tab29close3;
		real	tab30open0;
		real	tab30close0;
		real	tab30open1;
		real	tab30close1;
		real	tab30open2;
		real	tab30close2;
		real	tab31open0;
		real	tab31close0;
		real	tab31open1;
		real	tab31close1;
		real	tab31open2;
		real	tab31close2;
		real	tab32open0;
		real	tab32close0;
		real	tab32open1;
		real	tab32close1;
		real	tab32open2;
		real	tab32close2;
		real	tab33open0;
		real	tab33close0;
		real	tab33open1;
		real	tab33close1;
		real	tab33open2;
		real	tab33close2;
		real	tab33open3;
		real	tab33close3;
		real	tab33open4;
		real	tab33close4;
		real	tab34open0;
		real	tab34close0;
		real	tab34open1;
		real	tab34close1;
		real	tab35open0;
		real	tab35close0;
		real	tab35open1;
		real	tab35close1;
		real	tab35open2;
		real	tab35close2;
		real	tab35open3;
		real	tab35close3;
		real	tab35open4;
		real	tab35close4;
		real	tab35open5;
		real	tab35close5;
		real	tab35open6;
		real	tab35close6;
		real	tab35open7;
		real	tab35close7;
		real	tab35open8;
		real	tab35close8;
		real	tab35open9;
		real	tab35close9;
		real	tab35open10;
		real	tab35close10;
		//   group=long
		//   group=ulong
		index	tab0cell;
		index	tab0total_cell;
		index	tab1cell;
		index	tab1total_cell;
		index	tab2cell;
		index	tab2total_cell;
		index	tab3cell;
		index	tab3total_cell;
		index	tab4cell;
		index	tab4total_cell;
		index	tab5cell;
		index	tab5total_cell;
		index	tab6cell;
		index	tab6total_cell;
		index	tab7cell;
		index	tab7total_cell;
		index	tab8cell;
		index	tab8total_cell;
		index	tab9cell;
		index	tab9total_cell;
		index	tab10cell;
		index	tab10total_cell;
		index	tab11cell;
		index	tab11total_cell;
		index	tab12cell;
		index	tab12total_cell;
		index	tab13cell;
		index	tab13total_cell;
		index	tab14cell;
		index	tab14total_cell;
		index	tab15cell;
		index	tab15total_cell;
		index	tab16cell;
		index	tab16total_cell;
		index	tab17cell;
		index	tab17total_cell;
		index	tab18cell;
		index	tab18total_cell;
		index	tab19cell;
		index	tab19total_cell;
		index	tab20cell;
		index	tab20total_cell;
		index	tab21cell;
		index	tab21total_cell;
		index	tab22cell;
		index	tab22total_cell;
		index	tab24cell;
		index	tab24total_cell;
		index	tab25cell;
		index	tab25total_cell;
		index	tab26cell;
		index	tab26total_cell;
		index	tab27cell;
		index	tab27total_cell;
		index	tab28cell;
		index	tab28total_cell;
		index	tab29cell;
		index	tab29total_cell;
		index	tab30cell;
		index	tab30total_cell;
		index	tab31cell;
		index	tab31total_cell;
		index	tab32cell;
		index	tab32total_cell;
		index	tab33cell;
		index	tab33total_cell;
		index	tab34cell;
		index	tab34total_cell;
		index	tab35cell;
		index	tab35total_cell;
		//   group=int
		//   group=uint
		logical	tab0dim_filter : 1;
		logical	tab1dim_filter : 1;
		logical	tab2dim_filter : 1;
		logical	tab3dim_filter : 1;
		logical	tab4dim_filter : 1;
		logical	tab5dim_filter : 1;
		logical	tab6dim_filter : 1;
		logical	tab7dim_filter : 1;
		logical	tab8dim_filter : 1;
		logical	tab9dim_filter : 1;
		logical	tab10dim_filter : 1;
		logical	tab11dim_filter : 1;
		logical	tab12dim_filter : 1;
		logical	tab13dim_filter : 1;
		logical	tab14dim_filter : 1;
		logical	tab15dim_filter : 1;
		logical	tab16dim_filter : 1;
		logical	tab17dim_filter : 1;
		logical	tab18dim_filter : 1;
		logical	tab19dim_filter : 1;
		logical	tab20dim_filter : 1;
		logical	tab21dim_filter : 1;
		logical	tab22dim_filter : 1;
		logical	tab24dim_filter : 1;
		logical	tab25dim_filter : 1;
		logical	tab26dim_filter : 1;
		logical	tab27dim_filter : 1;
		logical	tab28dim_filter : 1;
		logical	tab29dim_filter : 1;
		logical	tab30dim_filter : 1;
		logical	tab31dim_filter : 1;
		logical	tab32dim_filter : 1;
		logical	tab33dim_filter : 1;
		logical	tab34dim_filter : 1;
		logical	tab35dim_filter : 1;
		//   group=short
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

class Person : public ActorClass {
	public:
		TIME	__time;
		double	__events;
		class PersonStateFunctions	*poStateFunctions;
		class PersonDerivedStates	*poDerivedStates;
		class PersonTable	tableinfo;
		void *operator new( size_t count );

		// links to other actors
		class ChildMortalityCalibrator	*lChildMortalityCalibrator;
		void	Set_lChildMortalityCalibrator( ChildMortalityCalibrator *value );
		class Person	*lMother;
		void	Set_lMother( Person *value );
		class Person_mlChild	*mlChild;

		// user-supplied data members

		// user-supplied and required functions
		Person() { };
		void DecidePrimaryEntry ( double dProb ) ; 
		void DecidePrimaryGrad ( double dProb ) ; 
		void Finish ( ) ; 
		void GetImmigrantCharacteristics ( ) ; 
		void GetStartCharacteristics ( ) ; 
		void SetImmigrantPrimaryEducationStatus ( ) ; 
		void SetImmigrantUnionFormation ( ) ; 
		void Start ( long PersonNr , Person * peMother ) ; 
		void WriteMicroRecord_Start ( ) ; 
		void	StartIMPLEMENT_HOOK();

		// special functions generated by the pre-compiler
		Person( BOOL bInit ) { InitActor(); };
		~Person() { DeleteActor(); };
		void	InitActor();
		void	DeleteActor();
		void	InitializeStates();
		void	UpdateLinkedStates();
		void	StartSpecial1();
		void	StartSpecial2();
		void	FinishSpecial();
		void	WaitUntil( double target_time );
		Person	*UpdateTime( );
		void	WaitUntilThisActor(double target_time);
		void	WaitSpecial( TIME wait_time );

		// actor set nodes
		ActorSetNode	*poasAllPersons;
		ActorSetNode	*poasFemaleAgeEduc;
		ActorSetNode	*poasForeignersAgeSexProv;
		ActorSetNode	*poasForeignersPrimAgeSexProv;
		ActorSetNode	*poasPotentialMothers;
		ActorSetNode	*poasPotentialMothersByAge;
		ActorSetNode	*poasPrimaryEntryCohort;
		ActorSetNode	*poasPrimaryGraduationCohort;
		ActorSetNode	*poasResidentsAgeSex;
		ActorSetNode	*poasResidentsPrimAgeSex;

		// event functions and variables
		void __fastcall	ScheduleEvent();
		TIME __fastcall	timeScheduleEvent();
		Event	*poScheduleEvent;
		TIME schedule_event_time;
		void __fastcall	BirthEvent();
		TIME __fastcall	timeBirthEvent();
		Event	*poBirthEvent;
		void __fastcall	ChildMortalityEvent();
		TIME __fastcall	timeChildMortalityEvent();
		Event	*poChildMortalityEvent;
		void __fastcall	EmigrationEvent();
		TIME __fastcall	timeEmigrationEvent();
		Event	*poEmigrationEvent;
		void __fastcall	MigrationEvent();
		TIME __fastcall	timeMigrationEvent();
		Event	*poMigrationEvent;
		void __fastcall	MortalityEvent();
		TIME __fastcall	timeMortalityEvent();
		Event	*poMortalityEvent;
		void __fastcall	SetAliveEvent();
		TIME __fastcall	timeSetAliveEvent();
		Event	*poSetAliveEvent;
		void __fastcall	UnionFormationEvent();
		TIME __fastcall	timeUnionFormationEvent();
		Event	*poUnionFormationEvent;
		void __fastcall	WriteMicroRecord();
		TIME __fastcall	timeWriteMicroRecord();
		Event	*poWriteMicroRecord;

		// event processing functions
		double	EventTime( int nEventNum, int *pnEventInfo );
		void	Implement( int nEventNum, int nEventInfo );

		// state variables and their special functions
		long	Get_actor_id() { return actor_id; };
		double	Set_actor_subsample_weight( double value );
		double	Set_actor_weight( double value );

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		double	actor_subsample_weight;
		double	actor_weight;
		TIME	age;
		double	case_seed;
		TIME	time;
		TIME	time_microdata_output;
		TIME	time_of_birth;
		TIME	time_of_immigration;
		TIME	time_of_last_birth;
		TIME	time_of_union_formation;
		double	time_since_last_birth_start;
		double	child_mortality;
		double	union_formation_hazard;
		double	union_formation_hazard_max_value_in;
		double	lChildMortalityCalibrator_mort_female_0;
		double	lChildMortalityCalibrator_mort_female_1;
		double	lChildMortalityCalibrator_mort_female_2;
		double	lChildMortalityCalibrator_mort_female_3;
		double	lChildMortalityCalibrator_mort_female_4;
		double	lChildMortalityCalibrator_mort_male_0;
		double	lChildMortalityCalibrator_mort_male_1;
		double	lChildMortalityCalibrator_mort_male_2;
		double	lChildMortalityCalibrator_mort_male_3;
		double	lChildMortalityCalibrator_mort_male_4;
		//   group=float
		//   group=long
		long	actor_id;
		long	case_id;
		//   group=ulong
		//   group=int
		int	birth_age_part;
		int	time_in_parity;
		int	time_since_last_birth;
		PRIMARY_LEVEL	educ_mother : 3;
		PERSON_TYPE	person_type : 3;
		PRIMARY_LEVEL	primary_level : 3;
		PROVINCE_INT	province_birth : 5;
		PROVINCE_NAT	province_nat : 5;
		RTAB_COL	rtab_column : 1;
		SEX	sex : 2;
		RTAB_REG	rtab_reg : 2;
		RTAB_SEL_YEARS	rtab_sel_years : 3;
		UNION_STATUS	union_status : 2;
		//   group=uint
		logical	alive : 1;
		logical	has_emigrated : 1;
		logical	in_this_parity : 1;
		logical	in_union : 1;
		PARITY_RANGE	infant_deaths : 4;
		logical	mother_educ_0 : 1;
		logical	mother_educ_1 : 1;
		logical	mother_known : 1;
		logical	mother_max_14 : 1;
		logical	mother_max_16 : 1;
		logical	override_mode : 1;
		PARITY_RANGE	parity : 4;
		logical	set_alive : 1;
		logical	actorset_filter_asAllPersons : 1;
		logical	actorset_filter_asFemaleAgeEduc : 1;
		logical	actorset_filter_asForeignersAgeSexProv : 1;
		logical	actorset_filter_asForeignersPrimAgeSexProv : 1;
		logical	actorset_filter_asPotentialMothers : 1;
		logical	actorset_filter_asPotentialMothersByAge : 1;
		logical	actorset_filter_asPrimaryEntryCohort : 1;
		logical	actorset_filter_asPrimaryGraduationCohort : 1;
		logical	actorset_filter_asResidentsAgeSex : 1;
		logical	actorset_filter_asResidentsPrimAgeSex : 1;
		AGE_UNION	age_union : 6;
		logical	child_mortality_overrides : 1;
		logical	count_parity : 1;
		FERTILE_AGE_RANGE	fert_age : 6;
		FERTILE_AGE_RANGE	fertile_age : 6;
		logical	in_capital : 1;
		logical	in_projected_time : 1;
		logical	in_rtab_sel_years : 1;
		logical	in_rtab_years : 1;
		logical	in_simulation_time : 1;
		logical	in_this_parity_in_simulation : 1;
		AGE_RANGE	integer_age : 7;
		logical	is_act_age : 1;
		logical	is_aded0 : 1;
		logical	is_aded1 : 1;
		logical	is_aded2 : 1;
		logical	is_child : 1;
		logical	is_old : 1;
		logical	is_potential_mother : 1;
		RTAB_YEARS	rtab_years : 12;
		CHILD_MORT_RANGE	sim_cmr : 4;
		SIM_YEAR_RANGE	sim_year : 12;
		SIM_YEAR_RANGE	sim_yob : 12;
		logical	table_filter_RTAB_AgeAtFirstUnionFormation : 1;
		logical	table_filter_RTAB_BirthsByAgeMother : 1;
		logical	table_filter_RTAB_BirthsByEducMother : 1;
		logical	table_filter_RTAB_BirthsByRegion : 1;
		logical	table_filter_RTAB_DeathsByRegion : 1;
		logical	table_filter_RTAB_EducationActivePop : 1;
		logical	table_filter_RTAB_PopPyramidByEduc : 1;
		logical	table_filter_RTAB_PrimSchoolEntries : 1;
		logical	table_filter_RTAB_PrimSchoolGraduations : 1;
		logical	table_filter_RTAB_TabDeathsByAgeGroup : 1;
		logical	table_filter_RTAB_TotalPopReg : 1;
		logical	table_filter_RTAB_TotalPopWillaya : 1;
		logical	table_filter_RTAB_UnionFormationByAge : 1;
		logical	table_filter_TabAgeAtBirth : 1;
		logical	table_filter_TabAgeAtFirstUnionFormation : 1;
		logical	table_filter_TabBirthsByAgeGroup : 1;
		logical	table_filter_TabCohortChildMortality : 1;
		logical	table_filter_TabDeathRates : 1;
		logical	table_filter_TabDeathsByAgeGroup : 1;
		logical	table_filter_TabEduc12ByPob : 1;
		logical	table_filter_TabEduc12ByPor : 1;
		logical	table_filter_TabEduc2015 : 1;
		logical	table_filter_TabEduc2040 : 1;
		logical	table_filter_TabEduc2065 : 1;
		logical	table_filter_TabEmigration : 1;
		logical	table_filter_TabFertilityRates : 1;
		logical	table_filter_TabImmigrationSexProvinceAge : 1;
		logical	table_filter_TabInternalMigrationRate : 1;
		logical	table_filter_TabMothersExperience : 1;
		logical	table_filter_TabNumberBirths : 1;
		logical	table_filter_TabPopProvAgeEducSex : 1;
		logical	table_filter_TabPopulationAgeSexProvince : 1;
		logical	table_filter_TabSimulatedStartingPopulation : 1;
		logical	table_filter_TabTotalPopulationSexProvince : 1;
		logical	table_filter_TabUnionFormation : 1;
		ALL_YEAR_RANGE	year_of_birth : 12;
		logical	__finished : 1;
		//   group=short
		integer	age_at_last_move;
		integer	number_moves;
		integer	calendar_year;
		//   group=ushort
		counter	table_unit_TabImmigrationSexProvinceAge;
		counter	table_unit_TabMothersExperience;
		counter	table_unit_TabSimulatedStartingPopulation;
		//   group=char
		//   group=uchar
		// end MembersDump

};

class PersonStateFunctions {
	public:
		PersonStateFunctions() { };
		class Person	*poParent;
		class PersonDerivedStates	*poDerivedStates;

		// state special functions
		double	Set_actor_subsample_weight( double value ){ return poParent->actor_subsample_weight = value; };
		double	Set_actor_weight( double value ){ return poParent->actor_weight = value; };
		TIME	Set_age( TIME value, bool bStartFunction = false );
		integer	Set_age_at_last_move( integer value );
		logical	Set_alive( logical value );
		PRIMARY_LEVEL	Set_educ_mother( PRIMARY_LEVEL value );
		logical	Set_has_emigrated( logical value );
		logical	Set_in_this_parity( logical value );
		logical	Set_in_union( logical value );
		PARITY_RANGE	Set_infant_deaths( PARITY_RANGE value );
		PARITY_RANGE	PlusPlusSet_infant_deaths() { PARITY_RANGE old_value; old_value = poParent->infant_deaths;
			Set_infant_deaths((PARITY_RANGE) (poParent->infant_deaths + 1)); return old_value; };
		logical	Set_mother_educ_0( logical value );
		logical	Set_mother_educ_1( logical value );
		logical	Set_mother_known( logical value );
		logical	Set_mother_max_14( logical value );
		logical	Set_mother_max_16( logical value );
		integer	Set_number_moves( integer value );
		integer	PlusPlusSet_number_moves() { integer old_value; old_value = poParent->number_moves;
			Set_number_moves((integer) (poParent->number_moves + 1)); return old_value; };
		logical	Set_override_mode( logical value );
		PARITY_RANGE	Set_parity( PARITY_RANGE value );
		PARITY_RANGE	PlusPlusSet_parity() { PARITY_RANGE old_value; old_value = poParent->parity;
			Set_parity((PARITY_RANGE) (poParent->parity + 1)); return old_value; };
		PERSON_TYPE	Set_person_type( PERSON_TYPE value );
		PRIMARY_LEVEL	Set_primary_level( PRIMARY_LEVEL value );
		PROVINCE_INT	Set_province_birth( PROVINCE_INT value );
		PROVINCE_NAT	Set_province_nat( PROVINCE_NAT value );
		RTAB_COL	Set_rtab_column( RTAB_COL value );
		logical	Set_set_alive( logical value );
		SEX	Set_sex( SEX value );
		counter	Set_table_unit_TabImmigrationSexProvinceAge( counter value );
		counter	Set_table_unit_TabMothersExperience( counter value );
		counter	Set_table_unit_TabSimulatedStartingPopulation( counter value );
		TIME	Set_time( TIME value, bool bStartFunction = false );
		TIME	Set_time_microdata_output( TIME value );
		TIME	Set_time_of_birth( TIME value );
		TIME	Set_time_of_union_formation( TIME value );
		double	Set_time_since_last_birth_start( double value );
		logical	Set_actorset_filter_asAllPersons( logical value );
		logical	Set_actorset_filter_asFemaleAgeEduc( logical value );
		logical	Set_actorset_filter_asForeignersAgeSexProv( logical value );
		logical	Set_actorset_filter_asForeignersPrimAgeSexProv( logical value );
		logical	Set_actorset_filter_asPotentialMothers( logical value );
		logical	Set_actorset_filter_asPotentialMothersByAge( logical value );
		logical	Set_actorset_filter_asPrimaryEntryCohort( logical value );
		logical	Set_actorset_filter_asPrimaryGraduationCohort( logical value );
		logical	Set_actorset_filter_asResidentsAgeSex( logical value );
		logical	Set_actorset_filter_asResidentsPrimAgeSex( logical value );
		AGE_UNION	Set_age_union( AGE_UNION value );
		int	Set_birth_age_part( int value );
		integer	Set_calendar_year( integer value );
		double	Set_child_mortality( double value );
		logical	Set_child_mortality_overrides( logical value );
		logical	Set_count_parity( logical value );
		FERTILE_AGE_RANGE	Set_fert_age( FERTILE_AGE_RANGE value );
		FERTILE_AGE_RANGE	Set_fertile_age( FERTILE_AGE_RANGE value );
		logical	Set_in_capital( logical value );
		logical	Set_in_projected_time( logical value );
		logical	Set_in_rtab_sel_years( logical value );
		logical	Set_in_rtab_years( logical value );
		logical	Set_in_simulation_time( logical value );
		logical	Set_in_this_parity_in_simulation( logical value );
		AGE_RANGE	Set_integer_age( AGE_RANGE value );
		logical	Set_is_aded0( logical value );
		logical	Set_is_aded1( logical value );
		logical	Set_is_aded2( logical value );
		logical	Set_is_child( logical value );
		logical	Set_is_old( logical value );
		logical	Set_is_potential_mother( logical value );
		RTAB_REG	Set_rtab_reg( RTAB_REG value );
		RTAB_SEL_YEARS	Set_rtab_sel_years( RTAB_SEL_YEARS value );
		RTAB_YEARS	Set_rtab_years( RTAB_YEARS value );
		CHILD_MORT_RANGE	Set_sim_cmr( CHILD_MORT_RANGE value );
		SIM_YEAR_RANGE	Set_sim_year( SIM_YEAR_RANGE value );
		SIM_YEAR_RANGE	Set_sim_yob( SIM_YEAR_RANGE value );
		logical	Set_table_filter_RTAB_AgeAtFirstUnionFormation( logical value );
		logical	Set_table_filter_RTAB_BirthsByAgeMother( logical value );
		logical	Set_table_filter_RTAB_BirthsByEducMother( logical value );
		logical	Set_table_filter_RTAB_BirthsByRegion( logical value );
		logical	Set_table_filter_RTAB_DeathsByRegion( logical value );
		logical	Set_table_filter_RTAB_EducationActivePop( logical value );
		logical	Set_table_filter_RTAB_PopPyramidByEduc( logical value );
		logical	Set_table_filter_RTAB_PrimSchoolEntries( logical value );
		logical	Set_table_filter_RTAB_PrimSchoolGraduations( logical value );
		logical	Set_table_filter_RTAB_TabDeathsByAgeGroup( logical value );
		logical	Set_table_filter_RTAB_TotalPopReg( logical value );
		logical	Set_table_filter_RTAB_TotalPopWillaya( logical value );
		logical	Set_table_filter_RTAB_UnionFormationByAge( logical value );
		logical	Set_table_filter_TabAgeAtBirth( logical value );
		logical	Set_table_filter_TabAgeAtFirstUnionFormation( logical value );
		logical	Set_table_filter_TabBirthsByAgeGroup( logical value );
		logical	Set_table_filter_TabCohortChildMortality( logical value );
		logical	Set_table_filter_TabDeathRates( logical value );
		logical	Set_table_filter_TabDeathsByAgeGroup( logical value );
		logical	Set_table_filter_TabEduc12ByPob( logical value );
		logical	Set_table_filter_TabEduc12ByPor( logical value );
		logical	Set_table_filter_TabEduc2015( logical value );
		logical	Set_table_filter_TabEduc2040( logical value );
		logical	Set_table_filter_TabEduc2065( logical value );
		logical	Set_table_filter_TabEmigration( logical value );
		logical	Set_table_filter_TabFertilityRates( logical value );
		logical	Set_table_filter_TabImmigrationSexProvinceAge( logical value );
		logical	Set_table_filter_TabInternalMigrationRate( logical value );
		logical	Set_table_filter_TabMothersExperience( logical value );
		logical	Set_table_filter_TabNumberBirths( logical value );
		logical	Set_table_filter_TabPopProvAgeEducSex( logical value );
		logical	Set_table_filter_TabPopulationAgeSexProvince( logical value );
		logical	Set_table_filter_TabSimulatedStartingPopulation( logical value );
		logical	Set_table_filter_TabTotalPopulationSexProvince( logical value );
		logical	Set_table_filter_TabUnionFormation( logical value );
		int	Set_time_in_parity( int value );
		int	Set_time_since_last_birth( int value );
		double	Set_union_formation_hazard( double value );
		double	Set_union_formation_hazard_max_value_in( double value );
		UNION_STATUS	Set_union_status( UNION_STATUS value );
		ALL_YEAR_RANGE	Set_year_of_birth( ALL_YEAR_RANGE value );
		double	Set_lChildMortalityCalibrator_mort_female_0( double value );
		double	Set_lChildMortalityCalibrator_mort_female_1( double value );
		double	Set_lChildMortalityCalibrator_mort_female_2( double value );
		double	Set_lChildMortalityCalibrator_mort_female_3( double value );
		double	Set_lChildMortalityCalibrator_mort_female_4( double value );
		double	Set_lChildMortalityCalibrator_mort_male_0( double value );
		double	Set_lChildMortalityCalibrator_mort_male_1( double value );
		double	Set_lChildMortalityCalibrator_mort_male_2( double value );
		double	Set_lChildMortalityCalibrator_mort_male_3( double value );
		double	Set_lChildMortalityCalibrator_mort_male_4( double value );
};

class PersonDerivedStates {
	public:
		PersonDerivedStates() { };
		class Person	*poParent;
		class PersonStateFunctions	*poStateFunctions;

		// derived state variables and their special functions
		TIME	Get_active_spell_duration_in_this_parity_in_simulation_TRUE() { return active_spell_duration_in_this_parity_in_simulation_TRUE + ( ( poParent->in_this_parity_in_simulation == TRUE ) ? ( poParent->__time - active_spell_duration_in_this_parity_in_simulation_TRUE_time ) : 0 ); };
		counter	Set_changes_parity( counter value );
		TIME	Set_duration_( TIME value );
		TIME	Get_duration_() { return duration_ + poParent->__time - duration__time; };
		TIME	Set_duration_in_union_FALSE( TIME value );
		TIME	Get_duration_in_union_FALSE() { return duration_in_union_FALSE + ( ( poParent->in_union == FALSE ) ? ( poParent->__time - duration_in_union_FALSE_time ) : 0 ); };
		TIME	Set_duration_in_union_TRUE( TIME value );
		TIME	Get_duration_in_union_TRUE() { return duration_in_union_TRUE + ( ( poParent->in_union == TRUE ) ? ( poParent->__time - duration_in_union_TRUE_time ) : 0 ); };
		TIME	Set_duration_is_aded0_TRUE( TIME value );
		TIME	Get_duration_is_aded0_TRUE() { return duration_is_aded0_TRUE + ( ( poParent->is_aded0 == TRUE ) ? ( poParent->__time - duration_is_aded0_TRUE_time ) : 0 ); };
		TIME	Set_duration_is_aded1_TRUE( TIME value );
		TIME	Get_duration_is_aded1_TRUE() { return duration_is_aded1_TRUE + ( ( poParent->is_aded1 == TRUE ) ? ( poParent->__time - duration_is_aded1_TRUE_time ) : 0 ); };
		TIME	Set_duration_is_aded2_TRUE( TIME value );
		TIME	Get_duration_is_aded2_TRUE() { return duration_is_aded2_TRUE + ( ( poParent->is_aded2 == TRUE ) ? ( poParent->__time - duration_is_aded2_TRUE_time ) : 0 ); };
		TIME	Set_duration_is_child_TRUE( TIME value );
		TIME	Get_duration_is_child_TRUE() { return duration_is_child_TRUE + ( ( poParent->is_child == TRUE ) ? ( poParent->__time - duration_is_child_TRUE_time ) : 0 ); };
		TIME	Set_duration_is_old_TRUE( TIME value );
		TIME	Get_duration_is_old_TRUE() { return duration_is_old_TRUE + ( ( poParent->is_old == TRUE ) ? ( poParent->__time - duration_is_old_TRUE_time ) : 0 ); };
		TIME	Set_duration_primary_level_PL_ENTER( TIME value );
		TIME	Get_duration_primary_level_PL_ENTER() { return duration_primary_level_PL_ENTER + ( ( poParent->primary_level == PL_ENTER ) ? ( poParent->__time - duration_primary_level_PL_ENTER_time ) : 0 ); };
		TIME	Set_duration_primary_level_PL_GRAD( TIME value );
		TIME	Get_duration_primary_level_PL_GRAD() { return duration_primary_level_PL_GRAD + ( ( poParent->primary_level == PL_GRAD ) ? ( poParent->__time - duration_primary_level_PL_GRAD_time ) : 0 ); };
		TIME	Set_duration_primary_level_PL_NO( TIME value );
		TIME	Get_duration_primary_level_PL_NO() { return duration_primary_level_PL_NO + ( ( poParent->primary_level == PL_NO ) ? ( poParent->__time - duration_primary_level_PL_NO_time ) : 0 ); };
		TIME	Get_duration_set_alive_TRUE() { return duration_set_alive_TRUE + ( ( poParent->set_alive == TRUE ) ? ( poParent->__time - duration_set_alive_TRUE_time ) : 0 ); };
		counter	Set_entrances_primary_level_PL_GRAD( counter value );
		counter	Set_entrances_set_alive_TRUE( counter value );
		AGE5_PART	Set_split_integer_age_AGE5_PART( AGE5_PART value );
		AGE_15_16_18	Set_split_integer_age_AGE_15_16_18( AGE_15_16_18 value );
		AGE_15_17	Set_split_integer_age_AGE_15_17( AGE_15_17 value );
		AGE_FIVE	Set_split_integer_age_AGE_FIVE( AGE_FIVE value );
		AGE_MORT	Set_split_integer_age_AGE_MORT( AGE_MORT value );
		AGE_RTAB_MORT	Set_split_integer_age_AGE_RTAB_MORT( AGE_RTAB_MORT value );
		BIRTH_AGE_PART	Set_split_integer_age_BIRTH_AGE_PART( BIRTH_AGE_PART value );
		DUR_TIME_IN_PARITY	Set_split_time_since_last_birth_DUR_TIME_IN_PARITY( DUR_TIME_IN_PARITY value );
		short	Set_ssint_active_spell_duration_in_this_parity_in_simulation_TRUE( short value );
		short	Set_ssint_age( short value );
		short	Set_ssint_time( short value );
		LITTLE_TIME	Set_sssplit_duration_set_alive_TRUE_LITTLE_TIME( LITTLE_TIME value );
		counter	Set_transitions_alive_TRUE_FALSE( counter value );
		counter	Set_transitions_has_emigrated_FALSE_TRUE( counter value );
		counter	Set_transitions_in_union_FALSE_TRUE( counter value );
		counter	Set_transitions_integer_age_14_15( counter value );
		counter	Set_transitions_integer_age_19_20( counter value );
		counter	Set_transitions_integer_age_9_10( counter value );
		counter	Set_transitions_parity_0_1( counter value );
		counter	Set_transitions_primary_level_PL_ENTER_PL_GRAD( counter value );
		counter	Set_transitions_primary_level_PL_NO_PL_ENTER( counter value );
		logical	Set_trigger_entrances_in_projected_time_TRUE( logical value );
		logical	Set_trigger_entrances_set_alive_TRUE( logical value );
		real	Set_value_at_changes_parity_age( real value );
		real	Set_value_at_transitions_in_union_FALSE_TRUE_age( real value );
		real	Set_value_at_transitions_parity_0_1_age( real value );

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		TIME	active_spell_duration_in_this_parity_in_simulation_TRUE;
		TIME	active_spell_duration_in_this_parity_in_simulation_TRUE_time;
		TIME	completed_spell_duration_in_this_parity_in_simulation_TRUE;
		TIME	duration_;
		TIME	duration__time;
		TIME	duration_in_union_FALSE;
		TIME	duration_in_union_FALSE_time;
		TIME	duration_in_union_TRUE;
		TIME	duration_in_union_TRUE_time;
		TIME	duration_is_aded0_TRUE;
		TIME	duration_is_aded0_TRUE_time;
		TIME	duration_is_aded1_TRUE;
		TIME	duration_is_aded1_TRUE_time;
		TIME	duration_is_aded2_TRUE;
		TIME	duration_is_aded2_TRUE_time;
		TIME	duration_is_child_TRUE;
		TIME	duration_is_child_TRUE_time;
		TIME	duration_is_old_TRUE;
		TIME	duration_is_old_TRUE_time;
		TIME	duration_primary_level_PL_ENTER;
		TIME	duration_primary_level_PL_ENTER_time;
		TIME	duration_primary_level_PL_GRAD;
		TIME	duration_primary_level_PL_GRAD_time;
		TIME	duration_primary_level_PL_NO;
		TIME	duration_primary_level_PL_NO_time;
		TIME	duration_set_alive_TRUE;
		TIME	duration_set_alive_TRUE_time;
		TIME	ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time;
		TIME	ssint_age_time;
		TIME	ssint_time_time;
		TIME	sssplit_duration_set_alive_TRUE_LITTLE_TIME_time;
		TIME	trigger_entrances_in_projected_time_TRUE_time;
		TIME	trigger_entrances_set_alive_TRUE_time;
		//   group=float
		real	value_at_changes_parity_age;
		real	value_at_transitions_in_union_FALSE_TRUE_age;
		real	value_at_transitions_parity_0_1_age;
		//   group=long
		//   group=ulong
		//   group=int
		//   group=uint
		AGE5_PART	split_integer_age_AGE5_PART : 4;
		AGE_15_16_18	split_integer_age_AGE_15_16_18 : 2;
		AGE_15_17	split_integer_age_AGE_15_17 : 2;
		AGE_FIVE	split_integer_age_AGE_FIVE : 5;
		AGE_MORT	split_integer_age_AGE_MORT : 3;
		AGE_RTAB_MORT	split_integer_age_AGE_RTAB_MORT : 1;
		BIRTH_AGE_PART	split_integer_age_BIRTH_AGE_PART : 2;
		DUR_TIME_IN_PARITY	split_time_since_last_birth_DUR_TIME_IN_PARITY : 3;
		LITTLE_TIME	sssplit_duration_set_alive_TRUE_LITTLE_TIME : 1;
		logical	trigger_entrances_in_projected_time_TRUE : 1;
		logical	trigger_entrances_set_alive_TRUE : 1;
		//   group=short
		short	ssint_active_spell_duration_in_this_parity_in_simulation_TRUE;
		short	ssint_age;
		short	ssint_time;
		//   group=ushort
		counter	changes_parity;
		counter	entrances_primary_level_PL_GRAD;
		counter	entrances_set_alive_TRUE;
		counter	transitions_alive_TRUE_FALSE;
		counter	transitions_has_emigrated_FALSE_TRUE;
		counter	transitions_in_union_FALSE_TRUE;
		counter	transitions_integer_age_14_15;
		counter	transitions_integer_age_19_20;
		counter	transitions_integer_age_9_10;
		counter	transitions_parity_0_1;
		counter	transitions_primary_level_PL_ENTER_PL_GRAD;
		counter	transitions_primary_level_PL_NO_PL_ENTER;
		//   group=char
		//   group=uchar
		// end MembersDump

};

class PrimarySchool : public ActorClass {
	public:
		TIME	__time;
		double	__events;
		class PrimarySchoolStateFunctions	*poStateFunctions;
		class PrimarySchoolDerivedStates	*poDerivedStates;
		void *operator new( size_t count );

		// links to other actors

		// user-supplied data members
		double	CalibrationFactorPrimaryEntry[rc_size_SEX][rc_size_PROVINCE_INT];
		double	CalibrationFactorPrimaryGrad[rc_size_SEX][rc_size_PROVINCE_INT];
		double	LastProbabilityPrimaryEntry[rc_size_SEX][rc_size_PROVINCE_INT];
		double	LastProbabilityPrimaryGrad[rc_size_SEX][rc_size_PROVINCE_INT];

		// user-supplied and required functions
		PrimarySchool() { };
		double AdjustedProbability ( double dProb , double dLogOddEduc , double dLogOddAdjust ) ; 
		void Finish ( ) ; 
		void PrimarySchoolEntry ( ) ; 
		void PrimarySchoolGraduation ( ) ; 
		void Start ( ) ; 

		// special functions generated by the pre-compiler
		PrimarySchool( BOOL bInit ) { InitActor(); };
		~PrimarySchool() { DeleteActor(); };
		void	InitActor();
		void	DeleteActor();
		void	InitializeStates();
		void	UpdateLinkedStates();
		void	StartSpecial1();
		void	StartSpecial2();
		void	FinishSpecial();
		void	WaitUntil( double target_time );
		PrimarySchool	*UpdateTime( );
		void	WaitUntilThisActor(double target_time);
		void	WaitSpecial( TIME wait_time );

		// event functions and variables
		void __fastcall	AssignInitialPrimary();
		TIME __fastcall	timeAssignInitialPrimary();
		Event	*poAssignInitialPrimary;
		void __fastcall	NextSchoolYearEvent();
		TIME __fastcall	timeNextSchoolYearEvent();
		Event	*poNextSchoolYearEvent;

		// event processing functions
		double	EventTime( int nEventNum, int *pnEventInfo );
		void	Implement( int nEventNum, int nEventInfo );

		// state variables and their special functions
		long	Get_actor_id() { return actor_id; };
		double	Set_actor_subsample_weight( double value );
		double	Set_actor_weight( double value );

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		double	actor_subsample_weight;
		double	actor_weight;
		TIME	age;
		TIME	assign_primary_time;
		double	case_seed;
		TIME	next_school_year;
		TIME	time;
		//   group=float
		//   group=long
		long	actor_id;
		long	case_id;
		//   group=ulong
		//   group=int
		int	school_year;
		//   group=uint
		logical	__finished : 1;
		//   group=short
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

class PrimarySchoolStateFunctions {
	public:
		PrimarySchoolStateFunctions() { };
		class PrimarySchool	*poParent;
		class PrimarySchoolDerivedStates	*poDerivedStates;

		// state special functions
		double	Set_actor_subsample_weight( double value ){ return poParent->actor_subsample_weight = value; };
		double	Set_actor_weight( double value ){ return poParent->actor_weight = value; };
		TIME	Set_age( TIME value, bool bStartFunction = false ){ return poParent->age = value; };
		TIME	Set_assign_primary_time( TIME value );
		TIME	Set_next_school_year( TIME value );
		TIME	Set_time( TIME value, bool bStartFunction = false ){ return poParent->time = value; };
};

class PrimarySchoolDerivedStates {
	public:
		PrimarySchoolDerivedStates() { };
		class PrimarySchool	*poParent;
		class PrimarySchoolStateFunctions	*poStateFunctions;

		// derived state variables and their special functions

		// begin MembersDump
		//   group=<unknown token type>
		//   group=double
		//   group=float
		//   group=long
		//   group=ulong
		//   group=int
		//   group=uint
		//   group=short
		//   group=ushort
		//   group=char
		//   group=uchar
		// end MembersDump

};

// global actor declarations
extern Thread ChildMortalityCalibrator	*gpoEmptyChildMortalityCalibrator;
typedef CTypedPtrArray<CObArray, ChildMortalityCalibrator*> ChildMortalityCalibratorArray;
void	DeleteAllChildMortalityCalibratorActors();
extern Thread Observation	*gpoEmptyObservation;
typedef CTypedPtrArray<CObArray, Observation*> ObservationArray;
void	DeleteAllObservationActors();
extern Thread Person	*gpoEmptyPerson;
typedef CTypedPtrArray<CObArray, Person*> PersonArray;
void	DeleteAllPersonActors();
extern Thread PrimarySchool	*gpoEmptyPrimarySchool;
typedef CTypedPtrArray<CObArray, PrimarySchool*> PrimarySchoolArray;
void	DeleteAllPrimarySchoolActors();

class ChildMortalityCalibrator_mlBabies : public CObject {
	public:
		ChildMortalityCalibrator_mlBabies( ChildMortalityCalibrator *pObject ) { prParentChildMortalityCalibrator = pObject; };
		void	Add( Person *prPerson );
		void	Remove( Person *prPerson );
		void	RemoveAll();
		void	Finish();
		void	FinishAll();
		Person	*GetNext( int nInitPos, int *pnPos );
		void	UpdateReferences();
		ChildMortalityCalibrator	*prParentChildMortalityCalibrator;
		PersonArray	arData;
};

class Person_mlChild : public CObject {
	public:
		Person_mlChild( Person *pObject ) { prParentPerson = pObject; };
		void	Add( Person *prPerson );
		void	Remove( Person *prPerson );
		void	RemoveAll();
		void	Finish();
		void	FinishAll();
		Person	*GetNext( int nInitPos, int *pnPos );
		void	UpdateReferences();
		Person	*prParentPerson;
		PersonArray	arData;
};

class asAllObservationsActorSetTree : public ActorSetTree	{
	public:
		asAllObservationsActorSetTree() {m_nId = 0;};
		~asAllObservationsActorSetTree() {};
		Observation	* Item(int nIndex) {return (Observation *) ActorSetTree::Item(nIndex);};
		Observation	* GetRandom(double dRandom) {return (Observation *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asAllObservationsActorSetTree	*asAllObservations;

class asAllPersonsActorSetTree : public ActorSetTree	{
	public:
		asAllPersonsActorSetTree() {m_nId = 1;};
		~asAllPersonsActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asAllPersonsActorSetTree	*asAllPersons;

class asFemaleAgeEducActorSetTree : public ActorSetTree	{
	public:
		asFemaleAgeEducActorSetTree() {m_nId = 2;};
		~asFemaleAgeEducActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asFemaleAgeEducActorSetTree	*asFemaleAgeEduc[rc_size_AGE_RANGE][rc_size_PRIMARY_LEVEL];

class asForeignersAgeSexProvActorSetTree : public ActorSetTree	{
	public:
		asForeignersAgeSexProvActorSetTree() {m_nId = 3;};
		~asForeignersAgeSexProvActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asForeignersAgeSexProvActorSetTree	*asForeignersAgeSexProv[rc_size_AGE_RANGE][rc_size_SEX][rc_size_PROVINCE_NAT];

class asForeignersPrimAgeSexProvActorSetTree : public ActorSetTree	{
	public:
		asForeignersPrimAgeSexProvActorSetTree() {m_nId = 4;};
		~asForeignersPrimAgeSexProvActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asForeignersPrimAgeSexProvActorSetTree	*asForeignersPrimAgeSexProv[rc_size_AGE_RANGE][rc_size_SEX][rc_size_PROVINCE_NAT];

class asPotentialMothersActorSetTree : public ActorSetTree	{
	public:
		asPotentialMothersActorSetTree() {m_nId = 5;};
		~asPotentialMothersActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asPotentialMothersActorSetTree	*asPotentialMothers;

class asPotentialMothersByAgeActorSetTree : public ActorSetTree	{
	public:
		asPotentialMothersByAgeActorSetTree() {m_nId = 6;};
		~asPotentialMothersByAgeActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asPotentialMothersByAgeActorSetTree	*asPotentialMothersByAge[rc_size_FERTILE_AGE_RANGE];

class asPrimaryEntryCohortActorSetTree : public ActorSetTree	{
	public:
		asPrimaryEntryCohortActorSetTree() {m_nId = 7;};
		~asPrimaryEntryCohortActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asPrimaryEntryCohortActorSetTree	*asPrimaryEntryCohort[rc_size_SEX][rc_size_PROVINCE_INT][rc_size_PRIMARY_LEVEL];

class asPrimaryGraduationCohortActorSetTree : public ActorSetTree	{
	public:
		asPrimaryGraduationCohortActorSetTree() {m_nId = 8;};
		~asPrimaryGraduationCohortActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asPrimaryGraduationCohortActorSetTree	*asPrimaryGraduationCohort[rc_size_SEX][rc_size_PROVINCE_INT][rc_size_PRIMARY_LEVEL];

class asResidentsAgeSexActorSetTree : public ActorSetTree	{
	public:
		asResidentsAgeSexActorSetTree() {m_nId = 9;};
		~asResidentsAgeSexActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asResidentsAgeSexActorSetTree	*asResidentsAgeSex[rc_size_AGE_RANGE][rc_size_SEX];

class asResidentsPrimAgeSexActorSetTree : public ActorSetTree	{
	public:
		asResidentsPrimAgeSexActorSetTree() {m_nId = 10;};
		~asResidentsPrimAgeSexActorSetTree() {};
		Person	* Item(int nIndex) {return (Person *) ActorSetTree::Item(nIndex);};
		Person	* GetRandom(double dRandom) {return (Person *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asResidentsPrimAgeSexActorSetTree	*asResidentsPrimAgeSex[rc_size_AGE_RANGE][rc_size_SEX];

class asTheChildMortalityCalibratorActorSetTree : public ActorSetTree	{
	public:
		asTheChildMortalityCalibratorActorSetTree() {m_nId = 11;};
		~asTheChildMortalityCalibratorActorSetTree() {};
		ChildMortalityCalibrator	* Item(int nIndex) {return (ChildMortalityCalibrator *) ActorSetTree::Item(nIndex);};
		ChildMortalityCalibrator	* GetRandom(double dRandom) {return (ChildMortalityCalibrator *) ActorSetTree::GetRandom(dRandom);};
		int	ActorCompare(ActorClass *poActor1, ActorClass *poActor2);

	private:
		ActorSetNode *RemoveAll() {return ActorSetTree::RemoveAll();}
};
extern Thread asTheChildMortalityCalibratorActorSetTree	*asTheChildMortalityCalibrator;

void PreSimulation0();
void PreSimulation1();
void PreSimulation2();
void PreSimulation3();
void PreSimulation4();
void PreSimulation5();
void PostSimulation0();

} // namespace 
