﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/DYNAMIS-POP-MRT.mpp"
//LABEL(DYNAMIS-POP-MRT, EN) Simulation Engine for a time-based model

/*	NOTE(DYNAMIS-POP-MRT, EN)

	This module is the simulation engine for a time-based model.
	It contains core simulation functions and definitions.
	When starting from a template (or using the Modgen wizard), model developers typically do
	not have to modify (or understand) the code of this file.

	The file contains the definition of the model type: in our case a time-based continuouse time model
	The second part consists of the functions handling the simulation

	Case-based versions of DYNAMIS-POP-MRT can be changed into a time-based model by exchanging this file resp.
	by copy-pasting the code of this file into the DYNAMIS-POP-MRT.mpp file replacing the code.

	The code contains one parameter - StartingPopulationSize - which is necessary in time-based models 
	(as it and not part of scenario-settings as in case-based models). 
	Note that this parameter has to be added to the .dat file when changing a case-based model into a time-based one
*/

// Definition of basic model characteristics

//version 1, 2017, 0, 19;					// The model version number
//model_type time_based, just_in_time;	// The model type
//time_type double;						// The data type used to represent time
//options packing_level = 2;				// An option reducing memory use at the expense of speed

// Other data types

//real_type		float;
//counter_type	ushort;
//integer_type	short;
//index_type		ulong;

// Supported languages

//languages 
//{
//	EN // English
//};

// Model strings

//string S_MODEL_FINISH;		//EN Finish
//string S_MODEL_REPLICATE;	//EN Replicate
//string S_MODEL_SIMULATION;	//EN Simulation
//string S_MODEL_START;		//EN Start

// Parameters

//parameters
//{
//	model_generated logical model_is_case_based;    //EN Model is case based
//};

void PreSimulation0()
{
	gprParam->model_is_case_based = FALSE;    
};

// The Simulation function is called by Modgen to simulate a replicate.

void Simulation()
{
    // Create and open the input data file
    input_csv in_csv;                               // Microdata input csv object
    in_csv.open(gprParam->MicroDataInputFile);                // Open the microdata input file 

    // Buffer for reporting progress
    const size_t nBufSize = 255;
    TCHAR szBuffer[nBufSize];

    // Note replicate number for progress reporting
    int nReplicate = GetReplicate();

    // Create observations
    for (long nJ = 0; nJ < gprParam->MicroDataInputFileSize; nJ++)
    {
        in_csv.read_record(nJ);
        Observation *paObservation = new Observation();
        paObservation->UpdateTime()->Start(in_csv);
        paObservation->UpdateTime()->WeightOrKill();
    }

    // Create the starting population
    _stprintf_s(szBuffer, nBufSize, _T("%s %d: %s"), ModelString("S_MODEL_REPLICATE"), nReplicate, ModelString("S_MODEL_START"));
    ProgressMessage(szBuffer);

    for (long nJ = 0; nJ < gprParam->TotalPopSampleSize; nJ++)
    {
        Person *paPerson = new Person();
        paPerson->UpdateTime()->Start(nJ, NULL);
    }

    // Create the Primary School actor
    PrimarySchool *paPrimarySchool = new PrimarySchool();
    paPrimarySchool->UpdateTime()->Start();

    // Create the child mortality calibration actor
    ChildMortalityCalibrator *paChildMortalityCalibrator = new ChildMortalityCalibrator();
    paChildMortalityCalibrator->UpdateTime()->Start();



    _stprintf_s(szBuffer, nBufSize, _T("%s %d: %s"), ModelString("S_MODEL_REPLICATE"), nReplicate, ModelString("S_MODEL_SIMULATION"));
    ProgressMessage(szBuffer);

    // event loop
    double dCurrentTime = TIME_INFINITE;
    double dStartTime = TIME_INFINITE;

    int nLastProgressPercent = -1;
    int nThisProgressPercent = -1;

    while (!gpoEventQueue->Empty())
    {

        // get the time of next event, verify against the simulation end
        dCurrentTime = gpoEventQueue->NextEvent();

        // Note the start time (time of first event) for progress indicator
        if (dStartTime == TIME_INFINITE)
        {
            dStartTime = dCurrentTime;
        }

        if (dCurrentTime > SIMULATION_END() || gbInterrupted || gbCancelled || gbErrors)
        {
            if (dCurrentTime > SIMULATION_END())
            {
                // age all actors to the simulation end time
                gpoEventQueue->WaitUntilAllActors(SIMULATION_END());
            }

            _stprintf_s(szBuffer, nBufSize, _T("%s %d: %s"), ModelString("S_MODEL_REPLICATE"), nReplicate, ModelString("S_MODEL_FINISH"));
            ProgressMessage(szBuffer);

            gpoEventQueue->FinishAllActors();
        }
        else
        {
            // age all actors to the time of next event
            gpoEventQueue->WaitUntil(dCurrentTime);

            // implement the next event
            gpoEventQueue->Implement();
        }

        // Update progress indicator only if the integer percentage complete changes
        // (updates to the progress bar at every event are expensive).
        nThisProgressPercent = (int)(100 * (dCurrentTime - dStartTime) /
            (SIMULATION_END() - dStartTime));

        if (nThisProgressPercent > nLastProgressPercent)
        {
            TimeReport(dCurrentTime);	// update simulation progress
            nLastProgressPercent = nThisProgressPercent;
        }
    }
    // Close teh microdata input file
    in_csv.close();
}

} // namespace 
