﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/Fertility.mpp"
//LABEL(Fertility, EN) Fertility 

/*  NOTE(Fertility, EN)

    Added at Step 4
    Modified and extended at Step 15

    This module implements fertility. Fertility is implemented in two ways:

    (a) The macro approach: rates by age only

    Fertility is based on age-specific fertility rates calculated from two parameters:
    an age distribution of fertility and the Total fertility rate TFR for future years
    Another parameter is the sex-ratio

    (b) The micro approach: fertility by age, parity, time since last birth, education and union status

    In this implementation, fertility is modeld based on a more detailed model
    estimated from micro data. Users can choose to align the outcome to the macro 
    outcomes, either for total outcome (number of births) or births by age

*/

// Actor Sets

//EN Potential mothers
//actor_set Person asPotentialMothers filter is_potential_mother;

//EN Potential mothers by age
//actor_set Person asPotentialMothersByAge[fertile_age] filter is_potential_mother;

// Dimensions

//range FERTILE_AGE_RANGE { 10, 49 };				    //EN Fertile age range
//range PARITY_RANGE { 0, 15 };					    //EN Parity range
//range PARITY_RANGE1{ 1, 15 };                       //EN Parity range 1+
//range PARITY_RANGE2 { 2, 15 };                      //EN Parity range 2+

//classification SELECTED_FERTILITY_MODEL             //EN Fertility model options 
//{
//    SFM_MACRO,                                      //EN Macro model (age only)
//    SFM_MICRO,                                      //EN Micro model un-aligned
//    SFM_ALIGNE_BIRTHS,                              //EN Micro model with aligned number of births
//    SFM_ALIGNE_AGE                                  //EN Micro model with aligned number of births and age  
//};

//classification UNION_STATUS                         //EN Union Status
//{
//    US_NO,                                          //EN Never entered a union 
//    US_YES                                          //EN Entered a union
//};

//classification HIGHER_BIRTHS_PARA                   //EN Parameters for higher order births
//{
//    HBP_PERIOD1,                                    //EN 0-1 years after previous birth
//    HBP_PERIOD2,                                    //EN 1-3 years after previous birth
//    HBP_PERIOD3,                                    //EN 3-6 years after previous birth
//    HBP_PERIOD4,                                    //EN 6-9 years after previous birth
//    HBP_PERIOD5,                                    //EN 9-12 years after previous birth
//    HBP_PERIOD6,                                    //EN 12+ years after previous birth
//    HBP_AGE35,                                      //EN Age 35-39
//    HBP_AGE40,                                      //EN Age 40-44
//    HBP_AGE45,                                      //EN Age 45+
//    HBP_EDUC1,                                      //EN Entered primary school but dropped out
//    HBP_EDUC2                                       //EN Graduated from primary school    
//};

//partition BIRTH_AGE_PART { 35, 40, 45 };            //EN Age Groups
//partition DUR_TIME_IN_PARITY { 1, 3, 6, 9, 12 };    //EN Duration episodes since last birth

// Parameters

//parameters
//{
//    // Model selection
//   
//    SELECTED_FERTILITY_MODEL    selected_fertility_model;               //EN Fertility model selection
//    
//    // Macro model parameters
//
//    double  AgeSpecificFertility[FERTILE_AGE_RANGE][SIM_YEAR_RANGE];    //EN Age distribution of fertility
//    double  TotalFertilityRate[SIM_YEAR_RANGE];                         //EN Total fertility rate
//    double  SexRatio[SIM_YEAR_RANGE];                                   //EN Sex ratio (male per 100 female)
//    
//    // Micro model parameters
//
//    //EN First Births
//    double  FirstBirthRates[PRIMARY_LEVEL][UNION_STATUS][FERTILE_AGE_RANGE][PROVINCE_NAT];
//
//    //EN Higher order births
//    double  HigherOrderBirthsPara[HIGHER_BIRTHS_PARA][PARITY_RANGE2];   //EN Higher order births
//
//    //EN Birth Trends
//    double  BirthTrends[PARITY_RANGE1][SIM_YEAR_RANGE];
//    
//    // Model-generated parameters
//
//    //EN Age specific fertility rate
//    model_generated	double	AgeSpecificFertilityRate[FERTILE_AGE_RANGE][SIM_YEAR_RANGE];	
//};

//parameter_group PG03_Fertility_Top                                      //EN Fertility
//{
//    selected_fertility_model,
//    PG03a_Fertility_Model_A,
//    PG03b_Fertility_Model_B
//};

//parameter_group PG03a_Fertility_Model_A                                 //EN Fertility Model A: Macro approach
//{
//    AgeSpecificFertility, TotalFertilityRate, SexRatio
//};

//parameter_group PG03b_Fertility_Model_B                                 //EN Fertility Model B: Micro approach
//{
//    FirstBirthRates, HigherOrderBirthsPara, BirthTrends
//};

// Actor declarations

//actor Person
//{
//    FERTILE_AGE_RANGE   fertile_age = COERCE(FERTILE_AGE_RANGE, integer_age);   //EN Age
//    int                 birth_age_part = split(integer_age, BIRTH_AGE_PART);    //EN Age group
//    PARITY_RANGE	    parity = { 0 };									        //EN Parity 
//    UNION_STATUS        union_status = (in_union) ? US_YES : US_NO;             //EN Union Status
//
//    //EN Switch for calculating waiting times from one model while using another
//    logical override_mode = { FALSE };                                  
//
//    //EN Indicator re-set at birth events for calculation of time since last birth
//    logical in_this_parity = { TRUE };
//    
//    //EN Indicator that current year is in simulated time
//    logical in_simulation_time = ( calendar_year >= MIN(SIM_YEAR_RANGE)) ? TRUE : FALSE;
//    
//    //EN Indicator re-set at birt events calculating time since last birth in simulation time
//    logical in_this_parity_in_simulation = in_simulation_time && in_this_parity;
//    
//    //EN Time since last birth in the starting population or at creation (immigrants)
//    double  time_since_last_birth_start = { 0.0 };
//
//    //EN Time since last birth in full years
//    int  time_since_last_birth = self_scheduling_int(active_spell_duration(in_this_parity_in_simulation, TRUE)) 
//        + round(time_since_last_birth_start); 
//
//    //EN Time index since the last birth event
//    int time_in_parity = split(time_since_last_birth, DUR_TIME_IN_PARITY);
//    
//    //STEP18
//    
//    TIME time_of_last_birth = { TIME_INFINITE };                    //EN Time of last birth
//    //EN Indicator that perion is a potential mother
//    logical is_potential_mother = (set_alive && sex == FEMALE && WITHIN(FERTILE_AGE_RANGE, integer_age)
//        && parity < MAX(PARITY_RANGE) && WITHIN(SIM_YEAR_RANGE, calendar_year)) ? TRUE : FALSE;
//
//    logical set_alive = { FALSE };									//EN Person is set to be alive already
//    event	timeSetAliveEvent, SetAliveEvent;						//EN Event to set set_alive to true
//    event	timeBirthEvent, BirthEvent;								//EN Birth event
//};

// Event implementation

TIME Person::timeBirthEvent()
{
    double dEventTime = TIME_INFINITE;
    double dHazard = 0.0;

    if (is_potential_mother )
    {
        // Timing comes from macro model (incl. models aligned to macro numbers)
        if (gprParam->selected_fertility_model != SFM_MICRO && !override_mode )
        {
            dHazard = gprParam->AgeSpecificFertilityRate[CHKBND(RANGE_POS(FERTILE_AGE_RANGE, integer_age ),40,"AgeSpecificFertilityRate",0)]
                [CHKBND(RANGE_POS(SIM_YEAR_RANGE, calendar_year ),101,"AgeSpecificFertilityRate",1)];
        }

        // Timing comes from micro model or waiting time needed from micro model
        else
        {
            // first birth
            if ( parity == 0)
            {
                dHazard = gprParam->FirstBirthRates[CHKBND(primary_level ,3,"FirstBirthRates",0)][CHKBND(union_status ,2,"FirstBirthRates",1)][CHKBND(RANGE_POS(FERTILE_AGE_RANGE, integer_age ),40,"FirstBirthRates",2)][CHKBND(province_nat ,13,"FirstBirthRates",3)]
                    * gprParam->BirthTrends[CHKBND(RANGE_POS(PARITY_RANGE1, parity +1),15,"BirthTrends",0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE, calendar_year ),101,"BirthTrends",1)];
            }
            // higher order births
            else            
            {
                double emil = time_since_last_birth ;
                double hugo = 1.1;
                dHazard = gprParam->HigherOrderBirthsPara[CHKBND(time_in_parity ,11,"HigherOrderBirthsPara",0)][CHKBND(RANGE_POS(PARITY_RANGE2, parity +1),14,"HigherOrderBirthsPara",1)]               //Baseline
                    * gprParam->BirthTrends[CHKBND(RANGE_POS(PARITY_RANGE1, parity +1),15,"BirthTrends",0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE, calendar_year ),101,"BirthTrends",1)];  //Trend
                integer gustav = time_in_parity ;
                // relative risk for older age
                if ( birth_age_part > 0) //not the baseline age group
                {
                    dHazard = dHazard * gprParam->HigherOrderBirthsPara[CHKBND(HBP_AGE35 - 1 + birth_age_part ,11,"HigherOrderBirthsPara",0)][CHKBND(RANGE_POS(PARITY_RANGE2, parity +1),14,"HigherOrderBirthsPara",1)];
                }
                // relative risk for primary dropouts and graduates
                if ( primary_level == PL_ENTER)
                {
                    dHazard = dHazard * gprParam->HigherOrderBirthsPara[CHKBND(HBP_EDUC1,11,"HigherOrderBirthsPara",0)][CHKBND(RANGE_POS(PARITY_RANGE2, parity +1),14,"HigherOrderBirthsPara",1)];
                }
                else if ( primary_level == PL_GRAD)
                {
                    dHazard = dHazard * gprParam->HigherOrderBirthsPara[CHKBND(HBP_EDUC2,11,"HigherOrderBirthsPara",0)][CHKBND(RANGE_POS(PARITY_RANGE2, parity +1),14,"HigherOrderBirthsPara",1)];
                }
            }
        }

        // schedule event
        if (dHazard > 0.0) dEventTime = WAIT(-TIME(log(RandUniform(28)) / dHazard));
    }
    return dEventTime;
}

/*	NOTE(Person.BirthEvent, EN)
At each birth event it has to be determined if the birth applies to the actor herself (unaligned models)
or if an appropriate mother for the birth still has to be found. The latter is used for aligned modes,
in which the birth events are produced by the macro model to which output is aligned, while the birth itself
is given by the potential mother with the shortest waiting time determined by the micro model.
*/

void Person::BirthEvent()
{
    // event applies to individual without alignment
    if (gprParam->selected_fertility_model == SFM_MICRO || gprParam->selected_fertility_model == SFM_MACRO)
    {
        // increase parity of this actor and create a baby linked to this actor
         poStateFunctions->Set_in_this_parity(  FALSE );             // initialize indicator
         poStateFunctions->Set_time_since_last_birth_start(  0.0 );  // reset time before start of simulation
        poStateFunctions->PlusPlusSet_parity();							// increment parity
        Person *peChild = new Person;		// Create and point to a new actor
        peChild->UpdateTime()->Start(-1, this);			// Call the Start() function of the new actor and pass own address 
         poStateFunctions->Set_in_this_parity(  TRUE );              // reset indicator
        //STEP18
         time_of_last_birth =   time  ;
    }
    // aligned to macro-model: find women with shortest waing time to birth
    else    
    {
        Person *peMother = NULL;
        Person *pePotentialMother = NULL;

        double dShortestWaitTime = TIME_INFINITE;

        // align number of births only
        if ( gprParam->selected_fertility_model == SFM_ALIGNE_BIRTHS )
        {
            double nNumber = asPotentialMothers->Count();
            // find an appropriate mother for this birth event
            for (double nJ = 0; nJ < nNumber; nJ++)
            {
                pePotentialMother = asPotentialMothers->Item(nJ);
                pePotentialMother-> UpdateTime() -> poStateFunctions->Set_override_mode(  TRUE );
                double dCurrentWaitTime = pePotentialMother->timeBirthEvent() - time ;
                if (dCurrentWaitTime < dShortestWaitTime)
                {
                    peMother = pePotentialMother;
                    dShortestWaitTime = dCurrentWaitTime;
                }
                pePotentialMother-> UpdateTime() -> poStateFunctions->Set_override_mode(  FALSE );
            }
        }

        // align number of birth by age
        else
        {
            double nNumber = asPotentialMothersByAge[RANGE_POS(FERTILE_AGE_RANGE, fertile_age )]->Count();
            // find an appropriate mother for this birth event
            for (double nJ = 0; nJ < nNumber; nJ++)
            {
                pePotentialMother = asPotentialMothersByAge[RANGE_POS(FERTILE_AGE_RANGE, fertile_age )]->Item(nJ);
                pePotentialMother-> UpdateTime() -> poStateFunctions->Set_override_mode(  TRUE );
                double dCurrentWaitTime = pePotentialMother->timeBirthEvent() - time ;
                if (dCurrentWaitTime < dShortestWaitTime)
                {
                    peMother = pePotentialMother;
                    dShortestWaitTime = dCurrentWaitTime;
                }
                pePotentialMother-> UpdateTime() -> poStateFunctions->Set_override_mode(  FALSE );
            }
        }

        // increase parity of the selected mother and create a baby linked to her mother
        peMother-> UpdateTime() -> poStateFunctions->Set_in_this_parity(  FALSE );               // initialize indicator
        peMother-> UpdateTime() -> poStateFunctions->Set_time_since_last_birth_start(  0.0 );    // reset time before start of simulation

        peMother->UpdateTime() -> poStateFunctions->PlusPlusSet_parity();							    // increment parity
        Person *peChild = new Person;		            // Create and point to a new actor
        peChild->UpdateTime()->Start(-1, peMother);		            // Call the Start() function of the new actor and pass own address 
        peMother-> UpdateTime() -> poStateFunctions->Set_in_this_parity(  TRUE );                // reset indicator
        //STEP18
        peMother-> UpdateTime() -> time_of_last_birth =   time  ;

    }
}   

/*	NOTE(Person.SetAliveEvent, EN)

The SetAlive Event is used, to switch the state set_alive to true, immediately after an actor is created and has processed its Start() function.
This is done, as transitions are not visible if a state is initialized in Start()
The state is used for example in tables, e.g. for counting births -- transitions(set_alive,FALSE,TRUE) -- and (later) to avoid linkages between actors
before their time of birth is known, which is immediately after Start()
*/

TIME Person::timeSetAliveEvent() { if (!set_alive ) return WAIT(0); else return TIME_INFINITE; }
void Person::SetAliveEvent() 
{ 
     poStateFunctions->Set_set_alive(  TRUE ); 
    if ( person_type == PT_IMMIGRANT)
    {
        SetImmigrantPrimaryEducationStatus();
        SetImmigrantUnionFormation();
    }
}

// Pre-Simulation

void PreSimulation1()
{
    double dSum;
    for (int nYear = 0; nYear < SIZE(SIM_YEAR_RANGE); nYear++)
    {
        dSum = 0.0;
        // check if distribution parameter adds up too 1 
        for (int nAge = 0; nAge < SIZE(FERTILE_AGE_RANGE); nAge++)
        {
            dSum = dSum + gprParam->AgeSpecificFertility[CHKBND(nAge,40,"AgeSpecificFertility",0)][CHKBND(nYear,101,"AgeSpecificFertility",1)];
        }
        // scale distribution to 1 and convert to fertility rates; copy to model generated parameter
        for (int nAge = 0; nAge < SIZE(FERTILE_AGE_RANGE); nAge++)
        {
            if (dSum > 0.0)
            {
                gprParam->AgeSpecificFertilityRate[CHKBND(nAge,40,"AgeSpecificFertilityRate",0)][CHKBND(nYear,101,"AgeSpecificFertilityRate",1)]
                    = gprParam->AgeSpecificFertility[CHKBND(nAge,40,"AgeSpecificFertility",0)][CHKBND(nYear,101,"AgeSpecificFertility",1)] / dSum * gprParam->TotalFertilityRate[CHKBND(nYear,101,"TotalFertilityRate",0)];
            }
            else gprParam->AgeSpecificFertilityRate[CHKBND(nAge,40,"AgeSpecificFertilityRate",0)][CHKBND(nYear,101,"AgeSpecificFertilityRate",1)] = 0.0;
        }
    }
}
} // namespace 
