﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/MicroDataOutput.mpp"
//LABEL(MicroDataOutput, EN) Micro data output

/*	NOTE(MicroDataOutput, EN)

    Changed at Step 18: output for multiple years and variable selection

    This module implements micro data output written to a csv file.
    Users can specify the time at which a micro-data file is written out and choose a file name.
    Output can also be produced in fixed time intervals, e.g. every year or every 5 years.
    Users can select which variables to write to a file. The output csv file includes a header
    line with variable names.
    The module can be switched on and off.
*/

output_csv out_csv;                                             //EN Microdata output csv object

//classification OUTPUT_VARIABLES                                 //EN List of output variables
//{
//    OV_TIME,                                                    //EN Time
//    OV_WEIGHT,                                                  //EN Weight
//    OV_BIRTH,									                //EN Time of birth
//    OV_SEX,                                                     //EN Sex
//    OV_PROVINCE,                                                //EN Province
//    OV_EDUC,                                                    //EN Education
//    OV_POB,                                                     //EN Province of birth
//    OV_UNION,                                                   //EN Union formation time
//    OV_PARITY,                                                  //EN Number of births
//    OV_LASTBIR                                                  //EN Time of last birth
//};

//classification OUTPUT_TIMES                                     //EN Micro-data output times
//{
//    OT_FIRST,                                                   //EN Time of first output            
//    OT_LAST,                                                    //EN Time of last output
//    OT_INTERVAL                                                 //EN Time interval (0 for no repetition)
//};

//parameters
//{
//    logical		WriteMicrodata;									//EN Write micro-data output file Y/N
//    double		TimeMicroOutput[OUTPUT_TIMES];					//EN Time(s) of micro-data output
//    file		MicroRecordFileName;							//EN File name micro-data output file
//    logical     SelectedOutput[OUTPUT_VARIABLES];               //EN Output variable selection
//};

//parameter_group PG05_Files										//EN Microdata output
//{
//    WriteMicrodata, MicroRecordFileName, TimeMicroOutput,
//    SelectedOutput
//};

//actor Person
//{
//    TIME	time_microdata_output = { TIME_INFINITE };			//EN Time for microdata output
//    void	WriteMicroRecord_Start();							//EN Initialization for microdata output event
//    hook	WriteMicroRecord_Start, Start;
//    event	timeWriteMicroRecord, WriteMicroRecord; 			//EN Write micro-data record event
//};

void Person::WriteMicroRecord_Start()
{
    if (gprParam->WriteMicrodata && gprParam->TimeMicroOutput[CHKBND(OT_FIRST,3,"TimeMicroOutput",0)] >= time )  poStateFunctions->Set_time_microdata_output(  gprParam->TimeMicroOutput[CHKBND(OT_FIRST,3,"TimeMicroOutput",0)] );
    else  poStateFunctions->Set_time_microdata_output(  TIME_INFINITE );
}

TIME Person::timeWriteMicroRecord()
{
    if (GetReplicate()==0)  return time_microdata_output ;
    else return TIME_INFINITE;
}

void Person::WriteMicroRecord()
{
    // calculate variables
    double dUnionFormation = -99;
    if ( sex == FEMALE &&  time_of_union_formation < time ) dUnionFormation = time_of_union_formation ;
    double dTimeLastBirth = -99;
    if ( sex == FEMALE &&  parity > 0) dTimeLastBirth = time_of_last_birth ;

    // Push the fields into the output record.
    out_csv << actor_id ;                                                // Actor ID
    if ( gprParam->SelectedOutput[CHKBND(OV_TIME,10,"SelectedOutput",0)])       out_csv << time ;                // Time
    if ( gprParam->SelectedOutput[CHKBND(OV_WEIGHT,10,"SelectedOutput",0)])     out_csv << gprParam->ActorWeight;         // Weight
    if ( gprParam->SelectedOutput[CHKBND(OV_BIRTH,10,"SelectedOutput",0)])      out_csv << time_of_birth ;       // Time of birth
    if ( gprParam->SelectedOutput[CHKBND(OV_SEX,10,"SelectedOutput",0)])        out_csv << (int)sex ;            // Sex
    if ( gprParam->SelectedOutput[CHKBND(OV_PROVINCE,10,"SelectedOutput",0)])   out_csv << (int)province_nat ;   // Province
    if ( gprParam->SelectedOutput[CHKBND(OV_EDUC,10,"SelectedOutput",0)])       out_csv << (int)primary_level ;  // Education
    if ( gprParam->SelectedOutput[CHKBND(OV_POB,10,"SelectedOutput",0)])        out_csv << (int)province_birth ; // Province of birth
    if ( gprParam->SelectedOutput[CHKBND(OV_UNION,10,"SelectedOutput",0)])      out_csv << dUnionFormation;     // Union formation time
    if ( gprParam->SelectedOutput[CHKBND(OV_PARITY,10,"SelectedOutput",0)])     out_csv << parity ;              // Number of births
    if (gprParam->SelectedOutput[CHKBND(OV_LASTBIR,10,"SelectedOutput",0)])     out_csv << dTimeLastBirth;      // time of last birth
  
    // All fields have been pushed, now write the record.
    out_csv.write_record();

    // set next output
    if ( time_microdata_output + gprParam->TimeMicroOutput[CHKBND(OT_INTERVAL,3,"TimeMicroOutput",0)] >  time &&
         time_microdata_output + gprParam->TimeMicroOutput[CHKBND(OT_INTERVAL,3,"TimeMicroOutput",0)] <= gprParam->TimeMicroOutput[CHKBND(OT_LAST,3,"TimeMicroOutput",0)])
    {
         poStateFunctions->Set_time_microdata_output(   time_microdata_output + gprParam->TimeMicroOutput[CHKBND(OT_INTERVAL,3,"TimeMicroOutput",0)] );
    }
    else
    {
         poStateFunctions->Set_time_microdata_output(  TIME_INFINITE );
    }
}

void PreSimulation2()
{
	if (gprParam->WriteMicrodata)
	{
        CStringA                            myString = "ID,";                   // Actor ID
        if (gprParam->SelectedOutput[CHKBND(OV_TIME,10,"SelectedOutput",0)])        myString = myString + "TIME,";      // Time
        if (gprParam->SelectedOutput[CHKBND(OV_WEIGHT,10,"SelectedOutput",0)])      myString = myString + "WEIGHT,";    // Weight
        if (gprParam->SelectedOutput[CHKBND(OV_BIRTH,10,"SelectedOutput",0)])       myString = myString + "BIRTH,";     // Time of birth
        if (gprParam->SelectedOutput[CHKBND(OV_SEX,10,"SelectedOutput",0)])         myString = myString + "MALE,";      // Sex
        if (gprParam->SelectedOutput[CHKBND(OV_PROVINCE,10,"SelectedOutput",0)])    myString = myString + "PROVINCE,";  // Province
        if (gprParam->SelectedOutput[CHKBND(OV_EDUC,10,"SelectedOutput",0)])        myString = myString + "EDUCATION,"; // Education
        if (gprParam->SelectedOutput[CHKBND(OV_POB,10,"SelectedOutput",0)])         myString = myString + "PROV_BIRTH,";// Province of birth
        if (gprParam->SelectedOutput[CHKBND(OV_UNION,10,"SelectedOutput",0)])       myString = myString + "UNION,";     // Union formation time
        if (gprParam->SelectedOutput[CHKBND(OV_PARITY,10,"SelectedOutput",0)])      myString = myString + "PARITY,";    // Number of births
        if (gprParam->SelectedOutput[CHKBND(OV_LASTBIR,10,"SelectedOutput",0)])     myString = myString + "LAST_BIRTH,";// time of last birth
        
        out_csv.open(gprParam->MicroRecordFileName);
		out_csv.write_header(myString.GetString());
	}
}

void PostSimulation0()
{
    if (gprParam->WriteMicrodata) { out_csv.close(); }
}

} // namespace 
