﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/Migration.mpp"
//LABEL(Migration, EN) Migration

/*  NOTE(Migration, EN)

    Changed at Step 17

    This module implements internal migration. It is based on age and sex specific transition
    matrices. Only one transition per year is allowed. The module has five parameters, one to
    switch migration on/off, a second to choose which parameter - with or without education 
    as a model dimension - is to be used. Two tables contain the probabilities to move away (by 
    province, age, sex - and if chosedn education), the third to sample the destination 
    province (by origin, age and sex).
*/

//partition AGE5_PART{ 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60 };	//EN Age Groups

//parameters
//{
//    logical ModelMigration;															//EN Migration switched on/off
//	logical ModelByEducation;														//EN Use parameters by education
//	double  MigrationProbabilityEduc[PRIMARY_LEVEL][SEX][AGE5_PART][PROVINCE_NAT];	//EN Migration probability by education
//	double  MigrationProbability[SEX][AGE5_PART][PROVINCE_NAT];						//EN Migration probability
//    cumrate MigrationDestination[SEX][PROVINCE_NAT][AGE5_PART][PROVINCE_NAT];		//EN Migration Destination
//};

//parameter_group PG_Migration														//EN Internal migration
//{
//    ModelMigration, ModelByEducation, 
//	MigrationProbability, MigrationProbabilityEduc, 
//	MigrationDestination
//};

//actor Person
//{
//    integer number_moves = { 0 };                       //EN Number of interprovincial moves
//    integer age_at_last_move = { 999 };                 //EN Age at last migration
//    event   timeMigrationEvent, MigrationEvent;         //EN Migration Event
//};

TIME Person::timeMigrationEvent()
{
    // get the current age index using SPLIT()
    int nAge5 = SPLIT(integer_age , AGE5_PART);

    // get the probability to move
	double dMoveProb = 0.0;
	if (gprParam->ModelByEducation) dMoveProb = gprParam->MigrationProbabilityEduc[CHKBND(primary_level ,3,"MigrationProbabilityEduc",0)][CHKBND(sex ,2,"MigrationProbabilityEduc",1)][CHKBND(nAge5,13,"MigrationProbabilityEduc",2)][CHKBND(province_nat ,13,"MigrationProbabilityEduc",3)];
	else dMoveProb = gprParam->MigrationProbability[CHKBND(sex ,2,"MigrationProbability",0)][CHKBND(nAge5,13,"MigrationProbability",1)][CHKBND(province_nat ,13,"MigrationProbability",2)];

	// Check if a person is at risk for moving 
    if (gprParam->ModelMigration &&  in_projected_time &&  age_at_last_move !=  integer_age && dMoveProb > 0.0)
    {
        if (dMoveProb >= 1.0) return WAIT(0); // if there is a 100% probability move immediately
        else                                  // there is o positive probability 100%
        {
            // calculate a random waiting time based on the given probability converted to a hazard
            // rate = -log(1-probability) 
            return WAIT(-log(RandUniform(7)) / -log(1 - dMoveProb));
        }
    }
    return TIME_INFINITE;
}

void Person::MigrationEvent()
{
    int nDestination;
    int nAge5 = SPLIT(integer_age , AGE5_PART);

    // Sample the destination
    Lookup_MigrationDestination(RandUniform(6), sex , province_nat , nAge5, &nDestination);

    // move the actor to the destination
     poStateFunctions->Set_province_nat(  (PROVINCE_NAT)nDestination );

    // update indicators
     poStateFunctions->Set_age_at_last_move(  integer_age  );
    poStateFunctions->PlusPlusSet_number_moves();
}



} // namespace 
