﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/PersonCore.mpp"
//LABEL(PersonCore, EN) Core functionality of the Person actor

/*	NOTE(PersonCore, EN)

	Changed at Step 8

    This module contains the basic information which defines the actor 'Person'.
	The module contains the functions Start() and Finish() for initializing actor states
	at its creation and to destroy the actor at death. 

	Each actor (in this model only Person) typically has such a corresponding file.
	This file is also a good place for placing code, which does not belon go specific behaviors or
	is used by various other modules, like age in full years, or teh time of birth.

*/

//classification SEX                      //EN Sex
//{
//    FEMALE,                             //EN Female
//    MALE                                //EN Male
//};

//classification PROVINCE_NAT             //EN Province
//{
//    PN_PROV00,                          //EN Hodh-Charghy
//    PN_PROV01,                          //EN Hodh-Gharby
//    PN_PROV02,                          //EN Assaba
//    PN_PROV03,                          //EN Gorgol
//    PN_PROV04,                          //EN Brakna
//    PN_PROV05,                          //EN Trarza
//    PN_PROV06,                          //EN Adrar
//    PN_PROV07,                          //EN Dakhlett-Nouadibou
//    PN_PROV08,                          //EN Tagant
//    PN_PROV09,                          //EN Guidimagha
//    PN_PROV10,                          //EN Tirs-Ezemour
//    PN_PROV11,                          //EN Inchiri
//    PN_PROV12                           //EN Nouakchott
//};

//classification PROVINCE_INT             //EN Province incl abroad
//{
//    PI_PROV00,                          //EN Hodh-Charghy
//    PI_PROV01,                          //EN Hodh-Gharby
//    PI_PROV02,                          //EN Assaba
//    PI_PROV03,                          //EN Gorgol
//    PI_PROV04,                          //EN Brakna
//    PI_PROV05,                          //EN Trarza
//    PI_PROV06,                          //EN Adrar
//    PI_PROV07,                          //EN Dakhlett-Nouadibou
//    PI_PROV08,                          //EN Tagant
//    PI_PROV09,                          //EN Guidimagha
//    PI_PROV10,                          //EN Tirs-Ezemour
//    PI_PROV11,                          //EN Inchiri
//    PI_PROV12,                          //EN Nouakchott
//    PI_PROV13                           //EN Abroad
//};

//classification PERSON_TYPE              //EN Person Type
//{
//    PT_START,                           //EN Person from Starting Population
//    PT_CHILD,                           //EN Person born in simulation    
//    PT_IMMIGRANT                        //EN Immigrant
//};

//range SIM_YEAR_RANGE{ 2013, 2113 };     //EN Projected calendar years	
//range ALL_YEAR_RANGE{ 1912, 2113 };     //EN All calendar years	
//range AGE_RANGE	{ 0, 100 };	            //EN Age range

//actor Person            
//{
//    PERSON_TYPE     person_type = { PT_START };                                 //EN Person type
//	AGE_RANGE	    integer_age = COERCE(AGE_RANGE, self_scheduling_int(age));	//EN Age in full years
//    TIME		    time_of_birth = { 1912 };							        //EN Time at birth
//    ALL_YEAR_RANGE  year_of_birth = int(time_of_birth);                         //EN Year of birth
//    SEX             sex = { FEMALE };                                           //EN Sex
//    PROVINCE_NAT    province_nat = { PN_PROV00 };                               //EN Province of residence
//
////12
//    PROVINCE_INT    province_birth = { PI_PROV00 };                             //EN Province of birth
//
//    integer         calendar_year = self_scheduling_int(time);                  //EN Calendar year	
//    SIM_YEAR_RANGE  sim_year = COERCE(SIM_YEAR_RANGE, calendar_year);           //EN Projected year
//    logical         in_projected_time = (calendar_year >= MIN(SIM_YEAR_RANGE)); //EN In projected time
//
//    // Functions
//    void		Start(long PersonNr, Person *peMother);                         //EN Starts the actor
//	void		Finish();                                                       //EN Finishes the actor
//};

void Person::Start(long PersonNr, Person *peMother)
{ if (__finished) {StartAfterFinish(); return; }; StartSpecial1();
    if (gprParam->model_is_case_based) PersonNr = case_id ;

    // Population scaling
    if (gprParam->ScalePopulation && !gprParam->model_is_case_based)
    {
       Set_actor_weight(gprParam->ActorWeight);
       Set_actor_subsample_weight(gprParam->ActorWeight);
    }
    
    // Determine the person type
    if (peMother != NULL)                                                       // Born in simulation
    {
         poStateFunctions->Set_person_type(  PT_CHILD );
    }
    //11 else if (PersonNr < StartPopSampleSize)     // Person from start population
    else if (asAllObservations->Count() > 1)     // Person from start population
    {
         poStateFunctions->Set_person_type(  PT_START );
    }
    else                                                                        // Immigrant
    {
         poStateFunctions->Set_person_type(  PT_IMMIGRANT );                                                             
    }

    // Initializations person from start population
    if ( person_type == PT_START)                                         
    {
        GetStartCharacteristics();
         poStateFunctions->Set_age(  0, true );
         poStateFunctions->Set_time(  time_of_birth , true );
    }
    // Initializations persons born in simulation
    else if ( person_type == PT_CHILD)
    {
        // assign a sex according to the sex ratio parameter
         poStateFunctions->Set_sex(  MALE );
        if (RandUniform(5) < 100.0 / (100.0 + gprParam->SexRatio[CHKBND(RANGE_POS(SIM_YEAR_RANGE, calendar_year ),101,"SexRatio",0)]))  poStateFunctions->Set_sex(  FEMALE );

        // inherit characteristics from the mother
         poStateFunctions->Set_time_of_birth(  peMother->UpdateTime() -> time  );
         poStateFunctions->Set_province_nat(  peMother->province_nat  );
         poStateFunctions->Set_province_birth(  (PROVINCE_INT)peMother->province_nat  );
         poStateFunctions->Set_age(  0, true );
         poStateFunctions->Set_time(  time_of_birth , true );

        //STEP16
         poStateFunctions->Set_mother_known(  TRUE );     //Mother is known
        if (peMother-> primary_level == PL_NO)  poStateFunctions->Set_mother_educ_0(  TRUE );
        else if (peMother-> primary_level == PL_ENTER)  poStateFunctions->Set_mother_educ_1(  TRUE );
        if (peMother-> integer_age <= 14)  poStateFunctions->Set_mother_max_14(  TRUE );
        else if (peMother-> integer_age <= 16)  poStateFunctions->Set_mother_max_16(  TRUE );
         Set_lChildMortalityCalibrator(  asTheChildMortalityCalibrator->Item(0) );
        //STEP18
         Set_lMother(  peMother );
        //FIX STEP 19
         poStateFunctions->Set_educ_mother(  peMother->primary_level  );
    }
    // Initializations immigrants
    else
    {
        GetImmigrantCharacteristics();
         poStateFunctions->Set_age(   time_of_immigration - time_of_birth , true );
         poStateFunctions->Set_time(  time_of_immigration , true );
    }
StartIMPLEMENT_HOOK();StartSpecial2();}

void Person::Finish()
{
	// Developers can add code here, currently no additional routines are required. 
	// After the code in this function is executed, Modgen automatically removes the actor from the 
	// simulation and recuperates any memory used by the actor.
if (__finished) {	DoubleFinish(case_seed);	return; } else __finished = TRUE;FinishSpecial();}



} // namespace 
