﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/PrimaryEducation.mpp"
//LABEL(PrimaryEducation, EN) Primary Education

/*  NOTE(PrimaryEducation, EN)

    Added at Step 13

    This module implements primary education. Users can specigy entry and graduaten ages, 
    as well as probabilities to enter and to graduate by year of birth, sex, and provinc eof birth.
    Optionlally, proportional factors can be specified accounting for mother's education. If
    these factors are used, the outcome calibrates itself for a selected year to the initial
    outcome.

    The primary education system is implemented as a 'PrimarySchool' actor, who at the change of each 
    school year selects potential students for entering school resp. for graduation. 
*/

//EN Primary education entry age cohort
//actor_set Person asPrimaryEntryCohort[sex][province_birth][educ_mother]  
//    filter set_alive && integer_age == AgeEnterPrimary;

//EN Primary education graduation age cohort
//actor_set Person asPrimaryGraduationCohort[sex][province_birth][educ_mother]
//    filter set_alive && integer_age == AgeGradPrimary && primary_level != PL_NO;

//EN All Persons 
//actor_set Person asAllPersons filter set_alive;

//EN Foreign born by age, sex and province
//actor_set Person asForeignersAgeSexProv[integer_age][sex][province_nat] filter set_alive && province_birth == PI_PROV13;

//EN Residents by age, sex and province
//actor_set Person asResidentsAgeSex[integer_age][sex] filter set_alive;

//EN Foreign born who at least entered primary education by age, sex and province
//actor_set Person asForeignersPrimAgeSexProv[integer_age][sex][province_nat] filter set_alive && province_birth == PI_PROV13 && primary_level != PL_NO;

// EN Residents who at least entered primary education
//actor_set Person asResidentsPrimAgeSex[integer_age][sex] filter set_alive && primary_level != PL_NO;

    
//classification PRIMARY_LEVEL								    //EN Primary Education level
//{
//    PL_NO,													    //EN Never entered primary school
//    PL_ENTER,												    //EN Entered primary school
//    PL_GRAD													    //EN Graduated from primary school
//};

//range YOB_START_PRIMARY { 2001, 2063 };						    //EN Year of birth
//range YOB_GRAD_PRIMARY { 1997, 2063 };						    //EN Year of birth

//parameters
//{
//    int     AgeEnterPrimary;                                    //EN School entry age primary
//    int     AgeGradPrimary;                                     //EN Graduation age primary        
//    double  StartOfSchoolYear;                                  //EN Start at school year (e.g. 0.5 = July)
//    double  StartPrimary[SEX][YOB_START_PRIMARY][PROVINCE_INT]; //EN Probability to start primary school
//    double  GradPrimary[SEX][YOB_GRAD_PRIMARY][PROVINCE_INT];   //EN Probability to graduate from primary school
//    double  OddsMotherEnterPrimary[SEX][PRIMARY_LEVEL];         //EN Odds by mother's education: primary	
//    double  OddsMotherGradPrimary[SEX][PRIMARY_LEVEL];          //EN Odds by mother's education: secondary	
//    int	    CalibratedYear;                                     //EN Calibrated year introducing mothers education 
//    logical	UseMothersEducation;                                //EN Use mother's education 
//};

//parameter_group PG_06_PrimaryEducation                          //EN Primary Education
//{
//    AgeEnterPrimary, AgeGradPrimary, StartOfSchoolYear,
//    StartPrimary, GradPrimary,OddsMotherEnterPrimary,
//    OddsMotherGradPrimary, CalibratedYear, UseMothersEducation
//};

//actor Person
//{
//    PRIMARY_LEVEL   primary_level = { PL_NO };                  //EN Primary school status
//    PRIMARY_LEVEL   educ_mother = { PL_NO };                    //EN Mothers primary school status
//
//    void DecidePrimaryEntry(double dProb);                      //EN Decides & sets primary school entry 
//    void DecidePrimaryGrad(double dProb);                       //EN Decides & sets primary graduation
//    void SetImmigrantPrimaryEducationStatus();                  //EN Assign primary education at immigration
//};

//actor PrimarySchool                                             //EN Primary School Actor
//{
//    int     school_year = { 0 };                                //EN Current school year
//    TIME    next_school_year = { TIME_INFINITE };               //EN Next school year
//    TIME    assign_primary_time = { TIME_INFINITE };            //EN Initial assignment of primary status
//
//    double	CalibrationFactorPrimaryEntry[SEX][PROVINCE_INT];	//EN Calibration factor primary entry 
//    double	LastProbabilityPrimaryEntry[SEX][PROVINCE_INT];		//EN Last primary entry probability 	
//    double	CalibrationFactorPrimaryGrad[SEX][PROVINCE_INT];	//EN Calibration factor primary graduation
//    double	LastProbabilityPrimaryGrad[SEX][PROVINCE_INT];		//EN Last primary entry probability 	
//
//    event   timeNextSchoolYearEvent, NextSchoolYearEvent;       //EN Next school year event
//    event   timeAssignInitialPrimary, AssignInitialPrimary;     //EN Assigns initial school level at start
//
//    void    PrimarySchoolEntry();                               //EN Decides who enters primary education
//    void    PrimarySchoolGraduation();                          //EN Decides who graduates from primary education          
//    
//    //EN Calculate adjusted probability adding relative factors
//    double  AdjustedProbability(double dProb, double dLogOddEduc, double dLogOddAdjust);	
//
//    void    Start();                                            //EN Start the primary school actor
//    void    Finish();                                           //EN Destroy the primary school actor
//};

void PrimarySchool::Start()
{ if (__finished) {StartAfterFinish(); return; }; StartSpecial1();
     poStateFunctions->Set_time(                   MIN(SIM_YEAR_RANGE), true );        // Create at start of simulation
     school_year =             MIN(SIM_YEAR_RANGE) - 1 ;    // Still 'old' school year
     poStateFunctions->Set_next_school_year(       WAIT(gprParam->StartOfSchoolYear) );    // Next school year starts mid year
     poStateFunctions->Set_assign_primary_time(    WAIT(0.0) );                  // Assign initial proiary status in population 
StartSpecial2();};

void PrimarySchool::Finish(){if (__finished) {	DoubleFinish(case_seed);	return; } else __finished = TRUE;FinishSpecial();}

TIME PrimarySchool::timeNextSchoolYearEvent()
{ 
    return next_school_year ;
}

/*	NOTE(PrimarySchool.NextSchoolYearEvent, EN)
    The primary school actor has just this one event, the change of school year.
    At each school year change, a selection is made which potential students enter
    primary education, and which students graduate.
*/    
void PrimarySchool::NextSchoolYearEvent()
{
    school_year++;                                  // update school year
    PrimarySchoolEntry();                           // decide who enters primary school
    PrimarySchoolGraduation();                      // decide who graduates from primary school
     poStateFunctions->Set_next_school_year(  WAIT(1.0) );                   // set clock for next school year
}

/*  NOTE(PrimarySchool.AssignInitialPrimary, EN)
    This event  assignes the initial primary education status at the beginning
    of the simulated time period. For given parameters of school entry and graduation age,
    values in the file are reset, re-assigned by models, or kept.
*/
TIME PrimarySchool::timeAssignInitialPrimary(){ return assign_primary_time ; }
void PrimarySchool::AssignInitialPrimary()
{
    // loop through all actors
    for (long nJ = 0; nJ < asAllPersons->Count(); nJ++)
    {
        Person * prPerson = new Person();
        prPerson = asAllPersons->Item(nJ);

        // Entry to primary education
        // (a) persons below school entry age reset education on file to non
        if (prPerson-> UpdateTime() -> age < gprParam->AgeEnterPrimary + gprParam->StartOfSchoolYear)
        {
            prPerson-> UpdateTime() -> poStateFunctions->Set_primary_level(  PL_NO );
        }
        // (b) primary education: modeled if yob in parameterized range 
        else if (prPerson-> year_of_birth >= MIN(YOB_START_PRIMARY))
        {
            prPerson-> UpdateTime() -> poStateFunctions->Set_primary_level(  PL_NO ); // reset
            if (RandUniform(8) < gprParam->StartPrimary[CHKBND(prPerson->sex ,2,"StartPrimary",0)][CHKBND(RANGE_POS(YOB_START_PRIMARY, prPerson->year_of_birth ),63,"StartPrimary",1)][CHKBND(prPerson->province_birth ,14,"StartPrimary",2)])
                prPerson-> UpdateTime() -> poStateFunctions->Set_primary_level(  PL_ENTER );
        }
        // (c) old enought: keep primary education from file (do nothing)
        else {}

        // Graduate from primary education
        // (a) persons below school graduation age reset graduations on record
        if (prPerson-> UpdateTime() -> age < gprParam->AgeGradPrimary + gprParam->StartOfSchoolYear && prPerson-> primary_level == PL_GRAD)
        {
            prPerson-> UpdateTime() -> poStateFunctions->Set_primary_level(  PL_ENTER );
        }
        // (b) primary education graduation: modeled if yob in parameterized range 
        else if (prPerson-> UpdateTime() -> age >= gprParam->AgeGradPrimary + gprParam->StartOfSchoolYear 
            && prPerson-> year_of_birth >= MIN(YOB_GRAD_PRIMARY) && prPerson-> primary_level != PL_NO)
        {
            prPerson-> UpdateTime() -> poStateFunctions->Set_primary_level(  PL_ENTER ); // reset
            if (RandUniform(15) < gprParam->GradPrimary[CHKBND(prPerson->sex ,2,"GradPrimary",0)][CHKBND(RANGE_POS(YOB_GRAD_PRIMARY, prPerson->year_of_birth ),67,"GradPrimary",1)][CHKBND(prPerson->province_birth ,14,"GradPrimary",2)])
                prPerson-> UpdateTime() -> poStateFunctions->Set_primary_level(  PL_GRAD );
        }
        // (c) old enought: keep primary education from file (do nothing)
        else {}
    }
     poStateFunctions->Set_assign_primary_time(  TIME_INFINITE ); //do just once
}

/*  NOTE(Person.SetImmigrantPrimaryEducationStatus, EN)
    This function is called at the moment of immigration in order to assign a education
    status to the new immigrant. Depending on year of birth and age, there are three possibilities:
    (a) the person is younger than school entry resp. graduation age and currently has not entered primary.
    (b) for the birth date, parameters are available: education status is modeled
    (c) the person was born too far in the past and no parameters are available: characteristics are sampled
        from the current immigrant population of the same age, sex, and province of residence. If no donor
        is found, the sampling is extended to the national population of the same age and sex.
*/
void Person::SetImmigrantPrimaryEducationStatus()
{
    if ( person_type == PT_IMMIGRANT)
    {
        // ENTRY TO PRIMARY

        double dTimeToNextSchoolyear = gprParam->StartOfSchoolYear - ( time - calendar_year );
        if (dTimeToNextSchoolyear < 0) dTimeToNextSchoolyear = dTimeToNextSchoolyear + 1.0;

        // (a) younger than entry age at next school year change:
        if ((int)( age + dTimeToNextSchoolyear) <= gprParam->AgeEnterPrimary) { /*do nothing*/ }

        // (b) parameters available for given year of birth
        else if ( year_of_birth >= MIN(YOB_START_PRIMARY))
        {
            if (RandUniform(18) < gprParam->StartPrimary[CHKBND(sex ,2,"StartPrimary",0)][CHKBND(RANGE_POS(YOB_START_PRIMARY, year_of_birth ),63,"StartPrimary",1)][CHKBND(province_birth ,14,"StartPrimary",2)])
                 poStateFunctions->Set_primary_level(  PL_ENTER );
        }

        // (c) sample from other foreign born of same age, sex and province of residence
        //     if no foreignborn with same characteristics available, sample from all residents of same age and sex
        else
        {
            Person * prPerson = new Person;
            int nPopSize = asForeignersAgeSexProv[integer_age ][sex ][province_nat ]->Count();
            if (nPopSize > 0) prPerson = asForeignersAgeSexProv[integer_age ][sex ][province_nat ]->GetRandom(RandUniform(19));
            else
            {
                nPopSize = asResidentsAgeSex[integer_age ][sex ]->Count();
                if (nPopSize > 0) prPerson = asResidentsAgeSex[integer_age ][sex ]->GetRandom(RandUniform(20));
            }
            // a donor was found: assign donor's primary entry level
            if (nPopSize > 0 && prPerson-> primary_level != PL_NO)  poStateFunctions->Set_primary_level(  PL_ENTER );
            // no donor was found; depending on samlpe size this should never or rarely happen
            else if (RandUniform(21) < 0.5)  poStateFunctions->Set_primary_level(  PL_ENTER );
        }

        // GRADUATION FROM PRIMARY

        if ( primary_level == PL_ENTER)
        {
            double dTimeToNextSchoolyear = gprParam->StartOfSchoolYear - ( time - calendar_year );
            if (dTimeToNextSchoolyear < 0) dTimeToNextSchoolyear = dTimeToNextSchoolyear + 1.0;

            // (a) younger than graduation age at next school year change:
            if ((int)( age + dTimeToNextSchoolyear) <= gprParam->AgeGradPrimary) { /* do nothing */ }

            // (b) parameters available for given year of birth
            else if ( year_of_birth >= MIN(YOB_GRAD_PRIMARY))
            {
                if (RandUniform(22) < gprParam->GradPrimary[CHKBND(sex ,2,"GradPrimary",0)][CHKBND(RANGE_POS(YOB_GRAD_PRIMARY, year_of_birth ),67,"GradPrimary",1)][CHKBND(province_birth ,14,"GradPrimary",2)])
                     poStateFunctions->Set_primary_level(  PL_GRAD );
            }
            // (c) sample from other foreign born who entered primary of same age, sex and province of residence
            //     if no foreignborn with same characteristics available, sample from all residents of same age and sex
            else
            {
                Person * prPerson = new Person;
                int nPopSize = asForeignersPrimAgeSexProv[integer_age ][sex ][province_nat ]->Count();
                if (nPopSize > 0) prPerson = asForeignersPrimAgeSexProv[integer_age ][sex ][province_nat ]->GetRandom(RandUniform(23));
                else
                {
                    nPopSize = asResidentsPrimAgeSex[integer_age ][sex ]->Count();
                    if (nPopSize > 0) prPerson = asResidentsPrimAgeSex[integer_age ][sex ]->GetRandom(RandUniform(24));
                }
                // a donor was found: assign donor's primary entry level
                if (nPopSize > 0 && prPerson-> primary_level == PL_GRAD)  poStateFunctions->Set_primary_level(  PL_GRAD );
                // no donor was found; depending on samlpe size this should never or rarely happen
                else if (RandUniform(25) < 0.5)  poStateFunctions->Set_primary_level(  PL_GRAD );
            }
        }
    }
 }

/*  NOTE(PrimarySchool.PrimarySchoolEntry, EN)
    This function is called by the Primary School Actor every year in order to select
    children to enroll in primary school. The user can choose two model variants, one with
    and one without accounting for mother's education. In the latter case, the outcome is 
    aligned to the initial outcome for the first year in which mothers education is introduced.
*/
void PrimarySchool::PrimarySchoolEntry()
{
    for (int nSex = 0; nSex < SIZE(SEX); nSex++)
    {
        for (int nProv = 0; nProv < SIZE(PROVINCE_INT); nProv++)
        {
            // Size of relevant sub-populations
            int nMotherEd0 = asPrimaryEntryCohort[nSex][nProv][PL_NO]->Count();
            int nMotherEd1 = asPrimaryEntryCohort[nSex][nProv][PL_ENTER]->Count();
            int nMotherEd2 = asPrimaryEntryCohort[nSex][nProv][PL_GRAD]->Count();
            int nPerson = nMotherEd0 + nMotherEd1 + nMotherEd2;

            // Calculate probabilities with all adjustments 
            if (nPerson > 0)
            {
                double dCalibrationFactor = 0.0;
                double dMotherFactorEduc0 = 0.0;
                double dMotherFactorEduc1 = 0.0;
                double dMotherFactorEduc2 = 0.0;
                double dCenter;

                // overall probability if not accounting for mother seducation
                double dProb = 
                    gprParam->StartPrimary[CHKBND(nSex,2,"StartPrimary",0)][CHKBND(RANGE_POS(YOB_START_PRIMARY,  school_year - gprParam->AgeEnterPrimary),63,"StartPrimary",1)][CHKBND(nProv,14,"StartPrimary",2)];

                // account for mothers education and calibrate results in year in which introduced
                if (gprParam->UseMothersEducation &&  school_year >= gprParam->CalibratedYear + gprParam->AgeEnterPrimary)
                {
                    dMotherFactorEduc0 = log(gprParam->OddsMotherEnterPrimary[CHKBND(nSex,2,"OddsMotherEnterPrimary",0)][CHKBND(PL_NO,3,"OddsMotherEnterPrimary",1)]);
                    dMotherFactorEduc1 = log(gprParam->OddsMotherEnterPrimary[CHKBND(nSex,2,"OddsMotherEnterPrimary",0)][CHKBND(PL_ENTER,3,"OddsMotherEnterPrimary",1)]);
                    dMotherFactorEduc2 = log(gprParam->OddsMotherEnterPrimary[CHKBND(nSex,2,"OddsMotherEnterPrimary",0)][CHKBND(PL_GRAD,3,"OddsMotherEnterPrimary",1)]);

                    // calibration year
                    if ( school_year == gprParam->CalibratedYear + gprParam->AgeEnterPrimary)
                    {
                        double dWeight0 = (double)nMotherEd0 / (double)nPerson;
                        double dWeight1 = (double)nMotherEd1 / (double)nPerson;
                        double dWeight2 = (double)nMotherEd2 / (double)nPerson;

                        int nIterations = 0;
                        double dResult = 500.0;
                        double dTarget = dProb;
                        double dLower = -10.0;
                        double dUpper = 10.0;
                        while (abs(dResult - dTarget) > 0.0001 && nIterations < 10000)
                        {
                            nIterations++;
                            dCenter = (dLower + dUpper) / 2.0;

                            dResult = dWeight0 * AdjustedProbability(dProb, dMotherFactorEduc0, dCenter)
                                + dWeight1 * AdjustedProbability(dProb, dMotherFactorEduc1, dCenter)
                                + dWeight2 * AdjustedProbability(dProb, dMotherFactorEduc2, dCenter);

                            if (dTarget > dResult) dLower = dCenter;
                            else dUpper = dCenter;
                        }
                        dCalibrationFactor = dCenter;

                        LastProbabilityPrimaryEntry[CHKBND(nSex,2,"PrimarySchool.LastProbabilityPrimaryEntry",0)][CHKBND(nProv,14,"PrimarySchool.LastProbabilityPrimaryEntry",1)] = dProb;
                        CalibrationFactorPrimaryEntry[CHKBND(nSex,2,"PrimarySchool.CalibrationFactorPrimaryEntry",0)][CHKBND(nProv,14,"PrimarySchool.CalibrationFactorPrimaryEntry",1)] = dCalibrationFactor;
                    }
                    // after calibrated year 
                    else
                    {
                        dCalibrationFactor = CalibrationFactorPrimaryEntry[CHKBND(nSex,2,"PrimarySchool.CalibrationFactorPrimaryEntry",0)][CHKBND(nProv,14,"PrimarySchool.CalibrationFactorPrimaryEntry",1)];
                        dProb = LastProbabilityPrimaryEntry[CHKBND(nSex,2,"PrimarySchool.LastProbabilityPrimaryEntry",0)][CHKBND(nProv,14,"PrimarySchool.LastProbabilityPrimaryEntry",1)];
                    }
                }
                // decide for each person in entry cohort, if entering school
                for (int nJ = 0; nJ < nMotherEd0; nJ++) asPrimaryEntryCohort[nSex][nProv][PL_NO]->Item(nJ)->
                    UpdateTime()->DecidePrimaryEntry(AdjustedProbability(dProb, dMotherFactorEduc0, dCalibrationFactor));
                for (int nJ = 0; nJ < nMotherEd1; nJ++) asPrimaryEntryCohort[nSex][nProv][PL_ENTER]->Item(nJ)->
                    UpdateTime()->DecidePrimaryEntry(AdjustedProbability(dProb, dMotherFactorEduc1, dCalibrationFactor));
                for (int nJ = 0; nJ < nMotherEd2; nJ++) asPrimaryEntryCohort[nSex][nProv][PL_GRAD]->Item(nJ)->
                    UpdateTime()->DecidePrimaryEntry(AdjustedProbability(dProb, dMotherFactorEduc2, dCalibrationFactor));
            }
        }
    }
}

/*  NOTE(PrimarySchool.PrimarySchoolGraduation, EN)
    This function is called by the Primary School Actor every year in order to select current primary students
    to graduate in primary school. If accounting for mother's education - the outcome is aligned to the initial 
    outcome for the first year in which mothers education is introduced.
*/
void PrimarySchool::PrimarySchoolGraduation()
{
    for (int nSex = 0; nSex < SIZE(SEX); nSex++)
    {
        for (int nProv = 0; nProv < SIZE(PROVINCE_INT); nProv++)
        {
            // Size of relevant sub-populations
            int nMotherEd0 = asPrimaryGraduationCohort[nSex][nProv][PL_NO]->Count();
            int nMotherEd1 = asPrimaryGraduationCohort[nSex][nProv][PL_ENTER]->Count();
            int nMotherEd2 = asPrimaryGraduationCohort[nSex][nProv][PL_GRAD]->Count();
            int nPerson = nMotherEd0 + nMotherEd1 + nMotherEd2;

            // Calculate probabilities with all adjustments
            if (nPerson > 0)
            {
                double dCalibrationFactor = 0.0;
                double dMotherFactorEduc0 = 0.0;
                double dMotherFactorEduc1 = 0.0;
                double dMotherFactorEduc2 = 0.0;
                double dCenter;

                // overall probability if not accounting for mother seducation
                double dProb = 
                    gprParam->GradPrimary[CHKBND(nSex,2,"GradPrimary",0)][CHKBND(RANGE_POS(YOB_GRAD_PRIMARY,  school_year - gprParam->AgeGradPrimary),67,"GradPrimary",1)][CHKBND(nProv,14,"GradPrimary",2)];

                if (gprParam->UseMothersEducation &&  school_year >= gprParam->CalibratedYear + gprParam->AgeGradPrimary)
                {
                    dMotherFactorEduc0 = log(gprParam->OddsMotherGradPrimary[CHKBND(nSex,2,"OddsMotherGradPrimary",0)][CHKBND(PL_NO,3,"OddsMotherGradPrimary",1)]);
                    dMotherFactorEduc1 = log(gprParam->OddsMotherGradPrimary[CHKBND(nSex,2,"OddsMotherGradPrimary",0)][CHKBND(PL_ENTER,3,"OddsMotherGradPrimary",1)]);
                    dMotherFactorEduc2 = log(gprParam->OddsMotherGradPrimary[CHKBND(nSex,2,"OddsMotherGradPrimary",0)][CHKBND(PL_GRAD,3,"OddsMotherGradPrimary",1)]);

                    // calibration year
                    if ( school_year == gprParam->CalibratedYear + gprParam->AgeGradPrimary)
                    {
                        double dWeight0 = (double)nMotherEd0 / (double)nPerson;
                        double dWeight1 = (double)nMotherEd1 / (double)nPerson;
                        double dWeight2 = (double)nMotherEd2 / (double)nPerson;

                        int nIterations = 0;
                        double dResult = 500.0;
                        double dTarget = dProb;
                        double dLower = -10.0;
                        double dUpper = 10.0;
                        while (abs(dResult - dTarget) > 0.0001 && nIterations < 10000)
                        {
                            nIterations++;
                            dCenter = (dLower + dUpper) / 2.0;

                            dResult = dWeight0 * AdjustedProbability(dProb, dMotherFactorEduc0, dCenter)
                                + dWeight1 * AdjustedProbability(dProb, dMotherFactorEduc1, dCenter)
                                + dWeight2 * AdjustedProbability(dProb, dMotherFactorEduc2, dCenter);

                            if (dTarget > dResult) dLower = dCenter;
                            else dUpper = dCenter;
                        }
                        dCalibrationFactor = dCenter;

                        LastProbabilityPrimaryGrad[CHKBND(nSex,2,"PrimarySchool.LastProbabilityPrimaryGrad",0)][CHKBND(nProv,14,"PrimarySchool.LastProbabilityPrimaryGrad",1)] = dProb;
                        CalibrationFactorPrimaryGrad[CHKBND(nSex,2,"PrimarySchool.CalibrationFactorPrimaryGrad",0)][CHKBND(nProv,14,"PrimarySchool.CalibrationFactorPrimaryGrad",1)] = dCalibrationFactor;
                    }
                    // after calibrated year 
                    else
                    {
                        dCalibrationFactor = CalibrationFactorPrimaryGrad[CHKBND(nSex,2,"PrimarySchool.CalibrationFactorPrimaryGrad",0)][CHKBND(nProv,14,"PrimarySchool.CalibrationFactorPrimaryGrad",1)];
                        dProb = LastProbabilityPrimaryGrad[CHKBND(nSex,2,"PrimarySchool.LastProbabilityPrimaryGrad",0)][CHKBND(nProv,14,"PrimarySchool.LastProbabilityPrimaryGrad",1)];
                    }
                }
                // decide for each primary student if graduating

                for (int nJ = 0; nJ < nMotherEd0; nJ++) asPrimaryGraduationCohort[nSex][nProv][PL_NO]->
                    Item(nJ)->UpdateTime()->DecidePrimaryGrad(AdjustedProbability(dProb, dMotherFactorEduc0, dCalibrationFactor));
                for (int nJ = 0; nJ < nMotherEd1; nJ++) asPrimaryGraduationCohort[nSex][nProv][PL_ENTER]->
                    Item(nJ)->UpdateTime()->DecidePrimaryGrad(AdjustedProbability(dProb, dMotherFactorEduc1, dCalibrationFactor));
                for (int nJ = 0; nJ < nMotherEd2; nJ++) asPrimaryGraduationCohort[nSex][nProv][PL_GRAD]->
                    Item(nJ)->UpdateTime()->DecidePrimaryGrad(AdjustedProbability(dProb, dMotherFactorEduc2, dCalibrationFactor));
               
            }
        }
    }
}

/*  NOTE(PrimarySchool.AdjustedProbability,EN)
    This function modifies a base probability by two log odds factors. It is used to apply 
    log odds by mothers education and a calibration factor to the base probability
*/
double PrimarySchool::AdjustedProbability(double dProb, double dLogOddEduc, double dLogOddAdjust)
{
    if (dProb >= 0.9999) dProb = 0.9999;
    double dExp = exp(log(dProb / (1 - dProb)) + dLogOddEduc + dLogOddAdjust);
    return dExp / (1 + dExp);
}

/*  NOTE(Person.DecidePrimaryEntry,EN)
    A simple function randomly setting the primary education entry status for a given probability
*/
void Person::DecidePrimaryEntry(double dProb)
{
    if (RandUniform(16) < dProb)  poStateFunctions->Set_primary_level(  PL_ENTER );
}

/*  NOTE(Person.DecidePrimaryGrad,EN)
    A simple function randomly setting the primary graduation status for a given probability
*/
void Person::DecidePrimaryGrad(double dProb)
{
   if ( primary_level == PL_ENTER && RandUniform(17) < dProb)  poStateFunctions->Set_primary_level(  PL_GRAD );
}





} // namespace 
