﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/StartPopFile.mpp"
//LABEL(StartPopFile, EN) Starting population File

/*  NOTE(StartPopFile, EN)

    Changed at step 12

    This module handles input data file operations. All records of the input file are read and 
    it is decided, if and how many clones are kept in the simulation.
    This allows to create populations smaller than the microdata file (i.e. records are sampled)
    or populations larger than the file (i.e. records are cloned).
    The routines also allow to use (optional) weights. 
*/

//classification PERSON_MICRODATA_COLUMNS         //EN fields in the microdata input file 
//{
//    PMC_WEIGHT,                                 //EN Weight
//    PMC_BIRTH,                                  //EN Time of birth
//    PMC_SEX,                                    //EN Sex
//    PMC_PROVINCE,                               //EN Province
//    PMC_EDUC,                                   //EN Education
//    PMC_POB,                                    //EN Province of birth
//    PMC_UNION,                                  //EN Union formation time
//    PMC_PARITY,                                 //EN Number of births
//    PMC_LASTBIR                                 //EN Time of last birth
//};

//parameters
//{
//    file		MicroDataInputFile;             //EN File name of starting population
//    long        MicroDataInputFileSize;         //EN File size of starting population
//    logical     UseWeightsInFile;               //EN Use weights in starting population file
//};

//parameter_group PG00_MicroDataInputFile         //EN Starting Population file
//{
//    MicroDataInputFile, MicroDataInputFileSize,
//    UseWeightsInFile
//};

// The Observation actor
//actor_set Observation asAllObservations;        //EN All Observations

//actor Observation                               //EN Actor Observations
//{
//    // values from file
//    double  pmc[PERSON_MICRODATA_COLUMNS];      //EN Person micro record columns
//
//    // states
//    integer int_weight = { 1.0 };               //EN Integer Weight
//
//    // functions
//    void Start(const input_csv& input);         //EN Starts the actor
//    void Finish();                              //EN Destroys the actor
//    void WeightOrKill();                        //EN Decides if an observation stays in sample and how often
//};

void Observation::Start(const input_csv& in_csv)
{ if (__finished) {StartAfterFinish(); return; }; StartSpecial1();
    for (int nJ = 0; nJ < SIZE(PERSON_MICRODATA_COLUMNS); nJ++) 
    {
        pmc[CHKBND(nJ,9,"Observation.pmc",0)] = in_csv[nJ];
    }
StartSpecial2();};

void Observation::Finish(){if (__finished) {	DoubleFinish(case_seed);	return; } else __finished = TRUE;FinishSpecial();};

/*  NOTE(Observation.WeightOrKill, EN)
    The function WeightOrKill calculates integer weights for each record in the file.
    Based on the non-integer part of the weight, a random number is used to decide if 
    weights are rounded up or down. Actors with a resulting integer weight 0 are removed.
    Weights add up to the sample size parameter.
*/
void Observation::WeightOrKill()
{
    double  dWeight = 0.0;
    int     nWeight = 0;
    
    if (!gprParam->UseWeightsInFile) dWeight = (double)gprParam->StartPopSampleSize / (double)gprParam->MicroDataInputFileSize;
    else dWeight = pmc[CHKBND(PMC_WEIGHT,9,"Observation.pmc",0)] * (double)gprParam->StartPopSampleSize / gprParam->StartPopSize;
    
    nWeight = (int)dWeight;
    dWeight = dWeight - nWeight;
    if (RandUniform(3) < dWeight) nWeight++;
     int_weight =   nWeight ;
    if ( int_weight == 0) Finish();
}

//actor Person
//{
//    void GetStartCharacteristics();			                //EN Get charcteristics at birth
//};

void Person::GetStartCharacteristics()
{
    // Sample an observation
    Observation * prObservation = asAllObservations->GetRandom(RandUniform(2));

    // Assigns values from observation
     poStateFunctions->Set_time_of_birth(  prObservation->pmc[PMC_BIRTH] );								
    if ((int)prObservation->pmc[PMC_SEX] == 1)  poStateFunctions->Set_sex(  MALE );	else  poStateFunctions->Set_sex(  FEMALE );	
     poStateFunctions->Set_province_nat(  (PROVINCE_NAT)(int)prObservation->pmc[PMC_PROVINCE] );			
     poStateFunctions->Set_province_birth(  (PROVINCE_INT)(int)prObservation->pmc[PMC_POB] );
     poStateFunctions->Set_primary_level(  (PRIMARY_LEVEL)(int)prObservation->pmc[PMC_EDUC] );

    if (prObservation->pmc[PMC_UNION] < MIN(SIM_YEAR_RANGE)) 
    {
         poStateFunctions->Set_time_of_union_formation(  prObservation->pmc[PMC_UNION] );
    }
    if (  sex == FEMALE )
    {
         poStateFunctions->Set_parity(  (int)prObservation->pmc[PMC_PARITY] );
        if (  parity > 0 && MIN(SIM_YEAR_RANGE) >  prObservation->pmc[PMC_LASTBIR] )
        {
             poStateFunctions->Set_time_since_last_birth_start(  MIN(SIM_YEAR_RANGE) -  prObservation->pmc[PMC_LASTBIR] );
            //STEP18
             time_of_last_birth =   prObservation->pmc[PMC_LASTBIR] ;
        }
    }

    // Decrement the integer weight of the observation actor and kill it if not needed anymore
    prObservation->UpdateTime() -> int_weight--;
    if (prObservation-> int_weight <= 0 ) prObservation->UpdateTime()->Finish();
}

} // namespace 
