﻿#include "model.h"
static int HaveAGoodDay;
namespace mm {
#line 1 "code/UnionFormation.mpp"
//LABEL(UnionFormation, EN) First Union Formation

/*	NOTE(UnionFormation, EN)

    Added at Step 14

	This module models first union formation of women. The module allows a choice between two approaches:
    The first option is applying a Coale & McNeil model. This allows parameterizing union formation 
    by three parameters
	- youngest age 
	- average age of union formation
	- proportion of the population ever entering a union
    Alternatively, users can specify period rates of age-specific union formation risks.
*/

// Females by age and education
//actor_set Person asFemaleAgeEduc[integer_age][primary_level] filter set_alive && sex==FEMALE;  

//range YOB_UNION { 1964, 2050 };             //EN Year of birth
//range AGE_UNION { 10, 60 };                 //EN Age

//classification UNION_PARA                   //EN Union formation parameters
//{
//	UP_MINAGE,                              //EN Age at which first 1% enter a union
//	UP_AVERAGE,                             //EN Average age at first union formation
//	UP_EVER                                 //EN Proportion ow femailes ever entering a union
//};

//classification UNION_MODEL_SELECTION        //EN Model selection
//{
//    UMS_COALE,                              //EN Coale & McNeil
//    UMS_RATES                               //EN Rates by age, period and education
//};



//parameters 
//{
//    //EN Model selection for union formation
//    UNION_MODEL_SELECTION UnionModelSelection;                                            
//
//    //EN Union formation (Option A): Coale and McNeil 
//    double UnionParameters[PRIMARY_LEVEL][YOB_UNION][UNION_PARA];				
//
//    //EN Union formation (Option B): Hazards
//    double UnionFormationHazardParameter[PRIMARY_LEVEL][AGE_UNION][SIM_YEAR_RANGE];	
//
//    //EN Union formation hazards
//    model_generated double UnionFormationHazard[PRIMARY_LEVEL][SIM_YEAR_RANGE][AGE_UNION];	
//};

//parameter_group PG_UnionFormation           //EN Union formation 
//{
//	UnionModelSelection, UnionParameters,
//    UnionFormationHazardParameter
//};

//actor Person
//{
//    AGE_UNION	age_union = COERCE(AGE_UNION, integer_age);	        //EN Age
//
//	//EN Union formation hazard
//	double		union_formation_hazard = (WITHIN(AGE_UNION, integer_age) && sex==FEMALE) ? 
//					UnionFormationHazard[primary_level][RANGE_POS(SIM_YEAR_RANGE, calendar_year)]
//                    [RANGE_POS(AGE_UNION, integer_age)] : 0.0;
//	
//	TIME		time_of_union_formation = { TIME_INFINITE };        //EN Time of union formation
//	logical		in_union = { FALSE };                               //EN Ever in union
//	event		timeUnionFormationEvent, UnionFormationEvent;       //EN First union formation event
//    void        SetImmigrantUnionFormation();                       //EN Union formation date for immigrants
//};


/*  NOTE(Person.SetImmigrantUnionFormation, EN)
    Function to sample a union formation age for immigrants based on age and primary education level. 
    This function is called at immigration. The host population is the total population 
    of the same age and education. Union formation age is modeled for females only.
*/
void Person::SetImmigrantUnionFormation()
{
    if ( person_type == PT_IMMIGRANT && sex ==FEMALE && asFemaleAgeEduc[integer_age ][primary_level ]->Count()>0)
    {
        Person * prPerson = new Person;
        prPerson = asFemaleAgeEduc[integer_age ][primary_level ]->GetRandom(RandUniform(27));
         poStateFunctions->Set_time_of_union_formation(  prPerson->time_of_union_formation  );
        if ( time_of_union_formation != TIME_INFINITE)  poStateFunctions->Set_in_union(  TRUE );

         poStateFunctions->Set_parity(  prPerson->parity  );
         poStateFunctions->Set_time_since_last_birth_start(  prPerson->time_since_last_birth  );
    }
}

TIME Person::timeUnionFormationEvent()
{
	if ( calendar_year >= MIN(SIM_YEAR_RANGE) &  sex == FEMALE && ! in_union && WITHIN(AGE_UNION, integer_age ) 
        &&  union_formation_hazard > 0.0)
	{
		return WAIT(-TIME(log(RandUniform(26)) / union_formation_hazard ));
	}
	else if ( time_of_union_formation < MIN(SIM_YEAR_RANGE)  &  sex == FEMALE 
        && ! in_union &  calendar_year < MIN(SIM_YEAR_RANGE))
	{
		return time_of_union_formation ;
	}
	else return TIME_INFINITE;
}

void Person::UnionFormationEvent()
{
	 poStateFunctions->Set_in_union(  TRUE );
	 poStateFunctions->Set_time_of_union_formation(  time  );
}

/* 
    GetUnionFormationHazard() returns the union formation hazard for given education, age and period based
    on a Coale McNeil model. This is a parametric model with three parameters
	- youngest age 
	- average age of union formation
	- proportion of the population ever entering a union
    This information is provided by a model parameter "UnionParameters". 
*/
double GetUnionFormationHazard(int nEduc, int nYear, int nAge);
double GetUnionFormationHazard(int nEduc, int nYear, int nAge)
{
	double	dReturnValue = 0.0;
	double	g[SIZE(AGE_UNION)];
	double	a0 = gprParam->UnionParameters[CHKBND(nEduc,3,"UnionParameters",0)][CHKBND(RANGE_POS(YOB_UNION, nYear - nAge),87,"UnionParameters",1)][CHKBND(UP_MINAGE,3,"UnionParameters",2)];
	double	my = gprParam->UnionParameters[CHKBND(nEduc,3,"UnionParameters",0)][CHKBND(RANGE_POS(YOB_UNION, nYear - nAge),87,"UnionParameters",1)][CHKBND(UP_AVERAGE,3,"UnionParameters",2)];
	double	C = gprParam->UnionParameters[CHKBND(nEduc,3,"UnionParameters",0)][CHKBND(RANGE_POS(YOB_UNION, nYear - nAge),87,"UnionParameters",1)][CHKBND(UP_EVER,3,"UnionParameters",2)];
	double	k = (my - a0) / 11.36;
	double dThisYearCumulative = 0.0;
	if (nAge < a0) dReturnValue = 0.0;
	else if ( nAge >= a0 )
	{
		for (int nX = 0; nX < SIZE(AGE_UNION); nX++)
		{
			g[nX] = 0.19465*exp(-0.174*(nX - 6.06) - exp(-0.288*(nX - 6.06)));
		}
		double	index = ((double)nAge - a0) / k;
		int		nIndex = int(index);		// integer part
		double	dIndex = index - nIndex;		// after comma part
		double G = 0.0;
		for (int nI = 0; nI <= nIndex; nI++)
		{
			if (nI < SIZE(AGE_UNION)) G = G + g[nI];
		}
		if (nIndex < SIZE(AGE_UNION) - 1) G = G + dIndex*g[nIndex + 1];
		dThisYearCumulative = G * C;
	}
	if (nAge == a0) dReturnValue = dThisYearCumulative;
	else if ( nAge > a0 )
	{
		double	index = ((double)nAge - a0 -1) / k;
		int		nIndex = int(index);		// integer part
		double	dIndex = index - nIndex;		// after comma part
		double G = 0.0;
		for (int nI = 0; nI <= nIndex; nI++)
		{
			if (nI < SIZE(AGE_UNION)) G = G + g[nI];
		}
		if (nIndex < SIZE(AGE_UNION) - 1) G = G + dIndex*g[nIndex + 1];
		if (1.0 - G * C > 0.0001) dReturnValue = (dThisYearCumulative - G * C) / (1 - G * C); 
        else dReturnValue = 0.0;

	}
	// convert probability to hazard
	if (dReturnValue < 1.0)  dReturnValue = -log(1 - dReturnValue);
	return dReturnValue;
}

/*
    In the presimulation of the union formation module, age specific hazards for union formation
    are calculated and copied into a model-generated parameter. Depending on the model selected by the user,
    values are calculated using a Coale McNeil model, or come directly from a parameter table.
*/
void PreSimulation5()
{
	for (int nEduc = 0; nEduc < SIZE(PRIMARY_LEVEL); nEduc++)
	{
		for (int nYear = MIN(SIM_YEAR_RANGE); nYear <= MAX(SIM_YEAR_RANGE); nYear++)
		{
			for (int nAge = MIN(AGE_UNION); nAge <= MAX(AGE_UNION); nAge++)
			{
                if (gprParam->UnionModelSelection == UMS_COALE)
                {
				    gprParam->UnionFormationHazard[CHKBND(nEduc,3,"UnionFormationHazard",0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE,nYear),101,"UnionFormationHazard",1)][CHKBND(RANGE_POS(AGE_UNION,nAge),51,"UnionFormationHazard",2)] 
                    = GetUnionFormationHazard(nEduc,nYear,nAge);
                }
                else
                {
                    gprParam->UnionFormationHazard[CHKBND(nEduc,3,"UnionFormationHazard",0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE,nYear),101,"UnionFormationHazard",1)][CHKBND(RANGE_POS(AGE_UNION,nAge),51,"UnionFormationHazard",2)] 
                    = gprParam->UnionFormationHazardParameter[CHKBND(nEduc,3,"UnionFormationHazardParameter",0)][CHKBND(RANGE_POS(AGE_UNION,nAge),51,"UnionFormationHazardParameter",1)]
                    [CHKBND(RANGE_POS(SIM_YEAR_RANGE,nYear),101,"UnionFormationHazardParameter",2)];
                }
			}
		}
	}
}



} // namespace 
