#include "model.h"

namespace mm {

#if defined(MG_CHKSUM)
	bool ::gbCheckSumActivated = true;
#else
	bool ::gbCheckSumActivated = false;
#endif

#pragma warning(disable : 4723)

ModelApp gApp( Simulation, InitParameters, DeleteParameters,
	ProcessModelGeneratedParameters, InitActors, DeleteActors,
	InitTables, PreSimulation, PostSimulation, UserTables,
	VerifyParameters, ValidateParameters, InitActorSets, DeleteActorSets );

void AFXAPI AfxAbort()
{
	TRACE0("AfxAbort called\n");
	DebugBreak();
#ifdef _WINDOWS
	AfxWinTerm();
#endif
	abort();
}

Thread EventQueue *gpoEventQueue;

Thread double ::gdThreadCurrentTime = 0;

Thread double ::gdThreadEvents = 0;

Thread double ::gdEventsForTabulation = 0;

Parameters	*gprParam;

BOOL VerifyParameters( CString *pszError)
{
	BOOL	bResult = TRUE;

	(*pszError).Empty();
	if ( !VerifyParameter( pszError, _T("AgeEnterPrimary") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("AgeGradPrimary") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("AgeImmigrants") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("AgeSpecificFertility") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("BirthTrends") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("CalibratedYear") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ChildMortalityBaseRisk") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ChildMortalityRelativeRisks") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ChildMortalityTrend") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("DestinationImmigrants") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("EmigrationRates") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("FirstBirthRates") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("GradPrimary") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("HigherOrderBirthsPara") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("LifeExpectancy") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("MicroDataInputFile") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("MicroDataInputFileSize") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("MicroRecordFileName") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("MigrationDestination") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("MigrationProbability") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("MigrationProbabilityEduc") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ModelByEducation") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ModelEmigration") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ModelImmigration") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ModelMigration") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("MortalityTable") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("NumberImmigrants") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("OddsMotherEnterPrimary") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("OddsMotherGradPrimary") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("ScalePopulation") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("selected_fertility_model") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("SelectedChildMortalityModel") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("SelectedOutput") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("SexRatio") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("StartOfSchoolYear") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("StartPopSampleSize") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("StartPopSize") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("StartPrimary") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("TimeMicroOutput") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("TotalFertilityRate") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("UnionFormationHazardParameter") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("UnionModelSelection") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("UnionParameters") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("UseMothersEducation") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("UseWeightsInFile") ) ) bResult = FALSE;
	if ( !VerifyParameter( pszError, _T("WriteMicrodata") ) ) bResult = FALSE;
	return bResult;
}

void InitParameters()
{
	gprParam = new Parameters();
}

void DeleteParameters()
{
	delete gprParam;
}

void SetCaseWeight( double dCaseWeight, double dCaseSubsampleWeight )
{
	gpoEventQueue->SetCaseWeight( dCaseWeight, dCaseSubsampleWeight ); 
}

// partitions
real p_points_AGE5_PART[] = { 5, 10, 15, 20, 25, 30, 35, 40, 45, 50,
	 55, 60 };
real p_points_AGE_15_16_18[] = { 15, 17, 19 };
real p_points_AGE_15_17[] = { 15, 17 };
real p_points_AGE_FIVE[] = { 5, 10, 15, 20, 25, 30, 35, 40, 45, 50,
	 55, 60, 65, 70, 75, 80 };
real p_points_AGE_MORT[] = { 1, 5, 15, 65 };
real p_points_AGE_RTAB_MORT[] = { 1 };
real p_points_BIRTH_AGE_PART[] = { 35, 40, 45 };
real p_points_DUR_TIME_IN_PARITY[] = { 1, 3, 6, 9, 12 };
real p_points_LITTLE_TIME[] = { 0.001 };

Parameters::Parameters()
{
	memset( &ActorWeight, '\0', 1 * sizeof( double ) );
	Initializer( &AgeEnterPrimary, _T("AgeEnterPrimary") );
	Initializer( &AgeGradPrimary, _T("AgeGradPrimary") );
	Initializer( &AgeImmigrants, _T("AgeImmigrants") );
	Initializer( &AgeSpecificFertility, _T("AgeSpecificFertility") );
	memset( &AgeSpecificFertilityRate, '\0', 4040 * sizeof( double ) );
	Initializer( &BirthTrends, _T("BirthTrends") );
	Initializer( &CalibratedYear, _T("CalibratedYear") );
	Initializer( &ChildMortalityBaseRisk, _T("ChildMortalityBaseRisk") );
	Initializer( &ChildMortalityRelativeRisks, _T("ChildMortalityRelativeRisks") );
	Initializer( &ChildMortalityTrend, _T("ChildMortalityTrend") );
	Initializer( &DestinationImmigrants, _T("DestinationImmigrants") );
	Initializer( &EmigrationRates, _T("EmigrationRates") );
	Initializer( &FirstBirthRates, _T("FirstBirthRates") );
	Initializer( &GradPrimary, _T("GradPrimary") );
	Initializer( &HigherOrderBirthsPara, _T("HigherOrderBirthsPara") );
	memset( &ImmiPopSampleSize, '\0', 1 * sizeof( long ) );
	memset( &ImmiPopSize, '\0', 1 * sizeof( double ) );
	Initializer( &LifeExpectancy, _T("LifeExpectancy") );
	Initializer( &MicroDataInputFile, _T("MicroDataInputFile") );
	Initializer( &MicroDataInputFileSize, _T("MicroDataInputFileSize") );
	Initializer( &MicroRecordFileName, _T("MicroRecordFileName") );
	Initializer( &MigrationDestination, _T("MigrationDestination") );
	Initializer( &MigrationProbability, _T("MigrationProbability") );
	Initializer( &MigrationProbabilityEduc, _T("MigrationProbabilityEduc") );
	memset( &model_is_case_based, '\0', 1 * sizeof( logical ) );
	Initializer( &ModelByEducation, _T("ModelByEducation") );
	Initializer( &ModelEmigration, _T("ModelEmigration") );
	Initializer( &ModelImmigration, _T("ModelImmigration") );
	Initializer( &ModelMigration, _T("ModelMigration") );
	Initializer( &MortalityTable, _T("MortalityTable") );
	memset( &MortalityTrend, '\0', 202 * sizeof( double ) );
	Initializer( &NumberImmigrants, _T("NumberImmigrants") );
	memset( &NumberImmigrantsTable, '\0', 202 * sizeof( double ) );
	Initializer( &OddsMotherEnterPrimary, _T("OddsMotherEnterPrimary") );
	Initializer( &OddsMotherGradPrimary, _T("OddsMotherGradPrimary") );
	Initializer( &ScalePopulation, _T("ScalePopulation") );
	Initializer( &selected_fertility_model, _T("selected_fertility_model") );
	Initializer( &SelectedChildMortalityModel, _T("SelectedChildMortalityModel") );
	Initializer( &SelectedOutput, _T("SelectedOutput") );
	Initializer( &SexRatio, _T("SexRatio") );
	Initializer( &StartOfSchoolYear, _T("StartOfSchoolYear") );
	Initializer( &StartPopSampleSize, _T("StartPopSampleSize") );
	Initializer( &StartPopSize, _T("StartPopSize") );
	Initializer( &StartPrimary, _T("StartPrimary") );
	Initializer( &TimeMicroOutput, _T("TimeMicroOutput") );
	Initializer( &TotalFertilityRate, _T("TotalFertilityRate") );
	memset( &TotalPopSampleSize, '\0', 1 * sizeof( long ) );
	memset( &TotalPopSize, '\0', 1 * sizeof( double ) );
	memset( &UnionFormationHazard, '\0', 15453 * sizeof( double ) );
	Initializer( &UnionFormationHazardParameter, _T("UnionFormationHazardParameter") );
	Initializer( &UnionModelSelection, _T("UnionModelSelection") );
	Initializer( &UnionParameters, _T("UnionParameters") );
	Initializer( &UseMothersEducation, _T("UseMothersEducation") );
	Initializer( &UseWeightsInFile, _T("UseWeightsInFile") );
	Initializer( &WriteMicrodata, _T("WriteMicrodata") );
}

void ProcessModelGeneratedParameters()
{
	ProcessParameter( &gprParam->ActorWeight, _T("ActorWeight"), NULL );
	ProcessParameter( &gprParam->AgeSpecificFertilityRate, _T("AgeSpecificFertilityRate"), NULL );
	ProcessParameter( &gprParam->ImmiPopSampleSize, _T("ImmiPopSampleSize"), NULL );
	ProcessParameter( &gprParam->ImmiPopSize, _T("ImmiPopSize"), NULL );
	ProcessParameter( &gprParam->model_is_case_based, _T("model_is_case_based"), NULL );
	ProcessParameter( &gprParam->MortalityTrend, _T("MortalityTrend"), NULL );
	ProcessParameter( &gprParam->NumberImmigrantsTable, _T("NumberImmigrantsTable"), &gprParam->NumberImmigrantsTable_lookup );
	ProcessParameter( &gprParam->TotalPopSampleSize, _T("TotalPopSampleSize"), NULL );
	ProcessParameter( &gprParam->TotalPopSize, _T("TotalPopSize"), NULL );
	ProcessParameter( &gprParam->UnionFormationHazard, _T("UnionFormationHazard"), NULL );
}

bool Lookup_AgeImmigrants( long lNumber, int nInd0, int *pnDest0 )
{
	return CumRateSearch( lNumber, (long *) &gprParam->AgeImmigrants[nInd0], 1, 101, pnDest0 );
}

bool Lookup_AgeImmigrants( double dNumber, int nInd0, int *pnDest0 )
{

	long lNumber;
	lNumber = (long) ( dNumber * RG_M_DOUBLE);

	return CumRateSearch( lNumber, (long *) &gprParam->AgeImmigrants[nInd0], 1, 101, pnDest0 );
}

int EmptyDistributions_AgeImmigrants( void )
{
	return ((ParameterSymbol *)gppiModelInfo->aoParameters[3])->m_nEmptyDistributions;
}

bool Lookup_DestinationImmigrants( long lNumber, int nInd0, int nInd1, int *pnDest0 )
{
	return CumRateSearch( lNumber, (long *) &gprParam->DestinationImmigrants[nInd0][nInd1], 1, 13, pnDest0 );
}

bool Lookup_DestinationImmigrants( double dNumber, int nInd0, int nInd1, int *pnDest0 )
{

	long lNumber;
	lNumber = (long) ( dNumber * RG_M_DOUBLE);

	return CumRateSearch( lNumber, (long *) &gprParam->DestinationImmigrants[nInd0][nInd1], 1, 13, pnDest0 );
}

int EmptyDistributions_DestinationImmigrants( void )
{
	return ((ParameterSymbol *)gppiModelInfo->aoParameters[11])->m_nEmptyDistributions;
}

bool Lookup_MigrationDestination( long lNumber, int nInd0, int nInd1, int nInd2, int *pnDest0 )
{
	return CumRateSearch( lNumber, (long *) &gprParam->MigrationDestination[nInd0][nInd1][nInd2], 1, 13, pnDest0 );
}

bool Lookup_MigrationDestination( double dNumber, int nInd0, int nInd1, int nInd2, int *pnDest0 )
{

	long lNumber;
	lNumber = (long) ( dNumber * RG_M_DOUBLE);

	return CumRateSearch( lNumber, (long *) &gprParam->MigrationDestination[nInd0][nInd1][nInd2], 1, 13, pnDest0 );
}

int EmptyDistributions_MigrationDestination( void )
{
	return ((ParameterSymbol *)gppiModelInfo->aoParameters[22])->m_nEmptyDistributions;
}

bool Lookup_NumberImmigrantsTable( long lNumber, int *pnDest0, int *pnDest1 )
{
	return CumRateSearch( lNumber, (long *) &gprParam->NumberImmigrantsTable_lookup, 2, 101, 2, pnDest0, pnDest1 );
}

bool Lookup_NumberImmigrantsTable( double dNumber, int *pnDest0, int *pnDest1 )
{

	long lNumber;
	lNumber = (long) ( dNumber * RG_M_DOUBLE);

	return CumRateSearch( lNumber, (long *) &gprParam->NumberImmigrantsTable_lookup, 2, 101, 2, pnDest0, pnDest1 );
}

int EmptyDistributions_NumberImmigrantsTable( void )
{
	return ((ParameterSymbol *)gppiModelInfo->aoParameters[33])->m_nEmptyDistributions;
}

bool IntIs_CHILD_MORTALITY_RISKS( int nNumber )
{
	switch (nNumber) {
		case 14: return true;
		case 16: return true;
	};
	return false;
}

CHILD_MORTALITY_RISKS IntTo_CHILD_MORTALITY_RISKS( int nNumber )
{
	switch (nNumber) {
		case 14: return CMR_AGE14;
		case 16: return CMR_AGE16;
	};
	return CMR_PRIMDROP;
}

int IntFrom_CHILD_MORTALITY_RISKS( CHILD_MORTALITY_RISKS nLevel )
{
	switch (nLevel) {
		case CMR_AGE14: return 14;
		case CMR_AGE16: return 16;
	};
	return -1;
}

bool IntIs_HIGHER_BIRTHS_PARA( int nNumber )
{
	switch (nNumber) {
		case 1: return true;
		case 2: return true;
		case 3: return true;
		case 4: return true;
		case 5: return true;
		case 6: return true;
		case 35: return true;
		case 40: return true;
		case 45: return true;
	};
	return false;
}

HIGHER_BIRTHS_PARA IntTo_HIGHER_BIRTHS_PARA( int nNumber )
{
	switch (nNumber) {
		case 1: return HBP_PERIOD1;
		case 2: return HBP_PERIOD2;
		case 3: return HBP_PERIOD3;
		case 4: return HBP_PERIOD4;
		case 5: return HBP_PERIOD5;
		case 6: return HBP_PERIOD6;
		case 35: return HBP_AGE35;
		case 40: return HBP_AGE40;
		case 45: return HBP_AGE45;
	};
	return HBP_EDUC2;
}

int IntFrom_HIGHER_BIRTHS_PARA( HIGHER_BIRTHS_PARA nLevel )
{
	switch (nLevel) {
		case HBP_PERIOD1: return 1;
		case HBP_PERIOD2: return 2;
		case HBP_PERIOD3: return 3;
		case HBP_PERIOD4: return 4;
		case HBP_PERIOD5: return 5;
		case HBP_PERIOD6: return 6;
		case HBP_AGE35: return 35;
		case HBP_AGE40: return 40;
		case HBP_AGE45: return 45;
		case HBP_EDUC1: return 1;
		case HBP_EDUC2: return 2;
	};
	return -1;
}

bool IntIs_MOTHER_AGE_CLASS( int nNumber )
{
	switch (nNumber) {
		case 14: return true;
		case 16: return true;
	};
	return false;
}

MOTHER_AGE_CLASS IntTo_MOTHER_AGE_CLASS( int nNumber )
{
	switch (nNumber) {
		case 14: return MAC_14;
		case 16: return MAC_16;
	};
	return MAC_17P;
}

int IntFrom_MOTHER_AGE_CLASS( MOTHER_AGE_CLASS nLevel )
{
	switch (nLevel) {
		case MAC_14: return 14;
		case MAC_16: return 16;
	};
	return -1;
}

bool IntIs_PROVINCE_INT( int nNumber )
{
	switch (nNumber) {
		case 0: return true;
		case 1: return true;
		case 2: return true;
		case 3: return true;
		case 4: return true;
		case 5: return true;
		case 6: return true;
		case 7: return true;
		case 8: return true;
		case 9: return true;
		case 10: return true;
		case 11: return true;
		case 12: return true;
		case 13: return true;
	};
	return false;
}

PROVINCE_INT IntTo_PROVINCE_INT( int nNumber )
{
	switch (nNumber) {
		case 0: return PI_PROV00;
		case 1: return PI_PROV01;
		case 2: return PI_PROV02;
		case 3: return PI_PROV03;
		case 4: return PI_PROV04;
		case 5: return PI_PROV05;
		case 6: return PI_PROV06;
		case 7: return PI_PROV07;
		case 8: return PI_PROV08;
		case 9: return PI_PROV09;
		case 10: return PI_PROV10;
		case 11: return PI_PROV11;
		case 12: return PI_PROV12;
		case 13: return PI_PROV13;
	};
	return PI_PROV13;
}

int IntFrom_PROVINCE_INT( PROVINCE_INT nLevel )
{
	switch (nLevel) {
		case PI_PROV00: return 0;
		case PI_PROV01: return 1;
		case PI_PROV02: return 2;
		case PI_PROV03: return 3;
		case PI_PROV04: return 4;
		case PI_PROV05: return 5;
		case PI_PROV06: return 6;
		case PI_PROV07: return 7;
		case PI_PROV08: return 8;
		case PI_PROV09: return 9;
		case PI_PROV10: return 10;
		case PI_PROV11: return 11;
		case PI_PROV12: return 12;
		case PI_PROV13: return 13;
	};
	return -1;
}

bool IntIs_PROVINCE_NAT( int nNumber )
{
	switch (nNumber) {
		case 0: return true;
		case 1: return true;
		case 2: return true;
		case 3: return true;
		case 4: return true;
		case 5: return true;
		case 6: return true;
		case 7: return true;
		case 8: return true;
		case 9: return true;
		case 10: return true;
		case 11: return true;
		case 12: return true;
	};
	return false;
}

PROVINCE_NAT IntTo_PROVINCE_NAT( int nNumber )
{
	switch (nNumber) {
		case 0: return PN_PROV00;
		case 1: return PN_PROV01;
		case 2: return PN_PROV02;
		case 3: return PN_PROV03;
		case 4: return PN_PROV04;
		case 5: return PN_PROV05;
		case 6: return PN_PROV06;
		case 7: return PN_PROV07;
		case 8: return PN_PROV08;
		case 9: return PN_PROV09;
		case 10: return PN_PROV10;
		case 11: return PN_PROV11;
		case 12: return PN_PROV12;
	};
	return PN_PROV12;
}

int IntFrom_PROVINCE_NAT( PROVINCE_NAT nLevel )
{
	switch (nLevel) {
		case PN_PROV00: return 0;
		case PN_PROV01: return 1;
		case PN_PROV02: return 2;
		case PN_PROV03: return 3;
		case PN_PROV04: return 4;
		case PN_PROV05: return 5;
		case PN_PROV06: return 6;
		case PN_PROV07: return 7;
		case PN_PROV08: return 8;
		case PN_PROV09: return 9;
		case PN_PROV10: return 10;
		case PN_PROV11: return 11;
		case PN_PROV12: return 12;
	};
	return -1;
}

bool IntIs_RTAB_SEL_YEARS( int nNumber )
{
	switch (nNumber) {
		case 2017: return true;
		case 2030: return true;
		case 2050: return true;
	};
	return false;
}

RTAB_SEL_YEARS IntTo_RTAB_SEL_YEARS( int nNumber )
{
	switch (nNumber) {
		case 2017: return SY_2017;
		case 2030: return SY_2030;
		case 2050: return SY_2050;
	};
	return SY_2050;
}

int IntFrom_RTAB_SEL_YEARS( RTAB_SEL_YEARS nLevel )
{
	switch (nLevel) {
		case SY_2017: return 2017;
		case SY_2030: return 2030;
		case SY_2050: return 2050;
	};
	return -1;
}

void InitActors()
{
	gpoEventQueue = new EventQueue();
	gpoEmptyChildMortalityCalibrator = NULL;
	gpoEmptyObservation = NULL;
	gpoEmptyPerson = NULL;
	gpoEmptyPrimarySchool = NULL;
}

void DeleteActors()
{
	delete gpoEventQueue;
	gctrlSection.Lock();
	gctrlSection.Unlock();



	ChildMortalityCalibrator *poChildMortalityCalibrator = gpoEmptyChildMortalityCalibrator;
	while ( poChildMortalityCalibrator != NULL ) {
		ChildMortalityCalibrator *poChildMortalityCalibratorTemp = (ChildMortalityCalibrator *) poChildMortalityCalibrator->poNext;
		::delete poChildMortalityCalibrator;
		poChildMortalityCalibrator = poChildMortalityCalibratorTemp;
	}


	Observation *poObservation = gpoEmptyObservation;
	while ( poObservation != NULL ) {
		Observation *poObservationTemp = (Observation *) poObservation->poNext;
		::delete poObservation;
		poObservation = poObservationTemp;
	}


	Person *poPerson = gpoEmptyPerson;
	while ( poPerson != NULL ) {
		Person *poPersonTemp = (Person *) poPerson->poNext;
		::delete poPerson;
		poPerson = poPersonTemp;
	}


	PrimarySchool *poPrimarySchool = gpoEmptyPrimarySchool;
	while ( poPrimarySchool != NULL ) {
		PrimarySchool *poPrimarySchoolTemp = (PrimarySchool *) poPrimarySchool->poNext;
		::delete poPrimarySchool;
		poPrimarySchool = poPrimarySchoolTemp;
	}
}

// sizeof actors
int ::ganSizeOfActors[4] = {
	sizeof(ChildMortalityCalibrator) + sizeof(ChildMortalityCalibratorDerivedStates) + sizeof(ChildMortalityCalibratorTable),
	sizeof(Observation) + sizeof(ObservationDerivedStates),
	sizeof(Person) + sizeof(PersonDerivedStates) + sizeof(PersonTable),
	sizeof(PrimarySchool) + sizeof(PrimarySchoolDerivedStates),
};

void *ChildMortalityCalibrator::operator new( size_t count )
{
	ChildMortalityCalibrator *poActor;
	if ( gpoEmptyChildMortalityCalibrator != NULL ) {
		poActor = gpoEmptyChildMortalityCalibrator;
		gpoEmptyChildMortalityCalibrator = (ChildMortalityCalibrator *) gpoEmptyChildMortalityCalibrator->poNext;
	}
	else {
		poActor = ::new ChildMortalityCalibrator( TRUE ) ;
	}
	poActor->__finished = false;
	return poActor;
}

void ChildMortalityCalibrator::Finish()
{
	if (__finished)
	{
		DoubleFinish(case_seed);
		return;
	}
	else __finished = TRUE;
	FinishSpecial();
}

void ChildMortalityCalibrator::InitActor()
{
	poDerivedStates = new ChildMortalityCalibratorDerivedStates(); 
	poDerivedStates->poParent = this;
	poStateFunctions = new ChildMortalityCalibratorStateFunctions();
	poStateFunctions->poParent = this;
	poStateFunctions->poDerivedStates = poDerivedStates;
	poDerivedStates->poStateFunctions = poStateFunctions;
	bUpdates = FALSE;
	nActorNumber = 0;
	InitializeStates();
	mlBabies = new ChildMortalityCalibrator_mlBabies( this );
}

void ChildMortalityCalibrator::DeleteActor()
{
	delete poDerivedStates;
	delete poStateFunctions;
	delete mlBabies;
}

#pragma optimize( "", off )

void ChildMortalityCalibrator::InitializeStates()
{
	// initialization of states
	actor_id = (long) 0;
	actor_subsample_weight = (double) 0;
	actor_weight = (double) 0;
	age = (TIME) 0;
#line 105 "code/ChildMortality.mpp"
	calibration_is_done = FALSE != 0;
#line 642 "modgen/src/ACTORS.CPP"
	case_id = (long) 0;
	case_seed = (double) 0;
	mort_female_0 = (double) 0;
	mort_female_1 = (double) 0;
	mort_female_2 = (double) 0;
	mort_female_3 = (double) 0;
	mort_female_4 = (double) 0;
	mort_male_0 = (double) 0;
	mort_male_1 = (double) 0;
	mort_male_2 = (double) 0;
	mort_male_3 = (double) 0;
	mort_male_4 = (double) 0;
	time = (TIME) gdThreadCurrentTime;
	actorset_filter_asTheChildMortalityCalibrator = FALSE;
	c_year = r_min_SIM_YEAR_RANGE;
	FemaleMortalityTrend = (double) 0;
	FemaleMortalityTrend_max_value_out = (double) 0;
	int_age = (integer) 0;
	MaleMortalityTrend = (double) 0;
	MaleMortalityTrend_value_out = (double) 0;
	poDerivedStates->ssint_age = (short) 0;
	poDerivedStates->ssint_age_time = (TIME) TIME_INFINITE;
	poDerivedStates->ssint_time = (short) 0;
	poDerivedStates->ssint_time_time = (TIME) TIME_INFINITE;
	table_filter_TabMortalityTrendFactor = FALSE;
	__finished = FALSE;
	__time = (TIME) gdThreadCurrentTime;
	__events = 0;
}

#pragma optimize( "", on )

void ChildMortalityCalibrator::UpdateLinkedStates()
{
}

void ChildMortalityCalibrator::StartSpecial1()
{
	nSubSample = GetSubSample();
	InitializeStates();
	case_seed = GetCaseSeed();
	case_id = GetCaseID();
	lActorId = actor_id = GetObjectID();
	GetCaseWeight( &actor_weight, &actor_subsample_weight );
	mlBabies->RemoveAll();

	// initialization of actor set nodes
	poasTheChildMortalityCalibrator = NULL;

	// initialization of events
	poChildMortalityCalibrationEvent = gpoEventQueue->NewEvent( this, 2, 0 );
	poScheduleEvent = gpoEventQueue->NewEvent( this, 8, 255 );
	schedule_event_time = TIME_INFINITE;
	//Initalize derived states
#line 98 "code/Tables.mpp"
	poStateFunctions->Set_c_year( SIM_YEAR_RANGE( COERCE(SIM_YEAR_RANGE,poDerivedStates->ssint_time) ) );
#line 699 "modgen/src/ACTORS.CPP"
	c_year = c_year < r_min_SIM_YEAR_RANGE ? r_min_SIM_YEAR_RANGE : c_year > r_max_SIM_YEAR_RANGE ? r_max_SIM_YEAR_RANGE : c_year;
#line 104 "code/Tables.mpp"
	poStateFunctions->Set_FemaleMortalityTrend( double( gprParam->MortalityTrend[CHKBND(RANGE_POS(SIM_YEAR_RANGE,c_year), 101, "MortalityTrend", 0)][CHKBND(FEMALE, 2, "MortalityTrend", 1)] ) );
#line 703 "modgen/src/ACTORS.CPP"
#line 235 "code/Tables.mpp"
	poStateFunctions->Set_FemaleMortalityTrend_max_value_out( double( FemaleMortalityTrend ) );
#line 706 "modgen/src/ACTORS.CPP"
#line 111 "code/ChildMortality.mpp"
	int_age = integer( poDerivedStates->ssint_age );
#line 709 "modgen/src/ACTORS.CPP"
#line 101 "code/Tables.mpp"
	poStateFunctions->Set_MaleMortalityTrend( double( gprParam->MortalityTrend[CHKBND(RANGE_POS(SIM_YEAR_RANGE,c_year), 101, "MortalityTrend", 0)][CHKBND(MALE, 2, "MortalityTrend", 1)] ) );
#line 712 "modgen/src/ACTORS.CPP"
#line 235 "code/Tables.mpp"
	poStateFunctions->Set_MaleMortalityTrend_value_out( double( MaleMortalityTrend ) );
#line 715 "modgen/src/ACTORS.CPP"
	CountActor(0);
}

#pragma optimize( "", off )

void ChildMortalityCalibrator::StartSpecial2()
{
	// initialization of expression and history states
	poStateFunctions->Set_time(TIME( CoarsenMantissa( time ) ), true);
	poStateFunctions->Set_age(TIME( CoarsenMantissa( age ) ), true);
	gdThreadCurrentTime = time;
	__time = time;
#line 38 "code/ChildMortality.mpp"
	poStateFunctions->Set_actorset_filter_asTheChildMortalityCalibrator( logical( 1 ) );
#line 730 "modgen/src/ACTORS.CPP"
#line 230 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabMortalityTrendFactor( logical( 1 ) );
#line 733 "modgen/src/ACTORS.CPP"
	UpdateLinkedStates();

	// initialization of cross-tabulation updates
	tableinfo.tab23events = -1;
	tableinfo.tab23open0 = (real) ( FemaleMortalityTrend_max_value_out );
	tableinfo.tab23open1 = (real) ( MaleMortalityTrend_value_out );
	poPrev = NULL;
	poNext = gpoEventQueue->poHeadActor;
	if ( gpoEventQueue->poHeadActor != NULL ) {
		gpoEventQueue->poHeadActor->poPrev = this;
	}
	gpoEventQueue->poHeadActor = this;
	bUpdates = TRUE;
}

void ChildMortalityCalibrator::FinishSpecial()
{
	CountFinishedActor(0);
	bUpdates = FALSE;
	gpoEventQueue->DestroyEvent( poChildMortalityCalibrationEvent );
	gpoEventQueue->DestroyEvent(poScheduleEvent);

	// close all updates
	if ( gplTableIndexes[23] >= 0 && ( tableinfo.tab23dim_filter || table_filter_TabMortalityTrendFactor ) ) {
		if ( tableinfo.tab23events != -1 ) {
			tableinfo.tab23open0 = (real) CloseUpdate( 23, 0, tableinfo.tab23open0, tableinfo.tab23close0, tableinfo.tab23dim_filter, tableinfo.tab23cell, tableinfo.tab23total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab23open1 = (real) CloseUpdate( 23, 1, tableinfo.tab23open1, tableinfo.tab23close1, tableinfo.tab23dim_filter, tableinfo.tab23cell, tableinfo.tab23total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab23cell = (index) TableCellIndex( 23, 1,
			c_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab23total_cell = (index) tableinfo.tab23cell;
		}
		else {
			tableinfo.tab23total_cell = (index) TableTotalCellIndex( 23, 1,
			c_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 23, 0, tableinfo.tab23open0, FemaleMortalityTrend_max_value_out, table_filter_TabMortalityTrendFactor, tableinfo.tab23cell, tableinfo.tab23total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 23, 1, tableinfo.tab23open1, MaleMortalityTrend_value_out, table_filter_TabMortalityTrendFactor, tableinfo.tab23cell, tableinfo.tab23total_cell, actor_weight, actor_subsample_weight );
	}
	mlBabies->Finish();
	if (poasTheChildMortalityCalibrator != NULL)
	{	asTheChildMortalityCalibrator->Remove(poasTheChildMortalityCalibrator);
		poasTheChildMortalityCalibrator = NULL;
	}

	// Remove actor from master list of active actors
	if ( poPrev != NULL )
	{
		poPrev->poNext = poNext;
	}
	else
	{
		if (gpoEventQueue->poHeadActor != NULL && gpoEventQueue->poHeadActor == this)
		{
			gpoEventQueue->poHeadActor = poNext;
		}
	}
	if ( poNext != NULL )
	{
		poNext->poPrev = poPrev;
	}
	poPrev = NULL;

	// Add actor to head of empty list of actors of this type
	poNext = gpoEmptyChildMortalityCalibrator;
	gpoEmptyChildMortalityCalibrator = this;
}

#pragma optimize( "", on )

void ChildMortalityCalibrator::WaitUntil( double target_time )
{
	//don't increase gdEventsForTabulation if special events because they are all treated as one event for tabulation purposes
	if (!gpoEventQueue->m_bSpecialEvents)
	{
		gdEventsForTabulation++;
	}
	__events = gdEventsForTabulation;
	if ( time < (TIME) target_time )
	{
		__time = (TIME) target_time;
		WaitSpecial( (TIME) target_time - time );
		poStateFunctions->Set_time( (TIME) target_time );
	}
}

ChildMortalityCalibrator *ChildMortalityCalibrator::UpdateTime(  )
{
	__events = gdEventsForTabulation;
	if ( time < (TIME) gdThreadCurrentTime ) {
		__time = (TIME) gdThreadCurrentTime;
		WaitSpecial( (TIME) gdThreadCurrentTime - time );
		poStateFunctions->Set_time( (TIME) gdThreadCurrentTime );
	}
	return this;
}

void ChildMortalityCalibrator::WaitUntilThisActor( double target_time )
{
	WaitUntil(target_time);
}

void ChildMortalityCalibrator::WaitSpecial( TIME wait_time )
{

	// increment the age
		poStateFunctions->Set_age( age + wait_time );
}

TIME ChildMortalityCalibrator::timeScheduleEvent()
{
	if (schedule_event_time == TIME_INFINITE)
	{
		TIME event_time = TIME_INFINITE;
		event_time = min( poDerivedStates->ssint_age_time, poDerivedStates->ssint_time_time );
		schedule_event_time = event_time;
	}
	return schedule_event_time;
}

void ChildMortalityCalibrator::ScheduleEvent()
{

	if (poDerivedStates->ssint_age_time == schedule_event_time)
	{
		//CHKSUM(time, 8);
		TRCEVNT(case_seed, _T("ChildMortalityCalibrator"), actor_id, _T("scheduled - 0"), time);
		gszRuntimeEventName = _T("ChildMortalityCalibrator.scheduled - 0");
		poDerivedStates->Set_ssint_age( poDerivedStates->ssint_age + 1 );
		poDerivedStates->ssint_age_time = WAIT( 1 );
	}

	if (poDerivedStates->ssint_time_time == schedule_event_time)
	{
		//CHKSUM(time, 9);
		TRCEVNT(case_seed, _T("ChildMortalityCalibrator"), actor_id, _T("scheduled - 1"), time);
		gszRuntimeEventName = _T("ChildMortalityCalibrator.scheduled - 1");
		poDerivedStates->Set_ssint_time( poDerivedStates->ssint_time + 1 );
		poDerivedStates->ssint_time_time = WAIT( 1 );
	}
	schedule_event_time = TIME_INFINITE;
}

double ChildMortalityCalibrator::EventTime( int nEventNum, int *pnEventInfo )
{
	double	event_time = double( TIME_INFINITE );

	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum ) {
		case 2:
			gszRuntimeEventName = _T("ChildMortalityCalibrator.ChildMortalityCalibrationEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeChildMortalityCalibrationEvent() );
			TRCEVNT(case_seed, _T("ChildMortalityCalibrator"), actor_id, _T("timeChildMortalityCalibrationEvent"), event_time);
			break;
		case 8:
			gszRuntimeEventName = _T("ScheduleEvent(time)");
			event_time = (double) timeScheduleEvent();
			break;
		case -1:
		default:;
	}
	return (TIME) event_time;
}

void ChildMortalityCalibrator::Implement( int nEventNum, int nEventInfo )
{
	gdThreadEvents += 1;
	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum )
	{
		case 2:
			CHKSUM(time, 0);
			TRCEVNT(case_seed, _T("ChildMortalityCalibrator"), actor_id, _T("ChildMortalityCalibrator.ChildMortalityCalibrationEvent"), time);
			gszRuntimeEventName = _T("ChildMortalityCalibrator.ChildMortalityCalibrationEvent(implement)");
			ChildMortalityCalibrationEvent();
			break;
		case 8:
			//CHKSUM(time, 1);
			ScheduleEvent();
			break;
		case -1:
		default:;
	}
}

double ChildMortalityCalibrator::Set_actor_subsample_weight( double value )
{
	return poStateFunctions->Set_actor_subsample_weight( value);
}

double ChildMortalityCalibrator::Set_actor_weight( double value )
{
	return poStateFunctions->Set_actor_weight( value);
}

TIME ChildMortalityCalibratorStateFunctions::Set_age( TIME value, bool bStartFunction )
{

	if ( poParent->age != value || bStartFunction )
	{
		if (bStartFunction)
		{
			poDerivedStates->Set_ssint_age( std::floor(value) );
			poDerivedStates->ssint_age_time = poParent->time + (1.0 - (value - poDerivedStates->ssint_age));
			if (poParent->schedule_event_time > poDerivedStates->ssint_age_time)
			{
				poParent->schedule_event_time = poDerivedStates->ssint_age_time;
			}
		}


		TIME	OldValue;
		OldValue = poParent->age;
		poParent->age = value;

	}
	return poParent->age = value;
}

logical ChildMortalityCalibratorStateFunctions::Set_calibration_is_done( logical value )
{

	value = value != 0;
	if ( poParent->calibration_is_done != value )
	{


		logical	OldValue;
		OldValue = poParent->calibration_is_done;
		poParent->calibration_is_done = value;

		if ( poParent->poChildMortalityCalibrationEvent->cRecalc == 0 ) {
			poParent->poChildMortalityCalibrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityCalibrationEvent );
		};
	}
	return poParent->calibration_is_done = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_female_0( double value )
{

	if ( poParent->mort_female_0 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_female_0;
		poParent->mort_female_0 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_female_0( poParent->mort_female_0 );
				}
			}
		}
	}
	return poParent->mort_female_0 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_female_1( double value )
{

	if ( poParent->mort_female_1 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_female_1;
		poParent->mort_female_1 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_female_1( poParent->mort_female_1 );
				}
			}
		}
	}
	return poParent->mort_female_1 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_female_2( double value )
{

	if ( poParent->mort_female_2 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_female_2;
		poParent->mort_female_2 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_female_2( poParent->mort_female_2 );
				}
			}
		}
	}
	return poParent->mort_female_2 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_female_3( double value )
{

	if ( poParent->mort_female_3 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_female_3;
		poParent->mort_female_3 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_female_3( poParent->mort_female_3 );
				}
			}
		}
	}
	return poParent->mort_female_3 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_female_4( double value )
{

	if ( poParent->mort_female_4 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_female_4;
		poParent->mort_female_4 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_female_4( poParent->mort_female_4 );
				}
			}
		}
	}
	return poParent->mort_female_4 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_male_0( double value )
{

	if ( poParent->mort_male_0 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_male_0;
		poParent->mort_male_0 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_male_0( poParent->mort_male_0 );
				}
			}
		}
	}
	return poParent->mort_male_0 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_male_1( double value )
{

	if ( poParent->mort_male_1 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_male_1;
		poParent->mort_male_1 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_male_1( poParent->mort_male_1 );
				}
			}
		}
	}
	return poParent->mort_male_1 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_male_2( double value )
{

	if ( poParent->mort_male_2 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_male_2;
		poParent->mort_male_2 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_male_2( poParent->mort_male_2 );
				}
			}
		}
	}
	return poParent->mort_male_2 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_male_3( double value )
{

	if ( poParent->mort_male_3 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_male_3;
		poParent->mort_male_3 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_male_3( poParent->mort_male_3 );
				}
			}
		}
	}
	return poParent->mort_male_3 = value;
}

double ChildMortalityCalibratorStateFunctions::Set_mort_male_4( double value )
{

	if ( poParent->mort_male_4 != value )
	{


		double	OldValue;
		OldValue = poParent->mort_male_4;
		poParent->mort_male_4 = value;

		{
			int	nIndex;
			int	nSize;
			Person	*prPerson;

			nSize = poParent->mlBabies->arData.GetSize();
			for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
				prPerson = poParent->mlBabies->arData[nIndex];
				if ( prPerson != NULL ) {
					prPerson->UpdateTime();
					prPerson->poStateFunctions->Set_lChildMortalityCalibrator_mort_male_4( poParent->mort_male_4 );
				}
			}
		}
	}
	return poParent->mort_male_4 = value;
}

TIME ChildMortalityCalibratorStateFunctions::Set_time( TIME value, bool bStartFunction )
{

	if ( poParent->time != value || bStartFunction )
	{
		if (bStartFunction)
		{
			poDerivedStates->Set_ssint_time( std::floor(value) );
			poDerivedStates->ssint_time_time = poDerivedStates->ssint_time + 1.0;
			if (poParent->schedule_event_time > poDerivedStates->ssint_time_time)
			{
				poParent->schedule_event_time = poDerivedStates->ssint_time_time;
			}
		}


		TIME	OldValue;
		OldValue = poParent->time;
		poParent->time = value;

	}
	return poParent->time = value;
}

logical ChildMortalityCalibratorStateFunctions::Set_actorset_filter_asTheChildMortalityCalibrator( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asTheChildMortalityCalibrator != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asTheChildMortalityCalibrator;
		poParent->actorset_filter_asTheChildMortalityCalibrator = value;

		// actorset filter
		if (poParent->actorset_filter_asTheChildMortalityCalibrator)
		{
			poParent->poasTheChildMortalityCalibrator = asTheChildMortalityCalibrator->NewNode(poParent);
		}
		else
		{
			asTheChildMortalityCalibrator->Remove(poParent->poasTheChildMortalityCalibrator);
			poParent->poasTheChildMortalityCalibrator = NULL;
		}
	}
	return poParent->actorset_filter_asTheChildMortalityCalibrator = value;
}

SIM_YEAR_RANGE ChildMortalityCalibratorStateFunctions::Set_c_year( SIM_YEAR_RANGE value )
{

	CHKLMT(value, c_year, r_min_SIM_YEAR_RANGE, r_max_SIM_YEAR_RANGE);
	if ( value < r_min_SIM_YEAR_RANGE ) {
		value = r_min_SIM_YEAR_RANGE;
	}
	else if ( value > r_max_SIM_YEAR_RANGE ) {
		value = r_max_SIM_YEAR_RANGE;
	}
	if ( poParent->c_year != value )
	{


		SIM_YEAR_RANGE	OldValue;
		OldValue = poParent->c_year;
		poParent->c_year = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[23] >= 0 && ( poParent->tableinfo.tab23dim_filter || poParent->table_filter_TabMortalityTrendFactor ) ) {
				if ( poParent->tableinfo.tab23events == -1 || poParent->tableinfo.tab23events < poParent->__events ) {
					if ( poParent->tableinfo.tab23events != -1 ) {
						poParent->tableinfo.tab23open0 = (real) poParent->CloseUpdate( 23, 0, poParent->tableinfo.tab23open0, poParent->tableinfo.tab23close0, poParent->tableinfo.tab23dim_filter, poParent->tableinfo.tab23cell, poParent->tableinfo.tab23total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab23open1 = (real) poParent->CloseUpdate( 23, 1, poParent->tableinfo.tab23open1, poParent->tableinfo.tab23close1, poParent->tableinfo.tab23dim_filter, poParent->tableinfo.tab23cell, poParent->tableinfo.tab23total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab23events = poParent->__events;
					poParent->tableinfo.tab23close0 = (real) poParent->FemaleMortalityTrend_max_value_out;
					poParent->tableinfo.tab23close1 = (real) poParent->MaleMortalityTrend_value_out;
					poParent->tableinfo.tab23cell = (index) poParent->TableCellIndex( 23, 1,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab23total_cell = (index) poParent->tableinfo.tab23cell;
					}
					else {
						poParent->tableinfo.tab23total_cell = (index) poParent->TableTotalCellIndex( 23, 1,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab23dim_filter = poParent->table_filter_TabMortalityTrendFactor;
				}
			}
		}
#line 104 "code/Tables.mpp"
		Set_FemaleMortalityTrend( gprParam->MortalityTrend[CHKBND(RANGE_POS(SIM_YEAR_RANGE,poParent->c_year), 101, "MortalityTrend", 0)][CHKBND(FEMALE, 2, "MortalityTrend", 1)] );
#line 1359 "modgen/src/ACTORS.CPP"
#line 101 "code/Tables.mpp"
		Set_MaleMortalityTrend( gprParam->MortalityTrend[CHKBND(RANGE_POS(SIM_YEAR_RANGE,poParent->c_year), 101, "MortalityTrend", 0)][CHKBND(MALE, 2, "MortalityTrend", 1)] );
#line 1362 "modgen/src/ACTORS.CPP"
	}
	return poParent->c_year = value;
}

double ChildMortalityCalibratorStateFunctions::Set_FemaleMortalityTrend( double value )
{

	if ( poParent->FemaleMortalityTrend != value )
	{


		double	OldValue;
		OldValue = poParent->FemaleMortalityTrend;
		poParent->FemaleMortalityTrend = value;

#line 235 "code/Tables.mpp"
		Set_FemaleMortalityTrend_max_value_out( poParent->FemaleMortalityTrend );
#line 1380 "modgen/src/ACTORS.CPP"
	}
	return poParent->FemaleMortalityTrend = value;
}

double ChildMortalityCalibratorStateFunctions::Set_FemaleMortalityTrend_max_value_out( double value )
{

	if ( poParent->FemaleMortalityTrend_max_value_out != value )
	{


		double	OldValue;
		OldValue = poParent->FemaleMortalityTrend_max_value_out;
		poParent->FemaleMortalityTrend_max_value_out = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[23] >= 0 && poParent->tableinfo.tab23events == poParent->__events ) {
				poParent->tableinfo.tab23close0 = (real) poParent->FemaleMortalityTrend_max_value_out;
			}
		}
	}
	return poParent->FemaleMortalityTrend_max_value_out = value;
}

double ChildMortalityCalibratorStateFunctions::Set_MaleMortalityTrend( double value )
{

	if ( poParent->MaleMortalityTrend != value )
	{


		double	OldValue;
		OldValue = poParent->MaleMortalityTrend;
		poParent->MaleMortalityTrend = value;

#line 235 "code/Tables.mpp"
		Set_MaleMortalityTrend_value_out( poParent->MaleMortalityTrend );
#line 1418 "modgen/src/ACTORS.CPP"
	}
	return poParent->MaleMortalityTrend = value;
}

double ChildMortalityCalibratorStateFunctions::Set_MaleMortalityTrend_value_out( double value )
{

	if ( poParent->MaleMortalityTrend_value_out != value )
	{


		double	OldValue;
		OldValue = poParent->MaleMortalityTrend_value_out;
		poParent->MaleMortalityTrend_value_out = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[23] >= 0 && poParent->tableinfo.tab23events == poParent->__events ) {
				poParent->tableinfo.tab23close1 = (real) poParent->MaleMortalityTrend_value_out;
			}
		}
	}
	return poParent->MaleMortalityTrend_value_out = value;
}

logical ChildMortalityCalibratorStateFunctions::Set_table_filter_TabMortalityTrendFactor( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabMortalityTrendFactor != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabMortalityTrendFactor;
		poParent->table_filter_TabMortalityTrendFactor = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[23] >= 0 ) {
				if ( poParent->tableinfo.tab23events == -1 || poParent->tableinfo.tab23events < poParent->__events ) {
					if ( poParent->tableinfo.tab23events != -1 ) {
						poParent->tableinfo.tab23open0 = (real) poParent->CloseUpdate( 23, 0, poParent->tableinfo.tab23open0, poParent->tableinfo.tab23close0, poParent->tableinfo.tab23dim_filter, poParent->tableinfo.tab23cell, poParent->tableinfo.tab23total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab23open1 = (real) poParent->CloseUpdate( 23, 1, poParent->tableinfo.tab23open1, poParent->tableinfo.tab23close1, poParent->tableinfo.tab23dim_filter, poParent->tableinfo.tab23cell, poParent->tableinfo.tab23total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab23events = poParent->__events;
					poParent->tableinfo.tab23close0 = (real) poParent->FemaleMortalityTrend_max_value_out;
					poParent->tableinfo.tab23close1 = (real) poParent->MaleMortalityTrend_value_out;
					poParent->tableinfo.tab23cell = (index) poParent->TableCellIndex( 23, 1,
						poParent->c_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab23total_cell = (index) poParent->tableinfo.tab23cell;
					}
					else {
						poParent->tableinfo.tab23total_cell = (index) poParent->TableTotalCellIndex( 23, 1,
						poParent->c_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab23dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabMortalityTrendFactor = value;
}

short ChildMortalityCalibratorDerivedStates::Set_ssint_age( short value )
{

	if ( ssint_age != value ) {

		short OldValue;
		OldValue = ssint_age;
		ssint_age = value;
		poParent->int_age = ssint_age;
	}
	return ssint_age = value;
}

short ChildMortalityCalibratorDerivedStates::Set_ssint_time( short value )
{

	if ( ssint_time != value ) {

		short OldValue;
		OldValue = ssint_time;
		ssint_time = value;
		poStateFunctions->Set_c_year( COERCE(SIM_YEAR_RANGE,ssint_time) );
	}
	return ssint_time = value;
}

void *Observation::operator new( size_t count )
{
	Observation *poActor;
	if ( gpoEmptyObservation != NULL ) {
		poActor = gpoEmptyObservation;
		gpoEmptyObservation = (Observation *) gpoEmptyObservation->poNext;
	}
	else {
		poActor = ::new Observation( TRUE ) ;
	}
	poActor->__finished = false;
	return poActor;
}

void Observation::InitActor()
{
	poDerivedStates = new ObservationDerivedStates(); 
	poDerivedStates->poParent = this;
	poStateFunctions = new ObservationStateFunctions();
	poStateFunctions->poParent = this;
	poStateFunctions->poDerivedStates = poDerivedStates;
	poDerivedStates->poStateFunctions = poStateFunctions;
	bUpdates = FALSE;
	nActorNumber = 1;
	InitializeStates();
}

void Observation::DeleteActor()
{
	delete poDerivedStates;
	delete poStateFunctions;
}

#pragma optimize( "", off )

void Observation::InitializeStates()
{
	// initialization of states
	actor_id = (long) 0;
	actor_subsample_weight = (double) 0;
	actor_weight = (double) 0;
	age = (TIME) 0;
	case_id = (long) 0;
	case_seed = (double) 0;
#line 49 "code/StartPopFile.mpp"
	int_weight = (integer) 1.0;
#line 1554 "modgen/src/ACTORS.CPP"
	time = (TIME) gdThreadCurrentTime;
	actorset_filter_asAllObservations = FALSE;
	__finished = FALSE;
	__time = (TIME) gdThreadCurrentTime;
	__events = 0;
}

#pragma optimize( "", on )

void Observation::UpdateLinkedStates()
{
}

void Observation::StartSpecial1()
{
	nSubSample = GetSubSample();
	InitializeStates();
	case_seed = GetCaseSeed();
	case_id = GetCaseID();
	lActorId = actor_id = GetObjectID();
	GetCaseWeight( &actor_weight, &actor_subsample_weight );

	// initialization of actor set nodes
	poasAllObservations = NULL;

	// initialization of events
	//Initalize derived states
	CountActor(1);
}

#pragma optimize( "", off )

void Observation::StartSpecial2()
{
	// initialization of expression and history states
	poStateFunctions->Set_time(TIME( CoarsenMantissa( time ) ), true);
	poStateFunctions->Set_age(TIME( CoarsenMantissa( age ) ), true);
	gdThreadCurrentTime = time;
	__time = time;
#line 41 "code/StartPopFile.mpp"
	poStateFunctions->Set_actorset_filter_asAllObservations( logical( 1 ) );
#line 1596 "modgen/src/ACTORS.CPP"
	UpdateLinkedStates();
	poPrev = NULL;
	poNext = gpoEventQueue->poHeadActor;
	if ( gpoEventQueue->poHeadActor != NULL ) {
		gpoEventQueue->poHeadActor->poPrev = this;
	}
	gpoEventQueue->poHeadActor = this;
	bUpdates = TRUE;
}

void Observation::FinishSpecial()
{
	CountFinishedActor(1);
	bUpdates = FALSE;
	if (poasAllObservations != NULL)
	{	asAllObservations->Remove(poasAllObservations);
		poasAllObservations = NULL;
	}

	// Remove actor from master list of active actors
	if ( poPrev != NULL )
	{
		poPrev->poNext = poNext;
	}
	else
	{
		if (gpoEventQueue->poHeadActor != NULL && gpoEventQueue->poHeadActor == this)
		{
			gpoEventQueue->poHeadActor = poNext;
		}
	}
	if ( poNext != NULL )
	{
		poNext->poPrev = poPrev;
	}
	poPrev = NULL;

	// Add actor to head of empty list of actors of this type
	poNext = gpoEmptyObservation;
	gpoEmptyObservation = this;
}

#pragma optimize( "", on )

void Observation::WaitUntil( double target_time )
{
	//don't increase gdEventsForTabulation if special events because they are all treated as one event for tabulation purposes
	if (!gpoEventQueue->m_bSpecialEvents)
	{
		gdEventsForTabulation++;
	}
	__events = gdEventsForTabulation;
	if ( time < (TIME) target_time )
	{
		__time = (TIME) target_time;
		WaitSpecial( (TIME) target_time - time );
		poStateFunctions->Set_time( (TIME) target_time );
	}
}

Observation *Observation::UpdateTime(  )
{
	__events = gdEventsForTabulation;
	if ( time < (TIME) gdThreadCurrentTime ) {
		__time = (TIME) gdThreadCurrentTime;
		WaitSpecial( (TIME) gdThreadCurrentTime - time );
		poStateFunctions->Set_time( (TIME) gdThreadCurrentTime );
	}
	return this;
}

void Observation::WaitUntilThisActor( double target_time )
{
	WaitUntil(target_time);
}

void Observation::WaitSpecial( TIME wait_time )
{

	// increment the age
		poStateFunctions->Set_age( age + wait_time );
}

double Observation::EventTime( int nEventNum, int *pnEventInfo )
{
	double	event_time = double( TIME_INFINITE );

	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum ) {
		case -1:
		default:;
	}
	return (TIME) event_time;
}

void Observation::Implement( int nEventNum, int nEventInfo )
{
	gdThreadEvents += 1;
	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum )
	{
		case -1:
		default:;
	}
}

double Observation::Set_actor_subsample_weight( double value )
{
	return poStateFunctions->Set_actor_subsample_weight( value);
}

double Observation::Set_actor_weight( double value )
{
	return poStateFunctions->Set_actor_weight( value);
}

logical ObservationStateFunctions::Set_actorset_filter_asAllObservations( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asAllObservations != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asAllObservations;
		poParent->actorset_filter_asAllObservations = value;

		// actorset filter
		if (poParent->actorset_filter_asAllObservations)
		{
			poParent->poasAllObservations = asAllObservations->NewNode(poParent);
		}
		else
		{
			asAllObservations->Remove(poParent->poasAllObservations);
			poParent->poasAllObservations = NULL;
		}
	}
	return poParent->actorset_filter_asAllObservations = value;
}

void *Person::operator new( size_t count )
{
	Person *poActor;
	if ( gpoEmptyPerson != NULL ) {
		poActor = gpoEmptyPerson;
		gpoEmptyPerson = (Person *) gpoEmptyPerson->poNext;
	}
	else {
		poActor = ::new Person( TRUE ) ;
	}
	poActor->__finished = false;
	return poActor;
}

void Person::Set_lChildMortalityCalibrator( ChildMortalityCalibrator *value )
{
	ChildMortalityCalibrator *poOldLink;

	if ( lChildMortalityCalibrator != value ) {
		if (lChildMortalityCalibrator != NULL)
		{
			lChildMortalityCalibrator->UpdateTime();
		}
		if (value != NULL)
		{
			value->UpdateTime();
		}
		poStateFunctions->Set_child_mortality( ( value== NULL )? 0.0: ( integer_age== 0&& sex== MALE )? lChildMortalityCalibrator_mort_male_0: ( integer_age== 1&& sex== MALE )? lChildMortalityCalibrator_mort_male_1: ( integer_age== 2&& sex== MALE )? lChildMortalityCalibrator_mort_male_2: ( integer_age== 3&& sex== MALE )? lChildMortalityCalibrator_mort_male_3: ( integer_age== 4&& sex== MALE )? lChildMortalityCalibrator_mort_male_4: ( integer_age== 0&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_0: ( integer_age== 1&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_1: ( integer_age== 2&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_2: ( integer_age== 3&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_3: ( integer_age== 4&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_4: 0.0 );
		poOldLink = lChildMortalityCalibrator;
		lChildMortalityCalibrator = value;
		UpdateLinkedStates();
		if ( poOldLink != NULL ) {
			poOldLink->mlBabies->Remove( this );
		}
		if ( value != NULL ) {
			value->mlBabies->Add( this );
		}
	}
}

void Person::Set_lMother( Person *value )
{
	Person *poOldLink;

	if ( lMother != value ) {
		if (lMother != NULL)
		{
			lMother->UpdateTime();
		}
		if (value != NULL)
		{
			value->UpdateTime();
		}
		poOldLink = lMother;
		lMother = value;
		UpdateLinkedStates();
		if ( poOldLink != NULL ) {
			poOldLink->mlChild->Remove( this );
		}
		if ( value != NULL ) {
			value->mlChild->Add( this );
		}
	}
}

void Person::StartIMPLEMENT_HOOK()
{
	WriteMicroRecord_Start();
}

void Person::InitActor()
{
	poDerivedStates = new PersonDerivedStates(); 
	poDerivedStates->poParent = this;
	poStateFunctions = new PersonStateFunctions();
	poStateFunctions->poParent = this;
	poStateFunctions->poDerivedStates = poDerivedStates;
	poDerivedStates->poStateFunctions = poStateFunctions;
	bUpdates = FALSE;
	nActorNumber = 2;
	InitializeStates();
	lChildMortalityCalibrator = NULL;
	lMother = NULL;
	mlChild = new Person_mlChild( this );
}

void Person::DeleteActor()
{
	delete poDerivedStates;
	delete poStateFunctions;
	delete mlChild;
}

#pragma optimize( "", off )

void Person::InitializeStates()
{
	// initialization of states
	actor_id = (long) 0;
	actor_subsample_weight = (double) 0;
	actor_weight = (double) 0;
	age = (TIME) 0;
#line 36 "code/Migration.mpp"
	age_at_last_move = (integer) 999;
#line 1845 "modgen/src/ACTORS.CPP"
#line 43 "code/Mortality.mpp"
	alive = TRUE != 0;
#line 1848 "modgen/src/ACTORS.CPP"
	case_id = (long) 0;
	case_seed = (double) 0;
#line 74 "code/PrimaryEducation.mpp"
	educ_mother = PL_NO;
#line 1853 "modgen/src/ACTORS.CPP"
#line 24 "code/Emigration.mpp"
	has_emigrated = FALSE != 0;
#line 1856 "modgen/src/ACTORS.CPP"
#line 132 "code/Fertility.mpp"
	in_this_parity = TRUE != 0;
#line 1859 "modgen/src/ACTORS.CPP"
#line 68 "code/UnionFormation.mpp"
	in_union = FALSE != 0;
#line 1862 "modgen/src/ACTORS.CPP"
#line 285 "code/ChildMortality.mpp"
	CHKLMT(0, infant_deaths, r_min_PARITY_RANGE, r_max_PARITY_RANGE);
	infant_deaths = 0 < r_min_PARITY_RANGE ? r_min_PARITY_RANGE : 0 > r_max_PARITY_RANGE ? r_max_PARITY_RANGE : (PARITY_RANGE) 0;
#line 1866 "modgen/src/ACTORS.CPP"
#line 280 "code/ChildMortality.mpp"
	mother_educ_0 = FALSE != 0;
#line 1869 "modgen/src/ACTORS.CPP"
#line 281 "code/ChildMortality.mpp"
	mother_educ_1 = FALSE != 0;
#line 1872 "modgen/src/ACTORS.CPP"
#line 279 "code/ChildMortality.mpp"
	mother_known = FALSE != 0;
#line 1875 "modgen/src/ACTORS.CPP"
#line 282 "code/ChildMortality.mpp"
	mother_max_14 = FALSE != 0;
#line 1878 "modgen/src/ACTORS.CPP"
#line 283 "code/ChildMortality.mpp"
	mother_max_16 = FALSE != 0;
#line 1881 "modgen/src/ACTORS.CPP"
#line 35 "code/Migration.mpp"
	number_moves = (integer) 0;
#line 1884 "modgen/src/ACTORS.CPP"
#line 129 "code/Fertility.mpp"
	override_mode = FALSE != 0;
#line 1887 "modgen/src/ACTORS.CPP"
#line 125 "code/Fertility.mpp"
	CHKLMT(0, parity, r_min_PARITY_RANGE, r_max_PARITY_RANGE);
	parity = 0 < r_min_PARITY_RANGE ? r_min_PARITY_RANGE : 0 > r_max_PARITY_RANGE ? r_max_PARITY_RANGE : (PARITY_RANGE) 0;
#line 1891 "modgen/src/ACTORS.CPP"
#line 71 "code/PersonCore.mpp"
	person_type = PT_START;
#line 1894 "modgen/src/ACTORS.CPP"
#line 73 "code/PrimaryEducation.mpp"
	primary_level = PL_NO;
#line 1897 "modgen/src/ACTORS.CPP"
#line 79 "code/PersonCore.mpp"
	province_birth = PI_PROV00;
#line 1900 "modgen/src/ACTORS.CPP"
#line 76 "code/PersonCore.mpp"
	province_nat = PN_PROV00;
#line 1903 "modgen/src/ACTORS.CPP"
#line 50 "code/CmapReportTables.mpp"
	rtab_column = RC_COLUMN;
#line 1906 "modgen/src/ACTORS.CPP"
#line 157 "code/Fertility.mpp"
	set_alive = FALSE != 0;
#line 1909 "modgen/src/ACTORS.CPP"
#line 75 "code/PersonCore.mpp"
	sex = FEMALE;
#line 1912 "modgen/src/ACTORS.CPP"
	table_unit_TabImmigrationSexProvinceAge = (counter) 0;
	table_unit_TabMothersExperience = (counter) 0;
	table_unit_TabSimulatedStartingPopulation = (counter) 0;
	time = (TIME) gdThreadCurrentTime;
#line 54 "code/MicroDataOutput.mpp"
	time_microdata_output = (TIME) TIME_INFINITE;
#line 1919 "modgen/src/ACTORS.CPP"
#line 73 "code/PersonCore.mpp"
	time_of_birth = (TIME) 1912;
#line 1922 "modgen/src/ACTORS.CPP"
#line 29 "code/Immigration.mpp"
	time_of_immigration = (TIME) TIME_INFINITE;
#line 1925 "modgen/src/ACTORS.CPP"
#line 152 "code/Fertility.mpp"
	time_of_last_birth = (TIME) TIME_INFINITE;
#line 1928 "modgen/src/ACTORS.CPP"
#line 67 "code/UnionFormation.mpp"
	time_of_union_formation = (TIME) TIME_INFINITE;
#line 1931 "modgen/src/ACTORS.CPP"
#line 141 "code/Fertility.mpp"
	time_since_last_birth_start = (double) 0.0;
#line 1934 "modgen/src/ACTORS.CPP"
	poDerivedStates->active_spell_duration_in_this_parity_in_simulation_TRUE = (TIME) 0;
	poDerivedStates->active_spell_duration_in_this_parity_in_simulation_TRUE_time = (TIME) gdThreadCurrentTime;
	actorset_filter_asAllPersons = FALSE;
	actorset_filter_asFemaleAgeEduc = FALSE;
	actorset_filter_asForeignersAgeSexProv = FALSE;
	actorset_filter_asForeignersPrimAgeSexProv = FALSE;
	actorset_filter_asPotentialMothers = FALSE;
	actorset_filter_asPotentialMothersByAge = FALSE;
	actorset_filter_asPrimaryEntryCohort = FALSE;
	actorset_filter_asPrimaryGraduationCohort = FALSE;
	actorset_filter_asResidentsAgeSex = FALSE;
	actorset_filter_asResidentsPrimAgeSex = FALSE;
	age_union = r_min_AGE_UNION;
	birth_age_part = (int) 0;
	calendar_year = (integer) 0;
	poDerivedStates->changes_parity = (counter) 0;
	child_mortality = (double) 0;
	child_mortality_overrides = FALSE;
	poDerivedStates->completed_spell_duration_in_this_parity_in_simulation_TRUE = (TIME) 0;
	count_parity = FALSE;
	poDerivedStates->duration_ = (TIME) 0;
	poDerivedStates->duration__time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_in_union_FALSE = (TIME) 0;
	poDerivedStates->duration_in_union_FALSE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_in_union_TRUE = (TIME) 0;
	poDerivedStates->duration_in_union_TRUE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_is_aded0_TRUE = (TIME) 0;
	poDerivedStates->duration_is_aded0_TRUE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_is_aded1_TRUE = (TIME) 0;
	poDerivedStates->duration_is_aded1_TRUE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_is_aded2_TRUE = (TIME) 0;
	poDerivedStates->duration_is_aded2_TRUE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_is_child_TRUE = (TIME) 0;
	poDerivedStates->duration_is_child_TRUE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_is_old_TRUE = (TIME) 0;
	poDerivedStates->duration_is_old_TRUE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_primary_level_PL_ENTER = (TIME) 0;
	poDerivedStates->duration_primary_level_PL_ENTER_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_primary_level_PL_GRAD = (TIME) 0;
	poDerivedStates->duration_primary_level_PL_GRAD_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_primary_level_PL_NO = (TIME) 0;
	poDerivedStates->duration_primary_level_PL_NO_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->duration_set_alive_TRUE = (TIME) 0;
	poDerivedStates->duration_set_alive_TRUE_time = (TIME) gdThreadCurrentTime;
	poDerivedStates->entrances_primary_level_PL_GRAD = (counter) 0;
	poDerivedStates->entrances_set_alive_TRUE = (counter) 0;
	fert_age = r_min_FERTILE_AGE_RANGE;
	fertile_age = r_min_FERTILE_AGE_RANGE;
	in_capital = FALSE;
	in_projected_time = FALSE;
	in_rtab_sel_years = FALSE;
	in_rtab_years = FALSE;
	in_simulation_time = FALSE;
	in_this_parity_in_simulation = FALSE;
	integer_age = r_min_AGE_RANGE;
	is_act_age = FALSE;
	is_aded0 = FALSE;
	is_aded1 = FALSE;
	is_aded2 = FALSE;
	is_child = FALSE;
	is_old = FALSE;
	is_potential_mother = FALSE;
	rtab_reg = (RTAB_REG) 0;
	rtab_sel_years = (RTAB_SEL_YEARS) 0;
	rtab_years = r_min_RTAB_YEARS;
	sim_cmr = r_min_CHILD_MORT_RANGE;
	sim_year = r_min_SIM_YEAR_RANGE;
	sim_yob = r_min_SIM_YEAR_RANGE;
	poDerivedStates->split_integer_age_AGE5_PART = (AGE5_PART) 0;
	poDerivedStates->split_integer_age_AGE_15_16_18 = (AGE_15_16_18) 0;
	poDerivedStates->split_integer_age_AGE_15_17 = (AGE_15_17) 0;
	poDerivedStates->split_integer_age_AGE_FIVE = (AGE_FIVE) 0;
	poDerivedStates->split_integer_age_AGE_MORT = (AGE_MORT) 0;
	poDerivedStates->split_integer_age_AGE_RTAB_MORT = (AGE_RTAB_MORT) 0;
	poDerivedStates->split_integer_age_BIRTH_AGE_PART = (BIRTH_AGE_PART) 0;
	poDerivedStates->split_time_since_last_birth_DUR_TIME_IN_PARITY = (DUR_TIME_IN_PARITY) 0;
	poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE = (short) 0;
	poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time = (TIME) TIME_INFINITE;
	poDerivedStates->ssint_age = (short) 0;
	poDerivedStates->ssint_age_time = (TIME) TIME_INFINITE;
	poDerivedStates->ssint_time = (short) 0;
	poDerivedStates->ssint_time_time = (TIME) TIME_INFINITE;
	poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME = (LITTLE_TIME) 0;
	poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = (TIME) TIME_INFINITE;
	table_filter_RTAB_AgeAtFirstUnionFormation = FALSE;
	table_filter_RTAB_BirthsByAgeMother = FALSE;
	table_filter_RTAB_BirthsByEducMother = FALSE;
	table_filter_RTAB_BirthsByRegion = FALSE;
	table_filter_RTAB_DeathsByRegion = FALSE;
	table_filter_RTAB_EducationActivePop = FALSE;
	table_filter_RTAB_PopPyramidByEduc = FALSE;
	table_filter_RTAB_PrimSchoolEntries = FALSE;
	table_filter_RTAB_PrimSchoolGraduations = FALSE;
	table_filter_RTAB_TabDeathsByAgeGroup = FALSE;
	table_filter_RTAB_TotalPopReg = FALSE;
	table_filter_RTAB_TotalPopWillaya = FALSE;
	table_filter_RTAB_UnionFormationByAge = FALSE;
	table_filter_TabAgeAtBirth = FALSE;
	table_filter_TabAgeAtFirstUnionFormation = FALSE;
	table_filter_TabBirthsByAgeGroup = FALSE;
	table_filter_TabCohortChildMortality = FALSE;
	table_filter_TabDeathRates = FALSE;
	table_filter_TabDeathsByAgeGroup = FALSE;
	table_filter_TabEduc12ByPob = FALSE;
	table_filter_TabEduc12ByPor = FALSE;
	table_filter_TabEduc2015 = FALSE;
	table_filter_TabEduc2040 = FALSE;
	table_filter_TabEduc2065 = FALSE;
	table_filter_TabEmigration = FALSE;
	table_filter_TabFertilityRates = FALSE;
	table_filter_TabImmigrationSexProvinceAge = FALSE;
	table_filter_TabInternalMigrationRate = FALSE;
	table_filter_TabMothersExperience = FALSE;
	table_filter_TabNumberBirths = FALSE;
	table_filter_TabPopProvAgeEducSex = FALSE;
	table_filter_TabPopulationAgeSexProvince = FALSE;
	table_filter_TabSimulatedStartingPopulation = FALSE;
	table_filter_TabTotalPopulationSexProvince = FALSE;
	table_filter_TabUnionFormation = FALSE;
	time_in_parity = (int) 0;
	time_since_last_birth = (int) 0;
	poDerivedStates->transitions_alive_TRUE_FALSE = (counter) 0;
	poDerivedStates->transitions_has_emigrated_FALSE_TRUE = (counter) 0;
	poDerivedStates->transitions_in_union_FALSE_TRUE = (counter) 0;
	poDerivedStates->transitions_integer_age_14_15 = (counter) 0;
	poDerivedStates->transitions_integer_age_19_20 = (counter) 0;
	poDerivedStates->transitions_integer_age_9_10 = (counter) 0;
	poDerivedStates->transitions_parity_0_1 = (counter) 0;
	poDerivedStates->transitions_primary_level_PL_ENTER_PL_GRAD = (counter) 0;
	poDerivedStates->transitions_primary_level_PL_NO_PL_ENTER = (counter) 0;
	poDerivedStates->trigger_entrances_in_projected_time_TRUE = (logical) 0;
	poDerivedStates->trigger_entrances_in_projected_time_TRUE_time = (TIME) TIME_INFINITE;
	poDerivedStates->trigger_entrances_set_alive_TRUE = (logical) 0;
	poDerivedStates->trigger_entrances_set_alive_TRUE_time = (TIME) TIME_INFINITE;
	union_formation_hazard = (double) 0;
	union_formation_hazard_max_value_in = (double) 0;
	union_status = (UNION_STATUS) 0;
	poDerivedStates->value_at_changes_parity_age = (real) 0;
	poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age = (real) 0;
	poDerivedStates->value_at_transitions_parity_0_1_age = (real) 0;
	year_of_birth = r_min_ALL_YEAR_RANGE;
	lChildMortalityCalibrator_mort_female_0 = (double) 0;
	lChildMortalityCalibrator_mort_female_1 = (double) 0;
	lChildMortalityCalibrator_mort_female_2 = (double) 0;
	lChildMortalityCalibrator_mort_female_3 = (double) 0;
	lChildMortalityCalibrator_mort_female_4 = (double) 0;
	lChildMortalityCalibrator_mort_male_0 = (double) 0;
	lChildMortalityCalibrator_mort_male_1 = (double) 0;
	lChildMortalityCalibrator_mort_male_2 = (double) 0;
	lChildMortalityCalibrator_mort_male_3 = (double) 0;
	lChildMortalityCalibrator_mort_male_4 = (double) 0;
	__finished = FALSE;
	__time = (TIME) gdThreadCurrentTime;
	__events = 0;
}

#pragma optimize( "", on )

void Person::UpdateLinkedStates()
{
	if ( lChildMortalityCalibrator != NULL ) {
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_0( lChildMortalityCalibrator->mort_female_0 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_1( lChildMortalityCalibrator->mort_female_1 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_2( lChildMortalityCalibrator->mort_female_2 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_3( lChildMortalityCalibrator->mort_female_3 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_4( lChildMortalityCalibrator->mort_female_4 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_0( lChildMortalityCalibrator->mort_male_0 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_1( lChildMortalityCalibrator->mort_male_1 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_2( lChildMortalityCalibrator->mort_male_2 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_3( lChildMortalityCalibrator->mort_male_3 );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_4( lChildMortalityCalibrator->mort_male_4 );
	}
	else {
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_0( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_1( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_2( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_3( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_female_4( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_0( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_1( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_2( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_3( double( 0 ) );
		poStateFunctions->Set_lChildMortalityCalibrator_mort_male_4( double( 0 ) );
	}
}

void Person::StartSpecial1()
{
	nSubSample = GetSubSample();
	InitializeStates();
	case_seed = GetCaseSeed();
	case_id = GetCaseID();
	lActorId = actor_id = GetObjectID();
	GetCaseWeight( &actor_weight, &actor_subsample_weight );
	lChildMortalityCalibrator = NULL;
	lMother = NULL;
	mlChild->RemoveAll();

	// initialization of actor set nodes
	poasAllPersons = NULL;
	poasFemaleAgeEduc = NULL;
	poasForeignersAgeSexProv = NULL;
	poasForeignersPrimAgeSexProv = NULL;
	poasPotentialMothers = NULL;
	poasPotentialMothersByAge = NULL;
	poasPrimaryEntryCohort = NULL;
	poasPrimaryGraduationCohort = NULL;
	poasResidentsAgeSex = NULL;
	poasResidentsPrimAgeSex = NULL;

	// initialization of events
	poBirthEvent = gpoEventQueue->NewEvent( this, 1, 0 );
	poChildMortalityEvent = gpoEventQueue->NewEvent( this, 3, 0 );
	poEmigrationEvent = gpoEventQueue->NewEvent( this, 4, 0 );
	poMigrationEvent = gpoEventQueue->NewEvent( this, 5, 0 );
	poMortalityEvent = gpoEventQueue->NewEvent( this, 6, 0 );
	poSetAliveEvent = gpoEventQueue->NewEvent( this, 9, 0 );
	poUnionFormationEvent = gpoEventQueue->NewEvent( this, 10, 0 );
	poWriteMicroRecord = gpoEventQueue->NewEvent( this, 11, 0 );
	poScheduleEvent = gpoEventQueue->NewEvent( this, 8, 255 );
	schedule_event_time = TIME_INFINITE;
	//Initalize derived states
#line 60 "code/UnionFormation.mpp"
	poStateFunctions->Set_age_union( AGE_UNION( COERCE(AGE_UNION,integer_age) ) );
#line 2159 "modgen/src/ACTORS.CPP"
	age_union = age_union < r_min_AGE_UNION ? r_min_AGE_UNION : age_union > r_max_AGE_UNION ? r_max_AGE_UNION : age_union;
#line 124 "code/Fertility.mpp"
	poStateFunctions->Set_birth_age_part( int( poDerivedStates->split_integer_age_BIRTH_AGE_PART ) );
#line 2163 "modgen/src/ACTORS.CPP"
#line 81 "code/PersonCore.mpp"
	poStateFunctions->Set_calendar_year( integer( poDerivedStates->ssint_time ) );
#line 2166 "modgen/src/ACTORS.CPP"
#line 298 "code/ChildMortality.mpp"
	poStateFunctions->Set_child_mortality( double( ( lChildMortalityCalibrator== NULL )? 0.0: ( integer_age== 0&& sex== MALE )? lChildMortalityCalibrator_mort_male_0: ( integer_age== 1&& sex== MALE )? lChildMortalityCalibrator_mort_male_1: ( integer_age== 2&& sex== MALE )? lChildMortalityCalibrator_mort_male_2: ( integer_age== 3&& sex== MALE )? lChildMortalityCalibrator_mort_male_3: ( integer_age== 4&& sex== MALE )? lChildMortalityCalibrator_mort_male_4: ( integer_age== 0&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_0: ( integer_age== 1&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_1: ( integer_age== 2&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_2: ( integer_age== 3&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_3: ( integer_age== 4&& sex== FEMALE )? lChildMortalityCalibrator_mort_female_4: 0.0 ) );
#line 2169 "modgen/src/ACTORS.CPP"
#line 277 "code/ChildMortality.mpp"
	poStateFunctions->Set_child_mortality_overrides( logical( calendar_year>= MIN(CHILD_MORTALITY_YEARS)&& gprParam->SelectedChildMortalityModel!= SCMM_MACRO&& integer_age<= MAX(CHILD_MORTALITY_AGE)&& mother_known ) );
#line 2172 "modgen/src/ACTORS.CPP"
#line 91 "code/Tables.mpp"
	poStateFunctions->Set_count_parity( logical( poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME ) );
#line 2175 "modgen/src/ACTORS.CPP"
#line 86 "code/Tables.mpp"
	poStateFunctions->Set_fert_age( FERTILE_AGE_RANGE( COERCE(FERTILE_AGE_RANGE,integer_age) ) );
#line 2178 "modgen/src/ACTORS.CPP"
	fert_age = fert_age < r_min_FERTILE_AGE_RANGE ? r_min_FERTILE_AGE_RANGE : fert_age > r_max_FERTILE_AGE_RANGE ? r_max_FERTILE_AGE_RANGE : fert_age;
#line 123 "code/Fertility.mpp"
	poStateFunctions->Set_fertile_age( FERTILE_AGE_RANGE( COERCE(FERTILE_AGE_RANGE,integer_age) ) );
#line 2182 "modgen/src/ACTORS.CPP"
	fertile_age = fertile_age < r_min_FERTILE_AGE_RANGE ? r_min_FERTILE_AGE_RANGE : fertile_age > r_max_FERTILE_AGE_RANGE ? r_max_FERTILE_AGE_RANGE : fertile_age;
#line 92 "code/Tables.mpp"
	poStateFunctions->Set_in_capital( logical( ( province_nat== PN_PROV12 )? TRUE: FALSE ) );
#line 2186 "modgen/src/ACTORS.CPP"
#line 83 "code/PersonCore.mpp"
	poStateFunctions->Set_in_projected_time( logical( ( calendar_year>= MIN(SIM_YEAR_RANGE) ) ) );
#line 2189 "modgen/src/ACTORS.CPP"
#line 52 "code/CmapReportTables.mpp"
	poStateFunctions->Set_in_rtab_sel_years( logical( calendar_year== 2017|| calendar_year== 2030|| calendar_year== 2050 ) );
#line 2192 "modgen/src/ACTORS.CPP"
#line 48 "code/CmapReportTables.mpp"
	poStateFunctions->Set_in_rtab_years( logical( WITHIN(RTAB_YEARS,calendar_year) ) );
#line 2195 "modgen/src/ACTORS.CPP"
#line 135 "code/Fertility.mpp"
	poStateFunctions->Set_in_simulation_time( logical( ( calendar_year>= MIN(SIM_YEAR_RANGE) )? TRUE: FALSE ) );
#line 2198 "modgen/src/ACTORS.CPP"
#line 138 "code/Fertility.mpp"
	poStateFunctions->Set_in_this_parity_in_simulation( logical( in_simulation_time&& in_this_parity ) );
#line 2201 "modgen/src/ACTORS.CPP"
#line 72 "code/PersonCore.mpp"
	poStateFunctions->Set_integer_age( AGE_RANGE( COERCE(AGE_RANGE,poDerivedStates->ssint_age) ) );
#line 2204 "modgen/src/ACTORS.CPP"
	integer_age = integer_age < r_min_AGE_RANGE ? r_min_AGE_RANGE : integer_age > r_max_AGE_RANGE ? r_max_AGE_RANGE : integer_age;
#line 55 "code/CmapReportTables.mpp"
	is_act_age = logical( ( integer_age> 15&& integer_age< 60 ) );
#line 2208 "modgen/src/ACTORS.CPP"
#line 82 "code/Tables.mpp"
	poStateFunctions->Set_is_aded0( logical( ( !( is_child )&& !( is_old )&& primary_level== PL_NO ) ) );
#line 2211 "modgen/src/ACTORS.CPP"
#line 83 "code/Tables.mpp"
	poStateFunctions->Set_is_aded1( logical( ( !( is_child )&& !( is_old )&& primary_level== PL_ENTER ) ) );
#line 2214 "modgen/src/ACTORS.CPP"
#line 84 "code/Tables.mpp"
	poStateFunctions->Set_is_aded2( logical( ( !( is_child )&& !( is_old )&& primary_level== PL_GRAD ) ) );
#line 2217 "modgen/src/ACTORS.CPP"
#line 80 "code/Tables.mpp"
	poStateFunctions->Set_is_child( logical( ( integer_age< 15 ) ) );
#line 2220 "modgen/src/ACTORS.CPP"
#line 81 "code/Tables.mpp"
	poStateFunctions->Set_is_old( logical( ( integer_age>= 60 ) ) );
#line 2223 "modgen/src/ACTORS.CPP"
#line 155 "code/Fertility.mpp"
	poStateFunctions->Set_is_potential_mother( logical( ( set_alive&& sex== FEMALE&& WITHIN(FERTILE_AGE_RANGE,integer_age)&& parity< MAX(PARITY_RANGE)&& WITHIN(SIM_YEAR_RANGE,calendar_year) )? TRUE: FALSE ) );
#line 2226 "modgen/src/ACTORS.CPP"
#line 54 "code/CmapReportTables.mpp"
	poStateFunctions->Set_rtab_reg( RTAB_REG( ( province_nat== PN_PROV12 )? RR_NOUAK: RR_REST ) );
#line 2229 "modgen/src/ACTORS.CPP"
#line 53 "code/CmapReportTables.mpp"
	poStateFunctions->Set_rtab_sel_years( RTAB_SEL_YEARS( ( calendar_year== 2017 )? SY_2017: ( ( calendar_year== 2030 )? SY_2030: SY_2050 ) ) );
#line 2232 "modgen/src/ACTORS.CPP"
#line 49 "code/CmapReportTables.mpp"
	poStateFunctions->Set_rtab_years( RTAB_YEARS( COERCE(RTAB_YEARS,calendar_year) ) );
#line 2235 "modgen/src/ACTORS.CPP"
	rtab_years = rtab_years < r_min_RTAB_YEARS ? r_min_RTAB_YEARS : rtab_years > r_max_RTAB_YEARS ? r_max_RTAB_YEARS : rtab_years;
#line 88 "code/Tables.mpp"
	poStateFunctions->Set_sim_cmr( CHILD_MORT_RANGE( COERCE(CHILD_MORT_RANGE,integer_age) ) );
#line 2239 "modgen/src/ACTORS.CPP"
	sim_cmr = sim_cmr < r_min_CHILD_MORT_RANGE ? r_min_CHILD_MORT_RANGE : sim_cmr > r_max_CHILD_MORT_RANGE ? r_max_CHILD_MORT_RANGE : sim_cmr;
#line 82 "code/PersonCore.mpp"
	poStateFunctions->Set_sim_year( SIM_YEAR_RANGE( COERCE(SIM_YEAR_RANGE,calendar_year) ) );
#line 2243 "modgen/src/ACTORS.CPP"
	sim_year = sim_year < r_min_SIM_YEAR_RANGE ? r_min_SIM_YEAR_RANGE : sim_year > r_max_SIM_YEAR_RANGE ? r_max_SIM_YEAR_RANGE : sim_year;
#line 87 "code/Tables.mpp"
	poStateFunctions->Set_sim_yob( SIM_YEAR_RANGE( COERCE(SIM_YOB_RANGE,year_of_birth) ) );
#line 2247 "modgen/src/ACTORS.CPP"
	sim_yob = sim_yob < r_min_SIM_YEAR_RANGE ? r_min_SIM_YEAR_RANGE : sim_yob > r_max_SIM_YEAR_RANGE ? r_max_SIM_YEAR_RANGE : sim_yob;
	poDerivedStates->Set_split_integer_age_AGE5_PART( Split( (double) integer_age, poDerivedStates->split_integer_age_AGE5_PART, 12, p_points_AGE5_PART ) );
	poDerivedStates->Set_split_integer_age_AGE_15_16_18( Split( (double) integer_age, poDerivedStates->split_integer_age_AGE_15_16_18, 3, p_points_AGE_15_16_18 ) );
	poDerivedStates->Set_split_integer_age_AGE_15_17( Split( (double) integer_age, poDerivedStates->split_integer_age_AGE_15_17, 2, p_points_AGE_15_17 ) );
	poDerivedStates->Set_split_integer_age_AGE_FIVE( Split( (double) integer_age, poDerivedStates->split_integer_age_AGE_FIVE, 16, p_points_AGE_FIVE ) );
	poDerivedStates->Set_split_integer_age_AGE_MORT( Split( (double) integer_age, poDerivedStates->split_integer_age_AGE_MORT, 4, p_points_AGE_MORT ) );
	poDerivedStates->Set_split_integer_age_AGE_RTAB_MORT( Split( (double) integer_age, poDerivedStates->split_integer_age_AGE_RTAB_MORT, 1, p_points_AGE_RTAB_MORT ) );
	poDerivedStates->Set_split_integer_age_BIRTH_AGE_PART( Split( (double) integer_age, poDerivedStates->split_integer_age_BIRTH_AGE_PART, 3, p_points_BIRTH_AGE_PART ) );
	poDerivedStates->Set_split_time_since_last_birth_DUR_TIME_IN_PARITY( Split( (double) time_since_last_birth, poDerivedStates->split_time_since_last_birth_DUR_TIME_IN_PARITY, 5, p_points_DUR_TIME_IN_PARITY ) );
#line 148 "code/Fertility.mpp"
	poStateFunctions->Set_time_in_parity( int( poDerivedStates->split_time_since_last_birth_DUR_TIME_IN_PARITY ) );
#line 2259 "modgen/src/ACTORS.CPP"
#line 145 "code/Fertility.mpp"
	poStateFunctions->Set_time_since_last_birth( int( poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE+ round(time_since_last_birth_start) ) );
#line 2262 "modgen/src/ACTORS.CPP"
#line 65 "code/UnionFormation.mpp"
	poStateFunctions->Set_union_formation_hazard( double( ( WITHIN(AGE_UNION,integer_age)&& sex== FEMALE )? gprParam->UnionFormationHazard[CHKBND(primary_level, 3, "UnionFormationHazard", 0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE,calendar_year), 101, "UnionFormationHazard", 1)][CHKBND(RANGE_POS(AGE_UNION,integer_age), 51, "UnionFormationHazard", 2)]: 0.0 ) );
#line 2265 "modgen/src/ACTORS.CPP"
#line 384 "code/Tables.mpp"
	poStateFunctions->Set_union_formation_hazard_max_value_in( double( union_formation_hazard ) );
#line 2268 "modgen/src/ACTORS.CPP"
#line 126 "code/Fertility.mpp"
	poStateFunctions->Set_union_status( UNION_STATUS( ( in_union )? US_YES: US_NO ) );
#line 2271 "modgen/src/ACTORS.CPP"
#line 74 "code/PersonCore.mpp"
	poStateFunctions->Set_year_of_birth( ALL_YEAR_RANGE( int(time_of_birth) ) );
#line 2274 "modgen/src/ACTORS.CPP"
	year_of_birth = year_of_birth < r_min_ALL_YEAR_RANGE ? r_min_ALL_YEAR_RANGE : year_of_birth > r_max_ALL_YEAR_RANGE ? r_max_ALL_YEAR_RANGE : year_of_birth;
	CountActor(2);
}

#pragma optimize( "", off )

void Person::StartSpecial2()
{
	// initialization of expression and history states
	poStateFunctions->Set_time(TIME( CoarsenMantissa( time ) ), true);
	poStateFunctions->Set_age(TIME( CoarsenMantissa( age ) ), true);
	gdThreadCurrentTime = time;
	__time = time;
	poDerivedStates->active_spell_duration_in_this_parity_in_simulation_TRUE = (TIME) 0;
	poDerivedStates->active_spell_duration_in_this_parity_in_simulation_TRUE_time = time;
#line 26 "code/PrimaryEducation.mpp"
	poStateFunctions->Set_actorset_filter_asAllPersons( logical( set_alive ) );
#line 2292 "modgen/src/ACTORS.CPP"
#line 17 "code/UnionFormation.mpp"
	poStateFunctions->Set_actorset_filter_asFemaleAgeEduc( logical( set_alive&& sex== FEMALE ) );
#line 2295 "modgen/src/ACTORS.CPP"
#line 29 "code/PrimaryEducation.mpp"
	poStateFunctions->Set_actorset_filter_asForeignersAgeSexProv( logical( set_alive&& province_birth== PI_PROV13 ) );
#line 2298 "modgen/src/ACTORS.CPP"
#line 35 "code/PrimaryEducation.mpp"
	poStateFunctions->Set_actorset_filter_asForeignersPrimAgeSexProv( logical( set_alive&& province_birth== PI_PROV13&& primary_level!= PL_NO ) );
#line 2301 "modgen/src/ACTORS.CPP"
#line 27 "code/Fertility.mpp"
	poStateFunctions->Set_actorset_filter_asPotentialMothers( logical( is_potential_mother ) );
#line 2304 "modgen/src/ACTORS.CPP"
#line 30 "code/Fertility.mpp"
	poStateFunctions->Set_actorset_filter_asPotentialMothersByAge( logical( is_potential_mother ) );
#line 2307 "modgen/src/ACTORS.CPP"
#line 19 "code/PrimaryEducation.mpp"
	poStateFunctions->Set_actorset_filter_asPrimaryEntryCohort( logical( set_alive&& integer_age== gprParam->AgeEnterPrimary ) );
#line 2310 "modgen/src/ACTORS.CPP"
#line 23 "code/PrimaryEducation.mpp"
	poStateFunctions->Set_actorset_filter_asPrimaryGraduationCohort( logical( set_alive&& integer_age== gprParam->AgeGradPrimary&& primary_level!= PL_NO ) );
#line 2313 "modgen/src/ACTORS.CPP"
#line 32 "code/PrimaryEducation.mpp"
	poStateFunctions->Set_actorset_filter_asResidentsAgeSex( logical( set_alive ) );
#line 2316 "modgen/src/ACTORS.CPP"
#line 38 "code/PrimaryEducation.mpp"
	poStateFunctions->Set_actorset_filter_asResidentsPrimAgeSex( logical( set_alive&& primary_level!= PL_NO ) );
#line 2319 "modgen/src/ACTORS.CPP"
	poDerivedStates->Set_changes_parity( (counter) 0 );
	poDerivedStates->completed_spell_duration_in_this_parity_in_simulation_TRUE = (TIME) 0;
	poDerivedStates->Set_duration_( (TIME) 0 );
	poDerivedStates->duration__time = time;
	poDerivedStates->Set_duration_in_union_FALSE( (TIME) 0 );
	poDerivedStates->duration_in_union_FALSE_time = time;
	poDerivedStates->Set_duration_in_union_TRUE( (TIME) 0 );
	poDerivedStates->duration_in_union_TRUE_time = time;
	poDerivedStates->Set_duration_is_aded0_TRUE( (TIME) 0 );
	poDerivedStates->duration_is_aded0_TRUE_time = time;
	poDerivedStates->Set_duration_is_aded1_TRUE( (TIME) 0 );
	poDerivedStates->duration_is_aded1_TRUE_time = time;
	poDerivedStates->Set_duration_is_aded2_TRUE( (TIME) 0 );
	poDerivedStates->duration_is_aded2_TRUE_time = time;
	poDerivedStates->Set_duration_is_child_TRUE( (TIME) 0 );
	poDerivedStates->duration_is_child_TRUE_time = time;
	poDerivedStates->Set_duration_is_old_TRUE( (TIME) 0 );
	poDerivedStates->duration_is_old_TRUE_time = time;
	poDerivedStates->Set_duration_primary_level_PL_ENTER( (TIME) 0 );
	poDerivedStates->duration_primary_level_PL_ENTER_time = time;
	poDerivedStates->Set_duration_primary_level_PL_GRAD( (TIME) 0 );
	poDerivedStates->duration_primary_level_PL_GRAD_time = time;
	poDerivedStates->Set_duration_primary_level_PL_NO( (TIME) 0 );
	poDerivedStates->duration_primary_level_PL_NO_time = time;
	poDerivedStates->duration_set_alive_TRUE = (TIME) 0;
	poDerivedStates->duration_set_alive_TRUE_time = time;
	poDerivedStates->Set_entrances_primary_level_PL_GRAD( (counter) 0 );
	poDerivedStates->Set_entrances_set_alive_TRUE( (counter) 0 );
	poDerivedStates->Set_ssint_active_spell_duration_in_this_parity_in_simulation_TRUE( 0 );
	if ( in_this_parity_in_simulation == TRUE )
	{
		poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time = WAIT( 1 );
		if (schedule_event_time > poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time)
		{
			schedule_event_time = poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time;
		}
	}
	else
	{
		poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time = TIME_INFINITE;
	}
	poDerivedStates->Set_sssplit_duration_set_alive_TRUE_LITTLE_TIME( 0 );
	if ( set_alive == TRUE )
	{
		poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = WAIT( POINTS(LITTLE_TIME)[0] );
		if (schedule_event_time > poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time)
		{
			schedule_event_time = poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time;
		}
	}
	else
	{
		poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = TIME_INFINITE;
	}
#line 148 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_AgeAtFirstUnionFormation( logical( sex== FEMALE&& in_rtab_years&& count_parity ) );
#line 2376 "modgen/src/ACTORS.CPP"
#line 126 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_BirthsByAgeMother( logical( in_rtab_years&& count_parity ) );
#line 2379 "modgen/src/ACTORS.CPP"
#line 116 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_BirthsByEducMother( logical( in_rtab_years&& person_type== PT_CHILD ) );
#line 2382 "modgen/src/ACTORS.CPP"
#line 106 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_BirthsByRegion( logical( in_rtab_years&& person_type== PT_CHILD ) );
#line 2385 "modgen/src/ACTORS.CPP"
#line 95 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_DeathsByRegion( logical( in_rtab_years ) );
#line 2388 "modgen/src/ACTORS.CPP"
#line 169 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_EducationActivePop( logical( integer_age> 15&& integer_age< 60&& in_rtab_years ) );
#line 2391 "modgen/src/ACTORS.CPP"
#line 80 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_PopPyramidByEduc( logical( in_rtab_sel_years ) );
#line 2394 "modgen/src/ACTORS.CPP"
#line 182 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_PrimSchoolEntries( logical( in_rtab_years ) );
#line 2397 "modgen/src/ACTORS.CPP"
#line 193 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_PrimSchoolGraduations( logical( in_rtab_years ) );
#line 2400 "modgen/src/ACTORS.CPP"
#line 159 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_TabDeathsByAgeGroup( logical( in_rtab_years&& integer_age< 5 ) );
#line 2403 "modgen/src/ACTORS.CPP"
#line 60 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_TotalPopReg( logical( in_rtab_years ) );
#line 2406 "modgen/src/ACTORS.CPP"
#line 70 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_TotalPopWillaya( logical( in_rtab_years ) );
#line 2409 "modgen/src/ACTORS.CPP"
#line 136 "code/CmapReportTables.mpp"
	poStateFunctions->Set_table_filter_RTAB_UnionFormationByAge( logical( in_rtab_years&& count_parity ) );
#line 2412 "modgen/src/ACTORS.CPP"
#line 183 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabAgeAtBirth( logical( in_projected_time&& sex== FEMALE&& count_parity ) );
#line 2415 "modgen/src/ACTORS.CPP"
#line 388 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabAgeAtFirstUnionFormation( logical( sex== FEMALE&& in_projected_time&& count_parity ) );
#line 2418 "modgen/src/ACTORS.CPP"
#line 172 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabBirthsByAgeGroup( logical( in_projected_time&& count_parity ) );
#line 2421 "modgen/src/ACTORS.CPP"
#line 221 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabCohortChildMortality( logical( in_projected_time&& WITHIN(SIM_YOB_RANGE,year_of_birth)&& WITHIN(CHILD_MORT_RANGE,integer_age) ) );
#line 2424 "modgen/src/ACTORS.CPP"
#line 198 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabDeathRates( logical( in_projected_time ) );
#line 2427 "modgen/src/ACTORS.CPP"
#line 210 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabDeathsByAgeGroup( logical( in_projected_time ) );
#line 2430 "modgen/src/ACTORS.CPP"
#line 299 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabEduc12ByPob( logical( integer_age== 12&& in_projected_time ) );
#line 2433 "modgen/src/ACTORS.CPP"
#line 312 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabEduc12ByPor( logical( integer_age== 12&& in_projected_time ) );
#line 2436 "modgen/src/ACTORS.CPP"
#line 325 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabEduc2015( logical( calendar_year== 2015 ) );
#line 2439 "modgen/src/ACTORS.CPP"
#line 338 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabEduc2040( logical( calendar_year== 2040 ) );
#line 2442 "modgen/src/ACTORS.CPP"
#line 351 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabEduc2065( logical( calendar_year== 2065 ) );
#line 2445 "modgen/src/ACTORS.CPP"
#line 267 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabEmigration( logical( in_projected_time ) );
#line 2448 "modgen/src/ACTORS.CPP"
#line 152 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabFertilityRates( logical( in_projected_time&& WITHIN(FERTILE_AGE_RANGE,integer_age)&& sex== FEMALE&& count_parity ) );
#line 2451 "modgen/src/ACTORS.CPP"
#line 255 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabImmigrationSexProvinceAge( logical( person_type== PT_IMMIGRANT&& in_projected_time&& poDerivedStates->trigger_entrances_set_alive_TRUE ) );
#line 2454 "modgen/src/ACTORS.CPP"
#line 244 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabInternalMigrationRate( logical( in_projected_time ) );
#line 2457 "modgen/src/ACTORS.CPP"
#line 405 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabMothersExperience( logical( in_projected_time&& WITHIN(SIM_YOB_RANGE,year_of_birth)&& person_type== PT_CHILD&& sex== FEMALE ) );
#line 2460 "modgen/src/ACTORS.CPP"
#line 162 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabNumberBirths( logical( in_projected_time&& person_type== PT_CHILD ) );
#line 2463 "modgen/src/ACTORS.CPP"
#line 284 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabPopProvAgeEducSex( logical( in_projected_time ) );
#line 2466 "modgen/src/ACTORS.CPP"
#line 124 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabPopulationAgeSexProvince( logical( in_projected_time ) );
#line 2469 "modgen/src/ACTORS.CPP"
#line 136 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabSimulatedStartingPopulation( logical( poDerivedStates->trigger_entrances_in_projected_time_TRUE ) );
#line 2472 "modgen/src/ACTORS.CPP"
#line 113 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabTotalPopulationSexProvince( logical( in_projected_time ) );
#line 2475 "modgen/src/ACTORS.CPP"
#line 370 "code/Tables.mpp"
	poStateFunctions->Set_table_filter_TabUnionFormation( logical( WITHIN(AGE_UNION,integer_age)&& sex== FEMALE&& in_projected_time&& count_parity ) );
#line 2478 "modgen/src/ACTORS.CPP"
	poDerivedStates->Set_transitions_alive_TRUE_FALSE( (counter) 0 );
	poDerivedStates->Set_transitions_has_emigrated_FALSE_TRUE( (counter) 0 );
	poDerivedStates->Set_transitions_in_union_FALSE_TRUE( (counter) 0 );
	poDerivedStates->Set_transitions_integer_age_14_15( (counter) 0 );
	poDerivedStates->Set_transitions_integer_age_19_20( (counter) 0 );
	poDerivedStates->Set_transitions_integer_age_9_10( (counter) 0 );
	poDerivedStates->Set_transitions_parity_0_1( (counter) 0 );
	poDerivedStates->Set_transitions_primary_level_PL_ENTER_PL_GRAD( (counter) 0 );
	poDerivedStates->Set_transitions_primary_level_PL_NO_PL_ENTER( (counter) 0 );
	poDerivedStates->Set_trigger_entrances_in_projected_time_TRUE( FALSE );
	poDerivedStates->trigger_entrances_in_projected_time_TRUE_time = TIME_INFINITE;
	poDerivedStates->Set_trigger_entrances_set_alive_TRUE( FALSE );
	poDerivedStates->trigger_entrances_set_alive_TRUE_time = TIME_INFINITE;
	poDerivedStates->Set_value_at_changes_parity_age( (real) 0 );
	poDerivedStates->Set_value_at_transitions_in_union_FALSE_TRUE_age( (real) 0 );
	poDerivedStates->Set_value_at_transitions_parity_0_1_age( (real) 0 );
	UpdateLinkedStates();

	// initialization of cross-tabulation updates
	tableinfo.tab0events = -1;
	tableinfo.tab0open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab1events = -1;
	tableinfo.tab1open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab2events = -1;
	tableinfo.tab2open0 = (real) ( poDerivedStates->duration_primary_level_PL_ENTER );
	tableinfo.tab2open1 = (real) ( poDerivedStates->duration_primary_level_PL_GRAD );
	tableinfo.tab2open2 = (real) ( poDerivedStates->duration_primary_level_PL_NO );
	tableinfo.tab3events = -1;
	tableinfo.tab3open0 = (real) ( poDerivedStates->transitions_alive_TRUE_FALSE );
	tableinfo.tab4events = -1;
	tableinfo.tab4open0 = (real) ( poDerivedStates->entrances_set_alive_TRUE );
	tableinfo.tab5events = -1;
	tableinfo.tab5open0 = (real) ( poDerivedStates->entrances_set_alive_TRUE );
	tableinfo.tab6events = -1;
	tableinfo.tab6open0 = (real) ( parity );
	tableinfo.tab7events = -1;
	tableinfo.tab7open0 = (real) ( poDerivedStates->transitions_in_union_FALSE_TRUE );
	tableinfo.tab8events = -1;
	tableinfo.tab8open0 = (real) ( poDerivedStates->transitions_in_union_FALSE_TRUE );
	tableinfo.tab8open1 = (real) ( poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age );
	tableinfo.tab9events = -1;
	tableinfo.tab9open0 = (real) ( poDerivedStates->transitions_alive_TRUE_FALSE );
	tableinfo.tab10events = -1;
	tableinfo.tab10open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab10open1 = (real) ( poDerivedStates->duration_primary_level_PL_ENTER );
	tableinfo.tab10open2 = (real) ( poDerivedStates->duration_primary_level_PL_GRAD );
	tableinfo.tab10open3 = (real) ( poDerivedStates->duration_primary_level_PL_NO );
	tableinfo.tab11events = -1;
	tableinfo.tab11open0 = (real) ( poDerivedStates->transitions_primary_level_PL_NO_PL_ENTER );
	tableinfo.tab12events = -1;
	tableinfo.tab12open0 = (real) ( poDerivedStates->transitions_primary_level_PL_ENTER_PL_GRAD );
	tableinfo.tab13events = -1;
	tableinfo.tab13open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab14events = -1;
	tableinfo.tab14open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab15events = -1;
	tableinfo.tab15open0 = (real) ( table_unit_TabSimulatedStartingPopulation );
	tableinfo.tab16events = -1;
	tableinfo.tab16open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab16open1 = (real) ( parity );
	tableinfo.tab17events = -1;
	tableinfo.tab17open0 = (real) ( poDerivedStates->entrances_set_alive_TRUE );
	tableinfo.tab18events = -1;
	tableinfo.tab18open0 = (real) ( parity );
	tableinfo.tab18open1 = (real) ( poDerivedStates->transitions_parity_0_1 );
	tableinfo.tab19events = -1;
	tableinfo.tab19open0 = (real) ( poDerivedStates->changes_parity );
	tableinfo.tab19open1 = (real) ( poDerivedStates->transitions_parity_0_1 );
	tableinfo.tab19open2 = (real) ( poDerivedStates->value_at_changes_parity_age );
	tableinfo.tab19open3 = (real) ( poDerivedStates->value_at_transitions_parity_0_1_age );
	tableinfo.tab20events = -1;
	tableinfo.tab20open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab20open1 = (real) ( poDerivedStates->transitions_alive_TRUE_FALSE );
	tableinfo.tab21events = -1;
	tableinfo.tab21open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab21open1 = (real) ( poDerivedStates->transitions_alive_TRUE_FALSE );
	tableinfo.tab22events = -1;
	tableinfo.tab22open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab22open1 = (real) ( poDerivedStates->transitions_alive_TRUE_FALSE );
	tableinfo.tab24events = -1;
	tableinfo.tab24open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab24open1 = (real) ( number_moves );
	tableinfo.tab25events = -1;
	tableinfo.tab25open0 = (real) ( table_unit_TabImmigrationSexProvinceAge );
	tableinfo.tab26events = -1;
	tableinfo.tab26open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab26open1 = (real) ( poDerivedStates->transitions_has_emigrated_FALSE_TRUE );
	tableinfo.tab27events = -1;
	tableinfo.tab27open0 = (real) ( poDerivedStates->duration_is_aded0_TRUE );
	tableinfo.tab27open1 = (real) ( poDerivedStates->duration_is_aded1_TRUE );
	tableinfo.tab27open2 = (real) ( poDerivedStates->duration_is_aded2_TRUE );
	tableinfo.tab27open3 = (real) ( poDerivedStates->duration_is_child_TRUE );
	tableinfo.tab27open4 = (real) ( poDerivedStates->duration_is_old_TRUE );
	tableinfo.tab28events = -1;
	tableinfo.tab28open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab28open1 = (real) ( poDerivedStates->duration_primary_level_PL_ENTER );
	tableinfo.tab28open2 = (real) ( poDerivedStates->duration_primary_level_PL_GRAD );
	tableinfo.tab28open3 = (real) ( poDerivedStates->duration_primary_level_PL_NO );
	tableinfo.tab29events = -1;
	tableinfo.tab29open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab29open1 = (real) ( poDerivedStates->duration_primary_level_PL_ENTER );
	tableinfo.tab29open2 = (real) ( poDerivedStates->duration_primary_level_PL_GRAD );
	tableinfo.tab29open3 = (real) ( poDerivedStates->duration_primary_level_PL_NO );
	tableinfo.tab30events = -1;
	tableinfo.tab30open0 = (real) ( poDerivedStates->duration_primary_level_PL_ENTER );
	tableinfo.tab30open1 = (real) ( poDerivedStates->duration_primary_level_PL_GRAD );
	tableinfo.tab30open2 = (real) ( poDerivedStates->duration_primary_level_PL_NO );
	tableinfo.tab31events = -1;
	tableinfo.tab31open0 = (real) ( poDerivedStates->duration_primary_level_PL_ENTER );
	tableinfo.tab31open1 = (real) ( poDerivedStates->duration_primary_level_PL_GRAD );
	tableinfo.tab31open2 = (real) ( poDerivedStates->duration_primary_level_PL_NO );
	tableinfo.tab32events = -1;
	tableinfo.tab32open0 = (real) ( poDerivedStates->duration_primary_level_PL_ENTER );
	tableinfo.tab32open1 = (real) ( poDerivedStates->duration_primary_level_PL_GRAD );
	tableinfo.tab32open2 = (real) ( poDerivedStates->duration_primary_level_PL_NO );
	tableinfo.tab33events = -1;
	tableinfo.tab33open0 = (real) ( poDerivedStates->duration_ );
	tableinfo.tab33open1 = (real) ( poDerivedStates->duration_in_union_FALSE );
	tableinfo.tab33open2 = (real) ( poDerivedStates->duration_in_union_TRUE );
	tableinfo.tab33open3 = (real) ( poDerivedStates->transitions_in_union_FALSE_TRUE );
	tableinfo.tab33open4 = (real) ( union_formation_hazard_max_value_in );
	tableinfo.tab34events = -1;
	tableinfo.tab34open0 = (real) ( poDerivedStates->transitions_in_union_FALSE_TRUE );
	tableinfo.tab34open1 = (real) ( poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age );
	tableinfo.tab35events = -1;
	tableinfo.tab35open0 = (real) ( poDerivedStates->entrances_primary_level_PL_GRAD );
	tableinfo.tab35open1 = (real) ( infant_deaths );
	tableinfo.tab35open2 = (real) ( parity );
	tableinfo.tab35open3 = (real) ( table_unit_TabMothersExperience );
	tableinfo.tab35open4 = (real) ( poDerivedStates->transitions_in_union_FALSE_TRUE );
	tableinfo.tab35open5 = (real) ( poDerivedStates->transitions_integer_age_14_15 );
	tableinfo.tab35open6 = (real) ( poDerivedStates->transitions_integer_age_19_20 );
	tableinfo.tab35open7 = (real) ( poDerivedStates->transitions_integer_age_9_10 );
	tableinfo.tab35open8 = (real) ( poDerivedStates->transitions_parity_0_1 );
	tableinfo.tab35open9 = (real) ( poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age );
	tableinfo.tab35open10 = (real) ( poDerivedStates->value_at_transitions_parity_0_1_age );
	poPrev = NULL;
	poNext = gpoEventQueue->poHeadActor;
	if ( gpoEventQueue->poHeadActor != NULL ) {
		gpoEventQueue->poHeadActor->poPrev = this;
	}
	gpoEventQueue->poHeadActor = this;
	bUpdates = TRUE;
}

void Person::FinishSpecial()
{
	CountFinishedActor(2);
	bUpdates = FALSE;
	gpoEventQueue->DestroyEvent( poBirthEvent );
	gpoEventQueue->DestroyEvent( poChildMortalityEvent );
	gpoEventQueue->DestroyEvent( poEmigrationEvent );
	gpoEventQueue->DestroyEvent( poMigrationEvent );
	gpoEventQueue->DestroyEvent( poMortalityEvent );
	gpoEventQueue->DestroyEvent( poSetAliveEvent );
	gpoEventQueue->DestroyEvent( poUnionFormationEvent );
	gpoEventQueue->DestroyEvent( poWriteMicroRecord );
	gpoEventQueue->DestroyEvent(poScheduleEvent);

	// close all updates
	if ( gplTableIndexes[0] >= 0 && ( tableinfo.tab0dim_filter || table_filter_RTAB_TotalPopReg ) ) {
		if ( tableinfo.tab0events != -1 ) {
			tableinfo.tab0open0 = (real) CloseUpdate( 0, 0, tableinfo.tab0open0, tableinfo.tab0close0, tableinfo.tab0dim_filter, tableinfo.tab0cell, tableinfo.tab0total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab0cell = (index) TableCellIndex( 0, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_reg );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab0total_cell = (index) tableinfo.tab0cell;
		}
		else {
			tableinfo.tab0total_cell = (index) TableTotalCellIndex( 0, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_reg );
		}
		CloseUpdate( 0, 0, tableinfo.tab0open0, poDerivedStates->Get_duration_(), table_filter_RTAB_TotalPopReg, tableinfo.tab0cell, tableinfo.tab0total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[1] >= 0 && ( tableinfo.tab1dim_filter || table_filter_RTAB_TotalPopWillaya ) ) {
		if ( tableinfo.tab1events != -1 ) {
			tableinfo.tab1open0 = (real) CloseUpdate( 1, 0, tableinfo.tab1open0, tableinfo.tab1close0, tableinfo.tab1dim_filter, tableinfo.tab1cell, tableinfo.tab1total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab1cell = (index) TableCellIndex( 1, 2,
			rtab_years - r_min_RTAB_YEARS,
			province_nat );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab1total_cell = (index) tableinfo.tab1cell;
		}
		else {
			tableinfo.tab1total_cell = (index) TableTotalCellIndex( 1, 2,
			rtab_years - r_min_RTAB_YEARS,
			province_nat );
		}
		CloseUpdate( 1, 0, tableinfo.tab1open0, poDerivedStates->Get_duration_(), table_filter_RTAB_TotalPopWillaya, tableinfo.tab1cell, tableinfo.tab1total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[2] >= 0 && ( tableinfo.tab2dim_filter || table_filter_RTAB_PopPyramidByEduc ) ) {
		if ( tableinfo.tab2events != -1 ) {
			tableinfo.tab2open0 = (real) CloseUpdate( 2, 0, tableinfo.tab2open0, tableinfo.tab2close0, tableinfo.tab2dim_filter, tableinfo.tab2cell, tableinfo.tab2total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab2open1 = (real) CloseUpdate( 2, 1, tableinfo.tab2open1, tableinfo.tab2close1, tableinfo.tab2dim_filter, tableinfo.tab2cell, tableinfo.tab2total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab2open2 = (real) CloseUpdate( 2, 2, tableinfo.tab2open2, tableinfo.tab2close2, tableinfo.tab2dim_filter, tableinfo.tab2cell, tableinfo.tab2total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab2cell = (index) TableCellIndex( 2, 4,
			rtab_sel_years,
			sex,
			rtab_reg,
			poDerivedStates->split_integer_age_AGE_FIVE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab2total_cell = (index) tableinfo.tab2cell;
		}
		else {
			tableinfo.tab2total_cell = (index) TableTotalCellIndex( 2, 4,
			rtab_sel_years,
			sex,
			rtab_reg,
			poDerivedStates->split_integer_age_AGE_FIVE );
		}
		CloseUpdate( 2, 0, tableinfo.tab2open0, poDerivedStates->Get_duration_primary_level_PL_ENTER(), table_filter_RTAB_PopPyramidByEduc, tableinfo.tab2cell, tableinfo.tab2total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 2, 1, tableinfo.tab2open1, poDerivedStates->Get_duration_primary_level_PL_GRAD(), table_filter_RTAB_PopPyramidByEduc, tableinfo.tab2cell, tableinfo.tab2total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 2, 2, tableinfo.tab2open2, poDerivedStates->Get_duration_primary_level_PL_NO(), table_filter_RTAB_PopPyramidByEduc, tableinfo.tab2cell, tableinfo.tab2total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[3] >= 0 && ( tableinfo.tab3dim_filter || table_filter_RTAB_DeathsByRegion ) ) {
		if ( tableinfo.tab3events != -1 ) {
			tableinfo.tab3open0 = (real) CloseUpdate( 3, 0, tableinfo.tab3open0, tableinfo.tab3close0, tableinfo.tab3dim_filter, tableinfo.tab3cell, tableinfo.tab3total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab3cell = (index) TableCellIndex( 3, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_reg );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab3total_cell = (index) tableinfo.tab3cell;
		}
		else {
			tableinfo.tab3total_cell = (index) TableTotalCellIndex( 3, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_reg );
		}
		CloseUpdate( 3, 0, tableinfo.tab3open0, poDerivedStates->transitions_alive_TRUE_FALSE, table_filter_RTAB_DeathsByRegion, tableinfo.tab3cell, tableinfo.tab3total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[4] >= 0 && ( tableinfo.tab4dim_filter || table_filter_RTAB_BirthsByRegion ) ) {
		if ( tableinfo.tab4events != -1 ) {
			tableinfo.tab4open0 = (real) CloseUpdate( 4, 0, tableinfo.tab4open0, tableinfo.tab4close0, tableinfo.tab4dim_filter, tableinfo.tab4cell, tableinfo.tab4total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab4cell = (index) TableCellIndex( 4, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_reg );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab4total_cell = (index) tableinfo.tab4cell;
		}
		else {
			tableinfo.tab4total_cell = (index) TableTotalCellIndex( 4, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_reg );
		}
		CloseUpdate( 4, 0, tableinfo.tab4open0, poDerivedStates->entrances_set_alive_TRUE, table_filter_RTAB_BirthsByRegion, tableinfo.tab4cell, tableinfo.tab4total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[5] >= 0 && ( tableinfo.tab5dim_filter || table_filter_RTAB_BirthsByEducMother ) ) {
		if ( tableinfo.tab5events != -1 ) {
			tableinfo.tab5open0 = (real) CloseUpdate( 5, 0, tableinfo.tab5open0, tableinfo.tab5close0, tableinfo.tab5dim_filter, tableinfo.tab5cell, tableinfo.tab5total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab5cell = (index) TableCellIndex( 5, 2,
			rtab_years - r_min_RTAB_YEARS,
			educ_mother );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab5total_cell = (index) tableinfo.tab5cell;
		}
		else {
			tableinfo.tab5total_cell = (index) TableTotalCellIndex( 5, 2,
			rtab_years - r_min_RTAB_YEARS,
			educ_mother );
		}
		CloseUpdate( 5, 0, tableinfo.tab5open0, poDerivedStates->entrances_set_alive_TRUE, table_filter_RTAB_BirthsByEducMother, tableinfo.tab5cell, tableinfo.tab5total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[6] >= 0 && ( tableinfo.tab6dim_filter || table_filter_RTAB_BirthsByAgeMother ) ) {
		if ( tableinfo.tab6events != -1 ) {
			tableinfo.tab6open0 = (real) CloseUpdate( 6, 0, tableinfo.tab6open0, tableinfo.tab6close0, tableinfo.tab6dim_filter, tableinfo.tab6cell, tableinfo.tab6total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab6cell = (index) TableCellIndex( 6, 2,
			rtab_years - r_min_RTAB_YEARS,
			poDerivedStates->split_integer_age_AGE_15_16_18 );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab6total_cell = (index) tableinfo.tab6cell;
		}
		else {
			tableinfo.tab6total_cell = (index) TableTotalCellIndex( 6, 2,
			rtab_years - r_min_RTAB_YEARS,
			poDerivedStates->split_integer_age_AGE_15_16_18 );
		}
		CloseUpdate( 6, 0, tableinfo.tab6open0, parity, table_filter_RTAB_BirthsByAgeMother, tableinfo.tab6cell, tableinfo.tab6total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[7] >= 0 && ( tableinfo.tab7dim_filter || table_filter_RTAB_UnionFormationByAge ) ) {
		if ( tableinfo.tab7events != -1 ) {
			tableinfo.tab7open0 = (real) CloseUpdate( 7, 0, tableinfo.tab7open0, tableinfo.tab7close0, tableinfo.tab7dim_filter, tableinfo.tab7cell, tableinfo.tab7total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab7cell = (index) TableCellIndex( 7, 2,
			rtab_years - r_min_RTAB_YEARS,
			poDerivedStates->split_integer_age_AGE_15_16_18 );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab7total_cell = (index) tableinfo.tab7cell;
		}
		else {
			tableinfo.tab7total_cell = (index) TableTotalCellIndex( 7, 2,
			rtab_years - r_min_RTAB_YEARS,
			poDerivedStates->split_integer_age_AGE_15_16_18 );
		}
		CloseUpdate( 7, 0, tableinfo.tab7open0, poDerivedStates->transitions_in_union_FALSE_TRUE, table_filter_RTAB_UnionFormationByAge, tableinfo.tab7cell, tableinfo.tab7total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[8] >= 0 && ( tableinfo.tab8dim_filter || table_filter_RTAB_AgeAtFirstUnionFormation ) ) {
		if ( tableinfo.tab8events != -1 ) {
			tableinfo.tab8open0 = (real) CloseUpdate( 8, 0, tableinfo.tab8open0, tableinfo.tab8close0, tableinfo.tab8dim_filter, tableinfo.tab8cell, tableinfo.tab8total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab8open1 = (real) CloseUpdate( 8, 1, tableinfo.tab8open1, tableinfo.tab8close1, tableinfo.tab8dim_filter, tableinfo.tab8cell, tableinfo.tab8total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab8cell = (index) TableCellIndex( 8, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_column );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab8total_cell = (index) tableinfo.tab8cell;
		}
		else {
			tableinfo.tab8total_cell = (index) TableTotalCellIndex( 8, 2,
			rtab_years - r_min_RTAB_YEARS,
			rtab_column );
		}
		CloseUpdate( 8, 0, tableinfo.tab8open0, poDerivedStates->transitions_in_union_FALSE_TRUE, table_filter_RTAB_AgeAtFirstUnionFormation, tableinfo.tab8cell, tableinfo.tab8total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 8, 1, tableinfo.tab8open1, poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age, table_filter_RTAB_AgeAtFirstUnionFormation, tableinfo.tab8cell, tableinfo.tab8total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[9] >= 0 && ( tableinfo.tab9dim_filter || table_filter_RTAB_TabDeathsByAgeGroup ) ) {
		if ( tableinfo.tab9events != -1 ) {
			tableinfo.tab9open0 = (real) CloseUpdate( 9, 0, tableinfo.tab9open0, tableinfo.tab9close0, tableinfo.tab9dim_filter, tableinfo.tab9cell, tableinfo.tab9total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab9cell = (index) TableCellIndex( 9, 2,
			rtab_years - r_min_RTAB_YEARS,
			poDerivedStates->split_integer_age_AGE_RTAB_MORT );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab9total_cell = (index) tableinfo.tab9cell;
		}
		else {
			tableinfo.tab9total_cell = (index) TableTotalCellIndex( 9, 2,
			rtab_years - r_min_RTAB_YEARS,
			poDerivedStates->split_integer_age_AGE_RTAB_MORT );
		}
		CloseUpdate( 9, 0, tableinfo.tab9open0, poDerivedStates->transitions_alive_TRUE_FALSE, table_filter_RTAB_TabDeathsByAgeGroup, tableinfo.tab9cell, tableinfo.tab9total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[10] >= 0 && ( tableinfo.tab10dim_filter || table_filter_RTAB_EducationActivePop ) ) {
		if ( tableinfo.tab10events != -1 ) {
			tableinfo.tab10open0 = (real) CloseUpdate( 10, 0, tableinfo.tab10open0, tableinfo.tab10close0, tableinfo.tab10dim_filter, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab10open1 = (real) CloseUpdate( 10, 1, tableinfo.tab10open1, tableinfo.tab10close1, tableinfo.tab10dim_filter, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab10open2 = (real) CloseUpdate( 10, 2, tableinfo.tab10open2, tableinfo.tab10close2, tableinfo.tab10dim_filter, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab10open3 = (real) CloseUpdate( 10, 3, tableinfo.tab10open3, tableinfo.tab10close3, tableinfo.tab10dim_filter, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab10cell = (index) TableCellIndex( 10, 2,
			rtab_reg,
			rtab_years - r_min_RTAB_YEARS );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab10total_cell = (index) tableinfo.tab10cell;
		}
		else {
			tableinfo.tab10total_cell = (index) TableTotalCellIndex( 10, 2,
			rtab_reg,
			rtab_years - r_min_RTAB_YEARS );
		}
		CloseUpdate( 10, 0, tableinfo.tab10open0, poDerivedStates->Get_duration_(), table_filter_RTAB_EducationActivePop, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 10, 1, tableinfo.tab10open1, poDerivedStates->Get_duration_primary_level_PL_ENTER(), table_filter_RTAB_EducationActivePop, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 10, 2, tableinfo.tab10open2, poDerivedStates->Get_duration_primary_level_PL_GRAD(), table_filter_RTAB_EducationActivePop, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 10, 3, tableinfo.tab10open3, poDerivedStates->Get_duration_primary_level_PL_NO(), table_filter_RTAB_EducationActivePop, tableinfo.tab10cell, tableinfo.tab10total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[11] >= 0 && ( tableinfo.tab11dim_filter || table_filter_RTAB_PrimSchoolEntries ) ) {
		if ( tableinfo.tab11events != -1 ) {
			tableinfo.tab11open0 = (real) CloseUpdate( 11, 0, tableinfo.tab11open0, tableinfo.tab11close0, tableinfo.tab11dim_filter, tableinfo.tab11cell, tableinfo.tab11total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab11cell = (index) TableCellIndex( 11, 2,
			rtab_years - r_min_RTAB_YEARS,
			province_nat );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab11total_cell = (index) tableinfo.tab11cell;
		}
		else {
			tableinfo.tab11total_cell = (index) TableTotalCellIndex( 11, 2,
			rtab_years - r_min_RTAB_YEARS,
			province_nat );
		}
		CloseUpdate( 11, 0, tableinfo.tab11open0, poDerivedStates->transitions_primary_level_PL_NO_PL_ENTER, table_filter_RTAB_PrimSchoolEntries, tableinfo.tab11cell, tableinfo.tab11total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[12] >= 0 && ( tableinfo.tab12dim_filter || table_filter_RTAB_PrimSchoolGraduations ) ) {
		if ( tableinfo.tab12events != -1 ) {
			tableinfo.tab12open0 = (real) CloseUpdate( 12, 0, tableinfo.tab12open0, tableinfo.tab12close0, tableinfo.tab12dim_filter, tableinfo.tab12cell, tableinfo.tab12total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab12cell = (index) TableCellIndex( 12, 2,
			rtab_years - r_min_RTAB_YEARS,
			province_nat );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab12total_cell = (index) tableinfo.tab12cell;
		}
		else {
			tableinfo.tab12total_cell = (index) TableTotalCellIndex( 12, 2,
			rtab_years - r_min_RTAB_YEARS,
			province_nat );
		}
		CloseUpdate( 12, 0, tableinfo.tab12open0, poDerivedStates->transitions_primary_level_PL_ENTER_PL_GRAD, table_filter_RTAB_PrimSchoolGraduations, tableinfo.tab12cell, tableinfo.tab12total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[13] >= 0 && ( tableinfo.tab13dim_filter || table_filter_TabTotalPopulationSexProvince ) ) {
		if ( tableinfo.tab13events != -1 ) {
			tableinfo.tab13open0 = (real) CloseUpdate( 13, 0, tableinfo.tab13open0, tableinfo.tab13close0, tableinfo.tab13dim_filter, tableinfo.tab13cell, tableinfo.tab13total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab13cell = (index) TableCellIndex( 13, 3,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE,
			province_nat );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab13total_cell = (index) tableinfo.tab13cell;
		}
		else {
			tableinfo.tab13total_cell = (index) TableTotalCellIndex( 13, 3,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE,
			province_nat );
		}
		CloseUpdate( 13, 0, tableinfo.tab13open0, poDerivedStates->Get_duration_(), table_filter_TabTotalPopulationSexProvince, tableinfo.tab13cell, tableinfo.tab13total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[14] >= 0 && ( tableinfo.tab14dim_filter || table_filter_TabPopulationAgeSexProvince ) ) {
		if ( tableinfo.tab14events != -1 ) {
			tableinfo.tab14open0 = (real) CloseUpdate( 14, 0, tableinfo.tab14open0, tableinfo.tab14close0, tableinfo.tab14dim_filter, tableinfo.tab14cell, tableinfo.tab14total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab14cell = (index) TableCellIndex( 14, 4,
			sex,
			province_nat,
			poDerivedStates->split_integer_age_AGE5_PART,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab14total_cell = (index) tableinfo.tab14cell;
		}
		else {
			tableinfo.tab14total_cell = (index) TableTotalCellIndex( 14, 4,
			sex,
			province_nat,
			poDerivedStates->split_integer_age_AGE5_PART,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 14, 0, tableinfo.tab14open0, poDerivedStates->Get_duration_(), table_filter_TabPopulationAgeSexProvince, tableinfo.tab14cell, tableinfo.tab14total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[15] >= 0 && ( tableinfo.tab15dim_filter || table_filter_TabSimulatedStartingPopulation ) ) {
		if ( tableinfo.tab15events != -1 ) {
			poStateFunctions->Set_table_unit_TabSimulatedStartingPopulation(table_unit_TabSimulatedStartingPopulation+1);
			tableinfo.tab15close0 = (real) table_unit_TabSimulatedStartingPopulation;
			tableinfo.tab15open0 = (real) CloseUpdate( 15, 0, tableinfo.tab15open0, tableinfo.tab15close0, tableinfo.tab15dim_filter, tableinfo.tab15cell, tableinfo.tab15total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab15cell = (index) TableCellIndex( 15, 3,
			sex,
			integer_age - r_min_AGE_RANGE,
			province_nat );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab15total_cell = (index) tableinfo.tab15cell;
		}
		else {
			tableinfo.tab15total_cell = (index) TableTotalCellIndex( 15, 3,
			sex,
			integer_age - r_min_AGE_RANGE,
			province_nat );
		}
		poStateFunctions->Set_table_unit_TabSimulatedStartingPopulation(table_unit_TabSimulatedStartingPopulation+1);
		CloseUpdate( 15, 0, tableinfo.tab15open0, table_unit_TabSimulatedStartingPopulation, table_filter_TabSimulatedStartingPopulation, tableinfo.tab15cell, tableinfo.tab15total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[16] >= 0 && ( tableinfo.tab16dim_filter || table_filter_TabFertilityRates ) ) {
		if ( tableinfo.tab16events != -1 ) {
			tableinfo.tab16open0 = (real) CloseUpdate( 16, 0, tableinfo.tab16open0, tableinfo.tab16close0, tableinfo.tab16dim_filter, tableinfo.tab16cell, tableinfo.tab16total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab16open1 = (real) CloseUpdate( 16, 1, tableinfo.tab16open1, tableinfo.tab16close1, tableinfo.tab16dim_filter, tableinfo.tab16cell, tableinfo.tab16total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab16cell = (index) TableCellIndex( 16, 2,
			sim_year - r_min_SIM_YEAR_RANGE,
			fert_age - r_min_FERTILE_AGE_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab16total_cell = (index) tableinfo.tab16cell;
		}
		else {
			tableinfo.tab16total_cell = (index) TableTotalCellIndex( 16, 2,
			sim_year - r_min_SIM_YEAR_RANGE,
			fert_age - r_min_FERTILE_AGE_RANGE );
		}
		CloseUpdate( 16, 0, tableinfo.tab16open0, poDerivedStates->Get_duration_(), table_filter_TabFertilityRates, tableinfo.tab16cell, tableinfo.tab16total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 16, 1, tableinfo.tab16open1, parity, table_filter_TabFertilityRates, tableinfo.tab16cell, tableinfo.tab16total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[17] >= 0 && ( tableinfo.tab17dim_filter || table_filter_TabNumberBirths ) ) {
		if ( tableinfo.tab17events != -1 ) {
			tableinfo.tab17open0 = (real) CloseUpdate( 17, 0, tableinfo.tab17open0, tableinfo.tab17close0, tableinfo.tab17dim_filter, tableinfo.tab17cell, tableinfo.tab17total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab17cell = (index) TableCellIndex( 17, 2,
			sim_year - r_min_SIM_YEAR_RANGE,
			sex );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab17total_cell = (index) tableinfo.tab17cell;
		}
		else {
			tableinfo.tab17total_cell = (index) TableTotalCellIndex( 17, 2,
			sim_year - r_min_SIM_YEAR_RANGE,
			sex );
		}
		CloseUpdate( 17, 0, tableinfo.tab17open0, poDerivedStates->entrances_set_alive_TRUE, table_filter_TabNumberBirths, tableinfo.tab17cell, tableinfo.tab17total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[18] >= 0 && ( tableinfo.tab18dim_filter || table_filter_TabBirthsByAgeGroup ) ) {
		if ( tableinfo.tab18events != -1 ) {
			tableinfo.tab18open0 = (real) CloseUpdate( 18, 0, tableinfo.tab18open0, tableinfo.tab18close0, tableinfo.tab18dim_filter, tableinfo.tab18cell, tableinfo.tab18total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab18open1 = (real) CloseUpdate( 18, 1, tableinfo.tab18open1, tableinfo.tab18close1, tableinfo.tab18dim_filter, tableinfo.tab18cell, tableinfo.tab18total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab18cell = (index) TableCellIndex( 18, 2,
			poDerivedStates->split_integer_age_AGE_15_17,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab18total_cell = (index) tableinfo.tab18cell;
		}
		else {
			tableinfo.tab18total_cell = (index) TableTotalCellIndex( 18, 2,
			poDerivedStates->split_integer_age_AGE_15_17,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 18, 0, tableinfo.tab18open0, parity, table_filter_TabBirthsByAgeGroup, tableinfo.tab18cell, tableinfo.tab18total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 18, 1, tableinfo.tab18open1, poDerivedStates->transitions_parity_0_1, table_filter_TabBirthsByAgeGroup, tableinfo.tab18cell, tableinfo.tab18total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[19] >= 0 && ( tableinfo.tab19dim_filter || table_filter_TabAgeAtBirth ) ) {
		if ( tableinfo.tab19events != -1 ) {
			tableinfo.tab19open0 = (real) CloseUpdate( 19, 0, tableinfo.tab19open0, tableinfo.tab19close0, tableinfo.tab19dim_filter, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab19open1 = (real) CloseUpdate( 19, 1, tableinfo.tab19open1, tableinfo.tab19close1, tableinfo.tab19dim_filter, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab19open2 = (real) CloseUpdate( 19, 2, tableinfo.tab19open2, tableinfo.tab19close2, tableinfo.tab19dim_filter, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab19open3 = (real) CloseUpdate( 19, 3, tableinfo.tab19open3, tableinfo.tab19close3, tableinfo.tab19dim_filter, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab19cell = (index) TableCellIndex( 19, 2,
			primary_level,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab19total_cell = (index) tableinfo.tab19cell;
		}
		else {
			tableinfo.tab19total_cell = (index) TableTotalCellIndex( 19, 2,
			primary_level,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 19, 0, tableinfo.tab19open0, poDerivedStates->changes_parity, table_filter_TabAgeAtBirth, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 19, 1, tableinfo.tab19open1, poDerivedStates->transitions_parity_0_1, table_filter_TabAgeAtBirth, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 19, 2, tableinfo.tab19open2, poDerivedStates->value_at_changes_parity_age, table_filter_TabAgeAtBirth, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 19, 3, tableinfo.tab19open3, poDerivedStates->value_at_transitions_parity_0_1_age, table_filter_TabAgeAtBirth, tableinfo.tab19cell, tableinfo.tab19total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[20] >= 0 && ( tableinfo.tab20dim_filter || table_filter_TabDeathRates ) ) {
		if ( tableinfo.tab20events != -1 ) {
			tableinfo.tab20open0 = (real) CloseUpdate( 20, 0, tableinfo.tab20open0, tableinfo.tab20close0, tableinfo.tab20dim_filter, tableinfo.tab20cell, tableinfo.tab20total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab20open1 = (real) CloseUpdate( 20, 1, tableinfo.tab20open1, tableinfo.tab20close1, tableinfo.tab20dim_filter, tableinfo.tab20cell, tableinfo.tab20total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab20cell = (index) TableCellIndex( 20, 3,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE,
			integer_age - r_min_AGE_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab20total_cell = (index) tableinfo.tab20cell;
		}
		else {
			tableinfo.tab20total_cell = (index) TableTotalCellIndex( 20, 3,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE,
			integer_age - r_min_AGE_RANGE );
		}
		CloseUpdate( 20, 0, tableinfo.tab20open0, poDerivedStates->Get_duration_(), table_filter_TabDeathRates, tableinfo.tab20cell, tableinfo.tab20total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 20, 1, tableinfo.tab20open1, poDerivedStates->transitions_alive_TRUE_FALSE, table_filter_TabDeathRates, tableinfo.tab20cell, tableinfo.tab20total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[21] >= 0 && ( tableinfo.tab21dim_filter || table_filter_TabDeathsByAgeGroup ) ) {
		if ( tableinfo.tab21events != -1 ) {
			tableinfo.tab21open0 = (real) CloseUpdate( 21, 0, tableinfo.tab21open0, tableinfo.tab21close0, tableinfo.tab21dim_filter, tableinfo.tab21cell, tableinfo.tab21total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab21open1 = (real) CloseUpdate( 21, 1, tableinfo.tab21open1, tableinfo.tab21close1, tableinfo.tab21dim_filter, tableinfo.tab21cell, tableinfo.tab21total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab21cell = (index) TableCellIndex( 21, 2,
			sim_year - r_min_SIM_YEAR_RANGE,
			poDerivedStates->split_integer_age_AGE_MORT );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab21total_cell = (index) tableinfo.tab21cell;
		}
		else {
			tableinfo.tab21total_cell = (index) TableTotalCellIndex( 21, 2,
			sim_year - r_min_SIM_YEAR_RANGE,
			poDerivedStates->split_integer_age_AGE_MORT );
		}
		CloseUpdate( 21, 0, tableinfo.tab21open0, poDerivedStates->Get_duration_(), table_filter_TabDeathsByAgeGroup, tableinfo.tab21cell, tableinfo.tab21total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 21, 1, tableinfo.tab21open1, poDerivedStates->transitions_alive_TRUE_FALSE, table_filter_TabDeathsByAgeGroup, tableinfo.tab21cell, tableinfo.tab21total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[22] >= 0 && ( tableinfo.tab22dim_filter || table_filter_TabCohortChildMortality ) ) {
		if ( tableinfo.tab22events != -1 ) {
			tableinfo.tab22open0 = (real) CloseUpdate( 22, 0, tableinfo.tab22open0, tableinfo.tab22close0, tableinfo.tab22dim_filter, tableinfo.tab22cell, tableinfo.tab22total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab22open1 = (real) CloseUpdate( 22, 1, tableinfo.tab22open1, tableinfo.tab22close1, tableinfo.tab22dim_filter, tableinfo.tab22cell, tableinfo.tab22total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab22cell = (index) TableCellIndex( 22, 2,
			sim_cmr - r_min_CHILD_MORT_RANGE,
			sim_yob - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab22total_cell = (index) tableinfo.tab22cell;
		}
		else {
			tableinfo.tab22total_cell = (index) TableTotalCellIndex( 22, 2,
			sim_cmr - r_min_CHILD_MORT_RANGE,
			sim_yob - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 22, 0, tableinfo.tab22open0, poDerivedStates->Get_duration_(), table_filter_TabCohortChildMortality, tableinfo.tab22cell, tableinfo.tab22total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 22, 1, tableinfo.tab22open1, poDerivedStates->transitions_alive_TRUE_FALSE, table_filter_TabCohortChildMortality, tableinfo.tab22cell, tableinfo.tab22total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[24] >= 0 && ( tableinfo.tab24dim_filter || table_filter_TabInternalMigrationRate ) ) {
		if ( tableinfo.tab24events != -1 ) {
			tableinfo.tab24open0 = (real) CloseUpdate( 24, 0, tableinfo.tab24open0, tableinfo.tab24close0, tableinfo.tab24dim_filter, tableinfo.tab24cell, tableinfo.tab24total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab24open1 = (real) CloseUpdate( 24, 1, tableinfo.tab24open1, tableinfo.tab24close1, tableinfo.tab24dim_filter, tableinfo.tab24cell, tableinfo.tab24total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab24cell = (index) TableCellIndex( 24, 3,
			sex,
			poDerivedStates->split_integer_age_AGE5_PART,
			province_nat );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab24total_cell = (index) tableinfo.tab24cell;
		}
		else {
			tableinfo.tab24total_cell = (index) TableTotalCellIndex( 24, 3,
			sex,
			poDerivedStates->split_integer_age_AGE5_PART,
			province_nat );
		}
		CloseUpdate( 24, 0, tableinfo.tab24open0, poDerivedStates->Get_duration_(), table_filter_TabInternalMigrationRate, tableinfo.tab24cell, tableinfo.tab24total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 24, 1, tableinfo.tab24open1, number_moves, table_filter_TabInternalMigrationRate, tableinfo.tab24cell, tableinfo.tab24total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[25] >= 0 && ( tableinfo.tab25dim_filter || table_filter_TabImmigrationSexProvinceAge ) ) {
		if ( tableinfo.tab25events != -1 ) {
			poStateFunctions->Set_table_unit_TabImmigrationSexProvinceAge(table_unit_TabImmigrationSexProvinceAge+1);
			tableinfo.tab25close0 = (real) table_unit_TabImmigrationSexProvinceAge;
			tableinfo.tab25open0 = (real) CloseUpdate( 25, 0, tableinfo.tab25open0, tableinfo.tab25close0, tableinfo.tab25dim_filter, tableinfo.tab25cell, tableinfo.tab25total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab25cell = (index) TableCellIndex( 25, 4,
			sex,
			province_nat,
			poDerivedStates->split_integer_age_AGE5_PART,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab25total_cell = (index) tableinfo.tab25cell;
		}
		else {
			tableinfo.tab25total_cell = (index) TableTotalCellIndex( 25, 4,
			sex,
			province_nat,
			poDerivedStates->split_integer_age_AGE5_PART,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		poStateFunctions->Set_table_unit_TabImmigrationSexProvinceAge(table_unit_TabImmigrationSexProvinceAge+1);
		CloseUpdate( 25, 0, tableinfo.tab25open0, table_unit_TabImmigrationSexProvinceAge, table_filter_TabImmigrationSexProvinceAge, tableinfo.tab25cell, tableinfo.tab25total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[26] >= 0 && ( tableinfo.tab26dim_filter || table_filter_TabEmigration ) ) {
		if ( tableinfo.tab26events != -1 ) {
			tableinfo.tab26open0 = (real) CloseUpdate( 26, 0, tableinfo.tab26open0, tableinfo.tab26close0, tableinfo.tab26dim_filter, tableinfo.tab26cell, tableinfo.tab26total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab26open1 = (real) CloseUpdate( 26, 1, tableinfo.tab26open1, tableinfo.tab26close1, tableinfo.tab26dim_filter, tableinfo.tab26cell, tableinfo.tab26total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab26cell = (index) TableCellIndex( 26, 4,
			sex,
			province_nat,
			poDerivedStates->split_integer_age_AGE5_PART,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab26total_cell = (index) tableinfo.tab26cell;
		}
		else {
			tableinfo.tab26total_cell = (index) TableTotalCellIndex( 26, 4,
			sex,
			province_nat,
			poDerivedStates->split_integer_age_AGE5_PART,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 26, 0, tableinfo.tab26open0, poDerivedStates->Get_duration_(), table_filter_TabEmigration, tableinfo.tab26cell, tableinfo.tab26total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 26, 1, tableinfo.tab26open1, poDerivedStates->transitions_has_emigrated_FALSE_TRUE, table_filter_TabEmigration, tableinfo.tab26cell, tableinfo.tab26total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[27] >= 0 && ( tableinfo.tab27dim_filter || table_filter_TabPopProvAgeEducSex ) ) {
		if ( tableinfo.tab27events != -1 ) {
			tableinfo.tab27open0 = (real) CloseUpdate( 27, 0, tableinfo.tab27open0, tableinfo.tab27close0, tableinfo.tab27dim_filter, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab27open1 = (real) CloseUpdate( 27, 1, tableinfo.tab27open1, tableinfo.tab27close1, tableinfo.tab27dim_filter, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab27open2 = (real) CloseUpdate( 27, 2, tableinfo.tab27open2, tableinfo.tab27close2, tableinfo.tab27dim_filter, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab27open3 = (real) CloseUpdate( 27, 3, tableinfo.tab27open3, tableinfo.tab27close3, tableinfo.tab27dim_filter, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab27open4 = (real) CloseUpdate( 27, 4, tableinfo.tab27open4, tableinfo.tab27close4, tableinfo.tab27dim_filter, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab27cell = (index) TableCellIndex( 27, 3,
			province_nat,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab27total_cell = (index) tableinfo.tab27cell;
		}
		else {
			tableinfo.tab27total_cell = (index) TableTotalCellIndex( 27, 3,
			province_nat,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 27, 0, tableinfo.tab27open0, poDerivedStates->Get_duration_is_aded0_TRUE(), table_filter_TabPopProvAgeEducSex, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 27, 1, tableinfo.tab27open1, poDerivedStates->Get_duration_is_aded1_TRUE(), table_filter_TabPopProvAgeEducSex, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 27, 2, tableinfo.tab27open2, poDerivedStates->Get_duration_is_aded2_TRUE(), table_filter_TabPopProvAgeEducSex, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 27, 3, tableinfo.tab27open3, poDerivedStates->Get_duration_is_child_TRUE(), table_filter_TabPopProvAgeEducSex, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 27, 4, tableinfo.tab27open4, poDerivedStates->Get_duration_is_old_TRUE(), table_filter_TabPopProvAgeEducSex, tableinfo.tab27cell, tableinfo.tab27total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[28] >= 0 && ( tableinfo.tab28dim_filter || table_filter_TabEduc12ByPob ) ) {
		if ( tableinfo.tab28events != -1 ) {
			tableinfo.tab28open0 = (real) CloseUpdate( 28, 0, tableinfo.tab28open0, tableinfo.tab28close0, tableinfo.tab28dim_filter, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab28open1 = (real) CloseUpdate( 28, 1, tableinfo.tab28open1, tableinfo.tab28close1, tableinfo.tab28dim_filter, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab28open2 = (real) CloseUpdate( 28, 2, tableinfo.tab28open2, tableinfo.tab28close2, tableinfo.tab28dim_filter, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab28open3 = (real) CloseUpdate( 28, 3, tableinfo.tab28open3, tableinfo.tab28close3, tableinfo.tab28dim_filter, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab28cell = (index) TableCellIndex( 28, 3,
			province_birth,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab28total_cell = (index) tableinfo.tab28cell;
		}
		else {
			tableinfo.tab28total_cell = (index) TableTotalCellIndex( 28, 3,
			province_birth,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 28, 0, tableinfo.tab28open0, poDerivedStates->Get_duration_(), table_filter_TabEduc12ByPob, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 28, 1, tableinfo.tab28open1, poDerivedStates->Get_duration_primary_level_PL_ENTER(), table_filter_TabEduc12ByPob, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 28, 2, tableinfo.tab28open2, poDerivedStates->Get_duration_primary_level_PL_GRAD(), table_filter_TabEduc12ByPob, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 28, 3, tableinfo.tab28open3, poDerivedStates->Get_duration_primary_level_PL_NO(), table_filter_TabEduc12ByPob, tableinfo.tab28cell, tableinfo.tab28total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[29] >= 0 && ( tableinfo.tab29dim_filter || table_filter_TabEduc12ByPor ) ) {
		if ( tableinfo.tab29events != -1 ) {
			tableinfo.tab29open0 = (real) CloseUpdate( 29, 0, tableinfo.tab29open0, tableinfo.tab29close0, tableinfo.tab29dim_filter, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab29open1 = (real) CloseUpdate( 29, 1, tableinfo.tab29open1, tableinfo.tab29close1, tableinfo.tab29dim_filter, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab29open2 = (real) CloseUpdate( 29, 2, tableinfo.tab29open2, tableinfo.tab29close2, tableinfo.tab29dim_filter, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab29open3 = (real) CloseUpdate( 29, 3, tableinfo.tab29open3, tableinfo.tab29close3, tableinfo.tab29dim_filter, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab29cell = (index) TableCellIndex( 29, 3,
			province_nat,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab29total_cell = (index) tableinfo.tab29cell;
		}
		else {
			tableinfo.tab29total_cell = (index) TableTotalCellIndex( 29, 3,
			province_nat,
			sex,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 29, 0, tableinfo.tab29open0, poDerivedStates->Get_duration_(), table_filter_TabEduc12ByPor, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 29, 1, tableinfo.tab29open1, poDerivedStates->Get_duration_primary_level_PL_ENTER(), table_filter_TabEduc12ByPor, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 29, 2, tableinfo.tab29open2, poDerivedStates->Get_duration_primary_level_PL_GRAD(), table_filter_TabEduc12ByPor, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 29, 3, tableinfo.tab29open3, poDerivedStates->Get_duration_primary_level_PL_NO(), table_filter_TabEduc12ByPor, tableinfo.tab29cell, tableinfo.tab29total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[30] >= 0 && ( tableinfo.tab30dim_filter || table_filter_TabEduc2015 ) ) {
		if ( tableinfo.tab30events != -1 ) {
			tableinfo.tab30open0 = (real) CloseUpdate( 30, 0, tableinfo.tab30open0, tableinfo.tab30close0, tableinfo.tab30dim_filter, tableinfo.tab30cell, tableinfo.tab30total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab30open1 = (real) CloseUpdate( 30, 1, tableinfo.tab30open1, tableinfo.tab30close1, tableinfo.tab30dim_filter, tableinfo.tab30cell, tableinfo.tab30total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab30open2 = (real) CloseUpdate( 30, 2, tableinfo.tab30open2, tableinfo.tab30close2, tableinfo.tab30dim_filter, tableinfo.tab30cell, tableinfo.tab30total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab30cell = (index) TableCellIndex( 30, 3,
			in_capital,
			sex,
			poDerivedStates->split_integer_age_AGE_FIVE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab30total_cell = (index) tableinfo.tab30cell;
		}
		else {
			tableinfo.tab30total_cell = (index) TableTotalCellIndex( 30, 3,
			in_capital,
			sex,
			poDerivedStates->split_integer_age_AGE_FIVE );
		}
		CloseUpdate( 30, 0, tableinfo.tab30open0, poDerivedStates->Get_duration_primary_level_PL_ENTER(), table_filter_TabEduc2015, tableinfo.tab30cell, tableinfo.tab30total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 30, 1, tableinfo.tab30open1, poDerivedStates->Get_duration_primary_level_PL_GRAD(), table_filter_TabEduc2015, tableinfo.tab30cell, tableinfo.tab30total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 30, 2, tableinfo.tab30open2, poDerivedStates->Get_duration_primary_level_PL_NO(), table_filter_TabEduc2015, tableinfo.tab30cell, tableinfo.tab30total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[31] >= 0 && ( tableinfo.tab31dim_filter || table_filter_TabEduc2040 ) ) {
		if ( tableinfo.tab31events != -1 ) {
			tableinfo.tab31open0 = (real) CloseUpdate( 31, 0, tableinfo.tab31open0, tableinfo.tab31close0, tableinfo.tab31dim_filter, tableinfo.tab31cell, tableinfo.tab31total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab31open1 = (real) CloseUpdate( 31, 1, tableinfo.tab31open1, tableinfo.tab31close1, tableinfo.tab31dim_filter, tableinfo.tab31cell, tableinfo.tab31total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab31open2 = (real) CloseUpdate( 31, 2, tableinfo.tab31open2, tableinfo.tab31close2, tableinfo.tab31dim_filter, tableinfo.tab31cell, tableinfo.tab31total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab31cell = (index) TableCellIndex( 31, 3,
			in_capital,
			sex,
			poDerivedStates->split_integer_age_AGE_FIVE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab31total_cell = (index) tableinfo.tab31cell;
		}
		else {
			tableinfo.tab31total_cell = (index) TableTotalCellIndex( 31, 3,
			in_capital,
			sex,
			poDerivedStates->split_integer_age_AGE_FIVE );
		}
		CloseUpdate( 31, 0, tableinfo.tab31open0, poDerivedStates->Get_duration_primary_level_PL_ENTER(), table_filter_TabEduc2040, tableinfo.tab31cell, tableinfo.tab31total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 31, 1, tableinfo.tab31open1, poDerivedStates->Get_duration_primary_level_PL_GRAD(), table_filter_TabEduc2040, tableinfo.tab31cell, tableinfo.tab31total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 31, 2, tableinfo.tab31open2, poDerivedStates->Get_duration_primary_level_PL_NO(), table_filter_TabEduc2040, tableinfo.tab31cell, tableinfo.tab31total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[32] >= 0 && ( tableinfo.tab32dim_filter || table_filter_TabEduc2065 ) ) {
		if ( tableinfo.tab32events != -1 ) {
			tableinfo.tab32open0 = (real) CloseUpdate( 32, 0, tableinfo.tab32open0, tableinfo.tab32close0, tableinfo.tab32dim_filter, tableinfo.tab32cell, tableinfo.tab32total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab32open1 = (real) CloseUpdate( 32, 1, tableinfo.tab32open1, tableinfo.tab32close1, tableinfo.tab32dim_filter, tableinfo.tab32cell, tableinfo.tab32total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab32open2 = (real) CloseUpdate( 32, 2, tableinfo.tab32open2, tableinfo.tab32close2, tableinfo.tab32dim_filter, tableinfo.tab32cell, tableinfo.tab32total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab32cell = (index) TableCellIndex( 32, 3,
			in_capital,
			sex,
			poDerivedStates->split_integer_age_AGE_FIVE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab32total_cell = (index) tableinfo.tab32cell;
		}
		else {
			tableinfo.tab32total_cell = (index) TableTotalCellIndex( 32, 3,
			in_capital,
			sex,
			poDerivedStates->split_integer_age_AGE_FIVE );
		}
		CloseUpdate( 32, 0, tableinfo.tab32open0, poDerivedStates->Get_duration_primary_level_PL_ENTER(), table_filter_TabEduc2065, tableinfo.tab32cell, tableinfo.tab32total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 32, 1, tableinfo.tab32open1, poDerivedStates->Get_duration_primary_level_PL_GRAD(), table_filter_TabEduc2065, tableinfo.tab32cell, tableinfo.tab32total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 32, 2, tableinfo.tab32open2, poDerivedStates->Get_duration_primary_level_PL_NO(), table_filter_TabEduc2065, tableinfo.tab32cell, tableinfo.tab32total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[33] >= 0 && ( tableinfo.tab33dim_filter || table_filter_TabUnionFormation ) ) {
		if ( tableinfo.tab33events != -1 ) {
			tableinfo.tab33open0 = (real) CloseUpdate( 33, 0, tableinfo.tab33open0, tableinfo.tab33close0, tableinfo.tab33dim_filter, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab33open1 = (real) CloseUpdate( 33, 1, tableinfo.tab33open1, tableinfo.tab33close1, tableinfo.tab33dim_filter, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab33open2 = (real) CloseUpdate( 33, 2, tableinfo.tab33open2, tableinfo.tab33close2, tableinfo.tab33dim_filter, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab33open3 = (real) CloseUpdate( 33, 3, tableinfo.tab33open3, tableinfo.tab33close3, tableinfo.tab33dim_filter, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab33open4 = (real) CloseUpdate( 33, 4, tableinfo.tab33open4, tableinfo.tab33close4, tableinfo.tab33dim_filter, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab33cell = (index) TableCellIndex( 33, 3,
			primary_level,
			age_union - r_min_AGE_UNION,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab33total_cell = (index) tableinfo.tab33cell;
		}
		else {
			tableinfo.tab33total_cell = (index) TableTotalCellIndex( 33, 3,
			primary_level,
			age_union - r_min_AGE_UNION,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 33, 0, tableinfo.tab33open0, poDerivedStates->Get_duration_(), table_filter_TabUnionFormation, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 33, 1, tableinfo.tab33open1, poDerivedStates->Get_duration_in_union_FALSE(), table_filter_TabUnionFormation, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 33, 2, tableinfo.tab33open2, poDerivedStates->Get_duration_in_union_TRUE(), table_filter_TabUnionFormation, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 33, 3, tableinfo.tab33open3, poDerivedStates->transitions_in_union_FALSE_TRUE, table_filter_TabUnionFormation, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 33, 4, tableinfo.tab33open4, union_formation_hazard_max_value_in, table_filter_TabUnionFormation, tableinfo.tab33cell, tableinfo.tab33total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[34] >= 0 && ( tableinfo.tab34dim_filter || table_filter_TabAgeAtFirstUnionFormation ) ) {
		if ( tableinfo.tab34events != -1 ) {
			tableinfo.tab34open0 = (real) CloseUpdate( 34, 0, tableinfo.tab34open0, tableinfo.tab34close0, tableinfo.tab34dim_filter, tableinfo.tab34cell, tableinfo.tab34total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab34open1 = (real) CloseUpdate( 34, 1, tableinfo.tab34open1, tableinfo.tab34close1, tableinfo.tab34dim_filter, tableinfo.tab34cell, tableinfo.tab34total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab34cell = (index) TableCellIndex( 34, 2,
			primary_level,
			sim_year - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab34total_cell = (index) tableinfo.tab34cell;
		}
		else {
			tableinfo.tab34total_cell = (index) TableTotalCellIndex( 34, 2,
			primary_level,
			sim_year - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 34, 0, tableinfo.tab34open0, poDerivedStates->transitions_in_union_FALSE_TRUE, table_filter_TabAgeAtFirstUnionFormation, tableinfo.tab34cell, tableinfo.tab34total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 34, 1, tableinfo.tab34open1, poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age, table_filter_TabAgeAtFirstUnionFormation, tableinfo.tab34cell, tableinfo.tab34total_cell, actor_weight, actor_subsample_weight );
	}
	if ( gplTableIndexes[35] >= 0 && ( tableinfo.tab35dim_filter || table_filter_TabMothersExperience ) ) {
		if ( tableinfo.tab35events != -1 ) {
			tableinfo.tab35open0 = (real) CloseUpdate( 35, 0, tableinfo.tab35open0, tableinfo.tab35close0, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open1 = (real) CloseUpdate( 35, 1, tableinfo.tab35open1, tableinfo.tab35close1, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open2 = (real) CloseUpdate( 35, 2, tableinfo.tab35open2, tableinfo.tab35close2, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			poStateFunctions->Set_table_unit_TabMothersExperience(table_unit_TabMothersExperience+1);
			tableinfo.tab35close3 = (real) table_unit_TabMothersExperience;
			tableinfo.tab35open3 = (real) CloseUpdate( 35, 3, tableinfo.tab35open3, tableinfo.tab35close3, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open4 = (real) CloseUpdate( 35, 4, tableinfo.tab35open4, tableinfo.tab35close4, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open5 = (real) CloseUpdate( 35, 5, tableinfo.tab35open5, tableinfo.tab35close5, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open6 = (real) CloseUpdate( 35, 6, tableinfo.tab35open6, tableinfo.tab35close6, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open7 = (real) CloseUpdate( 35, 7, tableinfo.tab35open7, tableinfo.tab35close7, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open8 = (real) CloseUpdate( 35, 8, tableinfo.tab35open8, tableinfo.tab35close8, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open9 = (real) CloseUpdate( 35, 9, tableinfo.tab35open9, tableinfo.tab35close9, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
			tableinfo.tab35open10 = (real) CloseUpdate( 35, 10, tableinfo.tab35open10, tableinfo.tab35close10, tableinfo.tab35dim_filter, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		}
		tableinfo.tab35cell = (index) TableCellIndex( 35, 1,
			sim_yob - r_min_SIM_YEAR_RANGE );
		if ( gpModelApp->m_nSubSamples == 1 ) {
			tableinfo.tab35total_cell = (index) tableinfo.tab35cell;
		}
		else {
			tableinfo.tab35total_cell = (index) TableTotalCellIndex( 35, 1,
			sim_yob - r_min_SIM_YEAR_RANGE );
		}
		CloseUpdate( 35, 0, tableinfo.tab35open0, poDerivedStates->entrances_primary_level_PL_GRAD, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 1, tableinfo.tab35open1, infant_deaths, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 2, tableinfo.tab35open2, parity, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		poStateFunctions->Set_table_unit_TabMothersExperience(table_unit_TabMothersExperience+1);
		CloseUpdate( 35, 3, tableinfo.tab35open3, table_unit_TabMothersExperience, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 4, tableinfo.tab35open4, poDerivedStates->transitions_in_union_FALSE_TRUE, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 5, tableinfo.tab35open5, poDerivedStates->transitions_integer_age_14_15, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 6, tableinfo.tab35open6, poDerivedStates->transitions_integer_age_19_20, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 7, tableinfo.tab35open7, poDerivedStates->transitions_integer_age_9_10, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 8, tableinfo.tab35open8, poDerivedStates->transitions_parity_0_1, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 9, tableinfo.tab35open9, poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
		CloseUpdate( 35, 10, tableinfo.tab35open10, poDerivedStates->value_at_transitions_parity_0_1_age, table_filter_TabMothersExperience, tableinfo.tab35cell, tableinfo.tab35total_cell, actor_weight, actor_subsample_weight );
	}
	Set_lChildMortalityCalibrator( NULL );
	Set_lMother( NULL );
	mlChild->Finish();
	if (poasAllPersons != NULL)
	{	asAllPersons->Remove(poasAllPersons);
		poasAllPersons = NULL;
	}
	if (poasFemaleAgeEduc != NULL)
	{	asFemaleAgeEduc[RANGE_POS(AGE_RANGE,integer_age)][primary_level]->Remove(poasFemaleAgeEduc);
		poasFemaleAgeEduc = NULL;
	}
	if (poasForeignersAgeSexProv != NULL)
	{	asForeignersAgeSexProv[RANGE_POS(AGE_RANGE,integer_age)][sex][province_nat]->Remove(poasForeignersAgeSexProv);
		poasForeignersAgeSexProv = NULL;
	}
	if (poasForeignersPrimAgeSexProv != NULL)
	{	asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE,integer_age)][sex][province_nat]->Remove(poasForeignersPrimAgeSexProv);
		poasForeignersPrimAgeSexProv = NULL;
	}
	if (poasPotentialMothers != NULL)
	{	asPotentialMothers->Remove(poasPotentialMothers);
		poasPotentialMothers = NULL;
	}
	if (poasPotentialMothersByAge != NULL)
	{	asPotentialMothersByAge[RANGE_POS(FERTILE_AGE_RANGE,fertile_age)]->Remove(poasPotentialMothersByAge);
		poasPotentialMothersByAge = NULL;
	}
	if (poasPrimaryEntryCohort != NULL)
	{	asPrimaryEntryCohort[sex][province_birth][educ_mother]->Remove(poasPrimaryEntryCohort);
		poasPrimaryEntryCohort = NULL;
	}
	if (poasPrimaryGraduationCohort != NULL)
	{	asPrimaryGraduationCohort[sex][province_birth][educ_mother]->Remove(poasPrimaryGraduationCohort);
		poasPrimaryGraduationCohort = NULL;
	}
	if (poasResidentsAgeSex != NULL)
	{	asResidentsAgeSex[RANGE_POS(AGE_RANGE,integer_age)][sex]->Remove(poasResidentsAgeSex);
		poasResidentsAgeSex = NULL;
	}
	if (poasResidentsPrimAgeSex != NULL)
	{	asResidentsPrimAgeSex[RANGE_POS(AGE_RANGE,integer_age)][sex]->Remove(poasResidentsPrimAgeSex);
		poasResidentsPrimAgeSex = NULL;
	}

	// Remove actor from master list of active actors
	if ( poPrev != NULL )
	{
		poPrev->poNext = poNext;
	}
	else
	{
		if (gpoEventQueue->poHeadActor != NULL && gpoEventQueue->poHeadActor == this)
		{
			gpoEventQueue->poHeadActor = poNext;
		}
	}
	if ( poNext != NULL )
	{
		poNext->poPrev = poPrev;
	}
	poPrev = NULL;

	// Add actor to head of empty list of actors of this type
	poNext = gpoEmptyPerson;
	gpoEmptyPerson = this;
}

#pragma optimize( "", on )

void Person::WaitUntil( double target_time )
{
	//don't increase gdEventsForTabulation if special events because they are all treated as one event for tabulation purposes
	if (!gpoEventQueue->m_bSpecialEvents)
	{
		gdEventsForTabulation++;
	}
	__events = gdEventsForTabulation;
	if ( time < (TIME) target_time )
	{
		__time = (TIME) target_time;
		WaitSpecial( (TIME) target_time - time );
		poStateFunctions->Set_time( (TIME) target_time );
	}
}

Person *Person::UpdateTime(  )
{
	__events = gdEventsForTabulation;
	if ( time < (TIME) gdThreadCurrentTime ) {
		__time = (TIME) gdThreadCurrentTime;
		WaitSpecial( (TIME) gdThreadCurrentTime - time );
		poStateFunctions->Set_time( (TIME) gdThreadCurrentTime );
	}
	return this;
}

void Person::WaitUntilThisActor( double target_time )
{
	WaitUntil(target_time);
}

void Person::WaitSpecial( TIME wait_time )
{

	// increment the age
		poStateFunctions->Set_age( age + wait_time );
}

TIME Person::timeScheduleEvent()
{
	if (schedule_event_time == TIME_INFINITE)
	{
		TIME event_time = TIME_INFINITE;
		event_time = min( poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time, poDerivedStates->ssint_age_time );
		event_time = min( event_time, poDerivedStates->ssint_time_time );
		event_time = min( event_time, poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time );
		event_time = min( event_time, poDerivedStates->trigger_entrances_in_projected_time_TRUE_time );
		event_time = min( event_time, poDerivedStates->trigger_entrances_set_alive_TRUE_time );
		schedule_event_time = event_time;
	}
	return schedule_event_time;
}

void Person::ScheduleEvent()
{

	if (poDerivedStates->trigger_entrances_in_projected_time_TRUE_time == schedule_event_time)
	{
		//CHKSUM(time, 121);
		TRCEVNT(case_seed, _T("Person"), actor_id, _T("scheduled - 4"), time);
		gszRuntimeEventName = _T("Person.scheduled - 4");
		poDerivedStates->trigger_entrances_in_projected_time_TRUE_time = TIME_INFINITE;
		poDerivedStates->Set_trigger_entrances_in_projected_time_TRUE( FALSE );

	}

	if (poDerivedStates->trigger_entrances_set_alive_TRUE_time == schedule_event_time)
	{
		//CHKSUM(time, 122);
		TRCEVNT(case_seed, _T("Person"), actor_id, _T("scheduled - 5"), time);
		gszRuntimeEventName = _T("Person.scheduled - 5");
		poDerivedStates->trigger_entrances_set_alive_TRUE_time = TIME_INFINITE;
		poDerivedStates->Set_trigger_entrances_set_alive_TRUE( FALSE );

	}

	if (poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time == schedule_event_time)
	{
		//CHKSUM(time, 71);
		TRCEVNT(case_seed, _T("Person"), actor_id, _T("scheduled - 0"), time);
		gszRuntimeEventName = _T("Person.scheduled - 0");
		poDerivedStates->Set_ssint_active_spell_duration_in_this_parity_in_simulation_TRUE( poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE + 1 );
		poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time = WAIT( 1 );
	}

	if (poDerivedStates->ssint_age_time == schedule_event_time)
	{
		//CHKSUM(time, 72);
		TRCEVNT(case_seed, _T("Person"), actor_id, _T("scheduled - 1"), time);
		gszRuntimeEventName = _T("Person.scheduled - 1");
		poDerivedStates->Set_ssint_age( poDerivedStates->ssint_age + 1 );
		poDerivedStates->ssint_age_time = WAIT( 1 );
	}

	if (poDerivedStates->ssint_time_time == schedule_event_time)
	{
		//CHKSUM(time, 73);
		TRCEVNT(case_seed, _T("Person"), actor_id, _T("scheduled - 2"), time);
		gszRuntimeEventName = _T("Person.scheduled - 2");
		poDerivedStates->Set_ssint_time( poDerivedStates->ssint_time + 1 );
		poDerivedStates->ssint_time_time = WAIT( 1 );
	}

	if (poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time == schedule_event_time)
	{
		//CHKSUM(time, 74);
		TRCEVNT(case_seed, _T("Person"), actor_id, _T("scheduled - 3"), time);
		gszRuntimeEventName = _T("Person.scheduled - 3");
		poDerivedStates->Set_sssplit_duration_set_alive_TRUE_LITTLE_TIME( poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME + 1 );
		if ( poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME < 1 ) {
		poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = WAIT(TIME(POINTS(LITTLE_TIME)[poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME] - POINTS(LITTLE_TIME)[poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME - 1]));
		}
		else
		{		poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = TIME_INFINITE;
		}
	}
	schedule_event_time = TIME_INFINITE;
}

double Person::EventTime( int nEventNum, int *pnEventInfo )
{
	double	event_time = double( TIME_INFINITE );

	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum ) {
		case 1:
			gszRuntimeEventName = _T("Person.BirthEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeBirthEvent() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeBirthEvent"), event_time);
			break;
		case 3:
			gszRuntimeEventName = _T("Person.ChildMortalityEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeChildMortalityEvent() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeChildMortalityEvent"), event_time);
			break;
		case 4:
			gszRuntimeEventName = _T("Person.EmigrationEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeEmigrationEvent() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeEmigrationEvent"), event_time);
			break;
		case 5:
			gszRuntimeEventName = _T("Person.MigrationEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeMigrationEvent() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeMigrationEvent"), event_time);
			break;
		case 6:
			gszRuntimeEventName = _T("Person.MortalityEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeMortalityEvent() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeMortalityEvent"), event_time);
			break;
		case 9:
			gszRuntimeEventName = _T("Person.SetAliveEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeSetAliveEvent() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeSetAliveEvent"), event_time);
			break;
		case 10:
			gszRuntimeEventName = _T("Person.UnionFormationEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeUnionFormationEvent() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeUnionFormationEvent"), event_time);
			break;
		case 11:
			gszRuntimeEventName = _T("Person.WriteMicroRecord(time)");
			event_time = CoarsenMantissa( (TIME) timeWriteMicroRecord() );
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("timeWriteMicroRecord"), event_time);
			break;
		case 8:
			gszRuntimeEventName = _T("ScheduleEvent(time)");
			event_time = (double) timeScheduleEvent();
			break;
		case -1:
		default:;
	}
	return (TIME) event_time;
}

void Person::Implement( int nEventNum, int nEventInfo )
{
	gdThreadEvents += 1;
	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum )
	{
		case 1:
			CHKSUM(time, 0);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.BirthEvent"), time);
			gszRuntimeEventName = _T("Person.BirthEvent(implement)");
			BirthEvent();
			break;
		case 3:
			CHKSUM(time, 1);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.ChildMortalityEvent"), time);
			gszRuntimeEventName = _T("Person.ChildMortalityEvent(implement)");
			ChildMortalityEvent();
			break;
		case 4:
			CHKSUM(time, 2);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.EmigrationEvent"), time);
			gszRuntimeEventName = _T("Person.EmigrationEvent(implement)");
			EmigrationEvent();
			break;
		case 5:
			CHKSUM(time, 3);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.MigrationEvent"), time);
			gszRuntimeEventName = _T("Person.MigrationEvent(implement)");
			MigrationEvent();
			break;
		case 6:
			CHKSUM(time, 4);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.MortalityEvent"), time);
			gszRuntimeEventName = _T("Person.MortalityEvent(implement)");
			MortalityEvent();
			break;
		case 9:
			CHKSUM(time, 5);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.SetAliveEvent"), time);
			gszRuntimeEventName = _T("Person.SetAliveEvent(implement)");
			SetAliveEvent();
			break;
		case 10:
			CHKSUM(time, 6);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.UnionFormationEvent"), time);
			gszRuntimeEventName = _T("Person.UnionFormationEvent(implement)");
			UnionFormationEvent();
			break;
		case 11:
			CHKSUM(time, 7);
			TRCEVNT(case_seed, _T("Person"), actor_id, _T("Person.WriteMicroRecord"), time);
			gszRuntimeEventName = _T("Person.WriteMicroRecord(implement)");
			WriteMicroRecord();
			break;
		case 8:
			//CHKSUM(time, 8);
			ScheduleEvent();
			break;
		case -1:
		default:;
	}
}

double Person::Set_actor_subsample_weight( double value )
{
	return poStateFunctions->Set_actor_subsample_weight( value);
}

double Person::Set_actor_weight( double value )
{
	return poStateFunctions->Set_actor_weight( value);
}

TIME PersonStateFunctions::Set_age( TIME value, bool bStartFunction )
{

	if ( poParent->age != value || bStartFunction )
	{
		if (bStartFunction)
		{
			poDerivedStates->Set_ssint_age( std::floor(value) );
			poDerivedStates->ssint_age_time = poParent->time + (1.0 - (value - poDerivedStates->ssint_age));
			if (poParent->schedule_event_time > poDerivedStates->ssint_age_time)
			{
				poParent->schedule_event_time = poDerivedStates->ssint_age_time;
			}
		}


		TIME	OldValue;
		OldValue = poParent->age;
		poParent->age = value;

	}
	return poParent->age = value;
}

integer PersonStateFunctions::Set_age_at_last_move( integer value )
{

	if ( poParent->age_at_last_move != value )
	{


		integer	OldValue;
		OldValue = poParent->age_at_last_move;
		poParent->age_at_last_move = value;

		if ( poParent->poMigrationEvent->cRecalc == 0 ) {
			poParent->poMigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMigrationEvent );
		};
	}
	return poParent->age_at_last_move = value;
}

logical PersonStateFunctions::Set_alive( logical value )
{

	value = value != 0;
	if ( poParent->alive != value )
	{
		if ( poParent->alive == TRUE && value == FALSE ) {
			poDerivedStates->Set_transitions_alive_TRUE_FALSE( poDerivedStates->transitions_alive_TRUE_FALSE + 1 );
		}


		logical	OldValue;
		OldValue = poParent->alive;
		poParent->alive = value;

	}
	return poParent->alive = value;
}

PRIMARY_LEVEL PersonStateFunctions::Set_educ_mother( PRIMARY_LEVEL value )
{

	CHKLMT(value, educ_mother, 0, rc_size_PRIMARY_LEVEL - 1);
	if ( value < 0 ) {
		value = (PRIMARY_LEVEL) 0;
	}
	else if ( value >= rc_size_PRIMARY_LEVEL ) {
		value = (PRIMARY_LEVEL) ( rc_size_PRIMARY_LEVEL - 1 );
	}
	if ( poParent->educ_mother != value )
	{


		PRIMARY_LEVEL	OldValue;
		OldValue = poParent->educ_mother;
		poParent->educ_mother = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[5] >= 0 && ( poParent->tableinfo.tab5dim_filter || poParent->table_filter_RTAB_BirthsByEducMother ) ) {
				if ( poParent->tableinfo.tab5events == -1 || poParent->tableinfo.tab5events < poParent->__events ) {
					if ( poParent->tableinfo.tab5events != -1 ) {
						poParent->tableinfo.tab5open0 = (real) poParent->CloseUpdate( 5, 0, poParent->tableinfo.tab5open0, poParent->tableinfo.tab5close0, poParent->tableinfo.tab5dim_filter, poParent->tableinfo.tab5cell, poParent->tableinfo.tab5total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab5events = poParent->__events;
					poParent->tableinfo.tab5close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab5cell = (index) poParent->TableCellIndex( 5, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab5total_cell = (index) poParent->tableinfo.tab5cell;
					}
					else {
						poParent->tableinfo.tab5total_cell = (index) poParent->TableTotalCellIndex( 5, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab5dim_filter = poParent->table_filter_RTAB_BirthsByEducMother;
				}
			}
		}
		// actorset dimension or order state
		if (poParent->actorset_filter_asPrimaryEntryCohort)
		{
			asPrimaryEntryCohort[poParent->sex][poParent->province_birth][OldValue]->Remove(poParent->poasPrimaryEntryCohort);
			poParent->poasPrimaryEntryCohort = asPrimaryEntryCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asPrimaryGraduationCohort)
		{
			asPrimaryGraduationCohort[poParent->sex][poParent->province_birth][OldValue]->Remove(poParent->poasPrimaryGraduationCohort);
			poParent->poasPrimaryGraduationCohort = asPrimaryGraduationCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
	}
	return poParent->educ_mother = value;
}

logical PersonStateFunctions::Set_has_emigrated( logical value )
{

	value = value != 0;
	if ( poParent->has_emigrated != value )
	{
		if ( poParent->has_emigrated == FALSE && value == TRUE ) {
			poDerivedStates->Set_transitions_has_emigrated_FALSE_TRUE( poDerivedStates->transitions_has_emigrated_FALSE_TRUE + 1 );
		}


		logical	OldValue;
		OldValue = poParent->has_emigrated;
		poParent->has_emigrated = value;

	}
	return poParent->has_emigrated = value;
}

logical PersonStateFunctions::Set_in_this_parity( logical value )
{

	value = value != 0;
	if ( poParent->in_this_parity != value )
	{


		logical	OldValue;
		OldValue = poParent->in_this_parity;
		poParent->in_this_parity = value;

#line 138 "code/Fertility.mpp"
		Set_in_this_parity_in_simulation( poParent->in_simulation_time&& poParent->in_this_parity );
#line 3844 "modgen/src/ACTORS.CPP"
	}
	return poParent->in_this_parity = value;
}

logical PersonStateFunctions::Set_in_union( logical value )
{

	value = value != 0;
	if ( poParent->in_union != value )
	{
		if ( poParent->in_union == FALSE && value == TRUE ) {
			poDerivedStates->Set_transitions_in_union_FALSE_TRUE( poDerivedStates->transitions_in_union_FALSE_TRUE + 1 );
		}
		if ( poParent->in_union == FALSE && value == TRUE ) {
			poDerivedStates->Set_value_at_transitions_in_union_FALSE_TRUE_age( poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age + poParent->age );
		}
		if ( poParent->in_union == FALSE ) {
			poDerivedStates->Set_duration_in_union_FALSE( poDerivedStates->Get_duration_in_union_FALSE() );
		}
		else if ( value == FALSE ) {
			poDerivedStates->duration_in_union_FALSE_time = poParent->__time;
		}
		if ( poParent->in_union == TRUE ) {
			poDerivedStates->Set_duration_in_union_TRUE( poDerivedStates->Get_duration_in_union_TRUE() );
		}
		else if ( value == TRUE ) {
			poDerivedStates->duration_in_union_TRUE_time = poParent->__time;
		}


		logical	OldValue;
		OldValue = poParent->in_union;
		poParent->in_union = value;

#line 126 "code/Fertility.mpp"
		Set_union_status( ( poParent->in_union )? US_YES: US_NO );
#line 3881 "modgen/src/ACTORS.CPP"
		if ( poParent->poUnionFormationEvent->cRecalc == 0 ) {
			poParent->poUnionFormationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poUnionFormationEvent );
		};
	}
	return poParent->in_union = value;
}

PARITY_RANGE PersonStateFunctions::Set_infant_deaths( PARITY_RANGE value )
{

	CHKLMT(value, infant_deaths, r_min_PARITY_RANGE, r_max_PARITY_RANGE);
	if ( value < r_min_PARITY_RANGE ) {
		value = r_min_PARITY_RANGE;
	}
	else if ( value > r_max_PARITY_RANGE ) {
		value = r_max_PARITY_RANGE;
	}
	if ( poParent->infant_deaths != value )
	{


		PARITY_RANGE	OldValue;
		OldValue = poParent->infant_deaths;
		poParent->infant_deaths = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[35] >= 0 && poParent->tableinfo.tab35events == poParent->__events ) {
				poParent->tableinfo.tab35close1 = (real) poParent->infant_deaths;
			}
		}
	}
	return poParent->infant_deaths = value;
}

logical PersonStateFunctions::Set_mother_educ_0( logical value )
{

	value = value != 0;
	if ( poParent->mother_educ_0 != value )
	{


		logical	OldValue;
		OldValue = poParent->mother_educ_0;
		poParent->mother_educ_0 = value;

		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
	}
	return poParent->mother_educ_0 = value;
}

logical PersonStateFunctions::Set_mother_educ_1( logical value )
{

	value = value != 0;
	if ( poParent->mother_educ_1 != value )
	{


		logical	OldValue;
		OldValue = poParent->mother_educ_1;
		poParent->mother_educ_1 = value;

		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
	}
	return poParent->mother_educ_1 = value;
}

logical PersonStateFunctions::Set_mother_known( logical value )
{

	value = value != 0;
	if ( poParent->mother_known != value )
	{


		logical	OldValue;
		OldValue = poParent->mother_known;
		poParent->mother_known = value;

#line 277 "code/ChildMortality.mpp"
		Set_child_mortality_overrides( poParent->calendar_year>= MIN(CHILD_MORTALITY_YEARS)&& gprParam->SelectedChildMortalityModel!= SCMM_MACRO&& poParent->integer_age<= MAX(CHILD_MORTALITY_AGE)&& poParent->mother_known );
#line 3971 "modgen/src/ACTORS.CPP"
	}
	return poParent->mother_known = value;
}

logical PersonStateFunctions::Set_mother_max_14( logical value )
{

	value = value != 0;
	if ( poParent->mother_max_14 != value )
	{


		logical	OldValue;
		OldValue = poParent->mother_max_14;
		poParent->mother_max_14 = value;

		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
	}
	return poParent->mother_max_14 = value;
}

logical PersonStateFunctions::Set_mother_max_16( logical value )
{

	value = value != 0;
	if ( poParent->mother_max_16 != value )
	{


		logical	OldValue;
		OldValue = poParent->mother_max_16;
		poParent->mother_max_16 = value;

		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
	}
	return poParent->mother_max_16 = value;
}

integer PersonStateFunctions::Set_number_moves( integer value )
{

	if ( poParent->number_moves != value )
	{


		integer	OldValue;
		OldValue = poParent->number_moves;
		poParent->number_moves = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[24] >= 0 && poParent->tableinfo.tab24events == poParent->__events ) {
				poParent->tableinfo.tab24close1 = (real) poParent->number_moves;
			}
		}
	}
	return poParent->number_moves = value;
}

logical PersonStateFunctions::Set_override_mode( logical value )
{

	value = value != 0;
	if ( poParent->override_mode != value )
	{


		logical	OldValue;
		OldValue = poParent->override_mode;
		poParent->override_mode = value;

		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
	}
	return poParent->override_mode = value;
}

PARITY_RANGE PersonStateFunctions::Set_parity( PARITY_RANGE value )
{

	CHKLMT(value, parity, r_min_PARITY_RANGE, r_max_PARITY_RANGE);
	if ( value < r_min_PARITY_RANGE ) {
		value = r_min_PARITY_RANGE;
	}
	else if ( value > r_max_PARITY_RANGE ) {
		value = r_max_PARITY_RANGE;
	}
	if ( poParent->parity != value )
	{
		poDerivedStates->Set_changes_parity( poDerivedStates->changes_parity + 1 );
		if ( poParent->parity == 0 && value == 1 ) {
			poDerivedStates->Set_transitions_parity_0_1( poDerivedStates->transitions_parity_0_1 + 1 );
		}
		poDerivedStates->Set_value_at_changes_parity_age( poDerivedStates->value_at_changes_parity_age + poParent->age );
		if ( poParent->parity == 0 && value == 1 ) {
			poDerivedStates->Set_value_at_transitions_parity_0_1_age( poDerivedStates->value_at_transitions_parity_0_1_age + poParent->age );
		}


		PARITY_RANGE	OldValue;
		OldValue = poParent->parity;
		poParent->parity = value;

#line 155 "code/Fertility.mpp"
		Set_is_potential_mother( ( poParent->set_alive&& poParent->sex== FEMALE&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& poParent->parity< MAX(PARITY_RANGE)&& WITHIN(SIM_YEAR_RANGE,poParent->calendar_year) )? TRUE: FALSE );
#line 4084 "modgen/src/ACTORS.CPP"
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[6] >= 0 && poParent->tableinfo.tab6events == poParent->__events ) {
				poParent->tableinfo.tab6close0 = (real) poParent->parity;
			}
			if ( gplTableIndexes[16] >= 0 && poParent->tableinfo.tab16events == poParent->__events ) {
				poParent->tableinfo.tab16close1 = (real) poParent->parity;
			}
			if ( gplTableIndexes[18] >= 0 && poParent->tableinfo.tab18events == poParent->__events ) {
				poParent->tableinfo.tab18close0 = (real) poParent->parity;
			}
			if ( gplTableIndexes[35] >= 0 && poParent->tableinfo.tab35events == poParent->__events ) {
				poParent->tableinfo.tab35close2 = (real) poParent->parity;
			}
		}
		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
	}
	return poParent->parity = value;
}

PERSON_TYPE PersonStateFunctions::Set_person_type( PERSON_TYPE value )
{

	CHKLMT(value, person_type, 0, rc_size_PERSON_TYPE - 1);
	if ( value < 0 ) {
		value = (PERSON_TYPE) 0;
	}
	else if ( value >= rc_size_PERSON_TYPE ) {
		value = (PERSON_TYPE) ( rc_size_PERSON_TYPE - 1 );
	}
	if ( poParent->person_type != value )
	{
#line 116 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_BirthsByEducMother( poParent->in_rtab_years&& value== PT_CHILD );
#line 4121 "modgen/src/ACTORS.CPP"
#line 106 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_BirthsByRegion( poParent->in_rtab_years&& value== PT_CHILD );
#line 4124 "modgen/src/ACTORS.CPP"
#line 255 "code/Tables.mpp"
		Set_table_filter_TabImmigrationSexProvinceAge( value== PT_IMMIGRANT&& poParent->in_projected_time&& poDerivedStates->trigger_entrances_set_alive_TRUE );
#line 4127 "modgen/src/ACTORS.CPP"
#line 405 "code/Tables.mpp"
		Set_table_filter_TabMothersExperience( poParent->in_projected_time&& WITHIN(SIM_YOB_RANGE,poParent->year_of_birth)&& value== PT_CHILD&& poParent->sex== FEMALE );
#line 4130 "modgen/src/ACTORS.CPP"
#line 162 "code/Tables.mpp"
		Set_table_filter_TabNumberBirths( poParent->in_projected_time&& value== PT_CHILD );
#line 4133 "modgen/src/ACTORS.CPP"


		PERSON_TYPE	OldValue;
		OldValue = poParent->person_type;
		poParent->person_type = value;

	}
	return poParent->person_type = value;
}

PRIMARY_LEVEL PersonStateFunctions::Set_primary_level( PRIMARY_LEVEL value )
{

	CHKLMT(value, primary_level, 0, rc_size_PRIMARY_LEVEL - 1);
	if ( value < 0 ) {
		value = (PRIMARY_LEVEL) 0;
	}
	else if ( value >= rc_size_PRIMARY_LEVEL ) {
		value = (PRIMARY_LEVEL) ( rc_size_PRIMARY_LEVEL - 1 );
	}
	if ( poParent->primary_level != value )
	{
#line 35 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asForeignersPrimAgeSexProv( poParent->set_alive&& poParent->province_birth== PI_PROV13&& value!= PL_NO );
#line 4158 "modgen/src/ACTORS.CPP"
#line 23 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asPrimaryGraduationCohort( poParent->set_alive&& poParent->integer_age== gprParam->AgeGradPrimary&& value!= PL_NO );
#line 4161 "modgen/src/ACTORS.CPP"
#line 38 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asResidentsPrimAgeSex( poParent->set_alive&& value!= PL_NO );
#line 4164 "modgen/src/ACTORS.CPP"
		if ( value == PL_GRAD ) {
			poDerivedStates->Set_entrances_primary_level_PL_GRAD( poDerivedStates->entrances_primary_level_PL_GRAD + 1 );
		}
		if ( poParent->primary_level == PL_ENTER && value == PL_GRAD ) {
			poDerivedStates->Set_transitions_primary_level_PL_ENTER_PL_GRAD( poDerivedStates->transitions_primary_level_PL_ENTER_PL_GRAD + 1 );
		}
		if ( poParent->primary_level == PL_NO && value == PL_ENTER ) {
			poDerivedStates->Set_transitions_primary_level_PL_NO_PL_ENTER( poDerivedStates->transitions_primary_level_PL_NO_PL_ENTER + 1 );
		}
		if ( poParent->primary_level == PL_ENTER ) {
			poDerivedStates->Set_duration_primary_level_PL_ENTER( poDerivedStates->Get_duration_primary_level_PL_ENTER() );
		}
		else if ( value == PL_ENTER ) {
			poDerivedStates->duration_primary_level_PL_ENTER_time = poParent->__time;
		}
		if ( poParent->primary_level == PL_GRAD ) {
			poDerivedStates->Set_duration_primary_level_PL_GRAD( poDerivedStates->Get_duration_primary_level_PL_GRAD() );
		}
		else if ( value == PL_GRAD ) {
			poDerivedStates->duration_primary_level_PL_GRAD_time = poParent->__time;
		}
		if ( poParent->primary_level == PL_NO ) {
			poDerivedStates->Set_duration_primary_level_PL_NO( poDerivedStates->Get_duration_primary_level_PL_NO() );
		}
		else if ( value == PL_NO ) {
			poDerivedStates->duration_primary_level_PL_NO_time = poParent->__time;
		}


		PRIMARY_LEVEL	OldValue;
		OldValue = poParent->primary_level;
		poParent->primary_level = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[19] >= 0 && ( poParent->tableinfo.tab19dim_filter || poParent->table_filter_TabAgeAtBirth ) ) {
				if ( poParent->tableinfo.tab19events == -1 || poParent->tableinfo.tab19events < poParent->__events ) {
					if ( poParent->tableinfo.tab19events != -1 ) {
						poParent->tableinfo.tab19open0 = (real) poParent->CloseUpdate( 19, 0, poParent->tableinfo.tab19open0, poParent->tableinfo.tab19close0, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open1 = (real) poParent->CloseUpdate( 19, 1, poParent->tableinfo.tab19open1, poParent->tableinfo.tab19close1, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open2 = (real) poParent->CloseUpdate( 19, 2, poParent->tableinfo.tab19open2, poParent->tableinfo.tab19close2, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open3 = (real) poParent->CloseUpdate( 19, 3, poParent->tableinfo.tab19open3, poParent->tableinfo.tab19close3, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab19events = poParent->__events;
					poParent->tableinfo.tab19close0 = (real) poDerivedStates->changes_parity;
					poParent->tableinfo.tab19close1 = (real) poDerivedStates->transitions_parity_0_1;
					poParent->tableinfo.tab19close2 = (real) poDerivedStates->value_at_changes_parity_age;
					poParent->tableinfo.tab19close3 = (real) poDerivedStates->value_at_transitions_parity_0_1_age;
					poParent->tableinfo.tab19cell = (index) poParent->TableCellIndex( 19, 2,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab19total_cell = (index) poParent->tableinfo.tab19cell;
					}
					else {
						poParent->tableinfo.tab19total_cell = (index) poParent->TableTotalCellIndex( 19, 2,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab19dim_filter = poParent->table_filter_TabAgeAtBirth;
				}
			}
			if ( gplTableIndexes[33] >= 0 && ( poParent->tableinfo.tab33dim_filter || poParent->table_filter_TabUnionFormation ) ) {
				if ( poParent->tableinfo.tab33events == -1 || poParent->tableinfo.tab33events < poParent->__events ) {
					if ( poParent->tableinfo.tab33events != -1 ) {
						poParent->tableinfo.tab33open0 = (real) poParent->CloseUpdate( 33, 0, poParent->tableinfo.tab33open0, poParent->tableinfo.tab33close0, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open1 = (real) poParent->CloseUpdate( 33, 1, poParent->tableinfo.tab33open1, poParent->tableinfo.tab33close1, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open2 = (real) poParent->CloseUpdate( 33, 2, poParent->tableinfo.tab33open2, poParent->tableinfo.tab33close2, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open3 = (real) poParent->CloseUpdate( 33, 3, poParent->tableinfo.tab33open3, poParent->tableinfo.tab33close3, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open4 = (real) poParent->CloseUpdate( 33, 4, poParent->tableinfo.tab33open4, poParent->tableinfo.tab33close4, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab33events = poParent->__events;
					poParent->tableinfo.tab33close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab33close1 = (real) poDerivedStates->Get_duration_in_union_FALSE();
					poParent->tableinfo.tab33close2 = (real) poDerivedStates->Get_duration_in_union_TRUE();
					poParent->tableinfo.tab33close3 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab33close4 = (real) poParent->union_formation_hazard_max_value_in;
					poParent->tableinfo.tab33cell = (index) poParent->TableCellIndex( 33, 3,
						OldValue,
						poParent->age_union - r_min_AGE_UNION,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab33total_cell = (index) poParent->tableinfo.tab33cell;
					}
					else {
						poParent->tableinfo.tab33total_cell = (index) poParent->TableTotalCellIndex( 33, 3,
						OldValue,
						poParent->age_union - r_min_AGE_UNION,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab33dim_filter = poParent->table_filter_TabUnionFormation;
				}
			}
			if ( gplTableIndexes[34] >= 0 && ( poParent->tableinfo.tab34dim_filter || poParent->table_filter_TabAgeAtFirstUnionFormation ) ) {
				if ( poParent->tableinfo.tab34events == -1 || poParent->tableinfo.tab34events < poParent->__events ) {
					if ( poParent->tableinfo.tab34events != -1 ) {
						poParent->tableinfo.tab34open0 = (real) poParent->CloseUpdate( 34, 0, poParent->tableinfo.tab34open0, poParent->tableinfo.tab34close0, poParent->tableinfo.tab34dim_filter, poParent->tableinfo.tab34cell, poParent->tableinfo.tab34total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab34open1 = (real) poParent->CloseUpdate( 34, 1, poParent->tableinfo.tab34open1, poParent->tableinfo.tab34close1, poParent->tableinfo.tab34dim_filter, poParent->tableinfo.tab34cell, poParent->tableinfo.tab34total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab34events = poParent->__events;
					poParent->tableinfo.tab34close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab34close1 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab34cell = (index) poParent->TableCellIndex( 34, 2,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab34total_cell = (index) poParent->tableinfo.tab34cell;
					}
					else {
						poParent->tableinfo.tab34total_cell = (index) poParent->TableTotalCellIndex( 34, 2,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab34dim_filter = poParent->table_filter_TabAgeAtFirstUnionFormation;
				}
			}
		}
#line 82 "code/Tables.mpp"
		Set_is_aded0( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_NO ) );
#line 4283 "modgen/src/ACTORS.CPP"
#line 83 "code/Tables.mpp"
		Set_is_aded1( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_ENTER ) );
#line 4286 "modgen/src/ACTORS.CPP"
#line 84 "code/Tables.mpp"
		Set_is_aded2( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_GRAD ) );
#line 4289 "modgen/src/ACTORS.CPP"
#line 65 "code/UnionFormation.mpp"
		Set_union_formation_hazard( ( WITHIN(AGE_UNION,poParent->integer_age)&& poParent->sex== FEMALE )? gprParam->UnionFormationHazard[CHKBND(poParent->primary_level, 3, "UnionFormationHazard", 0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE,poParent->calendar_year), 101, "UnionFormationHazard", 1)][CHKBND(RANGE_POS(AGE_UNION,poParent->integer_age), 51, "UnionFormationHazard", 2)]: 0.0 );
#line 4292 "modgen/src/ACTORS.CPP"
		// actorset dimension or order state
		if (poParent->actorset_filter_asFemaleAgeEduc)
		{
			asFemaleAgeEduc[RANGE_POS(AGE_RANGE, poParent->integer_age)][OldValue]->Remove(poParent->poasFemaleAgeEduc);
			poParent->poasFemaleAgeEduc = asFemaleAgeEduc[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->primary_level]->NewNode(poParent);
		}
		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
		if ( poParent->poMigrationEvent->cRecalc == 0 ) {
			poParent->poMigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMigrationEvent );
		};
	}
	return poParent->primary_level = value;
}

PROVINCE_INT PersonStateFunctions::Set_province_birth( PROVINCE_INT value )
{

	CHKLMT(value, province_birth, 0, rc_size_PROVINCE_INT - 1);
	if ( value < 0 ) {
		value = (PROVINCE_INT) 0;
	}
	else if ( value >= rc_size_PROVINCE_INT ) {
		value = (PROVINCE_INT) ( rc_size_PROVINCE_INT - 1 );
	}
	if ( poParent->province_birth != value )
	{
#line 29 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asForeignersAgeSexProv( poParent->set_alive&& value== PI_PROV13 );
#line 4325 "modgen/src/ACTORS.CPP"
#line 35 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asForeignersPrimAgeSexProv( poParent->set_alive&& value== PI_PROV13&& poParent->primary_level!= PL_NO );
#line 4328 "modgen/src/ACTORS.CPP"


		PROVINCE_INT	OldValue;
		OldValue = poParent->province_birth;
		poParent->province_birth = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[28] >= 0 && ( poParent->tableinfo.tab28dim_filter || poParent->table_filter_TabEduc12ByPob ) ) {
				if ( poParent->tableinfo.tab28events == -1 || poParent->tableinfo.tab28events < poParent->__events ) {
					if ( poParent->tableinfo.tab28events != -1 ) {
						poParent->tableinfo.tab28open0 = (real) poParent->CloseUpdate( 28, 0, poParent->tableinfo.tab28open0, poParent->tableinfo.tab28close0, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open1 = (real) poParent->CloseUpdate( 28, 1, poParent->tableinfo.tab28open1, poParent->tableinfo.tab28close1, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open2 = (real) poParent->CloseUpdate( 28, 2, poParent->tableinfo.tab28open2, poParent->tableinfo.tab28close2, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open3 = (real) poParent->CloseUpdate( 28, 3, poParent->tableinfo.tab28open3, poParent->tableinfo.tab28close3, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab28events = poParent->__events;
					poParent->tableinfo.tab28close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab28close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab28close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab28close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab28cell = (index) poParent->TableCellIndex( 28, 3,
						OldValue,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab28total_cell = (index) poParent->tableinfo.tab28cell;
					}
					else {
						poParent->tableinfo.tab28total_cell = (index) poParent->TableTotalCellIndex( 28, 3,
						OldValue,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab28dim_filter = poParent->table_filter_TabEduc12ByPob;
				}
			}
		}
		// actorset dimension or order state
		if (poParent->actorset_filter_asPrimaryEntryCohort)
		{
			asPrimaryEntryCohort[poParent->sex][OldValue][poParent->educ_mother]->Remove(poParent->poasPrimaryEntryCohort);
			poParent->poasPrimaryEntryCohort = asPrimaryEntryCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asPrimaryGraduationCohort)
		{
			asPrimaryGraduationCohort[poParent->sex][OldValue][poParent->educ_mother]->Remove(poParent->poasPrimaryGraduationCohort);
			poParent->poasPrimaryGraduationCohort = asPrimaryGraduationCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
	}
	return poParent->province_birth = value;
}

PROVINCE_NAT PersonStateFunctions::Set_province_nat( PROVINCE_NAT value )
{

	CHKLMT(value, province_nat, 0, rc_size_PROVINCE_NAT - 1);
	if ( value < 0 ) {
		value = (PROVINCE_NAT) 0;
	}
	else if ( value >= rc_size_PROVINCE_NAT ) {
		value = (PROVINCE_NAT) ( rc_size_PROVINCE_NAT - 1 );
	}
	if ( poParent->province_nat != value )
	{


		PROVINCE_NAT	OldValue;
		OldValue = poParent->province_nat;
		poParent->province_nat = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[1] >= 0 && ( poParent->tableinfo.tab1dim_filter || poParent->table_filter_RTAB_TotalPopWillaya ) ) {
				if ( poParent->tableinfo.tab1events == -1 || poParent->tableinfo.tab1events < poParent->__events ) {
					if ( poParent->tableinfo.tab1events != -1 ) {
						poParent->tableinfo.tab1open0 = (real) poParent->CloseUpdate( 1, 0, poParent->tableinfo.tab1open0, poParent->tableinfo.tab1close0, poParent->tableinfo.tab1dim_filter, poParent->tableinfo.tab1cell, poParent->tableinfo.tab1total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab1events = poParent->__events;
					poParent->tableinfo.tab1close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab1cell = (index) poParent->TableCellIndex( 1, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab1total_cell = (index) poParent->tableinfo.tab1cell;
					}
					else {
						poParent->tableinfo.tab1total_cell = (index) poParent->TableTotalCellIndex( 1, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab1dim_filter = poParent->table_filter_RTAB_TotalPopWillaya;
				}
			}
			if ( gplTableIndexes[11] >= 0 && ( poParent->tableinfo.tab11dim_filter || poParent->table_filter_RTAB_PrimSchoolEntries ) ) {
				if ( poParent->tableinfo.tab11events == -1 || poParent->tableinfo.tab11events < poParent->__events ) {
					if ( poParent->tableinfo.tab11events != -1 ) {
						poParent->tableinfo.tab11open0 = (real) poParent->CloseUpdate( 11, 0, poParent->tableinfo.tab11open0, poParent->tableinfo.tab11close0, poParent->tableinfo.tab11dim_filter, poParent->tableinfo.tab11cell, poParent->tableinfo.tab11total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab11events = poParent->__events;
					poParent->tableinfo.tab11close0 = (real) poDerivedStates->transitions_primary_level_PL_NO_PL_ENTER;
					poParent->tableinfo.tab11cell = (index) poParent->TableCellIndex( 11, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab11total_cell = (index) poParent->tableinfo.tab11cell;
					}
					else {
						poParent->tableinfo.tab11total_cell = (index) poParent->TableTotalCellIndex( 11, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab11dim_filter = poParent->table_filter_RTAB_PrimSchoolEntries;
				}
			}
			if ( gplTableIndexes[12] >= 0 && ( poParent->tableinfo.tab12dim_filter || poParent->table_filter_RTAB_PrimSchoolGraduations ) ) {
				if ( poParent->tableinfo.tab12events == -1 || poParent->tableinfo.tab12events < poParent->__events ) {
					if ( poParent->tableinfo.tab12events != -1 ) {
						poParent->tableinfo.tab12open0 = (real) poParent->CloseUpdate( 12, 0, poParent->tableinfo.tab12open0, poParent->tableinfo.tab12close0, poParent->tableinfo.tab12dim_filter, poParent->tableinfo.tab12cell, poParent->tableinfo.tab12total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab12events = poParent->__events;
					poParent->tableinfo.tab12close0 = (real) poDerivedStates->transitions_primary_level_PL_ENTER_PL_GRAD;
					poParent->tableinfo.tab12cell = (index) poParent->TableCellIndex( 12, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab12total_cell = (index) poParent->tableinfo.tab12cell;
					}
					else {
						poParent->tableinfo.tab12total_cell = (index) poParent->TableTotalCellIndex( 12, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab12dim_filter = poParent->table_filter_RTAB_PrimSchoolGraduations;
				}
			}
			if ( gplTableIndexes[13] >= 0 && ( poParent->tableinfo.tab13dim_filter || poParent->table_filter_TabTotalPopulationSexProvince ) ) {
				if ( poParent->tableinfo.tab13events == -1 || poParent->tableinfo.tab13events < poParent->__events ) {
					if ( poParent->tableinfo.tab13events != -1 ) {
						poParent->tableinfo.tab13open0 = (real) poParent->CloseUpdate( 13, 0, poParent->tableinfo.tab13open0, poParent->tableinfo.tab13close0, poParent->tableinfo.tab13dim_filter, poParent->tableinfo.tab13cell, poParent->tableinfo.tab13total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab13events = poParent->__events;
					poParent->tableinfo.tab13close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab13cell = (index) poParent->TableCellIndex( 13, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab13total_cell = (index) poParent->tableinfo.tab13cell;
					}
					else {
						poParent->tableinfo.tab13total_cell = (index) poParent->TableTotalCellIndex( 13, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue );
					}
					poParent->tableinfo.tab13dim_filter = poParent->table_filter_TabTotalPopulationSexProvince;
				}
			}
			if ( gplTableIndexes[14] >= 0 && ( poParent->tableinfo.tab14dim_filter || poParent->table_filter_TabPopulationAgeSexProvince ) ) {
				if ( poParent->tableinfo.tab14events == -1 || poParent->tableinfo.tab14events < poParent->__events ) {
					if ( poParent->tableinfo.tab14events != -1 ) {
						poParent->tableinfo.tab14open0 = (real) poParent->CloseUpdate( 14, 0, poParent->tableinfo.tab14open0, poParent->tableinfo.tab14close0, poParent->tableinfo.tab14dim_filter, poParent->tableinfo.tab14cell, poParent->tableinfo.tab14total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab14events = poParent->__events;
					poParent->tableinfo.tab14close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab14cell = (index) poParent->TableCellIndex( 14, 4,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab14total_cell = (index) poParent->tableinfo.tab14cell;
					}
					else {
						poParent->tableinfo.tab14total_cell = (index) poParent->TableTotalCellIndex( 14, 4,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab14dim_filter = poParent->table_filter_TabPopulationAgeSexProvince;
				}
			}
			if ( gplTableIndexes[15] >= 0 && ( poParent->tableinfo.tab15dim_filter || poParent->table_filter_TabSimulatedStartingPopulation ) ) {
				if ( poParent->tableinfo.tab15events == -1 || poParent->tableinfo.tab15events < poParent->__events ) {
					if ( poParent->tableinfo.tab15events != -1 ) {
						Set_table_unit_TabSimulatedStartingPopulation(poParent->table_unit_TabSimulatedStartingPopulation+1);
						poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
						poParent->tableinfo.tab15open0 = (real) poParent->CloseUpdate( 15, 0, poParent->tableinfo.tab15open0, poParent->tableinfo.tab15close0, poParent->tableinfo.tab15dim_filter, poParent->tableinfo.tab15cell, poParent->tableinfo.tab15total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab15events = poParent->__events;
					poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
					poParent->tableinfo.tab15cell = (index) poParent->TableCellIndex( 15, 3,
						poParent->sex,
						poParent->integer_age - r_min_AGE_RANGE,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab15total_cell = (index) poParent->tableinfo.tab15cell;
					}
					else {
						poParent->tableinfo.tab15total_cell = (index) poParent->TableTotalCellIndex( 15, 3,
						poParent->sex,
						poParent->integer_age - r_min_AGE_RANGE,
						OldValue );
					}
					poParent->tableinfo.tab15dim_filter = poParent->table_filter_TabSimulatedStartingPopulation;
				}
			}
			if ( gplTableIndexes[24] >= 0 && ( poParent->tableinfo.tab24dim_filter || poParent->table_filter_TabInternalMigrationRate ) ) {
				if ( poParent->tableinfo.tab24events == -1 || poParent->tableinfo.tab24events < poParent->__events ) {
					if ( poParent->tableinfo.tab24events != -1 ) {
						poParent->tableinfo.tab24open0 = (real) poParent->CloseUpdate( 24, 0, poParent->tableinfo.tab24open0, poParent->tableinfo.tab24close0, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab24open1 = (real) poParent->CloseUpdate( 24, 1, poParent->tableinfo.tab24open1, poParent->tableinfo.tab24close1, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab24events = poParent->__events;
					poParent->tableinfo.tab24close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab24close1 = (real) poParent->number_moves;
					poParent->tableinfo.tab24cell = (index) poParent->TableCellIndex( 24, 3,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab24total_cell = (index) poParent->tableinfo.tab24cell;
					}
					else {
						poParent->tableinfo.tab24total_cell = (index) poParent->TableTotalCellIndex( 24, 3,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue );
					}
					poParent->tableinfo.tab24dim_filter = poParent->table_filter_TabInternalMigrationRate;
				}
			}
			if ( gplTableIndexes[25] >= 0 && ( poParent->tableinfo.tab25dim_filter || poParent->table_filter_TabImmigrationSexProvinceAge ) ) {
				if ( poParent->tableinfo.tab25events == -1 || poParent->tableinfo.tab25events < poParent->__events ) {
					if ( poParent->tableinfo.tab25events != -1 ) {
						Set_table_unit_TabImmigrationSexProvinceAge(poParent->table_unit_TabImmigrationSexProvinceAge+1);
						poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
						poParent->tableinfo.tab25open0 = (real) poParent->CloseUpdate( 25, 0, poParent->tableinfo.tab25open0, poParent->tableinfo.tab25close0, poParent->tableinfo.tab25dim_filter, poParent->tableinfo.tab25cell, poParent->tableinfo.tab25total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab25events = poParent->__events;
					poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
					poParent->tableinfo.tab25cell = (index) poParent->TableCellIndex( 25, 4,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab25total_cell = (index) poParent->tableinfo.tab25cell;
					}
					else {
						poParent->tableinfo.tab25total_cell = (index) poParent->TableTotalCellIndex( 25, 4,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab25dim_filter = poParent->table_filter_TabImmigrationSexProvinceAge;
				}
			}
			if ( gplTableIndexes[26] >= 0 && ( poParent->tableinfo.tab26dim_filter || poParent->table_filter_TabEmigration ) ) {
				if ( poParent->tableinfo.tab26events == -1 || poParent->tableinfo.tab26events < poParent->__events ) {
					if ( poParent->tableinfo.tab26events != -1 ) {
						poParent->tableinfo.tab26open0 = (real) poParent->CloseUpdate( 26, 0, poParent->tableinfo.tab26open0, poParent->tableinfo.tab26close0, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab26open1 = (real) poParent->CloseUpdate( 26, 1, poParent->tableinfo.tab26open1, poParent->tableinfo.tab26close1, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab26events = poParent->__events;
					poParent->tableinfo.tab26close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab26close1 = (real) poDerivedStates->transitions_has_emigrated_FALSE_TRUE;
					poParent->tableinfo.tab26cell = (index) poParent->TableCellIndex( 26, 4,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab26total_cell = (index) poParent->tableinfo.tab26cell;
					}
					else {
						poParent->tableinfo.tab26total_cell = (index) poParent->TableTotalCellIndex( 26, 4,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab26dim_filter = poParent->table_filter_TabEmigration;
				}
			}
			if ( gplTableIndexes[27] >= 0 && ( poParent->tableinfo.tab27dim_filter || poParent->table_filter_TabPopProvAgeEducSex ) ) {
				if ( poParent->tableinfo.tab27events == -1 || poParent->tableinfo.tab27events < poParent->__events ) {
					if ( poParent->tableinfo.tab27events != -1 ) {
						poParent->tableinfo.tab27open0 = (real) poParent->CloseUpdate( 27, 0, poParent->tableinfo.tab27open0, poParent->tableinfo.tab27close0, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open1 = (real) poParent->CloseUpdate( 27, 1, poParent->tableinfo.tab27open1, poParent->tableinfo.tab27close1, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open2 = (real) poParent->CloseUpdate( 27, 2, poParent->tableinfo.tab27open2, poParent->tableinfo.tab27close2, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open3 = (real) poParent->CloseUpdate( 27, 3, poParent->tableinfo.tab27open3, poParent->tableinfo.tab27close3, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open4 = (real) poParent->CloseUpdate( 27, 4, poParent->tableinfo.tab27open4, poParent->tableinfo.tab27close4, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab27events = poParent->__events;
					poParent->tableinfo.tab27close0 = (real) poDerivedStates->Get_duration_is_aded0_TRUE();
					poParent->tableinfo.tab27close1 = (real) poDerivedStates->Get_duration_is_aded1_TRUE();
					poParent->tableinfo.tab27close2 = (real) poDerivedStates->Get_duration_is_aded2_TRUE();
					poParent->tableinfo.tab27close3 = (real) poDerivedStates->Get_duration_is_child_TRUE();
					poParent->tableinfo.tab27close4 = (real) poDerivedStates->Get_duration_is_old_TRUE();
					poParent->tableinfo.tab27cell = (index) poParent->TableCellIndex( 27, 3,
						OldValue,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab27total_cell = (index) poParent->tableinfo.tab27cell;
					}
					else {
						poParent->tableinfo.tab27total_cell = (index) poParent->TableTotalCellIndex( 27, 3,
						OldValue,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab27dim_filter = poParent->table_filter_TabPopProvAgeEducSex;
				}
			}
			if ( gplTableIndexes[29] >= 0 && ( poParent->tableinfo.tab29dim_filter || poParent->table_filter_TabEduc12ByPor ) ) {
				if ( poParent->tableinfo.tab29events == -1 || poParent->tableinfo.tab29events < poParent->__events ) {
					if ( poParent->tableinfo.tab29events != -1 ) {
						poParent->tableinfo.tab29open0 = (real) poParent->CloseUpdate( 29, 0, poParent->tableinfo.tab29open0, poParent->tableinfo.tab29close0, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open1 = (real) poParent->CloseUpdate( 29, 1, poParent->tableinfo.tab29open1, poParent->tableinfo.tab29close1, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open2 = (real) poParent->CloseUpdate( 29, 2, poParent->tableinfo.tab29open2, poParent->tableinfo.tab29close2, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open3 = (real) poParent->CloseUpdate( 29, 3, poParent->tableinfo.tab29open3, poParent->tableinfo.tab29close3, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab29events = poParent->__events;
					poParent->tableinfo.tab29close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab29close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab29close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab29close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab29cell = (index) poParent->TableCellIndex( 29, 3,
						OldValue,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab29total_cell = (index) poParent->tableinfo.tab29cell;
					}
					else {
						poParent->tableinfo.tab29total_cell = (index) poParent->TableTotalCellIndex( 29, 3,
						OldValue,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab29dim_filter = poParent->table_filter_TabEduc12ByPor;
				}
			}
		}
#line 92 "code/Tables.mpp"
		Set_in_capital( ( poParent->province_nat== PN_PROV12 )? TRUE: FALSE );
#line 4678 "modgen/src/ACTORS.CPP"
#line 54 "code/CmapReportTables.mpp"
		Set_rtab_reg( ( poParent->province_nat== PN_PROV12 )? RR_NOUAK: RR_REST );
#line 4681 "modgen/src/ACTORS.CPP"
		// actorset dimension or order state
		if (poParent->actorset_filter_asForeignersAgeSexProv)
		{
			asForeignersAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][OldValue]->Remove(poParent->poasForeignersAgeSexProv);
			poParent->poasForeignersAgeSexProv = asForeignersAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asForeignersPrimAgeSexProv)
		{
			asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][OldValue]->Remove(poParent->poasForeignersPrimAgeSexProv);
			poParent->poasForeignersPrimAgeSexProv = asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
		if ( poParent->poEmigrationEvent->cRecalc == 0 ) {
			poParent->poEmigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poEmigrationEvent );
		};
		if ( poParent->poMigrationEvent->cRecalc == 0 ) {
			poParent->poMigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMigrationEvent );
		};
	}
	return poParent->province_nat = value;
}

RTAB_COL PersonStateFunctions::Set_rtab_column( RTAB_COL value )
{

	CHKLMT(value, rtab_column, 0, rc_size_RTAB_COL - 1);
	if ( value < 0 ) {
		value = (RTAB_COL) 0;
	}
	else if ( value >= rc_size_RTAB_COL ) {
		value = (RTAB_COL) ( rc_size_RTAB_COL - 1 );
	}
	if ( poParent->rtab_column != value )
	{


		RTAB_COL	OldValue;
		OldValue = poParent->rtab_column;
		poParent->rtab_column = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[8] >= 0 && ( poParent->tableinfo.tab8dim_filter || poParent->table_filter_RTAB_AgeAtFirstUnionFormation ) ) {
				if ( poParent->tableinfo.tab8events == -1 || poParent->tableinfo.tab8events < poParent->__events ) {
					if ( poParent->tableinfo.tab8events != -1 ) {
						poParent->tableinfo.tab8open0 = (real) poParent->CloseUpdate( 8, 0, poParent->tableinfo.tab8open0, poParent->tableinfo.tab8close0, poParent->tableinfo.tab8dim_filter, poParent->tableinfo.tab8cell, poParent->tableinfo.tab8total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab8open1 = (real) poParent->CloseUpdate( 8, 1, poParent->tableinfo.tab8open1, poParent->tableinfo.tab8close1, poParent->tableinfo.tab8dim_filter, poParent->tableinfo.tab8cell, poParent->tableinfo.tab8total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab8events = poParent->__events;
					poParent->tableinfo.tab8close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab8close1 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab8cell = (index) poParent->TableCellIndex( 8, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab8total_cell = (index) poParent->tableinfo.tab8cell;
					}
					else {
						poParent->tableinfo.tab8total_cell = (index) poParent->TableTotalCellIndex( 8, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab8dim_filter = poParent->table_filter_RTAB_AgeAtFirstUnionFormation;
				}
			}
		}
	}
	return poParent->rtab_column = value;
}

logical PersonStateFunctions::Set_set_alive( logical value )
{

	value = value != 0;
	if ( poParent->set_alive != value )
	{
		bool bScheduleTimeChange = false;	//indicates that schedule event should be recalculated
		if (	(poParent->schedule_event_time == poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time)
			|| (poParent->schedule_event_time == poDerivedStates->trigger_entrances_set_alive_TRUE_time))
		{
			//force timeScheduleEvent to calculate min time
			poParent->schedule_event_time = TIME_INFINITE;
		//indicate that schedule event needs to be recalculated
			bScheduleTimeChange = true;
		}
#line 26 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asAllPersons( value );
#line 4773 "modgen/src/ACTORS.CPP"
#line 17 "code/UnionFormation.mpp"
		Set_actorset_filter_asFemaleAgeEduc( value&& poParent->sex== FEMALE );
#line 4776 "modgen/src/ACTORS.CPP"
#line 29 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asForeignersAgeSexProv( value&& poParent->province_birth== PI_PROV13 );
#line 4779 "modgen/src/ACTORS.CPP"
#line 35 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asForeignersPrimAgeSexProv( value&& poParent->province_birth== PI_PROV13&& poParent->primary_level!= PL_NO );
#line 4782 "modgen/src/ACTORS.CPP"
#line 19 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asPrimaryEntryCohort( value&& poParent->integer_age== gprParam->AgeEnterPrimary );
#line 4785 "modgen/src/ACTORS.CPP"
#line 23 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asPrimaryGraduationCohort( value&& poParent->integer_age== gprParam->AgeGradPrimary&& poParent->primary_level!= PL_NO );
#line 4788 "modgen/src/ACTORS.CPP"
#line 32 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asResidentsAgeSex( value );
#line 4791 "modgen/src/ACTORS.CPP"
#line 38 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asResidentsPrimAgeSex( value&& poParent->primary_level!= PL_NO );
#line 4794 "modgen/src/ACTORS.CPP"
		if ( value == TRUE ) {
			poDerivedStates->Set_entrances_set_alive_TRUE( poDerivedStates->entrances_set_alive_TRUE + 1 );
		}
		if ( value == TRUE ) {
			if ( poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME < 1 ) {
				poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = WAIT1(poParent->time, POINTS(LITTLE_TIME)[poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME] - poDerivedStates->duration_set_alive_TRUE );
				if ( (!bScheduleTimeChange || poParent->schedule_event_time != TIME_INFINITE) && poParent->schedule_event_time > poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time)
				{
					poParent->schedule_event_time = poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time;
					//indicate that schedule event should be recalculated
					bScheduleTimeChange = true;
				}
			}
			else
			{
				poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = TIME_INFINITE;
			}
		}
		else
		{
			poDerivedStates->sssplit_duration_set_alive_TRUE_LITTLE_TIME_time = TIME_INFINITE;
		}
		if ( poParent->set_alive == TRUE ) {
			poDerivedStates->duration_set_alive_TRUE = poDerivedStates->Get_duration_set_alive_TRUE();
		}
		else if ( value == TRUE ) {
			poDerivedStates->duration_set_alive_TRUE_time = poParent->__time;
		}


		logical	OldValue;
		OldValue = poParent->set_alive;
		poParent->set_alive = value;

#line 155 "code/Fertility.mpp"
		Set_is_potential_mother( ( poParent->set_alive&& poParent->sex== FEMALE&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& poParent->parity< MAX(PARITY_RANGE)&& WITHIN(SIM_YEAR_RANGE,poParent->calendar_year) )? TRUE: FALSE );
#line 4831 "modgen/src/ACTORS.CPP"
		if ( value == TRUE ) 
		{
			poDerivedStates->Set_trigger_entrances_set_alive_TRUE( TRUE );
			poDerivedStates->trigger_entrances_set_alive_TRUE_time = poParent->__time;
			if ( poParent->schedule_event_time > poDerivedStates->trigger_entrances_set_alive_TRUE_time)
			{
				poParent->schedule_event_time = poDerivedStates->trigger_entrances_set_alive_TRUE_time;
				//indicate that schedule event should be recalculated
				bScheduleTimeChange = true;
			}
		}
		if ( poParent->poSetAliveEvent->cRecalc == 0 ) {
			poParent->poSetAliveEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poSetAliveEvent );
		};

		if (bScheduleTimeChange && poParent->poScheduleEvent->cRecalc == 0)
		{
			poParent->poScheduleEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent(poParent->poScheduleEvent);
		}
	}
	return poParent->set_alive = value;
}

SEX PersonStateFunctions::Set_sex( SEX value )
{

	CHKLMT(value, sex, 0, rc_size_SEX - 1);
	if ( value < 0 ) {
		value = (SEX) 0;
	}
	else if ( value >= rc_size_SEX ) {
		value = (SEX) ( rc_size_SEX - 1 );
	}
	if ( poParent->sex != value )
	{
#line 17 "code/UnionFormation.mpp"
		Set_actorset_filter_asFemaleAgeEduc( poParent->set_alive&& value== FEMALE );
#line 4871 "modgen/src/ACTORS.CPP"
#line 148 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_AgeAtFirstUnionFormation( value== FEMALE&& poParent->in_rtab_years&& poParent->count_parity );
#line 4874 "modgen/src/ACTORS.CPP"
#line 183 "code/Tables.mpp"
		Set_table_filter_TabAgeAtBirth( poParent->in_projected_time&& value== FEMALE&& poParent->count_parity );
#line 4877 "modgen/src/ACTORS.CPP"
#line 388 "code/Tables.mpp"
		Set_table_filter_TabAgeAtFirstUnionFormation( value== FEMALE&& poParent->in_projected_time&& poParent->count_parity );
#line 4880 "modgen/src/ACTORS.CPP"
#line 152 "code/Tables.mpp"
		Set_table_filter_TabFertilityRates( poParent->in_projected_time&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& value== FEMALE&& poParent->count_parity );
#line 4883 "modgen/src/ACTORS.CPP"
#line 405 "code/Tables.mpp"
		Set_table_filter_TabMothersExperience( poParent->in_projected_time&& WITHIN(SIM_YOB_RANGE,poParent->year_of_birth)&& poParent->person_type== PT_CHILD&& value== FEMALE );
#line 4886 "modgen/src/ACTORS.CPP"
#line 370 "code/Tables.mpp"
		Set_table_filter_TabUnionFormation( WITHIN(AGE_UNION,poParent->integer_age)&& value== FEMALE&& poParent->in_projected_time&& poParent->count_parity );
#line 4889 "modgen/src/ACTORS.CPP"


		SEX	OldValue;
		OldValue = poParent->sex;
		poParent->sex = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[2] >= 0 && ( poParent->tableinfo.tab2dim_filter || poParent->table_filter_RTAB_PopPyramidByEduc ) ) {
				if ( poParent->tableinfo.tab2events == -1 || poParent->tableinfo.tab2events < poParent->__events ) {
					if ( poParent->tableinfo.tab2events != -1 ) {
						poParent->tableinfo.tab2open0 = (real) poParent->CloseUpdate( 2, 0, poParent->tableinfo.tab2open0, poParent->tableinfo.tab2close0, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open1 = (real) poParent->CloseUpdate( 2, 1, poParent->tableinfo.tab2open1, poParent->tableinfo.tab2close1, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open2 = (real) poParent->CloseUpdate( 2, 2, poParent->tableinfo.tab2open2, poParent->tableinfo.tab2close2, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab2events = poParent->__events;
					poParent->tableinfo.tab2close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab2close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab2close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab2cell = (index) poParent->TableCellIndex( 2, 4,
						poParent->rtab_sel_years,
						OldValue,
						poParent->rtab_reg,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab2total_cell = (index) poParent->tableinfo.tab2cell;
					}
					else {
						poParent->tableinfo.tab2total_cell = (index) poParent->TableTotalCellIndex( 2, 4,
						poParent->rtab_sel_years,
						OldValue,
						poParent->rtab_reg,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab2dim_filter = poParent->table_filter_RTAB_PopPyramidByEduc;
				}
			}
			if ( gplTableIndexes[13] >= 0 && ( poParent->tableinfo.tab13dim_filter || poParent->table_filter_TabTotalPopulationSexProvince ) ) {
				if ( poParent->tableinfo.tab13events == -1 || poParent->tableinfo.tab13events < poParent->__events ) {
					if ( poParent->tableinfo.tab13events != -1 ) {
						poParent->tableinfo.tab13open0 = (real) poParent->CloseUpdate( 13, 0, poParent->tableinfo.tab13open0, poParent->tableinfo.tab13close0, poParent->tableinfo.tab13dim_filter, poParent->tableinfo.tab13cell, poParent->tableinfo.tab13total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab13events = poParent->__events;
					poParent->tableinfo.tab13close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab13cell = (index) poParent->TableCellIndex( 13, 3,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab13total_cell = (index) poParent->tableinfo.tab13cell;
					}
					else {
						poParent->tableinfo.tab13total_cell = (index) poParent->TableTotalCellIndex( 13, 3,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->province_nat );
					}
					poParent->tableinfo.tab13dim_filter = poParent->table_filter_TabTotalPopulationSexProvince;
				}
			}
			if ( gplTableIndexes[14] >= 0 && ( poParent->tableinfo.tab14dim_filter || poParent->table_filter_TabPopulationAgeSexProvince ) ) {
				if ( poParent->tableinfo.tab14events == -1 || poParent->tableinfo.tab14events < poParent->__events ) {
					if ( poParent->tableinfo.tab14events != -1 ) {
						poParent->tableinfo.tab14open0 = (real) poParent->CloseUpdate( 14, 0, poParent->tableinfo.tab14open0, poParent->tableinfo.tab14close0, poParent->tableinfo.tab14dim_filter, poParent->tableinfo.tab14cell, poParent->tableinfo.tab14total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab14events = poParent->__events;
					poParent->tableinfo.tab14close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab14cell = (index) poParent->TableCellIndex( 14, 4,
						OldValue,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab14total_cell = (index) poParent->tableinfo.tab14cell;
					}
					else {
						poParent->tableinfo.tab14total_cell = (index) poParent->TableTotalCellIndex( 14, 4,
						OldValue,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab14dim_filter = poParent->table_filter_TabPopulationAgeSexProvince;
				}
			}
			if ( gplTableIndexes[15] >= 0 && ( poParent->tableinfo.tab15dim_filter || poParent->table_filter_TabSimulatedStartingPopulation ) ) {
				if ( poParent->tableinfo.tab15events == -1 || poParent->tableinfo.tab15events < poParent->__events ) {
					if ( poParent->tableinfo.tab15events != -1 ) {
						Set_table_unit_TabSimulatedStartingPopulation(poParent->table_unit_TabSimulatedStartingPopulation+1);
						poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
						poParent->tableinfo.tab15open0 = (real) poParent->CloseUpdate( 15, 0, poParent->tableinfo.tab15open0, poParent->tableinfo.tab15close0, poParent->tableinfo.tab15dim_filter, poParent->tableinfo.tab15cell, poParent->tableinfo.tab15total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab15events = poParent->__events;
					poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
					poParent->tableinfo.tab15cell = (index) poParent->TableCellIndex( 15, 3,
						OldValue,
						poParent->integer_age - r_min_AGE_RANGE,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab15total_cell = (index) poParent->tableinfo.tab15cell;
					}
					else {
						poParent->tableinfo.tab15total_cell = (index) poParent->TableTotalCellIndex( 15, 3,
						OldValue,
						poParent->integer_age - r_min_AGE_RANGE,
						poParent->province_nat );
					}
					poParent->tableinfo.tab15dim_filter = poParent->table_filter_TabSimulatedStartingPopulation;
				}
			}
			if ( gplTableIndexes[17] >= 0 && ( poParent->tableinfo.tab17dim_filter || poParent->table_filter_TabNumberBirths ) ) {
				if ( poParent->tableinfo.tab17events == -1 || poParent->tableinfo.tab17events < poParent->__events ) {
					if ( poParent->tableinfo.tab17events != -1 ) {
						poParent->tableinfo.tab17open0 = (real) poParent->CloseUpdate( 17, 0, poParent->tableinfo.tab17open0, poParent->tableinfo.tab17close0, poParent->tableinfo.tab17dim_filter, poParent->tableinfo.tab17cell, poParent->tableinfo.tab17total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab17events = poParent->__events;
					poParent->tableinfo.tab17close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab17cell = (index) poParent->TableCellIndex( 17, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab17total_cell = (index) poParent->tableinfo.tab17cell;
					}
					else {
						poParent->tableinfo.tab17total_cell = (index) poParent->TableTotalCellIndex( 17, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue );
					}
					poParent->tableinfo.tab17dim_filter = poParent->table_filter_TabNumberBirths;
				}
			}
			if ( gplTableIndexes[20] >= 0 && ( poParent->tableinfo.tab20dim_filter || poParent->table_filter_TabDeathRates ) ) {
				if ( poParent->tableinfo.tab20events == -1 || poParent->tableinfo.tab20events < poParent->__events ) {
					if ( poParent->tableinfo.tab20events != -1 ) {
						poParent->tableinfo.tab20open0 = (real) poParent->CloseUpdate( 20, 0, poParent->tableinfo.tab20open0, poParent->tableinfo.tab20close0, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab20open1 = (real) poParent->CloseUpdate( 20, 1, poParent->tableinfo.tab20open1, poParent->tableinfo.tab20close1, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab20events = poParent->__events;
					poParent->tableinfo.tab20close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab20close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab20cell = (index) poParent->TableCellIndex( 20, 3,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->integer_age - r_min_AGE_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab20total_cell = (index) poParent->tableinfo.tab20cell;
					}
					else {
						poParent->tableinfo.tab20total_cell = (index) poParent->TableTotalCellIndex( 20, 3,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->integer_age - r_min_AGE_RANGE );
					}
					poParent->tableinfo.tab20dim_filter = poParent->table_filter_TabDeathRates;
				}
			}
			if ( gplTableIndexes[24] >= 0 && ( poParent->tableinfo.tab24dim_filter || poParent->table_filter_TabInternalMigrationRate ) ) {
				if ( poParent->tableinfo.tab24events == -1 || poParent->tableinfo.tab24events < poParent->__events ) {
					if ( poParent->tableinfo.tab24events != -1 ) {
						poParent->tableinfo.tab24open0 = (real) poParent->CloseUpdate( 24, 0, poParent->tableinfo.tab24open0, poParent->tableinfo.tab24close0, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab24open1 = (real) poParent->CloseUpdate( 24, 1, poParent->tableinfo.tab24open1, poParent->tableinfo.tab24close1, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab24events = poParent->__events;
					poParent->tableinfo.tab24close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab24close1 = (real) poParent->number_moves;
					poParent->tableinfo.tab24cell = (index) poParent->TableCellIndex( 24, 3,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab24total_cell = (index) poParent->tableinfo.tab24cell;
					}
					else {
						poParent->tableinfo.tab24total_cell = (index) poParent->TableTotalCellIndex( 24, 3,
						OldValue,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->province_nat );
					}
					poParent->tableinfo.tab24dim_filter = poParent->table_filter_TabInternalMigrationRate;
				}
			}
			if ( gplTableIndexes[25] >= 0 && ( poParent->tableinfo.tab25dim_filter || poParent->table_filter_TabImmigrationSexProvinceAge ) ) {
				if ( poParent->tableinfo.tab25events == -1 || poParent->tableinfo.tab25events < poParent->__events ) {
					if ( poParent->tableinfo.tab25events != -1 ) {
						Set_table_unit_TabImmigrationSexProvinceAge(poParent->table_unit_TabImmigrationSexProvinceAge+1);
						poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
						poParent->tableinfo.tab25open0 = (real) poParent->CloseUpdate( 25, 0, poParent->tableinfo.tab25open0, poParent->tableinfo.tab25close0, poParent->tableinfo.tab25dim_filter, poParent->tableinfo.tab25cell, poParent->tableinfo.tab25total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab25events = poParent->__events;
					poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
					poParent->tableinfo.tab25cell = (index) poParent->TableCellIndex( 25, 4,
						OldValue,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab25total_cell = (index) poParent->tableinfo.tab25cell;
					}
					else {
						poParent->tableinfo.tab25total_cell = (index) poParent->TableTotalCellIndex( 25, 4,
						OldValue,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab25dim_filter = poParent->table_filter_TabImmigrationSexProvinceAge;
				}
			}
			if ( gplTableIndexes[26] >= 0 && ( poParent->tableinfo.tab26dim_filter || poParent->table_filter_TabEmigration ) ) {
				if ( poParent->tableinfo.tab26events == -1 || poParent->tableinfo.tab26events < poParent->__events ) {
					if ( poParent->tableinfo.tab26events != -1 ) {
						poParent->tableinfo.tab26open0 = (real) poParent->CloseUpdate( 26, 0, poParent->tableinfo.tab26open0, poParent->tableinfo.tab26close0, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab26open1 = (real) poParent->CloseUpdate( 26, 1, poParent->tableinfo.tab26open1, poParent->tableinfo.tab26close1, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab26events = poParent->__events;
					poParent->tableinfo.tab26close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab26close1 = (real) poDerivedStates->transitions_has_emigrated_FALSE_TRUE;
					poParent->tableinfo.tab26cell = (index) poParent->TableCellIndex( 26, 4,
						OldValue,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab26total_cell = (index) poParent->tableinfo.tab26cell;
					}
					else {
						poParent->tableinfo.tab26total_cell = (index) poParent->TableTotalCellIndex( 26, 4,
						OldValue,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab26dim_filter = poParent->table_filter_TabEmigration;
				}
			}
			if ( gplTableIndexes[27] >= 0 && ( poParent->tableinfo.tab27dim_filter || poParent->table_filter_TabPopProvAgeEducSex ) ) {
				if ( poParent->tableinfo.tab27events == -1 || poParent->tableinfo.tab27events < poParent->__events ) {
					if ( poParent->tableinfo.tab27events != -1 ) {
						poParent->tableinfo.tab27open0 = (real) poParent->CloseUpdate( 27, 0, poParent->tableinfo.tab27open0, poParent->tableinfo.tab27close0, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open1 = (real) poParent->CloseUpdate( 27, 1, poParent->tableinfo.tab27open1, poParent->tableinfo.tab27close1, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open2 = (real) poParent->CloseUpdate( 27, 2, poParent->tableinfo.tab27open2, poParent->tableinfo.tab27close2, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open3 = (real) poParent->CloseUpdate( 27, 3, poParent->tableinfo.tab27open3, poParent->tableinfo.tab27close3, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open4 = (real) poParent->CloseUpdate( 27, 4, poParent->tableinfo.tab27open4, poParent->tableinfo.tab27close4, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab27events = poParent->__events;
					poParent->tableinfo.tab27close0 = (real) poDerivedStates->Get_duration_is_aded0_TRUE();
					poParent->tableinfo.tab27close1 = (real) poDerivedStates->Get_duration_is_aded1_TRUE();
					poParent->tableinfo.tab27close2 = (real) poDerivedStates->Get_duration_is_aded2_TRUE();
					poParent->tableinfo.tab27close3 = (real) poDerivedStates->Get_duration_is_child_TRUE();
					poParent->tableinfo.tab27close4 = (real) poDerivedStates->Get_duration_is_old_TRUE();
					poParent->tableinfo.tab27cell = (index) poParent->TableCellIndex( 27, 3,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab27total_cell = (index) poParent->tableinfo.tab27cell;
					}
					else {
						poParent->tableinfo.tab27total_cell = (index) poParent->TableTotalCellIndex( 27, 3,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab27dim_filter = poParent->table_filter_TabPopProvAgeEducSex;
				}
			}
			if ( gplTableIndexes[28] >= 0 && ( poParent->tableinfo.tab28dim_filter || poParent->table_filter_TabEduc12ByPob ) ) {
				if ( poParent->tableinfo.tab28events == -1 || poParent->tableinfo.tab28events < poParent->__events ) {
					if ( poParent->tableinfo.tab28events != -1 ) {
						poParent->tableinfo.tab28open0 = (real) poParent->CloseUpdate( 28, 0, poParent->tableinfo.tab28open0, poParent->tableinfo.tab28close0, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open1 = (real) poParent->CloseUpdate( 28, 1, poParent->tableinfo.tab28open1, poParent->tableinfo.tab28close1, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open2 = (real) poParent->CloseUpdate( 28, 2, poParent->tableinfo.tab28open2, poParent->tableinfo.tab28close2, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open3 = (real) poParent->CloseUpdate( 28, 3, poParent->tableinfo.tab28open3, poParent->tableinfo.tab28close3, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab28events = poParent->__events;
					poParent->tableinfo.tab28close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab28close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab28close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab28close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab28cell = (index) poParent->TableCellIndex( 28, 3,
						poParent->province_birth,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab28total_cell = (index) poParent->tableinfo.tab28cell;
					}
					else {
						poParent->tableinfo.tab28total_cell = (index) poParent->TableTotalCellIndex( 28, 3,
						poParent->province_birth,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab28dim_filter = poParent->table_filter_TabEduc12ByPob;
				}
			}
			if ( gplTableIndexes[29] >= 0 && ( poParent->tableinfo.tab29dim_filter || poParent->table_filter_TabEduc12ByPor ) ) {
				if ( poParent->tableinfo.tab29events == -1 || poParent->tableinfo.tab29events < poParent->__events ) {
					if ( poParent->tableinfo.tab29events != -1 ) {
						poParent->tableinfo.tab29open0 = (real) poParent->CloseUpdate( 29, 0, poParent->tableinfo.tab29open0, poParent->tableinfo.tab29close0, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open1 = (real) poParent->CloseUpdate( 29, 1, poParent->tableinfo.tab29open1, poParent->tableinfo.tab29close1, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open2 = (real) poParent->CloseUpdate( 29, 2, poParent->tableinfo.tab29open2, poParent->tableinfo.tab29close2, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open3 = (real) poParent->CloseUpdate( 29, 3, poParent->tableinfo.tab29open3, poParent->tableinfo.tab29close3, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab29events = poParent->__events;
					poParent->tableinfo.tab29close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab29close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab29close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab29close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab29cell = (index) poParent->TableCellIndex( 29, 3,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab29total_cell = (index) poParent->tableinfo.tab29cell;
					}
					else {
						poParent->tableinfo.tab29total_cell = (index) poParent->TableTotalCellIndex( 29, 3,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab29dim_filter = poParent->table_filter_TabEduc12ByPor;
				}
			}
			if ( gplTableIndexes[30] >= 0 && ( poParent->tableinfo.tab30dim_filter || poParent->table_filter_TabEduc2015 ) ) {
				if ( poParent->tableinfo.tab30events == -1 || poParent->tableinfo.tab30events < poParent->__events ) {
					if ( poParent->tableinfo.tab30events != -1 ) {
						poParent->tableinfo.tab30open0 = (real) poParent->CloseUpdate( 30, 0, poParent->tableinfo.tab30open0, poParent->tableinfo.tab30close0, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open1 = (real) poParent->CloseUpdate( 30, 1, poParent->tableinfo.tab30open1, poParent->tableinfo.tab30close1, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open2 = (real) poParent->CloseUpdate( 30, 2, poParent->tableinfo.tab30open2, poParent->tableinfo.tab30close2, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab30events = poParent->__events;
					poParent->tableinfo.tab30close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab30close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab30close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab30cell = (index) poParent->TableCellIndex( 30, 3,
						poParent->in_capital,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab30total_cell = (index) poParent->tableinfo.tab30cell;
					}
					else {
						poParent->tableinfo.tab30total_cell = (index) poParent->TableTotalCellIndex( 30, 3,
						poParent->in_capital,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab30dim_filter = poParent->table_filter_TabEduc2015;
				}
			}
			if ( gplTableIndexes[31] >= 0 && ( poParent->tableinfo.tab31dim_filter || poParent->table_filter_TabEduc2040 ) ) {
				if ( poParent->tableinfo.tab31events == -1 || poParent->tableinfo.tab31events < poParent->__events ) {
					if ( poParent->tableinfo.tab31events != -1 ) {
						poParent->tableinfo.tab31open0 = (real) poParent->CloseUpdate( 31, 0, poParent->tableinfo.tab31open0, poParent->tableinfo.tab31close0, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open1 = (real) poParent->CloseUpdate( 31, 1, poParent->tableinfo.tab31open1, poParent->tableinfo.tab31close1, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open2 = (real) poParent->CloseUpdate( 31, 2, poParent->tableinfo.tab31open2, poParent->tableinfo.tab31close2, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab31events = poParent->__events;
					poParent->tableinfo.tab31close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab31close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab31close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab31cell = (index) poParent->TableCellIndex( 31, 3,
						poParent->in_capital,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab31total_cell = (index) poParent->tableinfo.tab31cell;
					}
					else {
						poParent->tableinfo.tab31total_cell = (index) poParent->TableTotalCellIndex( 31, 3,
						poParent->in_capital,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab31dim_filter = poParent->table_filter_TabEduc2040;
				}
			}
			if ( gplTableIndexes[32] >= 0 && ( poParent->tableinfo.tab32dim_filter || poParent->table_filter_TabEduc2065 ) ) {
				if ( poParent->tableinfo.tab32events == -1 || poParent->tableinfo.tab32events < poParent->__events ) {
					if ( poParent->tableinfo.tab32events != -1 ) {
						poParent->tableinfo.tab32open0 = (real) poParent->CloseUpdate( 32, 0, poParent->tableinfo.tab32open0, poParent->tableinfo.tab32close0, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open1 = (real) poParent->CloseUpdate( 32, 1, poParent->tableinfo.tab32open1, poParent->tableinfo.tab32close1, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open2 = (real) poParent->CloseUpdate( 32, 2, poParent->tableinfo.tab32open2, poParent->tableinfo.tab32close2, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab32events = poParent->__events;
					poParent->tableinfo.tab32close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab32close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab32close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab32cell = (index) poParent->TableCellIndex( 32, 3,
						poParent->in_capital,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab32total_cell = (index) poParent->tableinfo.tab32cell;
					}
					else {
						poParent->tableinfo.tab32total_cell = (index) poParent->TableTotalCellIndex( 32, 3,
						poParent->in_capital,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab32dim_filter = poParent->table_filter_TabEduc2065;
				}
			}
		}
#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 5297 "modgen/src/ACTORS.CPP"
#line 155 "code/Fertility.mpp"
		Set_is_potential_mother( ( poParent->set_alive&& poParent->sex== FEMALE&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& poParent->parity< MAX(PARITY_RANGE)&& WITHIN(SIM_YEAR_RANGE,poParent->calendar_year) )? TRUE: FALSE );
#line 5300 "modgen/src/ACTORS.CPP"
#line 65 "code/UnionFormation.mpp"
		Set_union_formation_hazard( ( WITHIN(AGE_UNION,poParent->integer_age)&& poParent->sex== FEMALE )? gprParam->UnionFormationHazard[CHKBND(poParent->primary_level, 3, "UnionFormationHazard", 0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE,poParent->calendar_year), 101, "UnionFormationHazard", 1)][CHKBND(RANGE_POS(AGE_UNION,poParent->integer_age), 51, "UnionFormationHazard", 2)]: 0.0 );
#line 5303 "modgen/src/ACTORS.CPP"
		// actorset dimension or order state
		if (poParent->actorset_filter_asForeignersAgeSexProv)
		{
			asForeignersAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][OldValue][poParent->province_nat]->Remove(poParent->poasForeignersAgeSexProv);
			poParent->poasForeignersAgeSexProv = asForeignersAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asForeignersPrimAgeSexProv)
		{
			asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][OldValue][poParent->province_nat]->Remove(poParent->poasForeignersPrimAgeSexProv);
			poParent->poasForeignersPrimAgeSexProv = asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asPrimaryEntryCohort)
		{
			asPrimaryEntryCohort[OldValue][poParent->province_birth][poParent->educ_mother]->Remove(poParent->poasPrimaryEntryCohort);
			poParent->poasPrimaryEntryCohort = asPrimaryEntryCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asPrimaryGraduationCohort)
		{
			asPrimaryGraduationCohort[OldValue][poParent->province_birth][poParent->educ_mother]->Remove(poParent->poasPrimaryGraduationCohort);
			poParent->poasPrimaryGraduationCohort = asPrimaryGraduationCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asResidentsAgeSex)
		{
			asResidentsAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][OldValue]->Remove(poParent->poasResidentsAgeSex);
			poParent->poasResidentsAgeSex = asResidentsAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asResidentsPrimAgeSex)
		{
			asResidentsPrimAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][OldValue]->Remove(poParent->poasResidentsPrimAgeSex);
			poParent->poasResidentsPrimAgeSex = asResidentsPrimAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->NewNode(poParent);
		}
		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
		if ( poParent->poEmigrationEvent->cRecalc == 0 ) {
			poParent->poEmigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poEmigrationEvent );
		};
		if ( poParent->poMigrationEvent->cRecalc == 0 ) {
			poParent->poMigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMigrationEvent );
		};
		if ( poParent->poMortalityEvent->cRecalc == 0 ) {
			poParent->poMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMortalityEvent );
		};
		if ( poParent->poUnionFormationEvent->cRecalc == 0 ) {
			poParent->poUnionFormationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poUnionFormationEvent );
		};
	}
	return poParent->sex = value;
}

counter PersonStateFunctions::Set_table_unit_TabImmigrationSexProvinceAge( counter value )
{

	if ( poParent->table_unit_TabImmigrationSexProvinceAge != value )
	{


		counter	OldValue;
		OldValue = poParent->table_unit_TabImmigrationSexProvinceAge;
		poParent->table_unit_TabImmigrationSexProvinceAge = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[25] >= 0 && poParent->tableinfo.tab25events == poParent->__events ) {
				poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
			}
		}
	}
	return poParent->table_unit_TabImmigrationSexProvinceAge = value;
}

counter PersonStateFunctions::Set_table_unit_TabMothersExperience( counter value )
{

	if ( poParent->table_unit_TabMothersExperience != value )
	{


		counter	OldValue;
		OldValue = poParent->table_unit_TabMothersExperience;
		poParent->table_unit_TabMothersExperience = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[35] >= 0 && poParent->tableinfo.tab35events == poParent->__events ) {
				poParent->tableinfo.tab35close3 = (real) poParent->table_unit_TabMothersExperience;
			}
		}
	}
	return poParent->table_unit_TabMothersExperience = value;
}

counter PersonStateFunctions::Set_table_unit_TabSimulatedStartingPopulation( counter value )
{

	if ( poParent->table_unit_TabSimulatedStartingPopulation != value )
	{


		counter	OldValue;
		OldValue = poParent->table_unit_TabSimulatedStartingPopulation;
		poParent->table_unit_TabSimulatedStartingPopulation = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[15] >= 0 && poParent->tableinfo.tab15events == poParent->__events ) {
				poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
			}
		}
	}
	return poParent->table_unit_TabSimulatedStartingPopulation = value;
}

TIME PersonStateFunctions::Set_time( TIME value, bool bStartFunction )
{

	if ( poParent->time != value || bStartFunction )
	{
		if (bStartFunction)
		{
			poDerivedStates->Set_ssint_time( std::floor(value) );
			poDerivedStates->ssint_time_time = poDerivedStates->ssint_time + 1.0;
			if (poParent->schedule_event_time > poDerivedStates->ssint_time_time)
			{
				poParent->schedule_event_time = poDerivedStates->ssint_time_time;
			}
		}


		TIME	OldValue;
		OldValue = poParent->time;
		poParent->time = value;

	}
	return poParent->time = value;
}

TIME PersonStateFunctions::Set_time_microdata_output( TIME value )
{

	if ( poParent->time_microdata_output != value )
	{


		TIME	OldValue;
		OldValue = poParent->time_microdata_output;
		poParent->time_microdata_output = value;

		if ( poParent->poWriteMicroRecord->cRecalc == 0 ) {
			poParent->poWriteMicroRecord->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poWriteMicroRecord );
		};
	}
	return poParent->time_microdata_output = value;
}

TIME PersonStateFunctions::Set_time_of_birth( TIME value )
{

	if ( poParent->time_of_birth != value )
	{


		TIME	OldValue;
		OldValue = poParent->time_of_birth;
		poParent->time_of_birth = value;

#line 74 "code/PersonCore.mpp"
		Set_year_of_birth( int(poParent->time_of_birth) );
#line 5475 "modgen/src/ACTORS.CPP"
		if ( poParent->poMortalityEvent->cRecalc == 0 ) {
			poParent->poMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMortalityEvent );
		};
	}
	return poParent->time_of_birth = value;
}

TIME PersonStateFunctions::Set_time_of_union_formation( TIME value )
{

	if ( poParent->time_of_union_formation != value )
	{


		TIME	OldValue;
		OldValue = poParent->time_of_union_formation;
		poParent->time_of_union_formation = value;

		if ( poParent->poUnionFormationEvent->cRecalc == 0 ) {
			poParent->poUnionFormationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poUnionFormationEvent );
		};
	}
	return poParent->time_of_union_formation = value;
}

double PersonStateFunctions::Set_time_since_last_birth_start( double value )
{

	if ( poParent->time_since_last_birth_start != value )
	{


		double	OldValue;
		OldValue = poParent->time_since_last_birth_start;
		poParent->time_since_last_birth_start = value;

#line 145 "code/Fertility.mpp"
		Set_time_since_last_birth( poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE+ round(poParent->time_since_last_birth_start) );
#line 5516 "modgen/src/ACTORS.CPP"
	}
	return poParent->time_since_last_birth_start = value;
}

logical PersonStateFunctions::Set_actorset_filter_asAllPersons( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asAllPersons != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asAllPersons;
		poParent->actorset_filter_asAllPersons = value;

		// actorset filter
		if (poParent->actorset_filter_asAllPersons)
		{
			poParent->poasAllPersons = asAllPersons->NewNode(poParent);
		}
		else
		{
			asAllPersons->Remove(poParent->poasAllPersons);
			poParent->poasAllPersons = NULL;
		}
	}
	return poParent->actorset_filter_asAllPersons = value;
}

logical PersonStateFunctions::Set_actorset_filter_asFemaleAgeEduc( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asFemaleAgeEduc != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asFemaleAgeEduc;
		poParent->actorset_filter_asFemaleAgeEduc = value;

		// actorset filter
		if (poParent->actorset_filter_asFemaleAgeEduc)
		{
			poParent->poasFemaleAgeEduc = asFemaleAgeEduc[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->primary_level]->NewNode(poParent);
		}
		else
		{
			asFemaleAgeEduc[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->primary_level]->Remove(poParent->poasFemaleAgeEduc);
			poParent->poasFemaleAgeEduc = NULL;
		}
	}
	return poParent->actorset_filter_asFemaleAgeEduc = value;
}

logical PersonStateFunctions::Set_actorset_filter_asForeignersAgeSexProv( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asForeignersAgeSexProv != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asForeignersAgeSexProv;
		poParent->actorset_filter_asForeignersAgeSexProv = value;

		// actorset filter
		if (poParent->actorset_filter_asForeignersAgeSexProv)
		{
			poParent->poasForeignersAgeSexProv = asForeignersAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		else
		{
			asForeignersAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->Remove(poParent->poasForeignersAgeSexProv);
			poParent->poasForeignersAgeSexProv = NULL;
		}
	}
	return poParent->actorset_filter_asForeignersAgeSexProv = value;
}

logical PersonStateFunctions::Set_actorset_filter_asForeignersPrimAgeSexProv( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asForeignersPrimAgeSexProv != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asForeignersPrimAgeSexProv;
		poParent->actorset_filter_asForeignersPrimAgeSexProv = value;

		// actorset filter
		if (poParent->actorset_filter_asForeignersPrimAgeSexProv)
		{
			poParent->poasForeignersPrimAgeSexProv = asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		else
		{
			asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->Remove(poParent->poasForeignersPrimAgeSexProv);
			poParent->poasForeignersPrimAgeSexProv = NULL;
		}
	}
	return poParent->actorset_filter_asForeignersPrimAgeSexProv = value;
}

logical PersonStateFunctions::Set_actorset_filter_asPotentialMothers( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asPotentialMothers != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asPotentialMothers;
		poParent->actorset_filter_asPotentialMothers = value;

		// actorset filter
		if (poParent->actorset_filter_asPotentialMothers)
		{
			poParent->poasPotentialMothers = asPotentialMothers->NewNode(poParent);
		}
		else
		{
			asPotentialMothers->Remove(poParent->poasPotentialMothers);
			poParent->poasPotentialMothers = NULL;
		}
	}
	return poParent->actorset_filter_asPotentialMothers = value;
}

logical PersonStateFunctions::Set_actorset_filter_asPotentialMothersByAge( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asPotentialMothersByAge != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asPotentialMothersByAge;
		poParent->actorset_filter_asPotentialMothersByAge = value;

		// actorset filter
		if (poParent->actorset_filter_asPotentialMothersByAge)
		{
			poParent->poasPotentialMothersByAge = asPotentialMothersByAge[RANGE_POS(FERTILE_AGE_RANGE, poParent->fertile_age)]->NewNode(poParent);
		}
		else
		{
			asPotentialMothersByAge[RANGE_POS(FERTILE_AGE_RANGE, poParent->fertile_age)]->Remove(poParent->poasPotentialMothersByAge);
			poParent->poasPotentialMothersByAge = NULL;
		}
	}
	return poParent->actorset_filter_asPotentialMothersByAge = value;
}

logical PersonStateFunctions::Set_actorset_filter_asPrimaryEntryCohort( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asPrimaryEntryCohort != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asPrimaryEntryCohort;
		poParent->actorset_filter_asPrimaryEntryCohort = value;

		// actorset filter
		if (poParent->actorset_filter_asPrimaryEntryCohort)
		{
			poParent->poasPrimaryEntryCohort = asPrimaryEntryCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
		else
		{
			asPrimaryEntryCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->Remove(poParent->poasPrimaryEntryCohort);
			poParent->poasPrimaryEntryCohort = NULL;
		}
	}
	return poParent->actorset_filter_asPrimaryEntryCohort = value;
}

logical PersonStateFunctions::Set_actorset_filter_asPrimaryGraduationCohort( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asPrimaryGraduationCohort != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asPrimaryGraduationCohort;
		poParent->actorset_filter_asPrimaryGraduationCohort = value;

		// actorset filter
		if (poParent->actorset_filter_asPrimaryGraduationCohort)
		{
			poParent->poasPrimaryGraduationCohort = asPrimaryGraduationCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->NewNode(poParent);
		}
		else
		{
			asPrimaryGraduationCohort[poParent->sex][poParent->province_birth][poParent->educ_mother]->Remove(poParent->poasPrimaryGraduationCohort);
			poParent->poasPrimaryGraduationCohort = NULL;
		}
	}
	return poParent->actorset_filter_asPrimaryGraduationCohort = value;
}

logical PersonStateFunctions::Set_actorset_filter_asResidentsAgeSex( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asResidentsAgeSex != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asResidentsAgeSex;
		poParent->actorset_filter_asResidentsAgeSex = value;

		// actorset filter
		if (poParent->actorset_filter_asResidentsAgeSex)
		{
			poParent->poasResidentsAgeSex = asResidentsAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->NewNode(poParent);
		}
		else
		{
			asResidentsAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->Remove(poParent->poasResidentsAgeSex);
			poParent->poasResidentsAgeSex = NULL;
		}
	}
	return poParent->actorset_filter_asResidentsAgeSex = value;
}

logical PersonStateFunctions::Set_actorset_filter_asResidentsPrimAgeSex( logical value )
{

	value = value != 0;
	if ( poParent->actorset_filter_asResidentsPrimAgeSex != value )
	{


		logical	OldValue;
		OldValue = poParent->actorset_filter_asResidentsPrimAgeSex;
		poParent->actorset_filter_asResidentsPrimAgeSex = value;

		// actorset filter
		if (poParent->actorset_filter_asResidentsPrimAgeSex)
		{
			poParent->poasResidentsPrimAgeSex = asResidentsPrimAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->NewNode(poParent);
		}
		else
		{
			asResidentsPrimAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->Remove(poParent->poasResidentsPrimAgeSex);
			poParent->poasResidentsPrimAgeSex = NULL;
		}
	}
	return poParent->actorset_filter_asResidentsPrimAgeSex = value;
}

AGE_UNION PersonStateFunctions::Set_age_union( AGE_UNION value )
{

	CHKLMT(value, age_union, r_min_AGE_UNION, r_max_AGE_UNION);
	if ( value < r_min_AGE_UNION ) {
		value = r_min_AGE_UNION;
	}
	else if ( value > r_max_AGE_UNION ) {
		value = r_max_AGE_UNION;
	}
	if ( poParent->age_union != value )
	{


		AGE_UNION	OldValue;
		OldValue = poParent->age_union;
		poParent->age_union = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[33] >= 0 && ( poParent->tableinfo.tab33dim_filter || poParent->table_filter_TabUnionFormation ) ) {
				if ( poParent->tableinfo.tab33events == -1 || poParent->tableinfo.tab33events < poParent->__events ) {
					if ( poParent->tableinfo.tab33events != -1 ) {
						poParent->tableinfo.tab33open0 = (real) poParent->CloseUpdate( 33, 0, poParent->tableinfo.tab33open0, poParent->tableinfo.tab33close0, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open1 = (real) poParent->CloseUpdate( 33, 1, poParent->tableinfo.tab33open1, poParent->tableinfo.tab33close1, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open2 = (real) poParent->CloseUpdate( 33, 2, poParent->tableinfo.tab33open2, poParent->tableinfo.tab33close2, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open3 = (real) poParent->CloseUpdate( 33, 3, poParent->tableinfo.tab33open3, poParent->tableinfo.tab33close3, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open4 = (real) poParent->CloseUpdate( 33, 4, poParent->tableinfo.tab33open4, poParent->tableinfo.tab33close4, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab33events = poParent->__events;
					poParent->tableinfo.tab33close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab33close1 = (real) poDerivedStates->Get_duration_in_union_FALSE();
					poParent->tableinfo.tab33close2 = (real) poDerivedStates->Get_duration_in_union_TRUE();
					poParent->tableinfo.tab33close3 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab33close4 = (real) poParent->union_formation_hazard_max_value_in;
					poParent->tableinfo.tab33cell = (index) poParent->TableCellIndex( 33, 3,
						poParent->primary_level,
						OldValue - r_min_AGE_UNION,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab33total_cell = (index) poParent->tableinfo.tab33cell;
					}
					else {
						poParent->tableinfo.tab33total_cell = (index) poParent->TableTotalCellIndex( 33, 3,
						poParent->primary_level,
						OldValue - r_min_AGE_UNION,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab33dim_filter = poParent->table_filter_TabUnionFormation;
				}
			}
		}
	}
	return poParent->age_union = value;
}

int PersonStateFunctions::Set_birth_age_part( int value )
{

	if ( poParent->birth_age_part != value )
	{


		int	OldValue;
		OldValue = poParent->birth_age_part;
		poParent->birth_age_part = value;

		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
	}
	return poParent->birth_age_part = value;
}

integer PersonStateFunctions::Set_calendar_year( integer value )
{

	if ( poParent->calendar_year != value )
	{
#line 325 "code/Tables.mpp"
		Set_table_filter_TabEduc2015( value== 2015 );
#line 5862 "modgen/src/ACTORS.CPP"
#line 338 "code/Tables.mpp"
		Set_table_filter_TabEduc2040( value== 2040 );
#line 5865 "modgen/src/ACTORS.CPP"
#line 351 "code/Tables.mpp"
		Set_table_filter_TabEduc2065( value== 2065 );
#line 5868 "modgen/src/ACTORS.CPP"


		integer	OldValue;
		OldValue = poParent->calendar_year;
		poParent->calendar_year = value;

#line 277 "code/ChildMortality.mpp"
		Set_child_mortality_overrides( poParent->calendar_year>= MIN(CHILD_MORTALITY_YEARS)&& gprParam->SelectedChildMortalityModel!= SCMM_MACRO&& poParent->integer_age<= MAX(CHILD_MORTALITY_AGE)&& poParent->mother_known );
#line 5877 "modgen/src/ACTORS.CPP"
#line 83 "code/PersonCore.mpp"
		Set_in_projected_time( ( poParent->calendar_year>= MIN(SIM_YEAR_RANGE) ) );
#line 5880 "modgen/src/ACTORS.CPP"
#line 52 "code/CmapReportTables.mpp"
		Set_in_rtab_sel_years( poParent->calendar_year== 2017|| poParent->calendar_year== 2030|| poParent->calendar_year== 2050 );
#line 5883 "modgen/src/ACTORS.CPP"
#line 48 "code/CmapReportTables.mpp"
		Set_in_rtab_years( WITHIN(RTAB_YEARS,poParent->calendar_year) );
#line 5886 "modgen/src/ACTORS.CPP"
#line 135 "code/Fertility.mpp"
		Set_in_simulation_time( ( poParent->calendar_year>= MIN(SIM_YEAR_RANGE) )? TRUE: FALSE );
#line 5889 "modgen/src/ACTORS.CPP"
#line 155 "code/Fertility.mpp"
		Set_is_potential_mother( ( poParent->set_alive&& poParent->sex== FEMALE&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& poParent->parity< MAX(PARITY_RANGE)&& WITHIN(SIM_YEAR_RANGE,poParent->calendar_year) )? TRUE: FALSE );
#line 5892 "modgen/src/ACTORS.CPP"
#line 53 "code/CmapReportTables.mpp"
		Set_rtab_sel_years( ( poParent->calendar_year== 2017 )? SY_2017: ( ( poParent->calendar_year== 2030 )? SY_2030: SY_2050 ) );
#line 5895 "modgen/src/ACTORS.CPP"
#line 49 "code/CmapReportTables.mpp"
		Set_rtab_years( COERCE(RTAB_YEARS,poParent->calendar_year) );
#line 5898 "modgen/src/ACTORS.CPP"
#line 82 "code/PersonCore.mpp"
		Set_sim_year( COERCE(SIM_YEAR_RANGE,poParent->calendar_year) );
#line 5901 "modgen/src/ACTORS.CPP"
#line 65 "code/UnionFormation.mpp"
		Set_union_formation_hazard( ( WITHIN(AGE_UNION,poParent->integer_age)&& poParent->sex== FEMALE )? gprParam->UnionFormationHazard[CHKBND(poParent->primary_level, 3, "UnionFormationHazard", 0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE,poParent->calendar_year), 101, "UnionFormationHazard", 1)][CHKBND(RANGE_POS(AGE_UNION,poParent->integer_age), 51, "UnionFormationHazard", 2)]: 0.0 );
#line 5904 "modgen/src/ACTORS.CPP"
		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
		if ( poParent->poEmigrationEvent->cRecalc == 0 ) {
			poParent->poEmigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poEmigrationEvent );
		};
		if ( poParent->poMortalityEvent->cRecalc == 0 ) {
			poParent->poMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMortalityEvent );
		};
		if ( poParent->poUnionFormationEvent->cRecalc == 0 ) {
			poParent->poUnionFormationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poUnionFormationEvent );
		};
	}
	return poParent->calendar_year = value;
}

double PersonStateFunctions::Set_child_mortality( double value )
{

	if ( poParent->child_mortality != value )
	{


		double	OldValue;
		OldValue = poParent->child_mortality;
		poParent->child_mortality = value;

		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
	}
	return poParent->child_mortality = value;
}

logical PersonStateFunctions::Set_child_mortality_overrides( logical value )
{

	value = value != 0;
	if ( poParent->child_mortality_overrides != value )
	{


		logical	OldValue;
		OldValue = poParent->child_mortality_overrides;
		poParent->child_mortality_overrides = value;

		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
		if ( poParent->poMortalityEvent->cRecalc == 0 ) {
			poParent->poMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMortalityEvent );
		};
	}
	return poParent->child_mortality_overrides = value;
}

logical PersonStateFunctions::Set_count_parity( logical value )
{

	value = value != 0;
	if ( poParent->count_parity != value )
	{
#line 148 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_AgeAtFirstUnionFormation( poParent->sex== FEMALE&& poParent->in_rtab_years&& value );
#line 5980 "modgen/src/ACTORS.CPP"
#line 126 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_BirthsByAgeMother( poParent->in_rtab_years&& value );
#line 5983 "modgen/src/ACTORS.CPP"
#line 136 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_UnionFormationByAge( poParent->in_rtab_years&& value );
#line 5986 "modgen/src/ACTORS.CPP"
#line 183 "code/Tables.mpp"
		Set_table_filter_TabAgeAtBirth( poParent->in_projected_time&& poParent->sex== FEMALE&& value );
#line 5989 "modgen/src/ACTORS.CPP"
#line 388 "code/Tables.mpp"
		Set_table_filter_TabAgeAtFirstUnionFormation( poParent->sex== FEMALE&& poParent->in_projected_time&& value );
#line 5992 "modgen/src/ACTORS.CPP"
#line 172 "code/Tables.mpp"
		Set_table_filter_TabBirthsByAgeGroup( poParent->in_projected_time&& value );
#line 5995 "modgen/src/ACTORS.CPP"
#line 152 "code/Tables.mpp"
		Set_table_filter_TabFertilityRates( poParent->in_projected_time&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& poParent->sex== FEMALE&& value );
#line 5998 "modgen/src/ACTORS.CPP"
#line 370 "code/Tables.mpp"
		Set_table_filter_TabUnionFormation( WITHIN(AGE_UNION,poParent->integer_age)&& poParent->sex== FEMALE&& poParent->in_projected_time&& value );
#line 6001 "modgen/src/ACTORS.CPP"


		logical	OldValue;
		OldValue = poParent->count_parity;
		poParent->count_parity = value;

	}
	return poParent->count_parity = value;
}

FERTILE_AGE_RANGE PersonStateFunctions::Set_fert_age( FERTILE_AGE_RANGE value )
{

	CHKLMT(value, fert_age, r_min_FERTILE_AGE_RANGE, r_max_FERTILE_AGE_RANGE);
	if ( value < r_min_FERTILE_AGE_RANGE ) {
		value = r_min_FERTILE_AGE_RANGE;
	}
	else if ( value > r_max_FERTILE_AGE_RANGE ) {
		value = r_max_FERTILE_AGE_RANGE;
	}
	if ( poParent->fert_age != value )
	{


		FERTILE_AGE_RANGE	OldValue;
		OldValue = poParent->fert_age;
		poParent->fert_age = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[16] >= 0 && ( poParent->tableinfo.tab16dim_filter || poParent->table_filter_TabFertilityRates ) ) {
				if ( poParent->tableinfo.tab16events == -1 || poParent->tableinfo.tab16events < poParent->__events ) {
					if ( poParent->tableinfo.tab16events != -1 ) {
						poParent->tableinfo.tab16open0 = (real) poParent->CloseUpdate( 16, 0, poParent->tableinfo.tab16open0, poParent->tableinfo.tab16close0, poParent->tableinfo.tab16dim_filter, poParent->tableinfo.tab16cell, poParent->tableinfo.tab16total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab16open1 = (real) poParent->CloseUpdate( 16, 1, poParent->tableinfo.tab16open1, poParent->tableinfo.tab16close1, poParent->tableinfo.tab16dim_filter, poParent->tableinfo.tab16cell, poParent->tableinfo.tab16total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab16events = poParent->__events;
					poParent->tableinfo.tab16close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab16close1 = (real) poParent->parity;
					poParent->tableinfo.tab16cell = (index) poParent->TableCellIndex( 16, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue - r_min_FERTILE_AGE_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab16total_cell = (index) poParent->tableinfo.tab16cell;
					}
					else {
						poParent->tableinfo.tab16total_cell = (index) poParent->TableTotalCellIndex( 16, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue - r_min_FERTILE_AGE_RANGE );
					}
					poParent->tableinfo.tab16dim_filter = poParent->table_filter_TabFertilityRates;
				}
			}
		}
	}
	return poParent->fert_age = value;
}

FERTILE_AGE_RANGE PersonStateFunctions::Set_fertile_age( FERTILE_AGE_RANGE value )
{

	CHKLMT(value, fertile_age, r_min_FERTILE_AGE_RANGE, r_max_FERTILE_AGE_RANGE);
	if ( value < r_min_FERTILE_AGE_RANGE ) {
		value = r_min_FERTILE_AGE_RANGE;
	}
	else if ( value > r_max_FERTILE_AGE_RANGE ) {
		value = r_max_FERTILE_AGE_RANGE;
	}
	if ( poParent->fertile_age != value )
	{


		FERTILE_AGE_RANGE	OldValue;
		OldValue = poParent->fertile_age;
		poParent->fertile_age = value;

		// actorset dimension or order state
		if (poParent->actorset_filter_asPotentialMothersByAge)
		{
			asPotentialMothersByAge[RANGE_POS(FERTILE_AGE_RANGE,OldValue)]->Remove(poParent->poasPotentialMothersByAge);
			poParent->poasPotentialMothersByAge = asPotentialMothersByAge[RANGE_POS(FERTILE_AGE_RANGE, poParent->fertile_age)]->NewNode(poParent);
		}
	}
	return poParent->fertile_age = value;
}

logical PersonStateFunctions::Set_in_capital( logical value )
{

	value = value != 0;
	if ( poParent->in_capital != value )
	{


		logical	OldValue;
		OldValue = poParent->in_capital;
		poParent->in_capital = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[30] >= 0 && ( poParent->tableinfo.tab30dim_filter || poParent->table_filter_TabEduc2015 ) ) {
				if ( poParent->tableinfo.tab30events == -1 || poParent->tableinfo.tab30events < poParent->__events ) {
					if ( poParent->tableinfo.tab30events != -1 ) {
						poParent->tableinfo.tab30open0 = (real) poParent->CloseUpdate( 30, 0, poParent->tableinfo.tab30open0, poParent->tableinfo.tab30close0, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open1 = (real) poParent->CloseUpdate( 30, 1, poParent->tableinfo.tab30open1, poParent->tableinfo.tab30close1, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open2 = (real) poParent->CloseUpdate( 30, 2, poParent->tableinfo.tab30open2, poParent->tableinfo.tab30close2, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab30events = poParent->__events;
					poParent->tableinfo.tab30close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab30close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab30close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab30cell = (index) poParent->TableCellIndex( 30, 3,
						OldValue,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab30total_cell = (index) poParent->tableinfo.tab30cell;
					}
					else {
						poParent->tableinfo.tab30total_cell = (index) poParent->TableTotalCellIndex( 30, 3,
						OldValue,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab30dim_filter = poParent->table_filter_TabEduc2015;
				}
			}
			if ( gplTableIndexes[31] >= 0 && ( poParent->tableinfo.tab31dim_filter || poParent->table_filter_TabEduc2040 ) ) {
				if ( poParent->tableinfo.tab31events == -1 || poParent->tableinfo.tab31events < poParent->__events ) {
					if ( poParent->tableinfo.tab31events != -1 ) {
						poParent->tableinfo.tab31open0 = (real) poParent->CloseUpdate( 31, 0, poParent->tableinfo.tab31open0, poParent->tableinfo.tab31close0, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open1 = (real) poParent->CloseUpdate( 31, 1, poParent->tableinfo.tab31open1, poParent->tableinfo.tab31close1, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open2 = (real) poParent->CloseUpdate( 31, 2, poParent->tableinfo.tab31open2, poParent->tableinfo.tab31close2, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab31events = poParent->__events;
					poParent->tableinfo.tab31close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab31close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab31close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab31cell = (index) poParent->TableCellIndex( 31, 3,
						OldValue,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab31total_cell = (index) poParent->tableinfo.tab31cell;
					}
					else {
						poParent->tableinfo.tab31total_cell = (index) poParent->TableTotalCellIndex( 31, 3,
						OldValue,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab31dim_filter = poParent->table_filter_TabEduc2040;
				}
			}
			if ( gplTableIndexes[32] >= 0 && ( poParent->tableinfo.tab32dim_filter || poParent->table_filter_TabEduc2065 ) ) {
				if ( poParent->tableinfo.tab32events == -1 || poParent->tableinfo.tab32events < poParent->__events ) {
					if ( poParent->tableinfo.tab32events != -1 ) {
						poParent->tableinfo.tab32open0 = (real) poParent->CloseUpdate( 32, 0, poParent->tableinfo.tab32open0, poParent->tableinfo.tab32close0, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open1 = (real) poParent->CloseUpdate( 32, 1, poParent->tableinfo.tab32open1, poParent->tableinfo.tab32close1, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open2 = (real) poParent->CloseUpdate( 32, 2, poParent->tableinfo.tab32open2, poParent->tableinfo.tab32close2, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab32events = poParent->__events;
					poParent->tableinfo.tab32close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab32close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab32close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab32cell = (index) poParent->TableCellIndex( 32, 3,
						OldValue,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab32total_cell = (index) poParent->tableinfo.tab32cell;
					}
					else {
						poParent->tableinfo.tab32total_cell = (index) poParent->TableTotalCellIndex( 32, 3,
						OldValue,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab32dim_filter = poParent->table_filter_TabEduc2065;
				}
			}
		}
	}
	return poParent->in_capital = value;
}

logical PersonStateFunctions::Set_in_projected_time( logical value )
{

	value = value != 0;
	if ( poParent->in_projected_time != value )
	{
		bool bScheduleTimeChange = false;	//indicates that schedule event should be recalculated
		if (	(poParent->schedule_event_time == poDerivedStates->trigger_entrances_in_projected_time_TRUE_time))
		{
			//force timeScheduleEvent to calculate min time
			poParent->schedule_event_time = TIME_INFINITE;
		//indicate that schedule event needs to be recalculated
			bScheduleTimeChange = true;
		}
#line 183 "code/Tables.mpp"
		Set_table_filter_TabAgeAtBirth( value&& poParent->sex== FEMALE&& poParent->count_parity );
#line 6202 "modgen/src/ACTORS.CPP"
#line 388 "code/Tables.mpp"
		Set_table_filter_TabAgeAtFirstUnionFormation( poParent->sex== FEMALE&& value&& poParent->count_parity );
#line 6205 "modgen/src/ACTORS.CPP"
#line 172 "code/Tables.mpp"
		Set_table_filter_TabBirthsByAgeGroup( value&& poParent->count_parity );
#line 6208 "modgen/src/ACTORS.CPP"
#line 221 "code/Tables.mpp"
		Set_table_filter_TabCohortChildMortality( value&& WITHIN(SIM_YOB_RANGE,poParent->year_of_birth)&& WITHIN(CHILD_MORT_RANGE,poParent->integer_age) );
#line 6211 "modgen/src/ACTORS.CPP"
#line 198 "code/Tables.mpp"
		Set_table_filter_TabDeathRates( value );
#line 6214 "modgen/src/ACTORS.CPP"
#line 210 "code/Tables.mpp"
		Set_table_filter_TabDeathsByAgeGroup( value );
#line 6217 "modgen/src/ACTORS.CPP"
#line 299 "code/Tables.mpp"
		Set_table_filter_TabEduc12ByPob( poParent->integer_age== 12&& value );
#line 6220 "modgen/src/ACTORS.CPP"
#line 312 "code/Tables.mpp"
		Set_table_filter_TabEduc12ByPor( poParent->integer_age== 12&& value );
#line 6223 "modgen/src/ACTORS.CPP"
#line 267 "code/Tables.mpp"
		Set_table_filter_TabEmigration( value );
#line 6226 "modgen/src/ACTORS.CPP"
#line 152 "code/Tables.mpp"
		Set_table_filter_TabFertilityRates( value&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& poParent->sex== FEMALE&& poParent->count_parity );
#line 6229 "modgen/src/ACTORS.CPP"
#line 255 "code/Tables.mpp"
		Set_table_filter_TabImmigrationSexProvinceAge( poParent->person_type== PT_IMMIGRANT&& value&& poDerivedStates->trigger_entrances_set_alive_TRUE );
#line 6232 "modgen/src/ACTORS.CPP"
#line 244 "code/Tables.mpp"
		Set_table_filter_TabInternalMigrationRate( value );
#line 6235 "modgen/src/ACTORS.CPP"
#line 405 "code/Tables.mpp"
		Set_table_filter_TabMothersExperience( value&& WITHIN(SIM_YOB_RANGE,poParent->year_of_birth)&& poParent->person_type== PT_CHILD&& poParent->sex== FEMALE );
#line 6238 "modgen/src/ACTORS.CPP"
#line 162 "code/Tables.mpp"
		Set_table_filter_TabNumberBirths( value&& poParent->person_type== PT_CHILD );
#line 6241 "modgen/src/ACTORS.CPP"
#line 284 "code/Tables.mpp"
		Set_table_filter_TabPopProvAgeEducSex( value );
#line 6244 "modgen/src/ACTORS.CPP"
#line 124 "code/Tables.mpp"
		Set_table_filter_TabPopulationAgeSexProvince( value );
#line 6247 "modgen/src/ACTORS.CPP"
#line 113 "code/Tables.mpp"
		Set_table_filter_TabTotalPopulationSexProvince( value );
#line 6250 "modgen/src/ACTORS.CPP"
#line 370 "code/Tables.mpp"
		Set_table_filter_TabUnionFormation( WITHIN(AGE_UNION,poParent->integer_age)&& poParent->sex== FEMALE&& value&& poParent->count_parity );
#line 6253 "modgen/src/ACTORS.CPP"


		logical	OldValue;
		OldValue = poParent->in_projected_time;
		poParent->in_projected_time = value;

		if ( value == TRUE ) 
		{
			poDerivedStates->Set_trigger_entrances_in_projected_time_TRUE( TRUE );
			poDerivedStates->trigger_entrances_in_projected_time_TRUE_time = poParent->__time;
			if ( poParent->schedule_event_time > poDerivedStates->trigger_entrances_in_projected_time_TRUE_time)
			{
				poParent->schedule_event_time = poDerivedStates->trigger_entrances_in_projected_time_TRUE_time;
				//indicate that schedule event should be recalculated
				bScheduleTimeChange = true;
			}
		}
		if ( poParent->poMigrationEvent->cRecalc == 0 ) {
			poParent->poMigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMigrationEvent );
		};
		if ( poParent->poMortalityEvent->cRecalc == 0 ) {
			poParent->poMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMortalityEvent );
		};

		if (bScheduleTimeChange && poParent->poScheduleEvent->cRecalc == 0)
		{
			poParent->poScheduleEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent(poParent->poScheduleEvent);
		}
	}
	return poParent->in_projected_time = value;
}

logical PersonStateFunctions::Set_in_rtab_sel_years( logical value )
{

	value = value != 0;
	if ( poParent->in_rtab_sel_years != value )
	{
#line 80 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_PopPyramidByEduc( value );
#line 6297 "modgen/src/ACTORS.CPP"


		logical	OldValue;
		OldValue = poParent->in_rtab_sel_years;
		poParent->in_rtab_sel_years = value;

	}
	return poParent->in_rtab_sel_years = value;
}

logical PersonStateFunctions::Set_in_rtab_years( logical value )
{

	value = value != 0;
	if ( poParent->in_rtab_years != value )
	{
#line 148 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_AgeAtFirstUnionFormation( poParent->sex== FEMALE&& value&& poParent->count_parity );
#line 6316 "modgen/src/ACTORS.CPP"
#line 126 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_BirthsByAgeMother( value&& poParent->count_parity );
#line 6319 "modgen/src/ACTORS.CPP"
#line 116 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_BirthsByEducMother( value&& poParent->person_type== PT_CHILD );
#line 6322 "modgen/src/ACTORS.CPP"
#line 106 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_BirthsByRegion( value&& poParent->person_type== PT_CHILD );
#line 6325 "modgen/src/ACTORS.CPP"
#line 95 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_DeathsByRegion( value );
#line 6328 "modgen/src/ACTORS.CPP"
#line 169 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_EducationActivePop( poParent->integer_age> 15&& poParent->integer_age< 60&& value );
#line 6331 "modgen/src/ACTORS.CPP"
#line 182 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_PrimSchoolEntries( value );
#line 6334 "modgen/src/ACTORS.CPP"
#line 193 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_PrimSchoolGraduations( value );
#line 6337 "modgen/src/ACTORS.CPP"
#line 159 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_TabDeathsByAgeGroup( value&& poParent->integer_age< 5 );
#line 6340 "modgen/src/ACTORS.CPP"
#line 60 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_TotalPopReg( value );
#line 6343 "modgen/src/ACTORS.CPP"
#line 70 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_TotalPopWillaya( value );
#line 6346 "modgen/src/ACTORS.CPP"
#line 136 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_UnionFormationByAge( value&& poParent->count_parity );
#line 6349 "modgen/src/ACTORS.CPP"


		logical	OldValue;
		OldValue = poParent->in_rtab_years;
		poParent->in_rtab_years = value;

	}
	return poParent->in_rtab_years = value;
}

logical PersonStateFunctions::Set_in_simulation_time( logical value )
{

	value = value != 0;
	if ( poParent->in_simulation_time != value )
	{


		logical	OldValue;
		OldValue = poParent->in_simulation_time;
		poParent->in_simulation_time = value;

#line 138 "code/Fertility.mpp"
		Set_in_this_parity_in_simulation( poParent->in_simulation_time&& poParent->in_this_parity );
#line 6374 "modgen/src/ACTORS.CPP"
	}
	return poParent->in_simulation_time = value;
}

logical PersonStateFunctions::Set_in_this_parity_in_simulation( logical value )
{

	value = value != 0;
	if ( poParent->in_this_parity_in_simulation != value )
	{
		bool bScheduleTimeChange = false;	//indicates that schedule event should be recalculated
		if (	(poParent->schedule_event_time == poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time))
		{
			//force timeScheduleEvent to calculate min time
			poParent->schedule_event_time = TIME_INFINITE;
		//indicate that schedule event needs to be recalculated
			bScheduleTimeChange = true;
		}
		if ( poParent->in_this_parity_in_simulation == TRUE ) {
			poDerivedStates->completed_spell_duration_in_this_parity_in_simulation_TRUE = poDerivedStates->Get_active_spell_duration_in_this_parity_in_simulation_TRUE();
			poDerivedStates->active_spell_duration_in_this_parity_in_simulation_TRUE = (TIME) 0;
		}
		else if ( value == TRUE ) {
			poDerivedStates->active_spell_duration_in_this_parity_in_simulation_TRUE_time = poParent->__time;
		}
		poDerivedStates->Set_ssint_active_spell_duration_in_this_parity_in_simulation_TRUE( 0 );
		if ( value == TRUE ) {
			poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time = WAIT1(poParent->time, 1);
			if ( (!bScheduleTimeChange || poParent->schedule_event_time != TIME_INFINITE) && poParent->schedule_event_time > poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time)
			{
				poParent->schedule_event_time = poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time;
				//indicate that schedule event should be recalculated
				bScheduleTimeChange = true;
			}
		}
		else
		{
			poDerivedStates->ssint_active_spell_duration_in_this_parity_in_simulation_TRUE_time = TIME_INFINITE;
		}


		logical	OldValue;
		OldValue = poParent->in_this_parity_in_simulation;
		poParent->in_this_parity_in_simulation = value;


		if (bScheduleTimeChange && poParent->poScheduleEvent->cRecalc == 0)
		{
			poParent->poScheduleEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent(poParent->poScheduleEvent);
		}
	}
	return poParent->in_this_parity_in_simulation = value;
}

AGE_RANGE PersonStateFunctions::Set_integer_age( AGE_RANGE value )
{

	CHKLMT(value, integer_age, r_min_AGE_RANGE, r_max_AGE_RANGE);
	if ( value < r_min_AGE_RANGE ) {
		value = r_min_AGE_RANGE;
	}
	else if ( value > r_max_AGE_RANGE ) {
		value = r_max_AGE_RANGE;
	}
	if ( poParent->integer_age != value )
	{
#line 19 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asPrimaryEntryCohort( poParent->set_alive&& value== gprParam->AgeEnterPrimary );
#line 6444 "modgen/src/ACTORS.CPP"
#line 23 "code/PrimaryEducation.mpp"
		Set_actorset_filter_asPrimaryGraduationCohort( poParent->set_alive&& value== gprParam->AgeGradPrimary&& poParent->primary_level!= PL_NO );
#line 6447 "modgen/src/ACTORS.CPP"
#line 169 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_EducationActivePop( value> 15&& value< 60&& poParent->in_rtab_years );
#line 6450 "modgen/src/ACTORS.CPP"
#line 159 "code/CmapReportTables.mpp"
		Set_table_filter_RTAB_TabDeathsByAgeGroup( poParent->in_rtab_years&& value< 5 );
#line 6453 "modgen/src/ACTORS.CPP"
#line 221 "code/Tables.mpp"
		Set_table_filter_TabCohortChildMortality( poParent->in_projected_time&& WITHIN(SIM_YOB_RANGE,poParent->year_of_birth)&& WITHIN(CHILD_MORT_RANGE,value) );
#line 6456 "modgen/src/ACTORS.CPP"
#line 299 "code/Tables.mpp"
		Set_table_filter_TabEduc12ByPob( value== 12&& poParent->in_projected_time );
#line 6459 "modgen/src/ACTORS.CPP"
#line 312 "code/Tables.mpp"
		Set_table_filter_TabEduc12ByPor( value== 12&& poParent->in_projected_time );
#line 6462 "modgen/src/ACTORS.CPP"
#line 152 "code/Tables.mpp"
		Set_table_filter_TabFertilityRates( poParent->in_projected_time&& WITHIN(FERTILE_AGE_RANGE,value)&& poParent->sex== FEMALE&& poParent->count_parity );
#line 6465 "modgen/src/ACTORS.CPP"
#line 370 "code/Tables.mpp"
		Set_table_filter_TabUnionFormation( WITHIN(AGE_UNION,value)&& poParent->sex== FEMALE&& poParent->in_projected_time&& poParent->count_parity );
#line 6468 "modgen/src/ACTORS.CPP"
		if ( poParent->integer_age == 14 && value == 15 ) {
			poDerivedStates->Set_transitions_integer_age_14_15( poDerivedStates->transitions_integer_age_14_15 + 1 );
		}
		if ( poParent->integer_age == 19 && value == 20 ) {
			poDerivedStates->Set_transitions_integer_age_19_20( poDerivedStates->transitions_integer_age_19_20 + 1 );
		}
		if ( poParent->integer_age == 9 && value == 10 ) {
			poDerivedStates->Set_transitions_integer_age_9_10( poDerivedStates->transitions_integer_age_9_10 + 1 );
		}


		AGE_RANGE	OldValue;
		OldValue = poParent->integer_age;
		poParent->integer_age = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[15] >= 0 && ( poParent->tableinfo.tab15dim_filter || poParent->table_filter_TabSimulatedStartingPopulation ) ) {
				if ( poParent->tableinfo.tab15events == -1 || poParent->tableinfo.tab15events < poParent->__events ) {
					if ( poParent->tableinfo.tab15events != -1 ) {
						Set_table_unit_TabSimulatedStartingPopulation(poParent->table_unit_TabSimulatedStartingPopulation+1);
						poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
						poParent->tableinfo.tab15open0 = (real) poParent->CloseUpdate( 15, 0, poParent->tableinfo.tab15open0, poParent->tableinfo.tab15close0, poParent->tableinfo.tab15dim_filter, poParent->tableinfo.tab15cell, poParent->tableinfo.tab15total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab15events = poParent->__events;
					poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
					poParent->tableinfo.tab15cell = (index) poParent->TableCellIndex( 15, 3,
						poParent->sex,
						OldValue - r_min_AGE_RANGE,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab15total_cell = (index) poParent->tableinfo.tab15cell;
					}
					else {
						poParent->tableinfo.tab15total_cell = (index) poParent->TableTotalCellIndex( 15, 3,
						poParent->sex,
						OldValue - r_min_AGE_RANGE,
						poParent->province_nat );
					}
					poParent->tableinfo.tab15dim_filter = poParent->table_filter_TabSimulatedStartingPopulation;
				}
			}
			if ( gplTableIndexes[20] >= 0 && ( poParent->tableinfo.tab20dim_filter || poParent->table_filter_TabDeathRates ) ) {
				if ( poParent->tableinfo.tab20events == -1 || poParent->tableinfo.tab20events < poParent->__events ) {
					if ( poParent->tableinfo.tab20events != -1 ) {
						poParent->tableinfo.tab20open0 = (real) poParent->CloseUpdate( 20, 0, poParent->tableinfo.tab20open0, poParent->tableinfo.tab20close0, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab20open1 = (real) poParent->CloseUpdate( 20, 1, poParent->tableinfo.tab20open1, poParent->tableinfo.tab20close1, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab20events = poParent->__events;
					poParent->tableinfo.tab20close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab20close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab20cell = (index) poParent->TableCellIndex( 20, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue - r_min_AGE_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab20total_cell = (index) poParent->tableinfo.tab20cell;
					}
					else {
						poParent->tableinfo.tab20total_cell = (index) poParent->TableTotalCellIndex( 20, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue - r_min_AGE_RANGE );
					}
					poParent->tableinfo.tab20dim_filter = poParent->table_filter_TabDeathRates;
				}
			}
		}
#line 60 "code/UnionFormation.mpp"
		Set_age_union( COERCE(AGE_UNION,poParent->integer_age) );
#line 6538 "modgen/src/ACTORS.CPP"
#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 6541 "modgen/src/ACTORS.CPP"
#line 277 "code/ChildMortality.mpp"
		Set_child_mortality_overrides( poParent->calendar_year>= MIN(CHILD_MORTALITY_YEARS)&& gprParam->SelectedChildMortalityModel!= SCMM_MACRO&& poParent->integer_age<= MAX(CHILD_MORTALITY_AGE)&& poParent->mother_known );
#line 6544 "modgen/src/ACTORS.CPP"
#line 86 "code/Tables.mpp"
		Set_fert_age( COERCE(FERTILE_AGE_RANGE,poParent->integer_age) );
#line 6547 "modgen/src/ACTORS.CPP"
#line 123 "code/Fertility.mpp"
		Set_fertile_age( COERCE(FERTILE_AGE_RANGE,poParent->integer_age) );
#line 6550 "modgen/src/ACTORS.CPP"
#line 55 "code/CmapReportTables.mpp"
		poParent->is_act_age = ( poParent->integer_age> 15&& poParent->integer_age< 60 );
#line 6553 "modgen/src/ACTORS.CPP"
#line 80 "code/Tables.mpp"
		Set_is_child( ( poParent->integer_age< 15 ) );
#line 6556 "modgen/src/ACTORS.CPP"
#line 81 "code/Tables.mpp"
		Set_is_old( ( poParent->integer_age>= 60 ) );
#line 6559 "modgen/src/ACTORS.CPP"
#line 155 "code/Fertility.mpp"
		Set_is_potential_mother( ( poParent->set_alive&& poParent->sex== FEMALE&& WITHIN(FERTILE_AGE_RANGE,poParent->integer_age)&& poParent->parity< MAX(PARITY_RANGE)&& WITHIN(SIM_YEAR_RANGE,poParent->calendar_year) )? TRUE: FALSE );
#line 6562 "modgen/src/ACTORS.CPP"
#line 88 "code/Tables.mpp"
		Set_sim_cmr( COERCE(CHILD_MORT_RANGE,poParent->integer_age) );
#line 6565 "modgen/src/ACTORS.CPP"
		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->integer_age, poDerivedStates->split_integer_age_AGE5_PART, 12, p_points_AGE5_PART );
			if ( nNewGrp != poDerivedStates->split_integer_age_AGE5_PART ) {
				poDerivedStates->Set_split_integer_age_AGE5_PART( nNewGrp );
			}
		}
		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->integer_age, poDerivedStates->split_integer_age_AGE_15_16_18, 3, p_points_AGE_15_16_18 );
			if ( nNewGrp != poDerivedStates->split_integer_age_AGE_15_16_18 ) {
				poDerivedStates->Set_split_integer_age_AGE_15_16_18( nNewGrp );
			}
		}
		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->integer_age, poDerivedStates->split_integer_age_AGE_15_17, 2, p_points_AGE_15_17 );
			if ( nNewGrp != poDerivedStates->split_integer_age_AGE_15_17 ) {
				poDerivedStates->Set_split_integer_age_AGE_15_17( nNewGrp );
			}
		}
		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->integer_age, poDerivedStates->split_integer_age_AGE_FIVE, 16, p_points_AGE_FIVE );
			if ( nNewGrp != poDerivedStates->split_integer_age_AGE_FIVE ) {
				poDerivedStates->Set_split_integer_age_AGE_FIVE( nNewGrp );
			}
		}
		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->integer_age, poDerivedStates->split_integer_age_AGE_MORT, 4, p_points_AGE_MORT );
			if ( nNewGrp != poDerivedStates->split_integer_age_AGE_MORT ) {
				poDerivedStates->Set_split_integer_age_AGE_MORT( nNewGrp );
			}
		}
		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->integer_age, poDerivedStates->split_integer_age_AGE_RTAB_MORT, 1, p_points_AGE_RTAB_MORT );
			if ( nNewGrp != poDerivedStates->split_integer_age_AGE_RTAB_MORT ) {
				poDerivedStates->Set_split_integer_age_AGE_RTAB_MORT( nNewGrp );
			}
		}
		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->integer_age, poDerivedStates->split_integer_age_BIRTH_AGE_PART, 3, p_points_BIRTH_AGE_PART );
			if ( nNewGrp != poDerivedStates->split_integer_age_BIRTH_AGE_PART ) {
				poDerivedStates->Set_split_integer_age_BIRTH_AGE_PART( nNewGrp );
			}
		}
#line 65 "code/UnionFormation.mpp"
		Set_union_formation_hazard( ( WITHIN(AGE_UNION,poParent->integer_age)&& poParent->sex== FEMALE )? gprParam->UnionFormationHazard[CHKBND(poParent->primary_level, 3, "UnionFormationHazard", 0)][CHKBND(RANGE_POS(SIM_YEAR_RANGE,poParent->calendar_year), 101, "UnionFormationHazard", 1)][CHKBND(RANGE_POS(AGE_UNION,poParent->integer_age), 51, "UnionFormationHazard", 2)]: 0.0 );
#line 6624 "modgen/src/ACTORS.CPP"
		// actorset dimension or order state
		if (poParent->actorset_filter_asFemaleAgeEduc)
		{
			asFemaleAgeEduc[RANGE_POS(AGE_RANGE,OldValue)][poParent->primary_level]->Remove(poParent->poasFemaleAgeEduc);
			poParent->poasFemaleAgeEduc = asFemaleAgeEduc[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->primary_level]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asForeignersAgeSexProv)
		{
			asForeignersAgeSexProv[RANGE_POS(AGE_RANGE,OldValue)][poParent->sex][poParent->province_nat]->Remove(poParent->poasForeignersAgeSexProv);
			poParent->poasForeignersAgeSexProv = asForeignersAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asForeignersPrimAgeSexProv)
		{
			asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE,OldValue)][poParent->sex][poParent->province_nat]->Remove(poParent->poasForeignersPrimAgeSexProv);
			poParent->poasForeignersPrimAgeSexProv = asForeignersPrimAgeSexProv[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex][poParent->province_nat]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asResidentsAgeSex)
		{
			asResidentsAgeSex[RANGE_POS(AGE_RANGE,OldValue)][poParent->sex]->Remove(poParent->poasResidentsAgeSex);
			poParent->poasResidentsAgeSex = asResidentsAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->NewNode(poParent);
		}
		if (poParent->actorset_filter_asResidentsPrimAgeSex)
		{
			asResidentsPrimAgeSex[RANGE_POS(AGE_RANGE,OldValue)][poParent->sex]->Remove(poParent->poasResidentsPrimAgeSex);
			poParent->poasResidentsPrimAgeSex = asResidentsPrimAgeSex[RANGE_POS(AGE_RANGE, poParent->integer_age)][poParent->sex]->NewNode(poParent);
		}
		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
		if ( poParent->poChildMortalityEvent->cRecalc == 0 ) {
			poParent->poChildMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poChildMortalityEvent );
		};
		if ( poParent->poEmigrationEvent->cRecalc == 0 ) {
			poParent->poEmigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poEmigrationEvent );
		};
		if ( poParent->poMigrationEvent->cRecalc == 0 ) {
			poParent->poMigrationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMigrationEvent );
		};
		if ( poParent->poMortalityEvent->cRecalc == 0 ) {
			poParent->poMortalityEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poMortalityEvent );
		};
		if ( poParent->poUnionFormationEvent->cRecalc == 0 ) {
			poParent->poUnionFormationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poUnionFormationEvent );
		};
	}
	return poParent->integer_age = value;
}

logical PersonStateFunctions::Set_is_aded0( logical value )
{

	value = value != 0;
	if ( poParent->is_aded0 != value )
	{
		if ( poParent->is_aded0 == TRUE ) {
			poDerivedStates->Set_duration_is_aded0_TRUE( poDerivedStates->Get_duration_is_aded0_TRUE() );
		}
		else if ( value == TRUE ) {
			poDerivedStates->duration_is_aded0_TRUE_time = poParent->__time;
		}


		logical	OldValue;
		OldValue = poParent->is_aded0;
		poParent->is_aded0 = value;

	}
	return poParent->is_aded0 = value;
}

logical PersonStateFunctions::Set_is_aded1( logical value )
{

	value = value != 0;
	if ( poParent->is_aded1 != value )
	{
		if ( poParent->is_aded1 == TRUE ) {
			poDerivedStates->Set_duration_is_aded1_TRUE( poDerivedStates->Get_duration_is_aded1_TRUE() );
		}
		else if ( value == TRUE ) {
			poDerivedStates->duration_is_aded1_TRUE_time = poParent->__time;
		}


		logical	OldValue;
		OldValue = poParent->is_aded1;
		poParent->is_aded1 = value;

	}
	return poParent->is_aded1 = value;
}

logical PersonStateFunctions::Set_is_aded2( logical value )
{

	value = value != 0;
	if ( poParent->is_aded2 != value )
	{
		if ( poParent->is_aded2 == TRUE ) {
			poDerivedStates->Set_duration_is_aded2_TRUE( poDerivedStates->Get_duration_is_aded2_TRUE() );
		}
		else if ( value == TRUE ) {
			poDerivedStates->duration_is_aded2_TRUE_time = poParent->__time;
		}


		logical	OldValue;
		OldValue = poParent->is_aded2;
		poParent->is_aded2 = value;

	}
	return poParent->is_aded2 = value;
}

logical PersonStateFunctions::Set_is_child( logical value )
{

	value = value != 0;
	if ( poParent->is_child != value )
	{
		if ( poParent->is_child == TRUE ) {
			poDerivedStates->Set_duration_is_child_TRUE( poDerivedStates->Get_duration_is_child_TRUE() );
		}
		else if ( value == TRUE ) {
			poDerivedStates->duration_is_child_TRUE_time = poParent->__time;
		}


		logical	OldValue;
		OldValue = poParent->is_child;
		poParent->is_child = value;

#line 82 "code/Tables.mpp"
		Set_is_aded0( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_NO ) );
#line 6765 "modgen/src/ACTORS.CPP"
#line 83 "code/Tables.mpp"
		Set_is_aded1( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_ENTER ) );
#line 6768 "modgen/src/ACTORS.CPP"
#line 84 "code/Tables.mpp"
		Set_is_aded2( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_GRAD ) );
#line 6771 "modgen/src/ACTORS.CPP"
	}
	return poParent->is_child = value;
}

logical PersonStateFunctions::Set_is_old( logical value )
{

	value = value != 0;
	if ( poParent->is_old != value )
	{
		if ( poParent->is_old == TRUE ) {
			poDerivedStates->Set_duration_is_old_TRUE( poDerivedStates->Get_duration_is_old_TRUE() );
		}
		else if ( value == TRUE ) {
			poDerivedStates->duration_is_old_TRUE_time = poParent->__time;
		}


		logical	OldValue;
		OldValue = poParent->is_old;
		poParent->is_old = value;

#line 82 "code/Tables.mpp"
		Set_is_aded0( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_NO ) );
#line 6796 "modgen/src/ACTORS.CPP"
#line 83 "code/Tables.mpp"
		Set_is_aded1( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_ENTER ) );
#line 6799 "modgen/src/ACTORS.CPP"
#line 84 "code/Tables.mpp"
		Set_is_aded2( ( !( poParent->is_child )&& !( poParent->is_old )&& poParent->primary_level== PL_GRAD ) );
#line 6802 "modgen/src/ACTORS.CPP"
	}
	return poParent->is_old = value;
}

logical PersonStateFunctions::Set_is_potential_mother( logical value )
{

	value = value != 0;
	if ( poParent->is_potential_mother != value )
	{
#line 27 "code/Fertility.mpp"
		Set_actorset_filter_asPotentialMothers( value );
#line 6815 "modgen/src/ACTORS.CPP"
#line 30 "code/Fertility.mpp"
		Set_actorset_filter_asPotentialMothersByAge( value );
#line 6818 "modgen/src/ACTORS.CPP"


		logical	OldValue;
		OldValue = poParent->is_potential_mother;
		poParent->is_potential_mother = value;

		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
	}
	return poParent->is_potential_mother = value;
}

RTAB_REG PersonStateFunctions::Set_rtab_reg( RTAB_REG value )
{

	CHKLMT(value, rtab_reg, 0, rc_size_RTAB_REG - 1);
	if ( value < 0 ) {
		value = (RTAB_REG) 0;
	}
	else if ( value >= rc_size_RTAB_REG ) {
		value = (RTAB_REG) ( rc_size_RTAB_REG - 1 );
	}
	if ( poParent->rtab_reg != value )
	{


		RTAB_REG	OldValue;
		OldValue = poParent->rtab_reg;
		poParent->rtab_reg = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[0] >= 0 && ( poParent->tableinfo.tab0dim_filter || poParent->table_filter_RTAB_TotalPopReg ) ) {
				if ( poParent->tableinfo.tab0events == -1 || poParent->tableinfo.tab0events < poParent->__events ) {
					if ( poParent->tableinfo.tab0events != -1 ) {
						poParent->tableinfo.tab0open0 = (real) poParent->CloseUpdate( 0, 0, poParent->tableinfo.tab0open0, poParent->tableinfo.tab0close0, poParent->tableinfo.tab0dim_filter, poParent->tableinfo.tab0cell, poParent->tableinfo.tab0total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab0events = poParent->__events;
					poParent->tableinfo.tab0close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab0cell = (index) poParent->TableCellIndex( 0, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab0total_cell = (index) poParent->tableinfo.tab0cell;
					}
					else {
						poParent->tableinfo.tab0total_cell = (index) poParent->TableTotalCellIndex( 0, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab0dim_filter = poParent->table_filter_RTAB_TotalPopReg;
				}
			}
			if ( gplTableIndexes[2] >= 0 && ( poParent->tableinfo.tab2dim_filter || poParent->table_filter_RTAB_PopPyramidByEduc ) ) {
				if ( poParent->tableinfo.tab2events == -1 || poParent->tableinfo.tab2events < poParent->__events ) {
					if ( poParent->tableinfo.tab2events != -1 ) {
						poParent->tableinfo.tab2open0 = (real) poParent->CloseUpdate( 2, 0, poParent->tableinfo.tab2open0, poParent->tableinfo.tab2close0, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open1 = (real) poParent->CloseUpdate( 2, 1, poParent->tableinfo.tab2open1, poParent->tableinfo.tab2close1, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open2 = (real) poParent->CloseUpdate( 2, 2, poParent->tableinfo.tab2open2, poParent->tableinfo.tab2close2, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab2events = poParent->__events;
					poParent->tableinfo.tab2close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab2close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab2close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab2cell = (index) poParent->TableCellIndex( 2, 4,
						poParent->rtab_sel_years,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab2total_cell = (index) poParent->tableinfo.tab2cell;
					}
					else {
						poParent->tableinfo.tab2total_cell = (index) poParent->TableTotalCellIndex( 2, 4,
						poParent->rtab_sel_years,
						poParent->sex,
						OldValue,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab2dim_filter = poParent->table_filter_RTAB_PopPyramidByEduc;
				}
			}
			if ( gplTableIndexes[3] >= 0 && ( poParent->tableinfo.tab3dim_filter || poParent->table_filter_RTAB_DeathsByRegion ) ) {
				if ( poParent->tableinfo.tab3events == -1 || poParent->tableinfo.tab3events < poParent->__events ) {
					if ( poParent->tableinfo.tab3events != -1 ) {
						poParent->tableinfo.tab3open0 = (real) poParent->CloseUpdate( 3, 0, poParent->tableinfo.tab3open0, poParent->tableinfo.tab3close0, poParent->tableinfo.tab3dim_filter, poParent->tableinfo.tab3cell, poParent->tableinfo.tab3total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab3events = poParent->__events;
					poParent->tableinfo.tab3close0 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab3cell = (index) poParent->TableCellIndex( 3, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab3total_cell = (index) poParent->tableinfo.tab3cell;
					}
					else {
						poParent->tableinfo.tab3total_cell = (index) poParent->TableTotalCellIndex( 3, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab3dim_filter = poParent->table_filter_RTAB_DeathsByRegion;
				}
			}
			if ( gplTableIndexes[4] >= 0 && ( poParent->tableinfo.tab4dim_filter || poParent->table_filter_RTAB_BirthsByRegion ) ) {
				if ( poParent->tableinfo.tab4events == -1 || poParent->tableinfo.tab4events < poParent->__events ) {
					if ( poParent->tableinfo.tab4events != -1 ) {
						poParent->tableinfo.tab4open0 = (real) poParent->CloseUpdate( 4, 0, poParent->tableinfo.tab4open0, poParent->tableinfo.tab4close0, poParent->tableinfo.tab4dim_filter, poParent->tableinfo.tab4cell, poParent->tableinfo.tab4total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab4events = poParent->__events;
					poParent->tableinfo.tab4close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab4cell = (index) poParent->TableCellIndex( 4, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab4total_cell = (index) poParent->tableinfo.tab4cell;
					}
					else {
						poParent->tableinfo.tab4total_cell = (index) poParent->TableTotalCellIndex( 4, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab4dim_filter = poParent->table_filter_RTAB_BirthsByRegion;
				}
			}
			if ( gplTableIndexes[10] >= 0 && ( poParent->tableinfo.tab10dim_filter || poParent->table_filter_RTAB_EducationActivePop ) ) {
				if ( poParent->tableinfo.tab10events == -1 || poParent->tableinfo.tab10events < poParent->__events ) {
					if ( poParent->tableinfo.tab10events != -1 ) {
						poParent->tableinfo.tab10open0 = (real) poParent->CloseUpdate( 10, 0, poParent->tableinfo.tab10open0, poParent->tableinfo.tab10close0, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open1 = (real) poParent->CloseUpdate( 10, 1, poParent->tableinfo.tab10open1, poParent->tableinfo.tab10close1, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open2 = (real) poParent->CloseUpdate( 10, 2, poParent->tableinfo.tab10open2, poParent->tableinfo.tab10close2, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open3 = (real) poParent->CloseUpdate( 10, 3, poParent->tableinfo.tab10open3, poParent->tableinfo.tab10close3, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab10events = poParent->__events;
					poParent->tableinfo.tab10close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab10close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab10close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab10close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab10cell = (index) poParent->TableCellIndex( 10, 2,
						OldValue,
						poParent->rtab_years - r_min_RTAB_YEARS );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab10total_cell = (index) poParent->tableinfo.tab10cell;
					}
					else {
						poParent->tableinfo.tab10total_cell = (index) poParent->TableTotalCellIndex( 10, 2,
						OldValue,
						poParent->rtab_years - r_min_RTAB_YEARS );
					}
					poParent->tableinfo.tab10dim_filter = poParent->table_filter_RTAB_EducationActivePop;
				}
			}
		}
	}
	return poParent->rtab_reg = value;
}

RTAB_SEL_YEARS PersonStateFunctions::Set_rtab_sel_years( RTAB_SEL_YEARS value )
{

	CHKLMT(value, rtab_sel_years, 0, rc_size_RTAB_SEL_YEARS - 1);
	if ( value < 0 ) {
		value = (RTAB_SEL_YEARS) 0;
	}
	else if ( value >= rc_size_RTAB_SEL_YEARS ) {
		value = (RTAB_SEL_YEARS) ( rc_size_RTAB_SEL_YEARS - 1 );
	}
	if ( poParent->rtab_sel_years != value )
	{


		RTAB_SEL_YEARS	OldValue;
		OldValue = poParent->rtab_sel_years;
		poParent->rtab_sel_years = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[2] >= 0 && ( poParent->tableinfo.tab2dim_filter || poParent->table_filter_RTAB_PopPyramidByEduc ) ) {
				if ( poParent->tableinfo.tab2events == -1 || poParent->tableinfo.tab2events < poParent->__events ) {
					if ( poParent->tableinfo.tab2events != -1 ) {
						poParent->tableinfo.tab2open0 = (real) poParent->CloseUpdate( 2, 0, poParent->tableinfo.tab2open0, poParent->tableinfo.tab2close0, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open1 = (real) poParent->CloseUpdate( 2, 1, poParent->tableinfo.tab2open1, poParent->tableinfo.tab2close1, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open2 = (real) poParent->CloseUpdate( 2, 2, poParent->tableinfo.tab2open2, poParent->tableinfo.tab2close2, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab2events = poParent->__events;
					poParent->tableinfo.tab2close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab2close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab2close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab2cell = (index) poParent->TableCellIndex( 2, 4,
						OldValue,
						poParent->sex,
						poParent->rtab_reg,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab2total_cell = (index) poParent->tableinfo.tab2cell;
					}
					else {
						poParent->tableinfo.tab2total_cell = (index) poParent->TableTotalCellIndex( 2, 4,
						OldValue,
						poParent->sex,
						poParent->rtab_reg,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab2dim_filter = poParent->table_filter_RTAB_PopPyramidByEduc;
				}
			}
		}
	}
	return poParent->rtab_sel_years = value;
}

RTAB_YEARS PersonStateFunctions::Set_rtab_years( RTAB_YEARS value )
{

	CHKLMT(value, rtab_years, r_min_RTAB_YEARS, r_max_RTAB_YEARS);
	if ( value < r_min_RTAB_YEARS ) {
		value = r_min_RTAB_YEARS;
	}
	else if ( value > r_max_RTAB_YEARS ) {
		value = r_max_RTAB_YEARS;
	}
	if ( poParent->rtab_years != value )
	{


		RTAB_YEARS	OldValue;
		OldValue = poParent->rtab_years;
		poParent->rtab_years = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[0] >= 0 && ( poParent->tableinfo.tab0dim_filter || poParent->table_filter_RTAB_TotalPopReg ) ) {
				if ( poParent->tableinfo.tab0events == -1 || poParent->tableinfo.tab0events < poParent->__events ) {
					if ( poParent->tableinfo.tab0events != -1 ) {
						poParent->tableinfo.tab0open0 = (real) poParent->CloseUpdate( 0, 0, poParent->tableinfo.tab0open0, poParent->tableinfo.tab0close0, poParent->tableinfo.tab0dim_filter, poParent->tableinfo.tab0cell, poParent->tableinfo.tab0total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab0events = poParent->__events;
					poParent->tableinfo.tab0close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab0cell = (index) poParent->TableCellIndex( 0, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab0total_cell = (index) poParent->tableinfo.tab0cell;
					}
					else {
						poParent->tableinfo.tab0total_cell = (index) poParent->TableTotalCellIndex( 0, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					}
					poParent->tableinfo.tab0dim_filter = poParent->table_filter_RTAB_TotalPopReg;
				}
			}
			if ( gplTableIndexes[1] >= 0 && ( poParent->tableinfo.tab1dim_filter || poParent->table_filter_RTAB_TotalPopWillaya ) ) {
				if ( poParent->tableinfo.tab1events == -1 || poParent->tableinfo.tab1events < poParent->__events ) {
					if ( poParent->tableinfo.tab1events != -1 ) {
						poParent->tableinfo.tab1open0 = (real) poParent->CloseUpdate( 1, 0, poParent->tableinfo.tab1open0, poParent->tableinfo.tab1close0, poParent->tableinfo.tab1dim_filter, poParent->tableinfo.tab1cell, poParent->tableinfo.tab1total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab1events = poParent->__events;
					poParent->tableinfo.tab1close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab1cell = (index) poParent->TableCellIndex( 1, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab1total_cell = (index) poParent->tableinfo.tab1cell;
					}
					else {
						poParent->tableinfo.tab1total_cell = (index) poParent->TableTotalCellIndex( 1, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->province_nat );
					}
					poParent->tableinfo.tab1dim_filter = poParent->table_filter_RTAB_TotalPopWillaya;
				}
			}
			if ( gplTableIndexes[3] >= 0 && ( poParent->tableinfo.tab3dim_filter || poParent->table_filter_RTAB_DeathsByRegion ) ) {
				if ( poParent->tableinfo.tab3events == -1 || poParent->tableinfo.tab3events < poParent->__events ) {
					if ( poParent->tableinfo.tab3events != -1 ) {
						poParent->tableinfo.tab3open0 = (real) poParent->CloseUpdate( 3, 0, poParent->tableinfo.tab3open0, poParent->tableinfo.tab3close0, poParent->tableinfo.tab3dim_filter, poParent->tableinfo.tab3cell, poParent->tableinfo.tab3total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab3events = poParent->__events;
					poParent->tableinfo.tab3close0 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab3cell = (index) poParent->TableCellIndex( 3, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab3total_cell = (index) poParent->tableinfo.tab3cell;
					}
					else {
						poParent->tableinfo.tab3total_cell = (index) poParent->TableTotalCellIndex( 3, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					}
					poParent->tableinfo.tab3dim_filter = poParent->table_filter_RTAB_DeathsByRegion;
				}
			}
			if ( gplTableIndexes[4] >= 0 && ( poParent->tableinfo.tab4dim_filter || poParent->table_filter_RTAB_BirthsByRegion ) ) {
				if ( poParent->tableinfo.tab4events == -1 || poParent->tableinfo.tab4events < poParent->__events ) {
					if ( poParent->tableinfo.tab4events != -1 ) {
						poParent->tableinfo.tab4open0 = (real) poParent->CloseUpdate( 4, 0, poParent->tableinfo.tab4open0, poParent->tableinfo.tab4close0, poParent->tableinfo.tab4dim_filter, poParent->tableinfo.tab4cell, poParent->tableinfo.tab4total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab4events = poParent->__events;
					poParent->tableinfo.tab4close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab4cell = (index) poParent->TableCellIndex( 4, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab4total_cell = (index) poParent->tableinfo.tab4cell;
					}
					else {
						poParent->tableinfo.tab4total_cell = (index) poParent->TableTotalCellIndex( 4, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					}
					poParent->tableinfo.tab4dim_filter = poParent->table_filter_RTAB_BirthsByRegion;
				}
			}
			if ( gplTableIndexes[5] >= 0 && ( poParent->tableinfo.tab5dim_filter || poParent->table_filter_RTAB_BirthsByEducMother ) ) {
				if ( poParent->tableinfo.tab5events == -1 || poParent->tableinfo.tab5events < poParent->__events ) {
					if ( poParent->tableinfo.tab5events != -1 ) {
						poParent->tableinfo.tab5open0 = (real) poParent->CloseUpdate( 5, 0, poParent->tableinfo.tab5open0, poParent->tableinfo.tab5close0, poParent->tableinfo.tab5dim_filter, poParent->tableinfo.tab5cell, poParent->tableinfo.tab5total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab5events = poParent->__events;
					poParent->tableinfo.tab5close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab5cell = (index) poParent->TableCellIndex( 5, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->educ_mother );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab5total_cell = (index) poParent->tableinfo.tab5cell;
					}
					else {
						poParent->tableinfo.tab5total_cell = (index) poParent->TableTotalCellIndex( 5, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->educ_mother );
					}
					poParent->tableinfo.tab5dim_filter = poParent->table_filter_RTAB_BirthsByEducMother;
				}
			}
			if ( gplTableIndexes[6] >= 0 && ( poParent->tableinfo.tab6dim_filter || poParent->table_filter_RTAB_BirthsByAgeMother ) ) {
				if ( poParent->tableinfo.tab6events == -1 || poParent->tableinfo.tab6events < poParent->__events ) {
					if ( poParent->tableinfo.tab6events != -1 ) {
						poParent->tableinfo.tab6open0 = (real) poParent->CloseUpdate( 6, 0, poParent->tableinfo.tab6open0, poParent->tableinfo.tab6close0, poParent->tableinfo.tab6dim_filter, poParent->tableinfo.tab6cell, poParent->tableinfo.tab6total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab6events = poParent->__events;
					poParent->tableinfo.tab6close0 = (real) poParent->parity;
					poParent->tableinfo.tab6cell = (index) poParent->TableCellIndex( 6, 2,
						OldValue - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab6total_cell = (index) poParent->tableinfo.tab6cell;
					}
					else {
						poParent->tableinfo.tab6total_cell = (index) poParent->TableTotalCellIndex( 6, 2,
						OldValue - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					}
					poParent->tableinfo.tab6dim_filter = poParent->table_filter_RTAB_BirthsByAgeMother;
				}
			}
			if ( gplTableIndexes[7] >= 0 && ( poParent->tableinfo.tab7dim_filter || poParent->table_filter_RTAB_UnionFormationByAge ) ) {
				if ( poParent->tableinfo.tab7events == -1 || poParent->tableinfo.tab7events < poParent->__events ) {
					if ( poParent->tableinfo.tab7events != -1 ) {
						poParent->tableinfo.tab7open0 = (real) poParent->CloseUpdate( 7, 0, poParent->tableinfo.tab7open0, poParent->tableinfo.tab7close0, poParent->tableinfo.tab7dim_filter, poParent->tableinfo.tab7cell, poParent->tableinfo.tab7total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab7events = poParent->__events;
					poParent->tableinfo.tab7close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab7cell = (index) poParent->TableCellIndex( 7, 2,
						OldValue - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab7total_cell = (index) poParent->tableinfo.tab7cell;
					}
					else {
						poParent->tableinfo.tab7total_cell = (index) poParent->TableTotalCellIndex( 7, 2,
						OldValue - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					}
					poParent->tableinfo.tab7dim_filter = poParent->table_filter_RTAB_UnionFormationByAge;
				}
			}
			if ( gplTableIndexes[8] >= 0 && ( poParent->tableinfo.tab8dim_filter || poParent->table_filter_RTAB_AgeAtFirstUnionFormation ) ) {
				if ( poParent->tableinfo.tab8events == -1 || poParent->tableinfo.tab8events < poParent->__events ) {
					if ( poParent->tableinfo.tab8events != -1 ) {
						poParent->tableinfo.tab8open0 = (real) poParent->CloseUpdate( 8, 0, poParent->tableinfo.tab8open0, poParent->tableinfo.tab8close0, poParent->tableinfo.tab8dim_filter, poParent->tableinfo.tab8cell, poParent->tableinfo.tab8total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab8open1 = (real) poParent->CloseUpdate( 8, 1, poParent->tableinfo.tab8open1, poParent->tableinfo.tab8close1, poParent->tableinfo.tab8dim_filter, poParent->tableinfo.tab8cell, poParent->tableinfo.tab8total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab8events = poParent->__events;
					poParent->tableinfo.tab8close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab8close1 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab8cell = (index) poParent->TableCellIndex( 8, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_column );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab8total_cell = (index) poParent->tableinfo.tab8cell;
					}
					else {
						poParent->tableinfo.tab8total_cell = (index) poParent->TableTotalCellIndex( 8, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->rtab_column );
					}
					poParent->tableinfo.tab8dim_filter = poParent->table_filter_RTAB_AgeAtFirstUnionFormation;
				}
			}
			if ( gplTableIndexes[9] >= 0 && ( poParent->tableinfo.tab9dim_filter || poParent->table_filter_RTAB_TabDeathsByAgeGroup ) ) {
				if ( poParent->tableinfo.tab9events == -1 || poParent->tableinfo.tab9events < poParent->__events ) {
					if ( poParent->tableinfo.tab9events != -1 ) {
						poParent->tableinfo.tab9open0 = (real) poParent->CloseUpdate( 9, 0, poParent->tableinfo.tab9open0, poParent->tableinfo.tab9close0, poParent->tableinfo.tab9dim_filter, poParent->tableinfo.tab9cell, poParent->tableinfo.tab9total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab9events = poParent->__events;
					poParent->tableinfo.tab9close0 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab9cell = (index) poParent->TableCellIndex( 9, 2,
						OldValue - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_RTAB_MORT );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab9total_cell = (index) poParent->tableinfo.tab9cell;
					}
					else {
						poParent->tableinfo.tab9total_cell = (index) poParent->TableTotalCellIndex( 9, 2,
						OldValue - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_RTAB_MORT );
					}
					poParent->tableinfo.tab9dim_filter = poParent->table_filter_RTAB_TabDeathsByAgeGroup;
				}
			}
			if ( gplTableIndexes[10] >= 0 && ( poParent->tableinfo.tab10dim_filter || poParent->table_filter_RTAB_EducationActivePop ) ) {
				if ( poParent->tableinfo.tab10events == -1 || poParent->tableinfo.tab10events < poParent->__events ) {
					if ( poParent->tableinfo.tab10events != -1 ) {
						poParent->tableinfo.tab10open0 = (real) poParent->CloseUpdate( 10, 0, poParent->tableinfo.tab10open0, poParent->tableinfo.tab10close0, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open1 = (real) poParent->CloseUpdate( 10, 1, poParent->tableinfo.tab10open1, poParent->tableinfo.tab10close1, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open2 = (real) poParent->CloseUpdate( 10, 2, poParent->tableinfo.tab10open2, poParent->tableinfo.tab10close2, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open3 = (real) poParent->CloseUpdate( 10, 3, poParent->tableinfo.tab10open3, poParent->tableinfo.tab10close3, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab10events = poParent->__events;
					poParent->tableinfo.tab10close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab10close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab10close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab10close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab10cell = (index) poParent->TableCellIndex( 10, 2,
						poParent->rtab_reg,
						OldValue - r_min_RTAB_YEARS );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab10total_cell = (index) poParent->tableinfo.tab10cell;
					}
					else {
						poParent->tableinfo.tab10total_cell = (index) poParent->TableTotalCellIndex( 10, 2,
						poParent->rtab_reg,
						OldValue - r_min_RTAB_YEARS );
					}
					poParent->tableinfo.tab10dim_filter = poParent->table_filter_RTAB_EducationActivePop;
				}
			}
			if ( gplTableIndexes[11] >= 0 && ( poParent->tableinfo.tab11dim_filter || poParent->table_filter_RTAB_PrimSchoolEntries ) ) {
				if ( poParent->tableinfo.tab11events == -1 || poParent->tableinfo.tab11events < poParent->__events ) {
					if ( poParent->tableinfo.tab11events != -1 ) {
						poParent->tableinfo.tab11open0 = (real) poParent->CloseUpdate( 11, 0, poParent->tableinfo.tab11open0, poParent->tableinfo.tab11close0, poParent->tableinfo.tab11dim_filter, poParent->tableinfo.tab11cell, poParent->tableinfo.tab11total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab11events = poParent->__events;
					poParent->tableinfo.tab11close0 = (real) poDerivedStates->transitions_primary_level_PL_NO_PL_ENTER;
					poParent->tableinfo.tab11cell = (index) poParent->TableCellIndex( 11, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab11total_cell = (index) poParent->tableinfo.tab11cell;
					}
					else {
						poParent->tableinfo.tab11total_cell = (index) poParent->TableTotalCellIndex( 11, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->province_nat );
					}
					poParent->tableinfo.tab11dim_filter = poParent->table_filter_RTAB_PrimSchoolEntries;
				}
			}
			if ( gplTableIndexes[12] >= 0 && ( poParent->tableinfo.tab12dim_filter || poParent->table_filter_RTAB_PrimSchoolGraduations ) ) {
				if ( poParent->tableinfo.tab12events == -1 || poParent->tableinfo.tab12events < poParent->__events ) {
					if ( poParent->tableinfo.tab12events != -1 ) {
						poParent->tableinfo.tab12open0 = (real) poParent->CloseUpdate( 12, 0, poParent->tableinfo.tab12open0, poParent->tableinfo.tab12close0, poParent->tableinfo.tab12dim_filter, poParent->tableinfo.tab12cell, poParent->tableinfo.tab12total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab12events = poParent->__events;
					poParent->tableinfo.tab12close0 = (real) poDerivedStates->transitions_primary_level_PL_ENTER_PL_GRAD;
					poParent->tableinfo.tab12cell = (index) poParent->TableCellIndex( 12, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab12total_cell = (index) poParent->tableinfo.tab12cell;
					}
					else {
						poParent->tableinfo.tab12total_cell = (index) poParent->TableTotalCellIndex( 12, 2,
						OldValue - r_min_RTAB_YEARS,
						poParent->province_nat );
					}
					poParent->tableinfo.tab12dim_filter = poParent->table_filter_RTAB_PrimSchoolGraduations;
				}
			}
		}
	}
	return poParent->rtab_years = value;
}

CHILD_MORT_RANGE PersonStateFunctions::Set_sim_cmr( CHILD_MORT_RANGE value )
{

	CHKLMT(value, sim_cmr, r_min_CHILD_MORT_RANGE, r_max_CHILD_MORT_RANGE);
	if ( value < r_min_CHILD_MORT_RANGE ) {
		value = r_min_CHILD_MORT_RANGE;
	}
	else if ( value > r_max_CHILD_MORT_RANGE ) {
		value = r_max_CHILD_MORT_RANGE;
	}
	if ( poParent->sim_cmr != value )
	{


		CHILD_MORT_RANGE	OldValue;
		OldValue = poParent->sim_cmr;
		poParent->sim_cmr = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[22] >= 0 && ( poParent->tableinfo.tab22dim_filter || poParent->table_filter_TabCohortChildMortality ) ) {
				if ( poParent->tableinfo.tab22events == -1 || poParent->tableinfo.tab22events < poParent->__events ) {
					if ( poParent->tableinfo.tab22events != -1 ) {
						poParent->tableinfo.tab22open0 = (real) poParent->CloseUpdate( 22, 0, poParent->tableinfo.tab22open0, poParent->tableinfo.tab22close0, poParent->tableinfo.tab22dim_filter, poParent->tableinfo.tab22cell, poParent->tableinfo.tab22total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab22open1 = (real) poParent->CloseUpdate( 22, 1, poParent->tableinfo.tab22open1, poParent->tableinfo.tab22close1, poParent->tableinfo.tab22dim_filter, poParent->tableinfo.tab22cell, poParent->tableinfo.tab22total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab22events = poParent->__events;
					poParent->tableinfo.tab22close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab22close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab22cell = (index) poParent->TableCellIndex( 22, 2,
						OldValue - r_min_CHILD_MORT_RANGE,
						poParent->sim_yob - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab22total_cell = (index) poParent->tableinfo.tab22cell;
					}
					else {
						poParent->tableinfo.tab22total_cell = (index) poParent->TableTotalCellIndex( 22, 2,
						OldValue - r_min_CHILD_MORT_RANGE,
						poParent->sim_yob - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab22dim_filter = poParent->table_filter_TabCohortChildMortality;
				}
			}
		}
	}
	return poParent->sim_cmr = value;
}

SIM_YEAR_RANGE PersonStateFunctions::Set_sim_year( SIM_YEAR_RANGE value )
{

	CHKLMT(value, sim_year, r_min_SIM_YEAR_RANGE, r_max_SIM_YEAR_RANGE);
	if ( value < r_min_SIM_YEAR_RANGE ) {
		value = r_min_SIM_YEAR_RANGE;
	}
	else if ( value > r_max_SIM_YEAR_RANGE ) {
		value = r_max_SIM_YEAR_RANGE;
	}
	if ( poParent->sim_year != value )
	{


		SIM_YEAR_RANGE	OldValue;
		OldValue = poParent->sim_year;
		poParent->sim_year = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[13] >= 0 && ( poParent->tableinfo.tab13dim_filter || poParent->table_filter_TabTotalPopulationSexProvince ) ) {
				if ( poParent->tableinfo.tab13events == -1 || poParent->tableinfo.tab13events < poParent->__events ) {
					if ( poParent->tableinfo.tab13events != -1 ) {
						poParent->tableinfo.tab13open0 = (real) poParent->CloseUpdate( 13, 0, poParent->tableinfo.tab13open0, poParent->tableinfo.tab13close0, poParent->tableinfo.tab13dim_filter, poParent->tableinfo.tab13cell, poParent->tableinfo.tab13total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab13events = poParent->__events;
					poParent->tableinfo.tab13close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab13cell = (index) poParent->TableCellIndex( 13, 3,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab13total_cell = (index) poParent->tableinfo.tab13cell;
					}
					else {
						poParent->tableinfo.tab13total_cell = (index) poParent->TableTotalCellIndex( 13, 3,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->province_nat );
					}
					poParent->tableinfo.tab13dim_filter = poParent->table_filter_TabTotalPopulationSexProvince;
				}
			}
			if ( gplTableIndexes[14] >= 0 && ( poParent->tableinfo.tab14dim_filter || poParent->table_filter_TabPopulationAgeSexProvince ) ) {
				if ( poParent->tableinfo.tab14events == -1 || poParent->tableinfo.tab14events < poParent->__events ) {
					if ( poParent->tableinfo.tab14events != -1 ) {
						poParent->tableinfo.tab14open0 = (real) poParent->CloseUpdate( 14, 0, poParent->tableinfo.tab14open0, poParent->tableinfo.tab14close0, poParent->tableinfo.tab14dim_filter, poParent->tableinfo.tab14cell, poParent->tableinfo.tab14total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab14events = poParent->__events;
					poParent->tableinfo.tab14close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab14cell = (index) poParent->TableCellIndex( 14, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab14total_cell = (index) poParent->tableinfo.tab14cell;
					}
					else {
						poParent->tableinfo.tab14total_cell = (index) poParent->TableTotalCellIndex( 14, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab14dim_filter = poParent->table_filter_TabPopulationAgeSexProvince;
				}
			}
			if ( gplTableIndexes[16] >= 0 && ( poParent->tableinfo.tab16dim_filter || poParent->table_filter_TabFertilityRates ) ) {
				if ( poParent->tableinfo.tab16events == -1 || poParent->tableinfo.tab16events < poParent->__events ) {
					if ( poParent->tableinfo.tab16events != -1 ) {
						poParent->tableinfo.tab16open0 = (real) poParent->CloseUpdate( 16, 0, poParent->tableinfo.tab16open0, poParent->tableinfo.tab16close0, poParent->tableinfo.tab16dim_filter, poParent->tableinfo.tab16cell, poParent->tableinfo.tab16total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab16open1 = (real) poParent->CloseUpdate( 16, 1, poParent->tableinfo.tab16open1, poParent->tableinfo.tab16close1, poParent->tableinfo.tab16dim_filter, poParent->tableinfo.tab16cell, poParent->tableinfo.tab16total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab16events = poParent->__events;
					poParent->tableinfo.tab16close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab16close1 = (real) poParent->parity;
					poParent->tableinfo.tab16cell = (index) poParent->TableCellIndex( 16, 2,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->fert_age - r_min_FERTILE_AGE_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab16total_cell = (index) poParent->tableinfo.tab16cell;
					}
					else {
						poParent->tableinfo.tab16total_cell = (index) poParent->TableTotalCellIndex( 16, 2,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->fert_age - r_min_FERTILE_AGE_RANGE );
					}
					poParent->tableinfo.tab16dim_filter = poParent->table_filter_TabFertilityRates;
				}
			}
			if ( gplTableIndexes[17] >= 0 && ( poParent->tableinfo.tab17dim_filter || poParent->table_filter_TabNumberBirths ) ) {
				if ( poParent->tableinfo.tab17events == -1 || poParent->tableinfo.tab17events < poParent->__events ) {
					if ( poParent->tableinfo.tab17events != -1 ) {
						poParent->tableinfo.tab17open0 = (real) poParent->CloseUpdate( 17, 0, poParent->tableinfo.tab17open0, poParent->tableinfo.tab17close0, poParent->tableinfo.tab17dim_filter, poParent->tableinfo.tab17cell, poParent->tableinfo.tab17total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab17events = poParent->__events;
					poParent->tableinfo.tab17close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab17cell = (index) poParent->TableCellIndex( 17, 2,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->sex );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab17total_cell = (index) poParent->tableinfo.tab17cell;
					}
					else {
						poParent->tableinfo.tab17total_cell = (index) poParent->TableTotalCellIndex( 17, 2,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->sex );
					}
					poParent->tableinfo.tab17dim_filter = poParent->table_filter_TabNumberBirths;
				}
			}
			if ( gplTableIndexes[18] >= 0 && ( poParent->tableinfo.tab18dim_filter || poParent->table_filter_TabBirthsByAgeGroup ) ) {
				if ( poParent->tableinfo.tab18events == -1 || poParent->tableinfo.tab18events < poParent->__events ) {
					if ( poParent->tableinfo.tab18events != -1 ) {
						poParent->tableinfo.tab18open0 = (real) poParent->CloseUpdate( 18, 0, poParent->tableinfo.tab18open0, poParent->tableinfo.tab18close0, poParent->tableinfo.tab18dim_filter, poParent->tableinfo.tab18cell, poParent->tableinfo.tab18total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab18open1 = (real) poParent->CloseUpdate( 18, 1, poParent->tableinfo.tab18open1, poParent->tableinfo.tab18close1, poParent->tableinfo.tab18dim_filter, poParent->tableinfo.tab18cell, poParent->tableinfo.tab18total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab18events = poParent->__events;
					poParent->tableinfo.tab18close0 = (real) poParent->parity;
					poParent->tableinfo.tab18close1 = (real) poDerivedStates->transitions_parity_0_1;
					poParent->tableinfo.tab18cell = (index) poParent->TableCellIndex( 18, 2,
						poDerivedStates->split_integer_age_AGE_15_17,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab18total_cell = (index) poParent->tableinfo.tab18cell;
					}
					else {
						poParent->tableinfo.tab18total_cell = (index) poParent->TableTotalCellIndex( 18, 2,
						poDerivedStates->split_integer_age_AGE_15_17,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab18dim_filter = poParent->table_filter_TabBirthsByAgeGroup;
				}
			}
			if ( gplTableIndexes[19] >= 0 && ( poParent->tableinfo.tab19dim_filter || poParent->table_filter_TabAgeAtBirth ) ) {
				if ( poParent->tableinfo.tab19events == -1 || poParent->tableinfo.tab19events < poParent->__events ) {
					if ( poParent->tableinfo.tab19events != -1 ) {
						poParent->tableinfo.tab19open0 = (real) poParent->CloseUpdate( 19, 0, poParent->tableinfo.tab19open0, poParent->tableinfo.tab19close0, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open1 = (real) poParent->CloseUpdate( 19, 1, poParent->tableinfo.tab19open1, poParent->tableinfo.tab19close1, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open2 = (real) poParent->CloseUpdate( 19, 2, poParent->tableinfo.tab19open2, poParent->tableinfo.tab19close2, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open3 = (real) poParent->CloseUpdate( 19, 3, poParent->tableinfo.tab19open3, poParent->tableinfo.tab19close3, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab19events = poParent->__events;
					poParent->tableinfo.tab19close0 = (real) poDerivedStates->changes_parity;
					poParent->tableinfo.tab19close1 = (real) poDerivedStates->transitions_parity_0_1;
					poParent->tableinfo.tab19close2 = (real) poDerivedStates->value_at_changes_parity_age;
					poParent->tableinfo.tab19close3 = (real) poDerivedStates->value_at_transitions_parity_0_1_age;
					poParent->tableinfo.tab19cell = (index) poParent->TableCellIndex( 19, 2,
						poParent->primary_level,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab19total_cell = (index) poParent->tableinfo.tab19cell;
					}
					else {
						poParent->tableinfo.tab19total_cell = (index) poParent->TableTotalCellIndex( 19, 2,
						poParent->primary_level,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab19dim_filter = poParent->table_filter_TabAgeAtBirth;
				}
			}
			if ( gplTableIndexes[20] >= 0 && ( poParent->tableinfo.tab20dim_filter || poParent->table_filter_TabDeathRates ) ) {
				if ( poParent->tableinfo.tab20events == -1 || poParent->tableinfo.tab20events < poParent->__events ) {
					if ( poParent->tableinfo.tab20events != -1 ) {
						poParent->tableinfo.tab20open0 = (real) poParent->CloseUpdate( 20, 0, poParent->tableinfo.tab20open0, poParent->tableinfo.tab20close0, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab20open1 = (real) poParent->CloseUpdate( 20, 1, poParent->tableinfo.tab20open1, poParent->tableinfo.tab20close1, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab20events = poParent->__events;
					poParent->tableinfo.tab20close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab20close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab20cell = (index) poParent->TableCellIndex( 20, 3,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->integer_age - r_min_AGE_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab20total_cell = (index) poParent->tableinfo.tab20cell;
					}
					else {
						poParent->tableinfo.tab20total_cell = (index) poParent->TableTotalCellIndex( 20, 3,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE,
						poParent->integer_age - r_min_AGE_RANGE );
					}
					poParent->tableinfo.tab20dim_filter = poParent->table_filter_TabDeathRates;
				}
			}
			if ( gplTableIndexes[21] >= 0 && ( poParent->tableinfo.tab21dim_filter || poParent->table_filter_TabDeathsByAgeGroup ) ) {
				if ( poParent->tableinfo.tab21events == -1 || poParent->tableinfo.tab21events < poParent->__events ) {
					if ( poParent->tableinfo.tab21events != -1 ) {
						poParent->tableinfo.tab21open0 = (real) poParent->CloseUpdate( 21, 0, poParent->tableinfo.tab21open0, poParent->tableinfo.tab21close0, poParent->tableinfo.tab21dim_filter, poParent->tableinfo.tab21cell, poParent->tableinfo.tab21total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab21open1 = (real) poParent->CloseUpdate( 21, 1, poParent->tableinfo.tab21open1, poParent->tableinfo.tab21close1, poParent->tableinfo.tab21dim_filter, poParent->tableinfo.tab21cell, poParent->tableinfo.tab21total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab21events = poParent->__events;
					poParent->tableinfo.tab21close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab21close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab21cell = (index) poParent->TableCellIndex( 21, 2,
						OldValue - r_min_SIM_YEAR_RANGE,
						poDerivedStates->split_integer_age_AGE_MORT );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab21total_cell = (index) poParent->tableinfo.tab21cell;
					}
					else {
						poParent->tableinfo.tab21total_cell = (index) poParent->TableTotalCellIndex( 21, 2,
						OldValue - r_min_SIM_YEAR_RANGE,
						poDerivedStates->split_integer_age_AGE_MORT );
					}
					poParent->tableinfo.tab21dim_filter = poParent->table_filter_TabDeathsByAgeGroup;
				}
			}
			if ( gplTableIndexes[25] >= 0 && ( poParent->tableinfo.tab25dim_filter || poParent->table_filter_TabImmigrationSexProvinceAge ) ) {
				if ( poParent->tableinfo.tab25events == -1 || poParent->tableinfo.tab25events < poParent->__events ) {
					if ( poParent->tableinfo.tab25events != -1 ) {
						Set_table_unit_TabImmigrationSexProvinceAge(poParent->table_unit_TabImmigrationSexProvinceAge+1);
						poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
						poParent->tableinfo.tab25open0 = (real) poParent->CloseUpdate( 25, 0, poParent->tableinfo.tab25open0, poParent->tableinfo.tab25close0, poParent->tableinfo.tab25dim_filter, poParent->tableinfo.tab25cell, poParent->tableinfo.tab25total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab25events = poParent->__events;
					poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
					poParent->tableinfo.tab25cell = (index) poParent->TableCellIndex( 25, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab25total_cell = (index) poParent->tableinfo.tab25cell;
					}
					else {
						poParent->tableinfo.tab25total_cell = (index) poParent->TableTotalCellIndex( 25, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab25dim_filter = poParent->table_filter_TabImmigrationSexProvinceAge;
				}
			}
			if ( gplTableIndexes[26] >= 0 && ( poParent->tableinfo.tab26dim_filter || poParent->table_filter_TabEmigration ) ) {
				if ( poParent->tableinfo.tab26events == -1 || poParent->tableinfo.tab26events < poParent->__events ) {
					if ( poParent->tableinfo.tab26events != -1 ) {
						poParent->tableinfo.tab26open0 = (real) poParent->CloseUpdate( 26, 0, poParent->tableinfo.tab26open0, poParent->tableinfo.tab26close0, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab26open1 = (real) poParent->CloseUpdate( 26, 1, poParent->tableinfo.tab26open1, poParent->tableinfo.tab26close1, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab26events = poParent->__events;
					poParent->tableinfo.tab26close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab26close1 = (real) poDerivedStates->transitions_has_emigrated_FALSE_TRUE;
					poParent->tableinfo.tab26cell = (index) poParent->TableCellIndex( 26, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab26total_cell = (index) poParent->tableinfo.tab26cell;
					}
					else {
						poParent->tableinfo.tab26total_cell = (index) poParent->TableTotalCellIndex( 26, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab26dim_filter = poParent->table_filter_TabEmigration;
				}
			}
			if ( gplTableIndexes[27] >= 0 && ( poParent->tableinfo.tab27dim_filter || poParent->table_filter_TabPopProvAgeEducSex ) ) {
				if ( poParent->tableinfo.tab27events == -1 || poParent->tableinfo.tab27events < poParent->__events ) {
					if ( poParent->tableinfo.tab27events != -1 ) {
						poParent->tableinfo.tab27open0 = (real) poParent->CloseUpdate( 27, 0, poParent->tableinfo.tab27open0, poParent->tableinfo.tab27close0, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open1 = (real) poParent->CloseUpdate( 27, 1, poParent->tableinfo.tab27open1, poParent->tableinfo.tab27close1, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open2 = (real) poParent->CloseUpdate( 27, 2, poParent->tableinfo.tab27open2, poParent->tableinfo.tab27close2, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open3 = (real) poParent->CloseUpdate( 27, 3, poParent->tableinfo.tab27open3, poParent->tableinfo.tab27close3, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open4 = (real) poParent->CloseUpdate( 27, 4, poParent->tableinfo.tab27open4, poParent->tableinfo.tab27close4, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab27events = poParent->__events;
					poParent->tableinfo.tab27close0 = (real) poDerivedStates->Get_duration_is_aded0_TRUE();
					poParent->tableinfo.tab27close1 = (real) poDerivedStates->Get_duration_is_aded1_TRUE();
					poParent->tableinfo.tab27close2 = (real) poDerivedStates->Get_duration_is_aded2_TRUE();
					poParent->tableinfo.tab27close3 = (real) poDerivedStates->Get_duration_is_child_TRUE();
					poParent->tableinfo.tab27close4 = (real) poDerivedStates->Get_duration_is_old_TRUE();
					poParent->tableinfo.tab27cell = (index) poParent->TableCellIndex( 27, 3,
						poParent->province_nat,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab27total_cell = (index) poParent->tableinfo.tab27cell;
					}
					else {
						poParent->tableinfo.tab27total_cell = (index) poParent->TableTotalCellIndex( 27, 3,
						poParent->province_nat,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab27dim_filter = poParent->table_filter_TabPopProvAgeEducSex;
				}
			}
			if ( gplTableIndexes[28] >= 0 && ( poParent->tableinfo.tab28dim_filter || poParent->table_filter_TabEduc12ByPob ) ) {
				if ( poParent->tableinfo.tab28events == -1 || poParent->tableinfo.tab28events < poParent->__events ) {
					if ( poParent->tableinfo.tab28events != -1 ) {
						poParent->tableinfo.tab28open0 = (real) poParent->CloseUpdate( 28, 0, poParent->tableinfo.tab28open0, poParent->tableinfo.tab28close0, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open1 = (real) poParent->CloseUpdate( 28, 1, poParent->tableinfo.tab28open1, poParent->tableinfo.tab28close1, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open2 = (real) poParent->CloseUpdate( 28, 2, poParent->tableinfo.tab28open2, poParent->tableinfo.tab28close2, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open3 = (real) poParent->CloseUpdate( 28, 3, poParent->tableinfo.tab28open3, poParent->tableinfo.tab28close3, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab28events = poParent->__events;
					poParent->tableinfo.tab28close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab28close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab28close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab28close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab28cell = (index) poParent->TableCellIndex( 28, 3,
						poParent->province_birth,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab28total_cell = (index) poParent->tableinfo.tab28cell;
					}
					else {
						poParent->tableinfo.tab28total_cell = (index) poParent->TableTotalCellIndex( 28, 3,
						poParent->province_birth,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab28dim_filter = poParent->table_filter_TabEduc12ByPob;
				}
			}
			if ( gplTableIndexes[29] >= 0 && ( poParent->tableinfo.tab29dim_filter || poParent->table_filter_TabEduc12ByPor ) ) {
				if ( poParent->tableinfo.tab29events == -1 || poParent->tableinfo.tab29events < poParent->__events ) {
					if ( poParent->tableinfo.tab29events != -1 ) {
						poParent->tableinfo.tab29open0 = (real) poParent->CloseUpdate( 29, 0, poParent->tableinfo.tab29open0, poParent->tableinfo.tab29close0, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open1 = (real) poParent->CloseUpdate( 29, 1, poParent->tableinfo.tab29open1, poParent->tableinfo.tab29close1, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open2 = (real) poParent->CloseUpdate( 29, 2, poParent->tableinfo.tab29open2, poParent->tableinfo.tab29close2, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open3 = (real) poParent->CloseUpdate( 29, 3, poParent->tableinfo.tab29open3, poParent->tableinfo.tab29close3, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab29events = poParent->__events;
					poParent->tableinfo.tab29close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab29close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab29close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab29close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab29cell = (index) poParent->TableCellIndex( 29, 3,
						poParent->province_nat,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab29total_cell = (index) poParent->tableinfo.tab29cell;
					}
					else {
						poParent->tableinfo.tab29total_cell = (index) poParent->TableTotalCellIndex( 29, 3,
						poParent->province_nat,
						poParent->sex,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab29dim_filter = poParent->table_filter_TabEduc12ByPor;
				}
			}
			if ( gplTableIndexes[33] >= 0 && ( poParent->tableinfo.tab33dim_filter || poParent->table_filter_TabUnionFormation ) ) {
				if ( poParent->tableinfo.tab33events == -1 || poParent->tableinfo.tab33events < poParent->__events ) {
					if ( poParent->tableinfo.tab33events != -1 ) {
						poParent->tableinfo.tab33open0 = (real) poParent->CloseUpdate( 33, 0, poParent->tableinfo.tab33open0, poParent->tableinfo.tab33close0, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open1 = (real) poParent->CloseUpdate( 33, 1, poParent->tableinfo.tab33open1, poParent->tableinfo.tab33close1, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open2 = (real) poParent->CloseUpdate( 33, 2, poParent->tableinfo.tab33open2, poParent->tableinfo.tab33close2, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open3 = (real) poParent->CloseUpdate( 33, 3, poParent->tableinfo.tab33open3, poParent->tableinfo.tab33close3, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open4 = (real) poParent->CloseUpdate( 33, 4, poParent->tableinfo.tab33open4, poParent->tableinfo.tab33close4, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab33events = poParent->__events;
					poParent->tableinfo.tab33close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab33close1 = (real) poDerivedStates->Get_duration_in_union_FALSE();
					poParent->tableinfo.tab33close2 = (real) poDerivedStates->Get_duration_in_union_TRUE();
					poParent->tableinfo.tab33close3 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab33close4 = (real) poParent->union_formation_hazard_max_value_in;
					poParent->tableinfo.tab33cell = (index) poParent->TableCellIndex( 33, 3,
						poParent->primary_level,
						poParent->age_union - r_min_AGE_UNION,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab33total_cell = (index) poParent->tableinfo.tab33cell;
					}
					else {
						poParent->tableinfo.tab33total_cell = (index) poParent->TableTotalCellIndex( 33, 3,
						poParent->primary_level,
						poParent->age_union - r_min_AGE_UNION,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab33dim_filter = poParent->table_filter_TabUnionFormation;
				}
			}
			if ( gplTableIndexes[34] >= 0 && ( poParent->tableinfo.tab34dim_filter || poParent->table_filter_TabAgeAtFirstUnionFormation ) ) {
				if ( poParent->tableinfo.tab34events == -1 || poParent->tableinfo.tab34events < poParent->__events ) {
					if ( poParent->tableinfo.tab34events != -1 ) {
						poParent->tableinfo.tab34open0 = (real) poParent->CloseUpdate( 34, 0, poParent->tableinfo.tab34open0, poParent->tableinfo.tab34close0, poParent->tableinfo.tab34dim_filter, poParent->tableinfo.tab34cell, poParent->tableinfo.tab34total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab34open1 = (real) poParent->CloseUpdate( 34, 1, poParent->tableinfo.tab34open1, poParent->tableinfo.tab34close1, poParent->tableinfo.tab34dim_filter, poParent->tableinfo.tab34cell, poParent->tableinfo.tab34total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab34events = poParent->__events;
					poParent->tableinfo.tab34close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab34close1 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab34cell = (index) poParent->TableCellIndex( 34, 2,
						poParent->primary_level,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab34total_cell = (index) poParent->tableinfo.tab34cell;
					}
					else {
						poParent->tableinfo.tab34total_cell = (index) poParent->TableTotalCellIndex( 34, 2,
						poParent->primary_level,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab34dim_filter = poParent->table_filter_TabAgeAtFirstUnionFormation;
				}
			}
		}
	}
	return poParent->sim_year = value;
}

SIM_YEAR_RANGE PersonStateFunctions::Set_sim_yob( SIM_YEAR_RANGE value )
{

	CHKLMT(value, sim_yob, r_min_SIM_YEAR_RANGE, r_max_SIM_YEAR_RANGE);
	if ( value < r_min_SIM_YEAR_RANGE ) {
		value = r_min_SIM_YEAR_RANGE;
	}
	else if ( value > r_max_SIM_YEAR_RANGE ) {
		value = r_max_SIM_YEAR_RANGE;
	}
	if ( poParent->sim_yob != value )
	{


		SIM_YEAR_RANGE	OldValue;
		OldValue = poParent->sim_yob;
		poParent->sim_yob = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[22] >= 0 && ( poParent->tableinfo.tab22dim_filter || poParent->table_filter_TabCohortChildMortality ) ) {
				if ( poParent->tableinfo.tab22events == -1 || poParent->tableinfo.tab22events < poParent->__events ) {
					if ( poParent->tableinfo.tab22events != -1 ) {
						poParent->tableinfo.tab22open0 = (real) poParent->CloseUpdate( 22, 0, poParent->tableinfo.tab22open0, poParent->tableinfo.tab22close0, poParent->tableinfo.tab22dim_filter, poParent->tableinfo.tab22cell, poParent->tableinfo.tab22total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab22open1 = (real) poParent->CloseUpdate( 22, 1, poParent->tableinfo.tab22open1, poParent->tableinfo.tab22close1, poParent->tableinfo.tab22dim_filter, poParent->tableinfo.tab22cell, poParent->tableinfo.tab22total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab22events = poParent->__events;
					poParent->tableinfo.tab22close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab22close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab22cell = (index) poParent->TableCellIndex( 22, 2,
						poParent->sim_cmr - r_min_CHILD_MORT_RANGE,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab22total_cell = (index) poParent->tableinfo.tab22cell;
					}
					else {
						poParent->tableinfo.tab22total_cell = (index) poParent->TableTotalCellIndex( 22, 2,
						poParent->sim_cmr - r_min_CHILD_MORT_RANGE,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab22dim_filter = poParent->table_filter_TabCohortChildMortality;
				}
			}
			if ( gplTableIndexes[35] >= 0 && ( poParent->tableinfo.tab35dim_filter || poParent->table_filter_TabMothersExperience ) ) {
				if ( poParent->tableinfo.tab35events == -1 || poParent->tableinfo.tab35events < poParent->__events ) {
					if ( poParent->tableinfo.tab35events != -1 ) {
						poParent->tableinfo.tab35open0 = (real) poParent->CloseUpdate( 35, 0, poParent->tableinfo.tab35open0, poParent->tableinfo.tab35close0, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open1 = (real) poParent->CloseUpdate( 35, 1, poParent->tableinfo.tab35open1, poParent->tableinfo.tab35close1, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open2 = (real) poParent->CloseUpdate( 35, 2, poParent->tableinfo.tab35open2, poParent->tableinfo.tab35close2, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						Set_table_unit_TabMothersExperience(poParent->table_unit_TabMothersExperience+1);
						poParent->tableinfo.tab35close3 = (real) poParent->table_unit_TabMothersExperience;
						poParent->tableinfo.tab35open3 = (real) poParent->CloseUpdate( 35, 3, poParent->tableinfo.tab35open3, poParent->tableinfo.tab35close3, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open4 = (real) poParent->CloseUpdate( 35, 4, poParent->tableinfo.tab35open4, poParent->tableinfo.tab35close4, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open5 = (real) poParent->CloseUpdate( 35, 5, poParent->tableinfo.tab35open5, poParent->tableinfo.tab35close5, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open6 = (real) poParent->CloseUpdate( 35, 6, poParent->tableinfo.tab35open6, poParent->tableinfo.tab35close6, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open7 = (real) poParent->CloseUpdate( 35, 7, poParent->tableinfo.tab35open7, poParent->tableinfo.tab35close7, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open8 = (real) poParent->CloseUpdate( 35, 8, poParent->tableinfo.tab35open8, poParent->tableinfo.tab35close8, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open9 = (real) poParent->CloseUpdate( 35, 9, poParent->tableinfo.tab35open9, poParent->tableinfo.tab35close9, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open10 = (real) poParent->CloseUpdate( 35, 10, poParent->tableinfo.tab35open10, poParent->tableinfo.tab35close10, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab35events = poParent->__events;
					poParent->tableinfo.tab35close0 = (real) poDerivedStates->entrances_primary_level_PL_GRAD;
					poParent->tableinfo.tab35close1 = (real) poParent->infant_deaths;
					poParent->tableinfo.tab35close2 = (real) poParent->parity;
					poParent->tableinfo.tab35close3 = (real) poParent->table_unit_TabMothersExperience;
					poParent->tableinfo.tab35close4 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab35close5 = (real) poDerivedStates->transitions_integer_age_14_15;
					poParent->tableinfo.tab35close6 = (real) poDerivedStates->transitions_integer_age_19_20;
					poParent->tableinfo.tab35close7 = (real) poDerivedStates->transitions_integer_age_9_10;
					poParent->tableinfo.tab35close8 = (real) poDerivedStates->transitions_parity_0_1;
					poParent->tableinfo.tab35close9 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab35close10 = (real) poDerivedStates->value_at_transitions_parity_0_1_age;
					poParent->tableinfo.tab35cell = (index) poParent->TableCellIndex( 35, 1,
						OldValue - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab35total_cell = (index) poParent->tableinfo.tab35cell;
					}
					else {
						poParent->tableinfo.tab35total_cell = (index) poParent->TableTotalCellIndex( 35, 1,
						OldValue - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab35dim_filter = poParent->table_filter_TabMothersExperience;
				}
			}
		}
	}
	return poParent->sim_yob = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_AgeAtFirstUnionFormation( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_AgeAtFirstUnionFormation != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_AgeAtFirstUnionFormation;
		poParent->table_filter_RTAB_AgeAtFirstUnionFormation = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[8] >= 0 ) {
				if ( poParent->tableinfo.tab8events == -1 || poParent->tableinfo.tab8events < poParent->__events ) {
					if ( poParent->tableinfo.tab8events != -1 ) {
						poParent->tableinfo.tab8open0 = (real) poParent->CloseUpdate( 8, 0, poParent->tableinfo.tab8open0, poParent->tableinfo.tab8close0, poParent->tableinfo.tab8dim_filter, poParent->tableinfo.tab8cell, poParent->tableinfo.tab8total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab8open1 = (real) poParent->CloseUpdate( 8, 1, poParent->tableinfo.tab8open1, poParent->tableinfo.tab8close1, poParent->tableinfo.tab8dim_filter, poParent->tableinfo.tab8cell, poParent->tableinfo.tab8total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab8events = poParent->__events;
					poParent->tableinfo.tab8close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab8close1 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab8cell = (index) poParent->TableCellIndex( 8, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_column );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab8total_cell = (index) poParent->tableinfo.tab8cell;
					}
					else {
						poParent->tableinfo.tab8total_cell = (index) poParent->TableTotalCellIndex( 8, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_column );
					}
					poParent->tableinfo.tab8dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_AgeAtFirstUnionFormation = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_BirthsByAgeMother( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_BirthsByAgeMother != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_BirthsByAgeMother;
		poParent->table_filter_RTAB_BirthsByAgeMother = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[6] >= 0 ) {
				if ( poParent->tableinfo.tab6events == -1 || poParent->tableinfo.tab6events < poParent->__events ) {
					if ( poParent->tableinfo.tab6events != -1 ) {
						poParent->tableinfo.tab6open0 = (real) poParent->CloseUpdate( 6, 0, poParent->tableinfo.tab6open0, poParent->tableinfo.tab6close0, poParent->tableinfo.tab6dim_filter, poParent->tableinfo.tab6cell, poParent->tableinfo.tab6total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab6events = poParent->__events;
					poParent->tableinfo.tab6close0 = (real) poParent->parity;
					poParent->tableinfo.tab6cell = (index) poParent->TableCellIndex( 6, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab6total_cell = (index) poParent->tableinfo.tab6cell;
					}
					else {
						poParent->tableinfo.tab6total_cell = (index) poParent->TableTotalCellIndex( 6, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					}
					poParent->tableinfo.tab6dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_BirthsByAgeMother = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_BirthsByEducMother( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_BirthsByEducMother != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_BirthsByEducMother;
		poParent->table_filter_RTAB_BirthsByEducMother = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[5] >= 0 ) {
				if ( poParent->tableinfo.tab5events == -1 || poParent->tableinfo.tab5events < poParent->__events ) {
					if ( poParent->tableinfo.tab5events != -1 ) {
						poParent->tableinfo.tab5open0 = (real) poParent->CloseUpdate( 5, 0, poParent->tableinfo.tab5open0, poParent->tableinfo.tab5close0, poParent->tableinfo.tab5dim_filter, poParent->tableinfo.tab5cell, poParent->tableinfo.tab5total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab5events = poParent->__events;
					poParent->tableinfo.tab5close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab5cell = (index) poParent->TableCellIndex( 5, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->educ_mother );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab5total_cell = (index) poParent->tableinfo.tab5cell;
					}
					else {
						poParent->tableinfo.tab5total_cell = (index) poParent->TableTotalCellIndex( 5, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->educ_mother );
					}
					poParent->tableinfo.tab5dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_BirthsByEducMother = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_BirthsByRegion( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_BirthsByRegion != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_BirthsByRegion;
		poParent->table_filter_RTAB_BirthsByRegion = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[4] >= 0 ) {
				if ( poParent->tableinfo.tab4events == -1 || poParent->tableinfo.tab4events < poParent->__events ) {
					if ( poParent->tableinfo.tab4events != -1 ) {
						poParent->tableinfo.tab4open0 = (real) poParent->CloseUpdate( 4, 0, poParent->tableinfo.tab4open0, poParent->tableinfo.tab4close0, poParent->tableinfo.tab4dim_filter, poParent->tableinfo.tab4cell, poParent->tableinfo.tab4total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab4events = poParent->__events;
					poParent->tableinfo.tab4close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab4cell = (index) poParent->TableCellIndex( 4, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab4total_cell = (index) poParent->tableinfo.tab4cell;
					}
					else {
						poParent->tableinfo.tab4total_cell = (index) poParent->TableTotalCellIndex( 4, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					}
					poParent->tableinfo.tab4dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_BirthsByRegion = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_DeathsByRegion( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_DeathsByRegion != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_DeathsByRegion;
		poParent->table_filter_RTAB_DeathsByRegion = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[3] >= 0 ) {
				if ( poParent->tableinfo.tab3events == -1 || poParent->tableinfo.tab3events < poParent->__events ) {
					if ( poParent->tableinfo.tab3events != -1 ) {
						poParent->tableinfo.tab3open0 = (real) poParent->CloseUpdate( 3, 0, poParent->tableinfo.tab3open0, poParent->tableinfo.tab3close0, poParent->tableinfo.tab3dim_filter, poParent->tableinfo.tab3cell, poParent->tableinfo.tab3total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab3events = poParent->__events;
					poParent->tableinfo.tab3close0 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab3cell = (index) poParent->TableCellIndex( 3, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab3total_cell = (index) poParent->tableinfo.tab3cell;
					}
					else {
						poParent->tableinfo.tab3total_cell = (index) poParent->TableTotalCellIndex( 3, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					}
					poParent->tableinfo.tab3dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_DeathsByRegion = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_EducationActivePop( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_EducationActivePop != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_EducationActivePop;
		poParent->table_filter_RTAB_EducationActivePop = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[10] >= 0 ) {
				if ( poParent->tableinfo.tab10events == -1 || poParent->tableinfo.tab10events < poParent->__events ) {
					if ( poParent->tableinfo.tab10events != -1 ) {
						poParent->tableinfo.tab10open0 = (real) poParent->CloseUpdate( 10, 0, poParent->tableinfo.tab10open0, poParent->tableinfo.tab10close0, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open1 = (real) poParent->CloseUpdate( 10, 1, poParent->tableinfo.tab10open1, poParent->tableinfo.tab10close1, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open2 = (real) poParent->CloseUpdate( 10, 2, poParent->tableinfo.tab10open2, poParent->tableinfo.tab10close2, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab10open3 = (real) poParent->CloseUpdate( 10, 3, poParent->tableinfo.tab10open3, poParent->tableinfo.tab10close3, poParent->tableinfo.tab10dim_filter, poParent->tableinfo.tab10cell, poParent->tableinfo.tab10total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab10events = poParent->__events;
					poParent->tableinfo.tab10close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab10close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab10close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab10close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab10cell = (index) poParent->TableCellIndex( 10, 2,
						poParent->rtab_reg,
						poParent->rtab_years - r_min_RTAB_YEARS );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab10total_cell = (index) poParent->tableinfo.tab10cell;
					}
					else {
						poParent->tableinfo.tab10total_cell = (index) poParent->TableTotalCellIndex( 10, 2,
						poParent->rtab_reg,
						poParent->rtab_years - r_min_RTAB_YEARS );
					}
					poParent->tableinfo.tab10dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_EducationActivePop = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_PopPyramidByEduc( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_PopPyramidByEduc != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_PopPyramidByEduc;
		poParent->table_filter_RTAB_PopPyramidByEduc = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[2] >= 0 ) {
				if ( poParent->tableinfo.tab2events == -1 || poParent->tableinfo.tab2events < poParent->__events ) {
					if ( poParent->tableinfo.tab2events != -1 ) {
						poParent->tableinfo.tab2open0 = (real) poParent->CloseUpdate( 2, 0, poParent->tableinfo.tab2open0, poParent->tableinfo.tab2close0, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open1 = (real) poParent->CloseUpdate( 2, 1, poParent->tableinfo.tab2open1, poParent->tableinfo.tab2close1, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open2 = (real) poParent->CloseUpdate( 2, 2, poParent->tableinfo.tab2open2, poParent->tableinfo.tab2close2, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab2events = poParent->__events;
					poParent->tableinfo.tab2close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab2close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab2close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab2cell = (index) poParent->TableCellIndex( 2, 4,
						poParent->rtab_sel_years,
						poParent->sex,
						poParent->rtab_reg,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab2total_cell = (index) poParent->tableinfo.tab2cell;
					}
					else {
						poParent->tableinfo.tab2total_cell = (index) poParent->TableTotalCellIndex( 2, 4,
						poParent->rtab_sel_years,
						poParent->sex,
						poParent->rtab_reg,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab2dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_PopPyramidByEduc = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_PrimSchoolEntries( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_PrimSchoolEntries != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_PrimSchoolEntries;
		poParent->table_filter_RTAB_PrimSchoolEntries = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[11] >= 0 ) {
				if ( poParent->tableinfo.tab11events == -1 || poParent->tableinfo.tab11events < poParent->__events ) {
					if ( poParent->tableinfo.tab11events != -1 ) {
						poParent->tableinfo.tab11open0 = (real) poParent->CloseUpdate( 11, 0, poParent->tableinfo.tab11open0, poParent->tableinfo.tab11close0, poParent->tableinfo.tab11dim_filter, poParent->tableinfo.tab11cell, poParent->tableinfo.tab11total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab11events = poParent->__events;
					poParent->tableinfo.tab11close0 = (real) poDerivedStates->transitions_primary_level_PL_NO_PL_ENTER;
					poParent->tableinfo.tab11cell = (index) poParent->TableCellIndex( 11, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab11total_cell = (index) poParent->tableinfo.tab11cell;
					}
					else {
						poParent->tableinfo.tab11total_cell = (index) poParent->TableTotalCellIndex( 11, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->province_nat );
					}
					poParent->tableinfo.tab11dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_PrimSchoolEntries = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_PrimSchoolGraduations( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_PrimSchoolGraduations != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_PrimSchoolGraduations;
		poParent->table_filter_RTAB_PrimSchoolGraduations = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[12] >= 0 ) {
				if ( poParent->tableinfo.tab12events == -1 || poParent->tableinfo.tab12events < poParent->__events ) {
					if ( poParent->tableinfo.tab12events != -1 ) {
						poParent->tableinfo.tab12open0 = (real) poParent->CloseUpdate( 12, 0, poParent->tableinfo.tab12open0, poParent->tableinfo.tab12close0, poParent->tableinfo.tab12dim_filter, poParent->tableinfo.tab12cell, poParent->tableinfo.tab12total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab12events = poParent->__events;
					poParent->tableinfo.tab12close0 = (real) poDerivedStates->transitions_primary_level_PL_ENTER_PL_GRAD;
					poParent->tableinfo.tab12cell = (index) poParent->TableCellIndex( 12, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab12total_cell = (index) poParent->tableinfo.tab12cell;
					}
					else {
						poParent->tableinfo.tab12total_cell = (index) poParent->TableTotalCellIndex( 12, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->province_nat );
					}
					poParent->tableinfo.tab12dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_PrimSchoolGraduations = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_TabDeathsByAgeGroup( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_TabDeathsByAgeGroup != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_TabDeathsByAgeGroup;
		poParent->table_filter_RTAB_TabDeathsByAgeGroup = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[9] >= 0 ) {
				if ( poParent->tableinfo.tab9events == -1 || poParent->tableinfo.tab9events < poParent->__events ) {
					if ( poParent->tableinfo.tab9events != -1 ) {
						poParent->tableinfo.tab9open0 = (real) poParent->CloseUpdate( 9, 0, poParent->tableinfo.tab9open0, poParent->tableinfo.tab9close0, poParent->tableinfo.tab9dim_filter, poParent->tableinfo.tab9cell, poParent->tableinfo.tab9total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab9events = poParent->__events;
					poParent->tableinfo.tab9close0 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab9cell = (index) poParent->TableCellIndex( 9, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_RTAB_MORT );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab9total_cell = (index) poParent->tableinfo.tab9cell;
					}
					else {
						poParent->tableinfo.tab9total_cell = (index) poParent->TableTotalCellIndex( 9, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_RTAB_MORT );
					}
					poParent->tableinfo.tab9dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_TabDeathsByAgeGroup = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_TotalPopReg( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_TotalPopReg != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_TotalPopReg;
		poParent->table_filter_RTAB_TotalPopReg = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[0] >= 0 ) {
				if ( poParent->tableinfo.tab0events == -1 || poParent->tableinfo.tab0events < poParent->__events ) {
					if ( poParent->tableinfo.tab0events != -1 ) {
						poParent->tableinfo.tab0open0 = (real) poParent->CloseUpdate( 0, 0, poParent->tableinfo.tab0open0, poParent->tableinfo.tab0close0, poParent->tableinfo.tab0dim_filter, poParent->tableinfo.tab0cell, poParent->tableinfo.tab0total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab0events = poParent->__events;
					poParent->tableinfo.tab0close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab0cell = (index) poParent->TableCellIndex( 0, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab0total_cell = (index) poParent->tableinfo.tab0cell;
					}
					else {
						poParent->tableinfo.tab0total_cell = (index) poParent->TableTotalCellIndex( 0, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->rtab_reg );
					}
					poParent->tableinfo.tab0dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_TotalPopReg = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_TotalPopWillaya( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_TotalPopWillaya != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_TotalPopWillaya;
		poParent->table_filter_RTAB_TotalPopWillaya = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[1] >= 0 ) {
				if ( poParent->tableinfo.tab1events == -1 || poParent->tableinfo.tab1events < poParent->__events ) {
					if ( poParent->tableinfo.tab1events != -1 ) {
						poParent->tableinfo.tab1open0 = (real) poParent->CloseUpdate( 1, 0, poParent->tableinfo.tab1open0, poParent->tableinfo.tab1close0, poParent->tableinfo.tab1dim_filter, poParent->tableinfo.tab1cell, poParent->tableinfo.tab1total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab1events = poParent->__events;
					poParent->tableinfo.tab1close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab1cell = (index) poParent->TableCellIndex( 1, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab1total_cell = (index) poParent->tableinfo.tab1cell;
					}
					else {
						poParent->tableinfo.tab1total_cell = (index) poParent->TableTotalCellIndex( 1, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poParent->province_nat );
					}
					poParent->tableinfo.tab1dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_TotalPopWillaya = value;
}

logical PersonStateFunctions::Set_table_filter_RTAB_UnionFormationByAge( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_RTAB_UnionFormationByAge != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_RTAB_UnionFormationByAge;
		poParent->table_filter_RTAB_UnionFormationByAge = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[7] >= 0 ) {
				if ( poParent->tableinfo.tab7events == -1 || poParent->tableinfo.tab7events < poParent->__events ) {
					if ( poParent->tableinfo.tab7events != -1 ) {
						poParent->tableinfo.tab7open0 = (real) poParent->CloseUpdate( 7, 0, poParent->tableinfo.tab7open0, poParent->tableinfo.tab7close0, poParent->tableinfo.tab7dim_filter, poParent->tableinfo.tab7cell, poParent->tableinfo.tab7total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab7events = poParent->__events;
					poParent->tableinfo.tab7close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab7cell = (index) poParent->TableCellIndex( 7, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab7total_cell = (index) poParent->tableinfo.tab7cell;
					}
					else {
						poParent->tableinfo.tab7total_cell = (index) poParent->TableTotalCellIndex( 7, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						poDerivedStates->split_integer_age_AGE_15_16_18 );
					}
					poParent->tableinfo.tab7dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_RTAB_UnionFormationByAge = value;
}

logical PersonStateFunctions::Set_table_filter_TabAgeAtBirth( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabAgeAtBirth != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabAgeAtBirth;
		poParent->table_filter_TabAgeAtBirth = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[19] >= 0 ) {
				if ( poParent->tableinfo.tab19events == -1 || poParent->tableinfo.tab19events < poParent->__events ) {
					if ( poParent->tableinfo.tab19events != -1 ) {
						poParent->tableinfo.tab19open0 = (real) poParent->CloseUpdate( 19, 0, poParent->tableinfo.tab19open0, poParent->tableinfo.tab19close0, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open1 = (real) poParent->CloseUpdate( 19, 1, poParent->tableinfo.tab19open1, poParent->tableinfo.tab19close1, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open2 = (real) poParent->CloseUpdate( 19, 2, poParent->tableinfo.tab19open2, poParent->tableinfo.tab19close2, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab19open3 = (real) poParent->CloseUpdate( 19, 3, poParent->tableinfo.tab19open3, poParent->tableinfo.tab19close3, poParent->tableinfo.tab19dim_filter, poParent->tableinfo.tab19cell, poParent->tableinfo.tab19total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab19events = poParent->__events;
					poParent->tableinfo.tab19close0 = (real) poDerivedStates->changes_parity;
					poParent->tableinfo.tab19close1 = (real) poDerivedStates->transitions_parity_0_1;
					poParent->tableinfo.tab19close2 = (real) poDerivedStates->value_at_changes_parity_age;
					poParent->tableinfo.tab19close3 = (real) poDerivedStates->value_at_transitions_parity_0_1_age;
					poParent->tableinfo.tab19cell = (index) poParent->TableCellIndex( 19, 2,
						poParent->primary_level,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab19total_cell = (index) poParent->tableinfo.tab19cell;
					}
					else {
						poParent->tableinfo.tab19total_cell = (index) poParent->TableTotalCellIndex( 19, 2,
						poParent->primary_level,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab19dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabAgeAtBirth = value;
}

logical PersonStateFunctions::Set_table_filter_TabAgeAtFirstUnionFormation( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabAgeAtFirstUnionFormation != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabAgeAtFirstUnionFormation;
		poParent->table_filter_TabAgeAtFirstUnionFormation = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[34] >= 0 ) {
				if ( poParent->tableinfo.tab34events == -1 || poParent->tableinfo.tab34events < poParent->__events ) {
					if ( poParent->tableinfo.tab34events != -1 ) {
						poParent->tableinfo.tab34open0 = (real) poParent->CloseUpdate( 34, 0, poParent->tableinfo.tab34open0, poParent->tableinfo.tab34close0, poParent->tableinfo.tab34dim_filter, poParent->tableinfo.tab34cell, poParent->tableinfo.tab34total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab34open1 = (real) poParent->CloseUpdate( 34, 1, poParent->tableinfo.tab34open1, poParent->tableinfo.tab34close1, poParent->tableinfo.tab34dim_filter, poParent->tableinfo.tab34cell, poParent->tableinfo.tab34total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab34events = poParent->__events;
					poParent->tableinfo.tab34close0 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab34close1 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab34cell = (index) poParent->TableCellIndex( 34, 2,
						poParent->primary_level,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab34total_cell = (index) poParent->tableinfo.tab34cell;
					}
					else {
						poParent->tableinfo.tab34total_cell = (index) poParent->TableTotalCellIndex( 34, 2,
						poParent->primary_level,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab34dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabAgeAtFirstUnionFormation = value;
}

logical PersonStateFunctions::Set_table_filter_TabBirthsByAgeGroup( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabBirthsByAgeGroup != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabBirthsByAgeGroup;
		poParent->table_filter_TabBirthsByAgeGroup = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[18] >= 0 ) {
				if ( poParent->tableinfo.tab18events == -1 || poParent->tableinfo.tab18events < poParent->__events ) {
					if ( poParent->tableinfo.tab18events != -1 ) {
						poParent->tableinfo.tab18open0 = (real) poParent->CloseUpdate( 18, 0, poParent->tableinfo.tab18open0, poParent->tableinfo.tab18close0, poParent->tableinfo.tab18dim_filter, poParent->tableinfo.tab18cell, poParent->tableinfo.tab18total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab18open1 = (real) poParent->CloseUpdate( 18, 1, poParent->tableinfo.tab18open1, poParent->tableinfo.tab18close1, poParent->tableinfo.tab18dim_filter, poParent->tableinfo.tab18cell, poParent->tableinfo.tab18total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab18events = poParent->__events;
					poParent->tableinfo.tab18close0 = (real) poParent->parity;
					poParent->tableinfo.tab18close1 = (real) poDerivedStates->transitions_parity_0_1;
					poParent->tableinfo.tab18cell = (index) poParent->TableCellIndex( 18, 2,
						poDerivedStates->split_integer_age_AGE_15_17,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab18total_cell = (index) poParent->tableinfo.tab18cell;
					}
					else {
						poParent->tableinfo.tab18total_cell = (index) poParent->TableTotalCellIndex( 18, 2,
						poDerivedStates->split_integer_age_AGE_15_17,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab18dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabBirthsByAgeGroup = value;
}

logical PersonStateFunctions::Set_table_filter_TabCohortChildMortality( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabCohortChildMortality != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabCohortChildMortality;
		poParent->table_filter_TabCohortChildMortality = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[22] >= 0 ) {
				if ( poParent->tableinfo.tab22events == -1 || poParent->tableinfo.tab22events < poParent->__events ) {
					if ( poParent->tableinfo.tab22events != -1 ) {
						poParent->tableinfo.tab22open0 = (real) poParent->CloseUpdate( 22, 0, poParent->tableinfo.tab22open0, poParent->tableinfo.tab22close0, poParent->tableinfo.tab22dim_filter, poParent->tableinfo.tab22cell, poParent->tableinfo.tab22total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab22open1 = (real) poParent->CloseUpdate( 22, 1, poParent->tableinfo.tab22open1, poParent->tableinfo.tab22close1, poParent->tableinfo.tab22dim_filter, poParent->tableinfo.tab22cell, poParent->tableinfo.tab22total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab22events = poParent->__events;
					poParent->tableinfo.tab22close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab22close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab22cell = (index) poParent->TableCellIndex( 22, 2,
						poParent->sim_cmr - r_min_CHILD_MORT_RANGE,
						poParent->sim_yob - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab22total_cell = (index) poParent->tableinfo.tab22cell;
					}
					else {
						poParent->tableinfo.tab22total_cell = (index) poParent->TableTotalCellIndex( 22, 2,
						poParent->sim_cmr - r_min_CHILD_MORT_RANGE,
						poParent->sim_yob - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab22dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabCohortChildMortality = value;
}

logical PersonStateFunctions::Set_table_filter_TabDeathRates( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabDeathRates != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabDeathRates;
		poParent->table_filter_TabDeathRates = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[20] >= 0 ) {
				if ( poParent->tableinfo.tab20events == -1 || poParent->tableinfo.tab20events < poParent->__events ) {
					if ( poParent->tableinfo.tab20events != -1 ) {
						poParent->tableinfo.tab20open0 = (real) poParent->CloseUpdate( 20, 0, poParent->tableinfo.tab20open0, poParent->tableinfo.tab20close0, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab20open1 = (real) poParent->CloseUpdate( 20, 1, poParent->tableinfo.tab20open1, poParent->tableinfo.tab20close1, poParent->tableinfo.tab20dim_filter, poParent->tableinfo.tab20cell, poParent->tableinfo.tab20total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab20events = poParent->__events;
					poParent->tableinfo.tab20close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab20close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab20cell = (index) poParent->TableCellIndex( 20, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->integer_age - r_min_AGE_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab20total_cell = (index) poParent->tableinfo.tab20cell;
					}
					else {
						poParent->tableinfo.tab20total_cell = (index) poParent->TableTotalCellIndex( 20, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->integer_age - r_min_AGE_RANGE );
					}
					poParent->tableinfo.tab20dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabDeathRates = value;
}

logical PersonStateFunctions::Set_table_filter_TabDeathsByAgeGroup( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabDeathsByAgeGroup != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabDeathsByAgeGroup;
		poParent->table_filter_TabDeathsByAgeGroup = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[21] >= 0 ) {
				if ( poParent->tableinfo.tab21events == -1 || poParent->tableinfo.tab21events < poParent->__events ) {
					if ( poParent->tableinfo.tab21events != -1 ) {
						poParent->tableinfo.tab21open0 = (real) poParent->CloseUpdate( 21, 0, poParent->tableinfo.tab21open0, poParent->tableinfo.tab21close0, poParent->tableinfo.tab21dim_filter, poParent->tableinfo.tab21cell, poParent->tableinfo.tab21total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab21open1 = (real) poParent->CloseUpdate( 21, 1, poParent->tableinfo.tab21open1, poParent->tableinfo.tab21close1, poParent->tableinfo.tab21dim_filter, poParent->tableinfo.tab21cell, poParent->tableinfo.tab21total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab21events = poParent->__events;
					poParent->tableinfo.tab21close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab21close1 = (real) poDerivedStates->transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab21cell = (index) poParent->TableCellIndex( 21, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poDerivedStates->split_integer_age_AGE_MORT );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab21total_cell = (index) poParent->tableinfo.tab21cell;
					}
					else {
						poParent->tableinfo.tab21total_cell = (index) poParent->TableTotalCellIndex( 21, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poDerivedStates->split_integer_age_AGE_MORT );
					}
					poParent->tableinfo.tab21dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabDeathsByAgeGroup = value;
}

logical PersonStateFunctions::Set_table_filter_TabEduc12ByPob( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabEduc12ByPob != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabEduc12ByPob;
		poParent->table_filter_TabEduc12ByPob = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[28] >= 0 ) {
				if ( poParent->tableinfo.tab28events == -1 || poParent->tableinfo.tab28events < poParent->__events ) {
					if ( poParent->tableinfo.tab28events != -1 ) {
						poParent->tableinfo.tab28open0 = (real) poParent->CloseUpdate( 28, 0, poParent->tableinfo.tab28open0, poParent->tableinfo.tab28close0, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open1 = (real) poParent->CloseUpdate( 28, 1, poParent->tableinfo.tab28open1, poParent->tableinfo.tab28close1, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open2 = (real) poParent->CloseUpdate( 28, 2, poParent->tableinfo.tab28open2, poParent->tableinfo.tab28close2, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab28open3 = (real) poParent->CloseUpdate( 28, 3, poParent->tableinfo.tab28open3, poParent->tableinfo.tab28close3, poParent->tableinfo.tab28dim_filter, poParent->tableinfo.tab28cell, poParent->tableinfo.tab28total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab28events = poParent->__events;
					poParent->tableinfo.tab28close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab28close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab28close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab28close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab28cell = (index) poParent->TableCellIndex( 28, 3,
						poParent->province_birth,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab28total_cell = (index) poParent->tableinfo.tab28cell;
					}
					else {
						poParent->tableinfo.tab28total_cell = (index) poParent->TableTotalCellIndex( 28, 3,
						poParent->province_birth,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab28dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabEduc12ByPob = value;
}

logical PersonStateFunctions::Set_table_filter_TabEduc12ByPor( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabEduc12ByPor != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabEduc12ByPor;
		poParent->table_filter_TabEduc12ByPor = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[29] >= 0 ) {
				if ( poParent->tableinfo.tab29events == -1 || poParent->tableinfo.tab29events < poParent->__events ) {
					if ( poParent->tableinfo.tab29events != -1 ) {
						poParent->tableinfo.tab29open0 = (real) poParent->CloseUpdate( 29, 0, poParent->tableinfo.tab29open0, poParent->tableinfo.tab29close0, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open1 = (real) poParent->CloseUpdate( 29, 1, poParent->tableinfo.tab29open1, poParent->tableinfo.tab29close1, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open2 = (real) poParent->CloseUpdate( 29, 2, poParent->tableinfo.tab29open2, poParent->tableinfo.tab29close2, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab29open3 = (real) poParent->CloseUpdate( 29, 3, poParent->tableinfo.tab29open3, poParent->tableinfo.tab29close3, poParent->tableinfo.tab29dim_filter, poParent->tableinfo.tab29cell, poParent->tableinfo.tab29total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab29events = poParent->__events;
					poParent->tableinfo.tab29close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab29close1 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab29close2 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab29close3 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab29cell = (index) poParent->TableCellIndex( 29, 3,
						poParent->province_nat,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab29total_cell = (index) poParent->tableinfo.tab29cell;
					}
					else {
						poParent->tableinfo.tab29total_cell = (index) poParent->TableTotalCellIndex( 29, 3,
						poParent->province_nat,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab29dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabEduc12ByPor = value;
}

logical PersonStateFunctions::Set_table_filter_TabEduc2015( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabEduc2015 != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabEduc2015;
		poParent->table_filter_TabEduc2015 = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[30] >= 0 ) {
				if ( poParent->tableinfo.tab30events == -1 || poParent->tableinfo.tab30events < poParent->__events ) {
					if ( poParent->tableinfo.tab30events != -1 ) {
						poParent->tableinfo.tab30open0 = (real) poParent->CloseUpdate( 30, 0, poParent->tableinfo.tab30open0, poParent->tableinfo.tab30close0, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open1 = (real) poParent->CloseUpdate( 30, 1, poParent->tableinfo.tab30open1, poParent->tableinfo.tab30close1, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open2 = (real) poParent->CloseUpdate( 30, 2, poParent->tableinfo.tab30open2, poParent->tableinfo.tab30close2, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab30events = poParent->__events;
					poParent->tableinfo.tab30close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab30close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab30close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab30cell = (index) poParent->TableCellIndex( 30, 3,
						poParent->in_capital,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab30total_cell = (index) poParent->tableinfo.tab30cell;
					}
					else {
						poParent->tableinfo.tab30total_cell = (index) poParent->TableTotalCellIndex( 30, 3,
						poParent->in_capital,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab30dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabEduc2015 = value;
}

logical PersonStateFunctions::Set_table_filter_TabEduc2040( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabEduc2040 != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabEduc2040;
		poParent->table_filter_TabEduc2040 = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[31] >= 0 ) {
				if ( poParent->tableinfo.tab31events == -1 || poParent->tableinfo.tab31events < poParent->__events ) {
					if ( poParent->tableinfo.tab31events != -1 ) {
						poParent->tableinfo.tab31open0 = (real) poParent->CloseUpdate( 31, 0, poParent->tableinfo.tab31open0, poParent->tableinfo.tab31close0, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open1 = (real) poParent->CloseUpdate( 31, 1, poParent->tableinfo.tab31open1, poParent->tableinfo.tab31close1, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open2 = (real) poParent->CloseUpdate( 31, 2, poParent->tableinfo.tab31open2, poParent->tableinfo.tab31close2, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab31events = poParent->__events;
					poParent->tableinfo.tab31close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab31close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab31close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab31cell = (index) poParent->TableCellIndex( 31, 3,
						poParent->in_capital,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab31total_cell = (index) poParent->tableinfo.tab31cell;
					}
					else {
						poParent->tableinfo.tab31total_cell = (index) poParent->TableTotalCellIndex( 31, 3,
						poParent->in_capital,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab31dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabEduc2040 = value;
}

logical PersonStateFunctions::Set_table_filter_TabEduc2065( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabEduc2065 != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabEduc2065;
		poParent->table_filter_TabEduc2065 = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[32] >= 0 ) {
				if ( poParent->tableinfo.tab32events == -1 || poParent->tableinfo.tab32events < poParent->__events ) {
					if ( poParent->tableinfo.tab32events != -1 ) {
						poParent->tableinfo.tab32open0 = (real) poParent->CloseUpdate( 32, 0, poParent->tableinfo.tab32open0, poParent->tableinfo.tab32close0, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open1 = (real) poParent->CloseUpdate( 32, 1, poParent->tableinfo.tab32open1, poParent->tableinfo.tab32close1, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open2 = (real) poParent->CloseUpdate( 32, 2, poParent->tableinfo.tab32open2, poParent->tableinfo.tab32close2, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab32events = poParent->__events;
					poParent->tableinfo.tab32close0 = (real) poDerivedStates->Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab32close1 = (real) poDerivedStates->Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab32close2 = (real) poDerivedStates->Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab32cell = (index) poParent->TableCellIndex( 32, 3,
						poParent->in_capital,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab32total_cell = (index) poParent->tableinfo.tab32cell;
					}
					else {
						poParent->tableinfo.tab32total_cell = (index) poParent->TableTotalCellIndex( 32, 3,
						poParent->in_capital,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE_FIVE );
					}
					poParent->tableinfo.tab32dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabEduc2065 = value;
}

logical PersonStateFunctions::Set_table_filter_TabEmigration( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabEmigration != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabEmigration;
		poParent->table_filter_TabEmigration = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[26] >= 0 ) {
				if ( poParent->tableinfo.tab26events == -1 || poParent->tableinfo.tab26events < poParent->__events ) {
					if ( poParent->tableinfo.tab26events != -1 ) {
						poParent->tableinfo.tab26open0 = (real) poParent->CloseUpdate( 26, 0, poParent->tableinfo.tab26open0, poParent->tableinfo.tab26close0, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab26open1 = (real) poParent->CloseUpdate( 26, 1, poParent->tableinfo.tab26open1, poParent->tableinfo.tab26close1, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab26events = poParent->__events;
					poParent->tableinfo.tab26close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab26close1 = (real) poDerivedStates->transitions_has_emigrated_FALSE_TRUE;
					poParent->tableinfo.tab26cell = (index) poParent->TableCellIndex( 26, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab26total_cell = (index) poParent->tableinfo.tab26cell;
					}
					else {
						poParent->tableinfo.tab26total_cell = (index) poParent->TableTotalCellIndex( 26, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab26dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabEmigration = value;
}

logical PersonStateFunctions::Set_table_filter_TabFertilityRates( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabFertilityRates != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabFertilityRates;
		poParent->table_filter_TabFertilityRates = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[16] >= 0 ) {
				if ( poParent->tableinfo.tab16events == -1 || poParent->tableinfo.tab16events < poParent->__events ) {
					if ( poParent->tableinfo.tab16events != -1 ) {
						poParent->tableinfo.tab16open0 = (real) poParent->CloseUpdate( 16, 0, poParent->tableinfo.tab16open0, poParent->tableinfo.tab16close0, poParent->tableinfo.tab16dim_filter, poParent->tableinfo.tab16cell, poParent->tableinfo.tab16total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab16open1 = (real) poParent->CloseUpdate( 16, 1, poParent->tableinfo.tab16open1, poParent->tableinfo.tab16close1, poParent->tableinfo.tab16dim_filter, poParent->tableinfo.tab16cell, poParent->tableinfo.tab16total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab16events = poParent->__events;
					poParent->tableinfo.tab16close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab16close1 = (real) poParent->parity;
					poParent->tableinfo.tab16cell = (index) poParent->TableCellIndex( 16, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->fert_age - r_min_FERTILE_AGE_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab16total_cell = (index) poParent->tableinfo.tab16cell;
					}
					else {
						poParent->tableinfo.tab16total_cell = (index) poParent->TableTotalCellIndex( 16, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->fert_age - r_min_FERTILE_AGE_RANGE );
					}
					poParent->tableinfo.tab16dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabFertilityRates = value;
}

logical PersonStateFunctions::Set_table_filter_TabImmigrationSexProvinceAge( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabImmigrationSexProvinceAge != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabImmigrationSexProvinceAge;
		poParent->table_filter_TabImmigrationSexProvinceAge = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[25] >= 0 ) {
				if ( poParent->tableinfo.tab25events == -1 || poParent->tableinfo.tab25events < poParent->__events ) {
					if ( poParent->tableinfo.tab25events != -1 ) {
						Set_table_unit_TabImmigrationSexProvinceAge(poParent->table_unit_TabImmigrationSexProvinceAge+1);
						poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
						poParent->tableinfo.tab25open0 = (real) poParent->CloseUpdate( 25, 0, poParent->tableinfo.tab25open0, poParent->tableinfo.tab25close0, poParent->tableinfo.tab25dim_filter, poParent->tableinfo.tab25cell, poParent->tableinfo.tab25total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab25events = poParent->__events;
					poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
					poParent->tableinfo.tab25cell = (index) poParent->TableCellIndex( 25, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab25total_cell = (index) poParent->tableinfo.tab25cell;
					}
					else {
						poParent->tableinfo.tab25total_cell = (index) poParent->TableTotalCellIndex( 25, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab25dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabImmigrationSexProvinceAge = value;
}

logical PersonStateFunctions::Set_table_filter_TabInternalMigrationRate( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabInternalMigrationRate != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabInternalMigrationRate;
		poParent->table_filter_TabInternalMigrationRate = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[24] >= 0 ) {
				if ( poParent->tableinfo.tab24events == -1 || poParent->tableinfo.tab24events < poParent->__events ) {
					if ( poParent->tableinfo.tab24events != -1 ) {
						poParent->tableinfo.tab24open0 = (real) poParent->CloseUpdate( 24, 0, poParent->tableinfo.tab24open0, poParent->tableinfo.tab24close0, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab24open1 = (real) poParent->CloseUpdate( 24, 1, poParent->tableinfo.tab24open1, poParent->tableinfo.tab24close1, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab24events = poParent->__events;
					poParent->tableinfo.tab24close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab24close1 = (real) poParent->number_moves;
					poParent->tableinfo.tab24cell = (index) poParent->TableCellIndex( 24, 3,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab24total_cell = (index) poParent->tableinfo.tab24cell;
					}
					else {
						poParent->tableinfo.tab24total_cell = (index) poParent->TableTotalCellIndex( 24, 3,
						poParent->sex,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->province_nat );
					}
					poParent->tableinfo.tab24dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabInternalMigrationRate = value;
}

logical PersonStateFunctions::Set_table_filter_TabMothersExperience( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabMothersExperience != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabMothersExperience;
		poParent->table_filter_TabMothersExperience = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == -1 || poParent->tableinfo.tab35events < poParent->__events ) {
					if ( poParent->tableinfo.tab35events != -1 ) {
						poParent->tableinfo.tab35open0 = (real) poParent->CloseUpdate( 35, 0, poParent->tableinfo.tab35open0, poParent->tableinfo.tab35close0, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open1 = (real) poParent->CloseUpdate( 35, 1, poParent->tableinfo.tab35open1, poParent->tableinfo.tab35close1, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open2 = (real) poParent->CloseUpdate( 35, 2, poParent->tableinfo.tab35open2, poParent->tableinfo.tab35close2, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						Set_table_unit_TabMothersExperience(poParent->table_unit_TabMothersExperience+1);
						poParent->tableinfo.tab35close3 = (real) poParent->table_unit_TabMothersExperience;
						poParent->tableinfo.tab35open3 = (real) poParent->CloseUpdate( 35, 3, poParent->tableinfo.tab35open3, poParent->tableinfo.tab35close3, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open4 = (real) poParent->CloseUpdate( 35, 4, poParent->tableinfo.tab35open4, poParent->tableinfo.tab35close4, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open5 = (real) poParent->CloseUpdate( 35, 5, poParent->tableinfo.tab35open5, poParent->tableinfo.tab35close5, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open6 = (real) poParent->CloseUpdate( 35, 6, poParent->tableinfo.tab35open6, poParent->tableinfo.tab35close6, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open7 = (real) poParent->CloseUpdate( 35, 7, poParent->tableinfo.tab35open7, poParent->tableinfo.tab35close7, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open8 = (real) poParent->CloseUpdate( 35, 8, poParent->tableinfo.tab35open8, poParent->tableinfo.tab35close8, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open9 = (real) poParent->CloseUpdate( 35, 9, poParent->tableinfo.tab35open9, poParent->tableinfo.tab35close9, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab35open10 = (real) poParent->CloseUpdate( 35, 10, poParent->tableinfo.tab35open10, poParent->tableinfo.tab35close10, poParent->tableinfo.tab35dim_filter, poParent->tableinfo.tab35cell, poParent->tableinfo.tab35total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab35events = poParent->__events;
					poParent->tableinfo.tab35close0 = (real) poDerivedStates->entrances_primary_level_PL_GRAD;
					poParent->tableinfo.tab35close1 = (real) poParent->infant_deaths;
					poParent->tableinfo.tab35close2 = (real) poParent->parity;
					poParent->tableinfo.tab35close3 = (real) poParent->table_unit_TabMothersExperience;
					poParent->tableinfo.tab35close4 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab35close5 = (real) poDerivedStates->transitions_integer_age_14_15;
					poParent->tableinfo.tab35close6 = (real) poDerivedStates->transitions_integer_age_19_20;
					poParent->tableinfo.tab35close7 = (real) poDerivedStates->transitions_integer_age_9_10;
					poParent->tableinfo.tab35close8 = (real) poDerivedStates->transitions_parity_0_1;
					poParent->tableinfo.tab35close9 = (real) poDerivedStates->value_at_transitions_in_union_FALSE_TRUE_age;
					poParent->tableinfo.tab35close10 = (real) poDerivedStates->value_at_transitions_parity_0_1_age;
					poParent->tableinfo.tab35cell = (index) poParent->TableCellIndex( 35, 1,
						poParent->sim_yob - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab35total_cell = (index) poParent->tableinfo.tab35cell;
					}
					else {
						poParent->tableinfo.tab35total_cell = (index) poParent->TableTotalCellIndex( 35, 1,
						poParent->sim_yob - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab35dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabMothersExperience = value;
}

logical PersonStateFunctions::Set_table_filter_TabNumberBirths( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabNumberBirths != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabNumberBirths;
		poParent->table_filter_TabNumberBirths = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[17] >= 0 ) {
				if ( poParent->tableinfo.tab17events == -1 || poParent->tableinfo.tab17events < poParent->__events ) {
					if ( poParent->tableinfo.tab17events != -1 ) {
						poParent->tableinfo.tab17open0 = (real) poParent->CloseUpdate( 17, 0, poParent->tableinfo.tab17open0, poParent->tableinfo.tab17close0, poParent->tableinfo.tab17dim_filter, poParent->tableinfo.tab17cell, poParent->tableinfo.tab17total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab17events = poParent->__events;
					poParent->tableinfo.tab17close0 = (real) poDerivedStates->entrances_set_alive_TRUE;
					poParent->tableinfo.tab17cell = (index) poParent->TableCellIndex( 17, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->sex );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab17total_cell = (index) poParent->tableinfo.tab17cell;
					}
					else {
						poParent->tableinfo.tab17total_cell = (index) poParent->TableTotalCellIndex( 17, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->sex );
					}
					poParent->tableinfo.tab17dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabNumberBirths = value;
}

logical PersonStateFunctions::Set_table_filter_TabPopProvAgeEducSex( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabPopProvAgeEducSex != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabPopProvAgeEducSex;
		poParent->table_filter_TabPopProvAgeEducSex = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[27] >= 0 ) {
				if ( poParent->tableinfo.tab27events == -1 || poParent->tableinfo.tab27events < poParent->__events ) {
					if ( poParent->tableinfo.tab27events != -1 ) {
						poParent->tableinfo.tab27open0 = (real) poParent->CloseUpdate( 27, 0, poParent->tableinfo.tab27open0, poParent->tableinfo.tab27close0, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open1 = (real) poParent->CloseUpdate( 27, 1, poParent->tableinfo.tab27open1, poParent->tableinfo.tab27close1, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open2 = (real) poParent->CloseUpdate( 27, 2, poParent->tableinfo.tab27open2, poParent->tableinfo.tab27close2, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open3 = (real) poParent->CloseUpdate( 27, 3, poParent->tableinfo.tab27open3, poParent->tableinfo.tab27close3, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab27open4 = (real) poParent->CloseUpdate( 27, 4, poParent->tableinfo.tab27open4, poParent->tableinfo.tab27close4, poParent->tableinfo.tab27dim_filter, poParent->tableinfo.tab27cell, poParent->tableinfo.tab27total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab27events = poParent->__events;
					poParent->tableinfo.tab27close0 = (real) poDerivedStates->Get_duration_is_aded0_TRUE();
					poParent->tableinfo.tab27close1 = (real) poDerivedStates->Get_duration_is_aded1_TRUE();
					poParent->tableinfo.tab27close2 = (real) poDerivedStates->Get_duration_is_aded2_TRUE();
					poParent->tableinfo.tab27close3 = (real) poDerivedStates->Get_duration_is_child_TRUE();
					poParent->tableinfo.tab27close4 = (real) poDerivedStates->Get_duration_is_old_TRUE();
					poParent->tableinfo.tab27cell = (index) poParent->TableCellIndex( 27, 3,
						poParent->province_nat,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab27total_cell = (index) poParent->tableinfo.tab27cell;
					}
					else {
						poParent->tableinfo.tab27total_cell = (index) poParent->TableTotalCellIndex( 27, 3,
						poParent->province_nat,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab27dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabPopProvAgeEducSex = value;
}

logical PersonStateFunctions::Set_table_filter_TabPopulationAgeSexProvince( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabPopulationAgeSexProvince != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabPopulationAgeSexProvince;
		poParent->table_filter_TabPopulationAgeSexProvince = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[14] >= 0 ) {
				if ( poParent->tableinfo.tab14events == -1 || poParent->tableinfo.tab14events < poParent->__events ) {
					if ( poParent->tableinfo.tab14events != -1 ) {
						poParent->tableinfo.tab14open0 = (real) poParent->CloseUpdate( 14, 0, poParent->tableinfo.tab14open0, poParent->tableinfo.tab14close0, poParent->tableinfo.tab14dim_filter, poParent->tableinfo.tab14cell, poParent->tableinfo.tab14total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab14events = poParent->__events;
					poParent->tableinfo.tab14close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab14cell = (index) poParent->TableCellIndex( 14, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab14total_cell = (index) poParent->tableinfo.tab14cell;
					}
					else {
						poParent->tableinfo.tab14total_cell = (index) poParent->TableTotalCellIndex( 14, 4,
						poParent->sex,
						poParent->province_nat,
						poDerivedStates->split_integer_age_AGE5_PART,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab14dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabPopulationAgeSexProvince = value;
}

logical PersonStateFunctions::Set_table_filter_TabSimulatedStartingPopulation( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabSimulatedStartingPopulation != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabSimulatedStartingPopulation;
		poParent->table_filter_TabSimulatedStartingPopulation = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[15] >= 0 ) {
				if ( poParent->tableinfo.tab15events == -1 || poParent->tableinfo.tab15events < poParent->__events ) {
					if ( poParent->tableinfo.tab15events != -1 ) {
						Set_table_unit_TabSimulatedStartingPopulation(poParent->table_unit_TabSimulatedStartingPopulation+1);
						poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
						poParent->tableinfo.tab15open0 = (real) poParent->CloseUpdate( 15, 0, poParent->tableinfo.tab15open0, poParent->tableinfo.tab15close0, poParent->tableinfo.tab15dim_filter, poParent->tableinfo.tab15cell, poParent->tableinfo.tab15total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab15events = poParent->__events;
					poParent->tableinfo.tab15close0 = (real) poParent->table_unit_TabSimulatedStartingPopulation;
					poParent->tableinfo.tab15cell = (index) poParent->TableCellIndex( 15, 3,
						poParent->sex,
						poParent->integer_age - r_min_AGE_RANGE,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab15total_cell = (index) poParent->tableinfo.tab15cell;
					}
					else {
						poParent->tableinfo.tab15total_cell = (index) poParent->TableTotalCellIndex( 15, 3,
						poParent->sex,
						poParent->integer_age - r_min_AGE_RANGE,
						poParent->province_nat );
					}
					poParent->tableinfo.tab15dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabSimulatedStartingPopulation = value;
}

logical PersonStateFunctions::Set_table_filter_TabTotalPopulationSexProvince( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabTotalPopulationSexProvince != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabTotalPopulationSexProvince;
		poParent->table_filter_TabTotalPopulationSexProvince = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[13] >= 0 ) {
				if ( poParent->tableinfo.tab13events == -1 || poParent->tableinfo.tab13events < poParent->__events ) {
					if ( poParent->tableinfo.tab13events != -1 ) {
						poParent->tableinfo.tab13open0 = (real) poParent->CloseUpdate( 13, 0, poParent->tableinfo.tab13open0, poParent->tableinfo.tab13close0, poParent->tableinfo.tab13dim_filter, poParent->tableinfo.tab13cell, poParent->tableinfo.tab13total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab13events = poParent->__events;
					poParent->tableinfo.tab13close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab13cell = (index) poParent->TableCellIndex( 13, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab13total_cell = (index) poParent->tableinfo.tab13cell;
					}
					else {
						poParent->tableinfo.tab13total_cell = (index) poParent->TableTotalCellIndex( 13, 3,
						poParent->sex,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						poParent->province_nat );
					}
					poParent->tableinfo.tab13dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabTotalPopulationSexProvince = value;
}

logical PersonStateFunctions::Set_table_filter_TabUnionFormation( logical value )
{

	value = value != 0;
	if ( poParent->table_filter_TabUnionFormation != value )
	{


		logical	OldValue;
		OldValue = poParent->table_filter_TabUnionFormation;
		poParent->table_filter_TabUnionFormation = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[33] >= 0 ) {
				if ( poParent->tableinfo.tab33events == -1 || poParent->tableinfo.tab33events < poParent->__events ) {
					if ( poParent->tableinfo.tab33events != -1 ) {
						poParent->tableinfo.tab33open0 = (real) poParent->CloseUpdate( 33, 0, poParent->tableinfo.tab33open0, poParent->tableinfo.tab33close0, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open1 = (real) poParent->CloseUpdate( 33, 1, poParent->tableinfo.tab33open1, poParent->tableinfo.tab33close1, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open2 = (real) poParent->CloseUpdate( 33, 2, poParent->tableinfo.tab33open2, poParent->tableinfo.tab33close2, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open3 = (real) poParent->CloseUpdate( 33, 3, poParent->tableinfo.tab33open3, poParent->tableinfo.tab33close3, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab33open4 = (real) poParent->CloseUpdate( 33, 4, poParent->tableinfo.tab33open4, poParent->tableinfo.tab33close4, poParent->tableinfo.tab33dim_filter, poParent->tableinfo.tab33cell, poParent->tableinfo.tab33total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab33events = poParent->__events;
					poParent->tableinfo.tab33close0 = (real) poDerivedStates->Get_duration_();
					poParent->tableinfo.tab33close1 = (real) poDerivedStates->Get_duration_in_union_FALSE();
					poParent->tableinfo.tab33close2 = (real) poDerivedStates->Get_duration_in_union_TRUE();
					poParent->tableinfo.tab33close3 = (real) poDerivedStates->transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab33close4 = (real) poParent->union_formation_hazard_max_value_in;
					poParent->tableinfo.tab33cell = (index) poParent->TableCellIndex( 33, 3,
						poParent->primary_level,
						poParent->age_union - r_min_AGE_UNION,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab33total_cell = (index) poParent->tableinfo.tab33cell;
					}
					else {
						poParent->tableinfo.tab33total_cell = (index) poParent->TableTotalCellIndex( 33, 3,
						poParent->primary_level,
						poParent->age_union - r_min_AGE_UNION,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab33dim_filter = OldValue;
				}
			}
		}
	}
	return poParent->table_filter_TabUnionFormation = value;
}

int PersonStateFunctions::Set_time_in_parity( int value )
{

	if ( poParent->time_in_parity != value )
	{


		int	OldValue;
		OldValue = poParent->time_in_parity;
		poParent->time_in_parity = value;

		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
	}
	return poParent->time_in_parity = value;
}

int PersonStateFunctions::Set_time_since_last_birth( int value )
{

	if ( poParent->time_since_last_birth != value )
	{


		int	OldValue;
		OldValue = poParent->time_since_last_birth;
		poParent->time_since_last_birth = value;

		{
			int	nNewGrp;

			nNewGrp = Split( (double) poParent->time_since_last_birth, poDerivedStates->split_time_since_last_birth_DUR_TIME_IN_PARITY, 5, p_points_DUR_TIME_IN_PARITY );
			if ( nNewGrp != poDerivedStates->split_time_since_last_birth_DUR_TIME_IN_PARITY ) {
				poDerivedStates->Set_split_time_since_last_birth_DUR_TIME_IN_PARITY( nNewGrp );
			}
		}
		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
	}
	return poParent->time_since_last_birth = value;
}

double PersonStateFunctions::Set_union_formation_hazard( double value )
{

	if ( poParent->union_formation_hazard != value )
	{


		double	OldValue;
		OldValue = poParent->union_formation_hazard;
		poParent->union_formation_hazard = value;

#line 384 "code/Tables.mpp"
		Set_union_formation_hazard_max_value_in( poParent->union_formation_hazard );
#line 9419 "modgen/src/ACTORS.CPP"
		if ( poParent->poUnionFormationEvent->cRecalc == 0 ) {
			poParent->poUnionFormationEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poUnionFormationEvent );
		};
	}
	return poParent->union_formation_hazard = value;
}

double PersonStateFunctions::Set_union_formation_hazard_max_value_in( double value )
{

	if ( poParent->union_formation_hazard_max_value_in != value )
	{


		double	OldValue;
		OldValue = poParent->union_formation_hazard_max_value_in;
		poParent->union_formation_hazard_max_value_in = value;

		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[33] >= 0 && poParent->tableinfo.tab33events == poParent->__events ) {
				poParent->tableinfo.tab33close4 = (real) poParent->union_formation_hazard_max_value_in;
			}
		}
	}
	return poParent->union_formation_hazard_max_value_in = value;
}

UNION_STATUS PersonStateFunctions::Set_union_status( UNION_STATUS value )
{

	CHKLMT(value, union_status, 0, rc_size_UNION_STATUS - 1);
	if ( value < 0 ) {
		value = (UNION_STATUS) 0;
	}
	else if ( value >= rc_size_UNION_STATUS ) {
		value = (UNION_STATUS) ( rc_size_UNION_STATUS - 1 );
	}
	if ( poParent->union_status != value )
	{


		UNION_STATUS	OldValue;
		OldValue = poParent->union_status;
		poParent->union_status = value;

		if ( poParent->poBirthEvent->cRecalc == 0 ) {
			poParent->poBirthEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poBirthEvent );
		};
	}
	return poParent->union_status = value;
}

ALL_YEAR_RANGE PersonStateFunctions::Set_year_of_birth( ALL_YEAR_RANGE value )
{

	CHKLMT(value, year_of_birth, r_min_ALL_YEAR_RANGE, r_max_ALL_YEAR_RANGE);
	if ( value < r_min_ALL_YEAR_RANGE ) {
		value = r_min_ALL_YEAR_RANGE;
	}
	else if ( value > r_max_ALL_YEAR_RANGE ) {
		value = r_max_ALL_YEAR_RANGE;
	}
	if ( poParent->year_of_birth != value )
	{
#line 221 "code/Tables.mpp"
		Set_table_filter_TabCohortChildMortality( poParent->in_projected_time&& WITHIN(SIM_YOB_RANGE,value)&& WITHIN(CHILD_MORT_RANGE,poParent->integer_age) );
#line 9488 "modgen/src/ACTORS.CPP"
#line 405 "code/Tables.mpp"
		Set_table_filter_TabMothersExperience( poParent->in_projected_time&& WITHIN(SIM_YOB_RANGE,value)&& poParent->person_type== PT_CHILD&& poParent->sex== FEMALE );
#line 9491 "modgen/src/ACTORS.CPP"


		ALL_YEAR_RANGE	OldValue;
		OldValue = poParent->year_of_birth;
		poParent->year_of_birth = value;

#line 87 "code/Tables.mpp"
		Set_sim_yob( COERCE(SIM_YOB_RANGE,poParent->year_of_birth) );
#line 9500 "modgen/src/ACTORS.CPP"
	}
	return poParent->year_of_birth = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_female_0( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_female_0 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_female_0;
		poParent->lChildMortalityCalibrator_mort_female_0 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9518 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_female_0 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_female_1( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_female_1 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_female_1;
		poParent->lChildMortalityCalibrator_mort_female_1 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9536 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_female_1 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_female_2( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_female_2 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_female_2;
		poParent->lChildMortalityCalibrator_mort_female_2 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9554 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_female_2 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_female_3( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_female_3 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_female_3;
		poParent->lChildMortalityCalibrator_mort_female_3 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9572 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_female_3 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_female_4( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_female_4 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_female_4;
		poParent->lChildMortalityCalibrator_mort_female_4 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9590 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_female_4 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_male_0( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_male_0 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_male_0;
		poParent->lChildMortalityCalibrator_mort_male_0 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9608 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_male_0 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_male_1( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_male_1 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_male_1;
		poParent->lChildMortalityCalibrator_mort_male_1 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9626 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_male_1 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_male_2( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_male_2 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_male_2;
		poParent->lChildMortalityCalibrator_mort_male_2 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9644 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_male_2 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_male_3( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_male_3 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_male_3;
		poParent->lChildMortalityCalibrator_mort_male_3 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9662 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_male_3 = value;
}

double PersonStateFunctions::Set_lChildMortalityCalibrator_mort_male_4( double value )
{

	if ( poParent->lChildMortalityCalibrator_mort_male_4 != value )
	{


		double	OldValue;
		OldValue = poParent->lChildMortalityCalibrator_mort_male_4;
		poParent->lChildMortalityCalibrator_mort_male_4 = value;

#line 298 "code/ChildMortality.mpp"
		Set_child_mortality( ( poParent->lChildMortalityCalibrator== NULL )? 0.0: ( poParent->integer_age== 0&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_0: ( poParent->integer_age== 1&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_1: ( poParent->integer_age== 2&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_2: ( poParent->integer_age== 3&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_3: ( poParent->integer_age== 4&& poParent->sex== MALE )? poParent->lChildMortalityCalibrator_mort_male_4: ( poParent->integer_age== 0&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_0: ( poParent->integer_age== 1&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_1: ( poParent->integer_age== 2&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_2: ( poParent->integer_age== 3&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_3: ( poParent->integer_age== 4&& poParent->sex== FEMALE )? poParent->lChildMortalityCalibrator_mort_female_4: 0.0 );
#line 9680 "modgen/src/ACTORS.CPP"
	}
	return poParent->lChildMortalityCalibrator_mort_male_4 = value;
}

counter PersonDerivedStates::Set_changes_parity( counter value )
{

	if ( changes_parity != value ) {

counter		OldValue;
		OldValue = changes_parity;
		changes_parity = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[19] >= 0 ) {
				if ( poParent->tableinfo.tab19events == poParent->__events ) {
					poParent->tableinfo.tab19close0 = (real) changes_parity;
				}
			}
		}
	}
	return changes_parity = value;
}

TIME PersonDerivedStates::Set_duration_( TIME value )
{

	if ( duration_ != value ) {

TIME		OldValue;
		OldValue = duration_;
		duration_ = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[0] >= 0 ) {
				if ( poParent->tableinfo.tab0events == poParent->__events ) {
					poParent->tableinfo.tab0close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[1] >= 0 ) {
				if ( poParent->tableinfo.tab1events == poParent->__events ) {
					poParent->tableinfo.tab1close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[10] >= 0 ) {
				if ( poParent->tableinfo.tab10events == poParent->__events ) {
					poParent->tableinfo.tab10close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[13] >= 0 ) {
				if ( poParent->tableinfo.tab13events == poParent->__events ) {
					poParent->tableinfo.tab13close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[14] >= 0 ) {
				if ( poParent->tableinfo.tab14events == poParent->__events ) {
					poParent->tableinfo.tab14close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[16] >= 0 ) {
				if ( poParent->tableinfo.tab16events == poParent->__events ) {
					poParent->tableinfo.tab16close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[20] >= 0 ) {
				if ( poParent->tableinfo.tab20events == poParent->__events ) {
					poParent->tableinfo.tab20close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[21] >= 0 ) {
				if ( poParent->tableinfo.tab21events == poParent->__events ) {
					poParent->tableinfo.tab21close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[22] >= 0 ) {
				if ( poParent->tableinfo.tab22events == poParent->__events ) {
					poParent->tableinfo.tab22close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[24] >= 0 ) {
				if ( poParent->tableinfo.tab24events == poParent->__events ) {
					poParent->tableinfo.tab24close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[26] >= 0 ) {
				if ( poParent->tableinfo.tab26events == poParent->__events ) {
					poParent->tableinfo.tab26close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[28] >= 0 ) {
				if ( poParent->tableinfo.tab28events == poParent->__events ) {
					poParent->tableinfo.tab28close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[29] >= 0 ) {
				if ( poParent->tableinfo.tab29events == poParent->__events ) {
					poParent->tableinfo.tab29close0 = (real) duration_;
				}
			}
			if ( gplTableIndexes[33] >= 0 ) {
				if ( poParent->tableinfo.tab33events == poParent->__events ) {
					poParent->tableinfo.tab33close0 = (real) duration_;
				}
			}
		}
	}
	return duration_ = value;
}

TIME PersonDerivedStates::Set_duration_in_union_FALSE( TIME value )
{

	if ( duration_in_union_FALSE != value ) {

TIME		OldValue;
		OldValue = duration_in_union_FALSE;
		duration_in_union_FALSE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[33] >= 0 ) {
				if ( poParent->tableinfo.tab33events == poParent->__events ) {
					poParent->tableinfo.tab33close1 = (real) duration_in_union_FALSE;
				}
			}
		}
	}
	return duration_in_union_FALSE = value;
}

TIME PersonDerivedStates::Set_duration_in_union_TRUE( TIME value )
{

	if ( duration_in_union_TRUE != value ) {

TIME		OldValue;
		OldValue = duration_in_union_TRUE;
		duration_in_union_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[33] >= 0 ) {
				if ( poParent->tableinfo.tab33events == poParent->__events ) {
					poParent->tableinfo.tab33close2 = (real) duration_in_union_TRUE;
				}
			}
		}
	}
	return duration_in_union_TRUE = value;
}

TIME PersonDerivedStates::Set_duration_is_aded0_TRUE( TIME value )
{

	if ( duration_is_aded0_TRUE != value ) {

TIME		OldValue;
		OldValue = duration_is_aded0_TRUE;
		duration_is_aded0_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[27] >= 0 ) {
				if ( poParent->tableinfo.tab27events == poParent->__events ) {
					poParent->tableinfo.tab27close0 = (real) duration_is_aded0_TRUE;
				}
			}
		}
	}
	return duration_is_aded0_TRUE = value;
}

TIME PersonDerivedStates::Set_duration_is_aded1_TRUE( TIME value )
{

	if ( duration_is_aded1_TRUE != value ) {

TIME		OldValue;
		OldValue = duration_is_aded1_TRUE;
		duration_is_aded1_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[27] >= 0 ) {
				if ( poParent->tableinfo.tab27events == poParent->__events ) {
					poParent->tableinfo.tab27close1 = (real) duration_is_aded1_TRUE;
				}
			}
		}
	}
	return duration_is_aded1_TRUE = value;
}

TIME PersonDerivedStates::Set_duration_is_aded2_TRUE( TIME value )
{

	if ( duration_is_aded2_TRUE != value ) {

TIME		OldValue;
		OldValue = duration_is_aded2_TRUE;
		duration_is_aded2_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[27] >= 0 ) {
				if ( poParent->tableinfo.tab27events == poParent->__events ) {
					poParent->tableinfo.tab27close2 = (real) duration_is_aded2_TRUE;
				}
			}
		}
	}
	return duration_is_aded2_TRUE = value;
}

TIME PersonDerivedStates::Set_duration_is_child_TRUE( TIME value )
{

	if ( duration_is_child_TRUE != value ) {

TIME		OldValue;
		OldValue = duration_is_child_TRUE;
		duration_is_child_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[27] >= 0 ) {
				if ( poParent->tableinfo.tab27events == poParent->__events ) {
					poParent->tableinfo.tab27close3 = (real) duration_is_child_TRUE;
				}
			}
		}
	}
	return duration_is_child_TRUE = value;
}

TIME PersonDerivedStates::Set_duration_is_old_TRUE( TIME value )
{

	if ( duration_is_old_TRUE != value ) {

TIME		OldValue;
		OldValue = duration_is_old_TRUE;
		duration_is_old_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[27] >= 0 ) {
				if ( poParent->tableinfo.tab27events == poParent->__events ) {
					poParent->tableinfo.tab27close4 = (real) duration_is_old_TRUE;
				}
			}
		}
	}
	return duration_is_old_TRUE = value;
}

TIME PersonDerivedStates::Set_duration_primary_level_PL_ENTER( TIME value )
{

	if ( duration_primary_level_PL_ENTER != value ) {

TIME		OldValue;
		OldValue = duration_primary_level_PL_ENTER;
		duration_primary_level_PL_ENTER = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[2] >= 0 ) {
				if ( poParent->tableinfo.tab2events == poParent->__events ) {
					poParent->tableinfo.tab2close0 = (real) duration_primary_level_PL_ENTER;
				}
			}
			if ( gplTableIndexes[10] >= 0 ) {
				if ( poParent->tableinfo.tab10events == poParent->__events ) {
					poParent->tableinfo.tab10close1 = (real) duration_primary_level_PL_ENTER;
				}
			}
			if ( gplTableIndexes[28] >= 0 ) {
				if ( poParent->tableinfo.tab28events == poParent->__events ) {
					poParent->tableinfo.tab28close1 = (real) duration_primary_level_PL_ENTER;
				}
			}
			if ( gplTableIndexes[29] >= 0 ) {
				if ( poParent->tableinfo.tab29events == poParent->__events ) {
					poParent->tableinfo.tab29close1 = (real) duration_primary_level_PL_ENTER;
				}
			}
			if ( gplTableIndexes[30] >= 0 ) {
				if ( poParent->tableinfo.tab30events == poParent->__events ) {
					poParent->tableinfo.tab30close0 = (real) duration_primary_level_PL_ENTER;
				}
			}
			if ( gplTableIndexes[31] >= 0 ) {
				if ( poParent->tableinfo.tab31events == poParent->__events ) {
					poParent->tableinfo.tab31close0 = (real) duration_primary_level_PL_ENTER;
				}
			}
			if ( gplTableIndexes[32] >= 0 ) {
				if ( poParent->tableinfo.tab32events == poParent->__events ) {
					poParent->tableinfo.tab32close0 = (real) duration_primary_level_PL_ENTER;
				}
			}
		}
	}
	return duration_primary_level_PL_ENTER = value;
}

TIME PersonDerivedStates::Set_duration_primary_level_PL_GRAD( TIME value )
{

	if ( duration_primary_level_PL_GRAD != value ) {

TIME		OldValue;
		OldValue = duration_primary_level_PL_GRAD;
		duration_primary_level_PL_GRAD = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[2] >= 0 ) {
				if ( poParent->tableinfo.tab2events == poParent->__events ) {
					poParent->tableinfo.tab2close1 = (real) duration_primary_level_PL_GRAD;
				}
			}
			if ( gplTableIndexes[10] >= 0 ) {
				if ( poParent->tableinfo.tab10events == poParent->__events ) {
					poParent->tableinfo.tab10close2 = (real) duration_primary_level_PL_GRAD;
				}
			}
			if ( gplTableIndexes[28] >= 0 ) {
				if ( poParent->tableinfo.tab28events == poParent->__events ) {
					poParent->tableinfo.tab28close2 = (real) duration_primary_level_PL_GRAD;
				}
			}
			if ( gplTableIndexes[29] >= 0 ) {
				if ( poParent->tableinfo.tab29events == poParent->__events ) {
					poParent->tableinfo.tab29close2 = (real) duration_primary_level_PL_GRAD;
				}
			}
			if ( gplTableIndexes[30] >= 0 ) {
				if ( poParent->tableinfo.tab30events == poParent->__events ) {
					poParent->tableinfo.tab30close1 = (real) duration_primary_level_PL_GRAD;
				}
			}
			if ( gplTableIndexes[31] >= 0 ) {
				if ( poParent->tableinfo.tab31events == poParent->__events ) {
					poParent->tableinfo.tab31close1 = (real) duration_primary_level_PL_GRAD;
				}
			}
			if ( gplTableIndexes[32] >= 0 ) {
				if ( poParent->tableinfo.tab32events == poParent->__events ) {
					poParent->tableinfo.tab32close1 = (real) duration_primary_level_PL_GRAD;
				}
			}
		}
	}
	return duration_primary_level_PL_GRAD = value;
}

TIME PersonDerivedStates::Set_duration_primary_level_PL_NO( TIME value )
{

	if ( duration_primary_level_PL_NO != value ) {

TIME		OldValue;
		OldValue = duration_primary_level_PL_NO;
		duration_primary_level_PL_NO = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[2] >= 0 ) {
				if ( poParent->tableinfo.tab2events == poParent->__events ) {
					poParent->tableinfo.tab2close2 = (real) duration_primary_level_PL_NO;
				}
			}
			if ( gplTableIndexes[10] >= 0 ) {
				if ( poParent->tableinfo.tab10events == poParent->__events ) {
					poParent->tableinfo.tab10close3 = (real) duration_primary_level_PL_NO;
				}
			}
			if ( gplTableIndexes[28] >= 0 ) {
				if ( poParent->tableinfo.tab28events == poParent->__events ) {
					poParent->tableinfo.tab28close3 = (real) duration_primary_level_PL_NO;
				}
			}
			if ( gplTableIndexes[29] >= 0 ) {
				if ( poParent->tableinfo.tab29events == poParent->__events ) {
					poParent->tableinfo.tab29close3 = (real) duration_primary_level_PL_NO;
				}
			}
			if ( gplTableIndexes[30] >= 0 ) {
				if ( poParent->tableinfo.tab30events == poParent->__events ) {
					poParent->tableinfo.tab30close2 = (real) duration_primary_level_PL_NO;
				}
			}
			if ( gplTableIndexes[31] >= 0 ) {
				if ( poParent->tableinfo.tab31events == poParent->__events ) {
					poParent->tableinfo.tab31close2 = (real) duration_primary_level_PL_NO;
				}
			}
			if ( gplTableIndexes[32] >= 0 ) {
				if ( poParent->tableinfo.tab32events == poParent->__events ) {
					poParent->tableinfo.tab32close2 = (real) duration_primary_level_PL_NO;
				}
			}
		}
	}
	return duration_primary_level_PL_NO = value;
}

counter PersonDerivedStates::Set_entrances_primary_level_PL_GRAD( counter value )
{

	if ( entrances_primary_level_PL_GRAD != value ) {

counter		OldValue;
		OldValue = entrances_primary_level_PL_GRAD;
		entrances_primary_level_PL_GRAD = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close0 = (real) entrances_primary_level_PL_GRAD;
				}
			}
		}
	}
	return entrances_primary_level_PL_GRAD = value;
}

counter PersonDerivedStates::Set_entrances_set_alive_TRUE( counter value )
{

	if ( entrances_set_alive_TRUE != value ) {

counter		OldValue;
		OldValue = entrances_set_alive_TRUE;
		entrances_set_alive_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[4] >= 0 ) {
				if ( poParent->tableinfo.tab4events == poParent->__events ) {
					poParent->tableinfo.tab4close0 = (real) entrances_set_alive_TRUE;
				}
			}
			if ( gplTableIndexes[5] >= 0 ) {
				if ( poParent->tableinfo.tab5events == poParent->__events ) {
					poParent->tableinfo.tab5close0 = (real) entrances_set_alive_TRUE;
				}
			}
			if ( gplTableIndexes[17] >= 0 ) {
				if ( poParent->tableinfo.tab17events == poParent->__events ) {
					poParent->tableinfo.tab17close0 = (real) entrances_set_alive_TRUE;
				}
			}
		}
	}
	return entrances_set_alive_TRUE = value;
}

AGE5_PART PersonDerivedStates::Set_split_integer_age_AGE5_PART( AGE5_PART value )
{

	if ( split_integer_age_AGE5_PART != value ) {

AGE5_PART		OldValue;
		OldValue = split_integer_age_AGE5_PART;
		split_integer_age_AGE5_PART = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[14] >= 0 && ( poParent->tableinfo.tab14dim_filter || poParent->table_filter_TabPopulationAgeSexProvince ) ) {
				if ( poParent->tableinfo.tab14events == -1 || poParent->tableinfo.tab14events < poParent->__events ) {
					if ( poParent->tableinfo.tab14events != -1 ) {
						poParent->tableinfo.tab14open0 = (real) poParent->CloseUpdate( 14, 0, poParent->tableinfo.tab14open0, poParent->tableinfo.tab14close0, poParent->tableinfo.tab14dim_filter, poParent->tableinfo.tab14cell, poParent->tableinfo.tab14total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab14events = poParent->__events;
					poParent->tableinfo.tab14close0 = (real) Get_duration_();
					poParent->tableinfo.tab14cell = (index) poParent->TableCellIndex( 14, 4,
						poParent->sex,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab14total_cell = (index) poParent->tableinfo.tab14cell;
					}
					else {
						poParent->tableinfo.tab14total_cell = (index) poParent->TableTotalCellIndex( 14, 4,
						poParent->sex,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab14dim_filter = poParent->table_filter_TabPopulationAgeSexProvince;
				}
			}
			if ( gplTableIndexes[24] >= 0 && ( poParent->tableinfo.tab24dim_filter || poParent->table_filter_TabInternalMigrationRate ) ) {
				if ( poParent->tableinfo.tab24events == -1 || poParent->tableinfo.tab24events < poParent->__events ) {
					if ( poParent->tableinfo.tab24events != -1 ) {
						poParent->tableinfo.tab24open0 = (real) poParent->CloseUpdate( 24, 0, poParent->tableinfo.tab24open0, poParent->tableinfo.tab24close0, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab24open1 = (real) poParent->CloseUpdate( 24, 1, poParent->tableinfo.tab24open1, poParent->tableinfo.tab24close1, poParent->tableinfo.tab24dim_filter, poParent->tableinfo.tab24cell, poParent->tableinfo.tab24total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab24events = poParent->__events;
					poParent->tableinfo.tab24close0 = (real) Get_duration_();
					poParent->tableinfo.tab24close1 = (real) poParent->number_moves;
					poParent->tableinfo.tab24cell = (index) poParent->TableCellIndex( 24, 3,
						poParent->sex,
						OldValue,
						poParent->province_nat );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab24total_cell = (index) poParent->tableinfo.tab24cell;
					}
					else {
						poParent->tableinfo.tab24total_cell = (index) poParent->TableTotalCellIndex( 24, 3,
						poParent->sex,
						OldValue,
						poParent->province_nat );
					}
					poParent->tableinfo.tab24dim_filter = poParent->table_filter_TabInternalMigrationRate;
				}
			}
			if ( gplTableIndexes[25] >= 0 && ( poParent->tableinfo.tab25dim_filter || poParent->table_filter_TabImmigrationSexProvinceAge ) ) {
				if ( poParent->tableinfo.tab25events == -1 || poParent->tableinfo.tab25events < poParent->__events ) {
					if ( poParent->tableinfo.tab25events != -1 ) {
						poStateFunctions->Set_table_unit_TabImmigrationSexProvinceAge(poParent->table_unit_TabImmigrationSexProvinceAge+1);
						poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
						poParent->tableinfo.tab25open0 = (real) poParent->CloseUpdate( 25, 0, poParent->tableinfo.tab25open0, poParent->tableinfo.tab25close0, poParent->tableinfo.tab25dim_filter, poParent->tableinfo.tab25cell, poParent->tableinfo.tab25total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab25events = poParent->__events;
					poParent->tableinfo.tab25close0 = (real) poParent->table_unit_TabImmigrationSexProvinceAge;
					poParent->tableinfo.tab25cell = (index) poParent->TableCellIndex( 25, 4,
						poParent->sex,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab25total_cell = (index) poParent->tableinfo.tab25cell;
					}
					else {
						poParent->tableinfo.tab25total_cell = (index) poParent->TableTotalCellIndex( 25, 4,
						poParent->sex,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab25dim_filter = poParent->table_filter_TabImmigrationSexProvinceAge;
				}
			}
			if ( gplTableIndexes[26] >= 0 && ( poParent->tableinfo.tab26dim_filter || poParent->table_filter_TabEmigration ) ) {
				if ( poParent->tableinfo.tab26events == -1 || poParent->tableinfo.tab26events < poParent->__events ) {
					if ( poParent->tableinfo.tab26events != -1 ) {
						poParent->tableinfo.tab26open0 = (real) poParent->CloseUpdate( 26, 0, poParent->tableinfo.tab26open0, poParent->tableinfo.tab26close0, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab26open1 = (real) poParent->CloseUpdate( 26, 1, poParent->tableinfo.tab26open1, poParent->tableinfo.tab26close1, poParent->tableinfo.tab26dim_filter, poParent->tableinfo.tab26cell, poParent->tableinfo.tab26total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab26events = poParent->__events;
					poParent->tableinfo.tab26close0 = (real) Get_duration_();
					poParent->tableinfo.tab26close1 = (real) transitions_has_emigrated_FALSE_TRUE;
					poParent->tableinfo.tab26cell = (index) poParent->TableCellIndex( 26, 4,
						poParent->sex,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab26total_cell = (index) poParent->tableinfo.tab26cell;
					}
					else {
						poParent->tableinfo.tab26total_cell = (index) poParent->TableTotalCellIndex( 26, 4,
						poParent->sex,
						poParent->province_nat,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab26dim_filter = poParent->table_filter_TabEmigration;
				}
			}
		}
	}
	return split_integer_age_AGE5_PART = value;
}

AGE_15_16_18 PersonDerivedStates::Set_split_integer_age_AGE_15_16_18( AGE_15_16_18 value )
{

	if ( split_integer_age_AGE_15_16_18 != value ) {

AGE_15_16_18		OldValue;
		OldValue = split_integer_age_AGE_15_16_18;
		split_integer_age_AGE_15_16_18 = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[6] >= 0 && ( poParent->tableinfo.tab6dim_filter || poParent->table_filter_RTAB_BirthsByAgeMother ) ) {
				if ( poParent->tableinfo.tab6events == -1 || poParent->tableinfo.tab6events < poParent->__events ) {
					if ( poParent->tableinfo.tab6events != -1 ) {
						poParent->tableinfo.tab6open0 = (real) poParent->CloseUpdate( 6, 0, poParent->tableinfo.tab6open0, poParent->tableinfo.tab6close0, poParent->tableinfo.tab6dim_filter, poParent->tableinfo.tab6cell, poParent->tableinfo.tab6total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab6events = poParent->__events;
					poParent->tableinfo.tab6close0 = (real) poParent->parity;
					poParent->tableinfo.tab6cell = (index) poParent->TableCellIndex( 6, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab6total_cell = (index) poParent->tableinfo.tab6cell;
					}
					else {
						poParent->tableinfo.tab6total_cell = (index) poParent->TableTotalCellIndex( 6, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab6dim_filter = poParent->table_filter_RTAB_BirthsByAgeMother;
				}
			}
			if ( gplTableIndexes[7] >= 0 && ( poParent->tableinfo.tab7dim_filter || poParent->table_filter_RTAB_UnionFormationByAge ) ) {
				if ( poParent->tableinfo.tab7events == -1 || poParent->tableinfo.tab7events < poParent->__events ) {
					if ( poParent->tableinfo.tab7events != -1 ) {
						poParent->tableinfo.tab7open0 = (real) poParent->CloseUpdate( 7, 0, poParent->tableinfo.tab7open0, poParent->tableinfo.tab7close0, poParent->tableinfo.tab7dim_filter, poParent->tableinfo.tab7cell, poParent->tableinfo.tab7total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab7events = poParent->__events;
					poParent->tableinfo.tab7close0 = (real) transitions_in_union_FALSE_TRUE;
					poParent->tableinfo.tab7cell = (index) poParent->TableCellIndex( 7, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab7total_cell = (index) poParent->tableinfo.tab7cell;
					}
					else {
						poParent->tableinfo.tab7total_cell = (index) poParent->TableTotalCellIndex( 7, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab7dim_filter = poParent->table_filter_RTAB_UnionFormationByAge;
				}
			}
		}
	}
	return split_integer_age_AGE_15_16_18 = value;
}

AGE_15_17 PersonDerivedStates::Set_split_integer_age_AGE_15_17( AGE_15_17 value )
{

	if ( split_integer_age_AGE_15_17 != value ) {

AGE_15_17		OldValue;
		OldValue = split_integer_age_AGE_15_17;
		split_integer_age_AGE_15_17 = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[18] >= 0 && ( poParent->tableinfo.tab18dim_filter || poParent->table_filter_TabBirthsByAgeGroup ) ) {
				if ( poParent->tableinfo.tab18events == -1 || poParent->tableinfo.tab18events < poParent->__events ) {
					if ( poParent->tableinfo.tab18events != -1 ) {
						poParent->tableinfo.tab18open0 = (real) poParent->CloseUpdate( 18, 0, poParent->tableinfo.tab18open0, poParent->tableinfo.tab18close0, poParent->tableinfo.tab18dim_filter, poParent->tableinfo.tab18cell, poParent->tableinfo.tab18total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab18open1 = (real) poParent->CloseUpdate( 18, 1, poParent->tableinfo.tab18open1, poParent->tableinfo.tab18close1, poParent->tableinfo.tab18dim_filter, poParent->tableinfo.tab18cell, poParent->tableinfo.tab18total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab18events = poParent->__events;
					poParent->tableinfo.tab18close0 = (real) poParent->parity;
					poParent->tableinfo.tab18close1 = (real) transitions_parity_0_1;
					poParent->tableinfo.tab18cell = (index) poParent->TableCellIndex( 18, 2,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab18total_cell = (index) poParent->tableinfo.tab18cell;
					}
					else {
						poParent->tableinfo.tab18total_cell = (index) poParent->TableTotalCellIndex( 18, 2,
						OldValue,
						poParent->sim_year - r_min_SIM_YEAR_RANGE );
					}
					poParent->tableinfo.tab18dim_filter = poParent->table_filter_TabBirthsByAgeGroup;
				}
			}
		}
	}
	return split_integer_age_AGE_15_17 = value;
}

AGE_FIVE PersonDerivedStates::Set_split_integer_age_AGE_FIVE( AGE_FIVE value )
{

	if ( split_integer_age_AGE_FIVE != value ) {

AGE_FIVE		OldValue;
		OldValue = split_integer_age_AGE_FIVE;
		split_integer_age_AGE_FIVE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[2] >= 0 && ( poParent->tableinfo.tab2dim_filter || poParent->table_filter_RTAB_PopPyramidByEduc ) ) {
				if ( poParent->tableinfo.tab2events == -1 || poParent->tableinfo.tab2events < poParent->__events ) {
					if ( poParent->tableinfo.tab2events != -1 ) {
						poParent->tableinfo.tab2open0 = (real) poParent->CloseUpdate( 2, 0, poParent->tableinfo.tab2open0, poParent->tableinfo.tab2close0, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open1 = (real) poParent->CloseUpdate( 2, 1, poParent->tableinfo.tab2open1, poParent->tableinfo.tab2close1, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab2open2 = (real) poParent->CloseUpdate( 2, 2, poParent->tableinfo.tab2open2, poParent->tableinfo.tab2close2, poParent->tableinfo.tab2dim_filter, poParent->tableinfo.tab2cell, poParent->tableinfo.tab2total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab2events = poParent->__events;
					poParent->tableinfo.tab2close0 = (real) Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab2close1 = (real) Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab2close2 = (real) Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab2cell = (index) poParent->TableCellIndex( 2, 4,
						poParent->rtab_sel_years,
						poParent->sex,
						poParent->rtab_reg,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab2total_cell = (index) poParent->tableinfo.tab2cell;
					}
					else {
						poParent->tableinfo.tab2total_cell = (index) poParent->TableTotalCellIndex( 2, 4,
						poParent->rtab_sel_years,
						poParent->sex,
						poParent->rtab_reg,
						OldValue );
					}
					poParent->tableinfo.tab2dim_filter = poParent->table_filter_RTAB_PopPyramidByEduc;
				}
			}
			if ( gplTableIndexes[30] >= 0 && ( poParent->tableinfo.tab30dim_filter || poParent->table_filter_TabEduc2015 ) ) {
				if ( poParent->tableinfo.tab30events == -1 || poParent->tableinfo.tab30events < poParent->__events ) {
					if ( poParent->tableinfo.tab30events != -1 ) {
						poParent->tableinfo.tab30open0 = (real) poParent->CloseUpdate( 30, 0, poParent->tableinfo.tab30open0, poParent->tableinfo.tab30close0, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open1 = (real) poParent->CloseUpdate( 30, 1, poParent->tableinfo.tab30open1, poParent->tableinfo.tab30close1, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab30open2 = (real) poParent->CloseUpdate( 30, 2, poParent->tableinfo.tab30open2, poParent->tableinfo.tab30close2, poParent->tableinfo.tab30dim_filter, poParent->tableinfo.tab30cell, poParent->tableinfo.tab30total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab30events = poParent->__events;
					poParent->tableinfo.tab30close0 = (real) Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab30close1 = (real) Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab30close2 = (real) Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab30cell = (index) poParent->TableCellIndex( 30, 3,
						poParent->in_capital,
						poParent->sex,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab30total_cell = (index) poParent->tableinfo.tab30cell;
					}
					else {
						poParent->tableinfo.tab30total_cell = (index) poParent->TableTotalCellIndex( 30, 3,
						poParent->in_capital,
						poParent->sex,
						OldValue );
					}
					poParent->tableinfo.tab30dim_filter = poParent->table_filter_TabEduc2015;
				}
			}
			if ( gplTableIndexes[31] >= 0 && ( poParent->tableinfo.tab31dim_filter || poParent->table_filter_TabEduc2040 ) ) {
				if ( poParent->tableinfo.tab31events == -1 || poParent->tableinfo.tab31events < poParent->__events ) {
					if ( poParent->tableinfo.tab31events != -1 ) {
						poParent->tableinfo.tab31open0 = (real) poParent->CloseUpdate( 31, 0, poParent->tableinfo.tab31open0, poParent->tableinfo.tab31close0, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open1 = (real) poParent->CloseUpdate( 31, 1, poParent->tableinfo.tab31open1, poParent->tableinfo.tab31close1, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab31open2 = (real) poParent->CloseUpdate( 31, 2, poParent->tableinfo.tab31open2, poParent->tableinfo.tab31close2, poParent->tableinfo.tab31dim_filter, poParent->tableinfo.tab31cell, poParent->tableinfo.tab31total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab31events = poParent->__events;
					poParent->tableinfo.tab31close0 = (real) Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab31close1 = (real) Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab31close2 = (real) Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab31cell = (index) poParent->TableCellIndex( 31, 3,
						poParent->in_capital,
						poParent->sex,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab31total_cell = (index) poParent->tableinfo.tab31cell;
					}
					else {
						poParent->tableinfo.tab31total_cell = (index) poParent->TableTotalCellIndex( 31, 3,
						poParent->in_capital,
						poParent->sex,
						OldValue );
					}
					poParent->tableinfo.tab31dim_filter = poParent->table_filter_TabEduc2040;
				}
			}
			if ( gplTableIndexes[32] >= 0 && ( poParent->tableinfo.tab32dim_filter || poParent->table_filter_TabEduc2065 ) ) {
				if ( poParent->tableinfo.tab32events == -1 || poParent->tableinfo.tab32events < poParent->__events ) {
					if ( poParent->tableinfo.tab32events != -1 ) {
						poParent->tableinfo.tab32open0 = (real) poParent->CloseUpdate( 32, 0, poParent->tableinfo.tab32open0, poParent->tableinfo.tab32close0, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open1 = (real) poParent->CloseUpdate( 32, 1, poParent->tableinfo.tab32open1, poParent->tableinfo.tab32close1, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab32open2 = (real) poParent->CloseUpdate( 32, 2, poParent->tableinfo.tab32open2, poParent->tableinfo.tab32close2, poParent->tableinfo.tab32dim_filter, poParent->tableinfo.tab32cell, poParent->tableinfo.tab32total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab32events = poParent->__events;
					poParent->tableinfo.tab32close0 = (real) Get_duration_primary_level_PL_ENTER();
					poParent->tableinfo.tab32close1 = (real) Get_duration_primary_level_PL_GRAD();
					poParent->tableinfo.tab32close2 = (real) Get_duration_primary_level_PL_NO();
					poParent->tableinfo.tab32cell = (index) poParent->TableCellIndex( 32, 3,
						poParent->in_capital,
						poParent->sex,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab32total_cell = (index) poParent->tableinfo.tab32cell;
					}
					else {
						poParent->tableinfo.tab32total_cell = (index) poParent->TableTotalCellIndex( 32, 3,
						poParent->in_capital,
						poParent->sex,
						OldValue );
					}
					poParent->tableinfo.tab32dim_filter = poParent->table_filter_TabEduc2065;
				}
			}
		}
	}
	return split_integer_age_AGE_FIVE = value;
}

AGE_MORT PersonDerivedStates::Set_split_integer_age_AGE_MORT( AGE_MORT value )
{

	if ( split_integer_age_AGE_MORT != value ) {

AGE_MORT		OldValue;
		OldValue = split_integer_age_AGE_MORT;
		split_integer_age_AGE_MORT = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[21] >= 0 && ( poParent->tableinfo.tab21dim_filter || poParent->table_filter_TabDeathsByAgeGroup ) ) {
				if ( poParent->tableinfo.tab21events == -1 || poParent->tableinfo.tab21events < poParent->__events ) {
					if ( poParent->tableinfo.tab21events != -1 ) {
						poParent->tableinfo.tab21open0 = (real) poParent->CloseUpdate( 21, 0, poParent->tableinfo.tab21open0, poParent->tableinfo.tab21close0, poParent->tableinfo.tab21dim_filter, poParent->tableinfo.tab21cell, poParent->tableinfo.tab21total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
						poParent->tableinfo.tab21open1 = (real) poParent->CloseUpdate( 21, 1, poParent->tableinfo.tab21open1, poParent->tableinfo.tab21close1, poParent->tableinfo.tab21dim_filter, poParent->tableinfo.tab21cell, poParent->tableinfo.tab21total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab21events = poParent->__events;
					poParent->tableinfo.tab21close0 = (real) Get_duration_();
					poParent->tableinfo.tab21close1 = (real) transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab21cell = (index) poParent->TableCellIndex( 21, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab21total_cell = (index) poParent->tableinfo.tab21cell;
					}
					else {
						poParent->tableinfo.tab21total_cell = (index) poParent->TableTotalCellIndex( 21, 2,
						poParent->sim_year - r_min_SIM_YEAR_RANGE,
						OldValue );
					}
					poParent->tableinfo.tab21dim_filter = poParent->table_filter_TabDeathsByAgeGroup;
				}
			}
		}
	}
	return split_integer_age_AGE_MORT = value;
}

AGE_RTAB_MORT PersonDerivedStates::Set_split_integer_age_AGE_RTAB_MORT( AGE_RTAB_MORT value )
{

	if ( split_integer_age_AGE_RTAB_MORT != value ) {

AGE_RTAB_MORT		OldValue;
		OldValue = split_integer_age_AGE_RTAB_MORT;
		split_integer_age_AGE_RTAB_MORT = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[9] >= 0 && ( poParent->tableinfo.tab9dim_filter || poParent->table_filter_RTAB_TabDeathsByAgeGroup ) ) {
				if ( poParent->tableinfo.tab9events == -1 || poParent->tableinfo.tab9events < poParent->__events ) {
					if ( poParent->tableinfo.tab9events != -1 ) {
						poParent->tableinfo.tab9open0 = (real) poParent->CloseUpdate( 9, 0, poParent->tableinfo.tab9open0, poParent->tableinfo.tab9close0, poParent->tableinfo.tab9dim_filter, poParent->tableinfo.tab9cell, poParent->tableinfo.tab9total_cell, poParent->actor_weight, poParent->actor_subsample_weight );
					}
					poParent->tableinfo.tab9events = poParent->__events;
					poParent->tableinfo.tab9close0 = (real) transitions_alive_TRUE_FALSE;
					poParent->tableinfo.tab9cell = (index) poParent->TableCellIndex( 9, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					if ( gpModelApp->m_nSubSamples == 1 ) {
						poParent->tableinfo.tab9total_cell = (index) poParent->tableinfo.tab9cell;
					}
					else {
						poParent->tableinfo.tab9total_cell = (index) poParent->TableTotalCellIndex( 9, 2,
						poParent->rtab_years - r_min_RTAB_YEARS,
						OldValue );
					}
					poParent->tableinfo.tab9dim_filter = poParent->table_filter_RTAB_TabDeathsByAgeGroup;
				}
			}
		}
	}
	return split_integer_age_AGE_RTAB_MORT = value;
}

BIRTH_AGE_PART PersonDerivedStates::Set_split_integer_age_BIRTH_AGE_PART( BIRTH_AGE_PART value )
{

	if ( split_integer_age_BIRTH_AGE_PART != value ) {

		BIRTH_AGE_PART OldValue;
		OldValue = split_integer_age_BIRTH_AGE_PART;
		split_integer_age_BIRTH_AGE_PART = value;
		poStateFunctions->Set_birth_age_part( split_integer_age_BIRTH_AGE_PART );
	}
	return split_integer_age_BIRTH_AGE_PART = value;
}

DUR_TIME_IN_PARITY PersonDerivedStates::Set_split_time_since_last_birth_DUR_TIME_IN_PARITY( DUR_TIME_IN_PARITY value )
{

	if ( split_time_since_last_birth_DUR_TIME_IN_PARITY != value ) {

		DUR_TIME_IN_PARITY OldValue;
		OldValue = split_time_since_last_birth_DUR_TIME_IN_PARITY;
		split_time_since_last_birth_DUR_TIME_IN_PARITY = value;
		poStateFunctions->Set_time_in_parity( split_time_since_last_birth_DUR_TIME_IN_PARITY );
	}
	return split_time_since_last_birth_DUR_TIME_IN_PARITY = value;
}

short PersonDerivedStates::Set_ssint_active_spell_duration_in_this_parity_in_simulation_TRUE( short value )
{

	if ( ssint_active_spell_duration_in_this_parity_in_simulation_TRUE != value ) {

		short OldValue;
		OldValue = ssint_active_spell_duration_in_this_parity_in_simulation_TRUE;
		ssint_active_spell_duration_in_this_parity_in_simulation_TRUE = value;
		poStateFunctions->Set_time_since_last_birth( ssint_active_spell_duration_in_this_parity_in_simulation_TRUE+ round(poParent->time_since_last_birth_start) );
	}
	return ssint_active_spell_duration_in_this_parity_in_simulation_TRUE = value;
}

short PersonDerivedStates::Set_ssint_age( short value )
{

	if ( ssint_age != value ) {

		short OldValue;
		OldValue = ssint_age;
		ssint_age = value;
		poStateFunctions->Set_integer_age( COERCE(AGE_RANGE,ssint_age) );
	}
	return ssint_age = value;
}

short PersonDerivedStates::Set_ssint_time( short value )
{

	if ( ssint_time != value ) {

		short OldValue;
		OldValue = ssint_time;
		ssint_time = value;
		poStateFunctions->Set_calendar_year( ssint_time );
	}
	return ssint_time = value;
}

LITTLE_TIME PersonDerivedStates::Set_sssplit_duration_set_alive_TRUE_LITTLE_TIME( LITTLE_TIME value )
{

	if ( sssplit_duration_set_alive_TRUE_LITTLE_TIME != value ) {

		LITTLE_TIME OldValue;
		OldValue = sssplit_duration_set_alive_TRUE_LITTLE_TIME;
		sssplit_duration_set_alive_TRUE_LITTLE_TIME = value;
		poStateFunctions->Set_count_parity( sssplit_duration_set_alive_TRUE_LITTLE_TIME );
	}
	return sssplit_duration_set_alive_TRUE_LITTLE_TIME = value;
}

counter PersonDerivedStates::Set_transitions_alive_TRUE_FALSE( counter value )
{

	if ( transitions_alive_TRUE_FALSE != value ) {

counter		OldValue;
		OldValue = transitions_alive_TRUE_FALSE;
		transitions_alive_TRUE_FALSE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[3] >= 0 ) {
				if ( poParent->tableinfo.tab3events == poParent->__events ) {
					poParent->tableinfo.tab3close0 = (real) transitions_alive_TRUE_FALSE;
				}
			}
			if ( gplTableIndexes[9] >= 0 ) {
				if ( poParent->tableinfo.tab9events == poParent->__events ) {
					poParent->tableinfo.tab9close0 = (real) transitions_alive_TRUE_FALSE;
				}
			}
			if ( gplTableIndexes[20] >= 0 ) {
				if ( poParent->tableinfo.tab20events == poParent->__events ) {
					poParent->tableinfo.tab20close1 = (real) transitions_alive_TRUE_FALSE;
				}
			}
			if ( gplTableIndexes[21] >= 0 ) {
				if ( poParent->tableinfo.tab21events == poParent->__events ) {
					poParent->tableinfo.tab21close1 = (real) transitions_alive_TRUE_FALSE;
				}
			}
			if ( gplTableIndexes[22] >= 0 ) {
				if ( poParent->tableinfo.tab22events == poParent->__events ) {
					poParent->tableinfo.tab22close1 = (real) transitions_alive_TRUE_FALSE;
				}
			}
		}
	}
	return transitions_alive_TRUE_FALSE = value;
}

counter PersonDerivedStates::Set_transitions_has_emigrated_FALSE_TRUE( counter value )
{

	if ( transitions_has_emigrated_FALSE_TRUE != value ) {

counter		OldValue;
		OldValue = transitions_has_emigrated_FALSE_TRUE;
		transitions_has_emigrated_FALSE_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[26] >= 0 ) {
				if ( poParent->tableinfo.tab26events == poParent->__events ) {
					poParent->tableinfo.tab26close1 = (real) transitions_has_emigrated_FALSE_TRUE;
				}
			}
		}
	}
	return transitions_has_emigrated_FALSE_TRUE = value;
}

counter PersonDerivedStates::Set_transitions_in_union_FALSE_TRUE( counter value )
{

	if ( transitions_in_union_FALSE_TRUE != value ) {

counter		OldValue;
		OldValue = transitions_in_union_FALSE_TRUE;
		transitions_in_union_FALSE_TRUE = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[7] >= 0 ) {
				if ( poParent->tableinfo.tab7events == poParent->__events ) {
					poParent->tableinfo.tab7close0 = (real) transitions_in_union_FALSE_TRUE;
				}
			}
			if ( gplTableIndexes[8] >= 0 ) {
				if ( poParent->tableinfo.tab8events == poParent->__events ) {
					poParent->tableinfo.tab8close0 = (real) transitions_in_union_FALSE_TRUE;
				}
			}
			if ( gplTableIndexes[33] >= 0 ) {
				if ( poParent->tableinfo.tab33events == poParent->__events ) {
					poParent->tableinfo.tab33close3 = (real) transitions_in_union_FALSE_TRUE;
				}
			}
			if ( gplTableIndexes[34] >= 0 ) {
				if ( poParent->tableinfo.tab34events == poParent->__events ) {
					poParent->tableinfo.tab34close0 = (real) transitions_in_union_FALSE_TRUE;
				}
			}
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close4 = (real) transitions_in_union_FALSE_TRUE;
				}
			}
		}
	}
	return transitions_in_union_FALSE_TRUE = value;
}

counter PersonDerivedStates::Set_transitions_integer_age_14_15( counter value )
{

	if ( transitions_integer_age_14_15 != value ) {

counter		OldValue;
		OldValue = transitions_integer_age_14_15;
		transitions_integer_age_14_15 = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close5 = (real) transitions_integer_age_14_15;
				}
			}
		}
	}
	return transitions_integer_age_14_15 = value;
}

counter PersonDerivedStates::Set_transitions_integer_age_19_20( counter value )
{

	if ( transitions_integer_age_19_20 != value ) {

counter		OldValue;
		OldValue = transitions_integer_age_19_20;
		transitions_integer_age_19_20 = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close6 = (real) transitions_integer_age_19_20;
				}
			}
		}
	}
	return transitions_integer_age_19_20 = value;
}

counter PersonDerivedStates::Set_transitions_integer_age_9_10( counter value )
{

	if ( transitions_integer_age_9_10 != value ) {

counter		OldValue;
		OldValue = transitions_integer_age_9_10;
		transitions_integer_age_9_10 = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close7 = (real) transitions_integer_age_9_10;
				}
			}
		}
	}
	return transitions_integer_age_9_10 = value;
}

counter PersonDerivedStates::Set_transitions_parity_0_1( counter value )
{

	if ( transitions_parity_0_1 != value ) {

counter		OldValue;
		OldValue = transitions_parity_0_1;
		transitions_parity_0_1 = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[18] >= 0 ) {
				if ( poParent->tableinfo.tab18events == poParent->__events ) {
					poParent->tableinfo.tab18close1 = (real) transitions_parity_0_1;
				}
			}
			if ( gplTableIndexes[19] >= 0 ) {
				if ( poParent->tableinfo.tab19events == poParent->__events ) {
					poParent->tableinfo.tab19close1 = (real) transitions_parity_0_1;
				}
			}
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close8 = (real) transitions_parity_0_1;
				}
			}
		}
	}
	return transitions_parity_0_1 = value;
}

counter PersonDerivedStates::Set_transitions_primary_level_PL_ENTER_PL_GRAD( counter value )
{

	if ( transitions_primary_level_PL_ENTER_PL_GRAD != value ) {

counter		OldValue;
		OldValue = transitions_primary_level_PL_ENTER_PL_GRAD;
		transitions_primary_level_PL_ENTER_PL_GRAD = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[12] >= 0 ) {
				if ( poParent->tableinfo.tab12events == poParent->__events ) {
					poParent->tableinfo.tab12close0 = (real) transitions_primary_level_PL_ENTER_PL_GRAD;
				}
			}
		}
	}
	return transitions_primary_level_PL_ENTER_PL_GRAD = value;
}

counter PersonDerivedStates::Set_transitions_primary_level_PL_NO_PL_ENTER( counter value )
{

	if ( transitions_primary_level_PL_NO_PL_ENTER != value ) {

counter		OldValue;
		OldValue = transitions_primary_level_PL_NO_PL_ENTER;
		transitions_primary_level_PL_NO_PL_ENTER = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[11] >= 0 ) {
				if ( poParent->tableinfo.tab11events == poParent->__events ) {
					poParent->tableinfo.tab11close0 = (real) transitions_primary_level_PL_NO_PL_ENTER;
				}
			}
		}
	}
	return transitions_primary_level_PL_NO_PL_ENTER = value;
}

logical PersonDerivedStates::Set_trigger_entrances_in_projected_time_TRUE( logical value )
{

	value = value != 0;
	if ( trigger_entrances_in_projected_time_TRUE != value ) {
		poStateFunctions->Set_table_filter_TabSimulatedStartingPopulation( value );

		logical OldValue;
		OldValue = trigger_entrances_in_projected_time_TRUE;
		trigger_entrances_in_projected_time_TRUE = value;
	}
	return trigger_entrances_in_projected_time_TRUE = value;
}

logical PersonDerivedStates::Set_trigger_entrances_set_alive_TRUE( logical value )
{

	value = value != 0;
	if ( trigger_entrances_set_alive_TRUE != value ) {
		poStateFunctions->Set_table_filter_TabImmigrationSexProvinceAge( poParent->person_type== PT_IMMIGRANT&& poParent->in_projected_time&& value );

		logical OldValue;
		OldValue = trigger_entrances_set_alive_TRUE;
		trigger_entrances_set_alive_TRUE = value;
	}
	return trigger_entrances_set_alive_TRUE = value;
}

real PersonDerivedStates::Set_value_at_changes_parity_age( real value )
{

	if ( value_at_changes_parity_age != value ) {

real		OldValue;
		OldValue = value_at_changes_parity_age;
		value_at_changes_parity_age = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[19] >= 0 ) {
				if ( poParent->tableinfo.tab19events == poParent->__events ) {
					poParent->tableinfo.tab19close2 = (real) value_at_changes_parity_age;
				}
			}
		}
	}
	return value_at_changes_parity_age = value;
}

real PersonDerivedStates::Set_value_at_transitions_in_union_FALSE_TRUE_age( real value )
{

	if ( value_at_transitions_in_union_FALSE_TRUE_age != value ) {

real		OldValue;
		OldValue = value_at_transitions_in_union_FALSE_TRUE_age;
		value_at_transitions_in_union_FALSE_TRUE_age = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[8] >= 0 ) {
				if ( poParent->tableinfo.tab8events == poParent->__events ) {
					poParent->tableinfo.tab8close1 = (real) value_at_transitions_in_union_FALSE_TRUE_age;
				}
			}
			if ( gplTableIndexes[34] >= 0 ) {
				if ( poParent->tableinfo.tab34events == poParent->__events ) {
					poParent->tableinfo.tab34close1 = (real) value_at_transitions_in_union_FALSE_TRUE_age;
				}
			}
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close9 = (real) value_at_transitions_in_union_FALSE_TRUE_age;
				}
			}
		}
	}
	return value_at_transitions_in_union_FALSE_TRUE_age = value;
}

real PersonDerivedStates::Set_value_at_transitions_parity_0_1_age( real value )
{

	if ( value_at_transitions_parity_0_1_age != value ) {

real		OldValue;
		OldValue = value_at_transitions_parity_0_1_age;
		value_at_transitions_parity_0_1_age = value;
		if ( poParent->bUpdates ) {
			if ( gplTableIndexes[19] >= 0 ) {
				if ( poParent->tableinfo.tab19events == poParent->__events ) {
					poParent->tableinfo.tab19close3 = (real) value_at_transitions_parity_0_1_age;
				}
			}
			if ( gplTableIndexes[35] >= 0 ) {
				if ( poParent->tableinfo.tab35events == poParent->__events ) {
					poParent->tableinfo.tab35close10 = (real) value_at_transitions_parity_0_1_age;
				}
			}
		}
	}
	return value_at_transitions_parity_0_1_age = value;
}

void *PrimarySchool::operator new( size_t count )
{
	PrimarySchool *poActor;
	if ( gpoEmptyPrimarySchool != NULL ) {
		poActor = gpoEmptyPrimarySchool;
		gpoEmptyPrimarySchool = (PrimarySchool *) gpoEmptyPrimarySchool->poNext;
	}
	else {
		poActor = ::new PrimarySchool( TRUE ) ;
	}
	poActor->__finished = false;
	return poActor;
}

void PrimarySchool::InitActor()
{
	poDerivedStates = new PrimarySchoolDerivedStates(); 
	poDerivedStates->poParent = this;
	poStateFunctions = new PrimarySchoolStateFunctions();
	poStateFunctions->poParent = this;
	poStateFunctions->poDerivedStates = poDerivedStates;
	poDerivedStates->poStateFunctions = poStateFunctions;
	bUpdates = FALSE;
	nActorNumber = 3;
	InitializeStates();
}

void PrimarySchool::DeleteActor()
{
	delete poDerivedStates;
	delete poStateFunctions;
}

#pragma optimize( "", off )

void PrimarySchool::InitializeStates()
{
	// initialization of states
	actor_id = (long) 0;
	actor_subsample_weight = (double) 0;
	actor_weight = (double) 0;
	age = (TIME) 0;
#line 85 "code/PrimaryEducation.mpp"
	assign_primary_time = (TIME) TIME_INFINITE;
#line 10966 "modgen/src/ACTORS.CPP"
	case_id = (long) 0;
	case_seed = (double) 0;
#line 84 "code/PrimaryEducation.mpp"
	next_school_year = (TIME) TIME_INFINITE;
#line 10971 "modgen/src/ACTORS.CPP"
#line 83 "code/PrimaryEducation.mpp"
	school_year = (int) 0;
#line 10974 "modgen/src/ACTORS.CPP"
	time = (TIME) gdThreadCurrentTime;
	__finished = FALSE;
	__time = (TIME) gdThreadCurrentTime;
	__events = 0;
}

#pragma optimize( "", on )

void PrimarySchool::UpdateLinkedStates()
{
}

void PrimarySchool::StartSpecial1()
{
	nSubSample = GetSubSample();
	InitializeStates();
	case_seed = GetCaseSeed();
	case_id = GetCaseID();
	lActorId = actor_id = GetObjectID();
	GetCaseWeight( &actor_weight, &actor_subsample_weight );

	// initialization of events
	poAssignInitialPrimary = gpoEventQueue->NewEvent( this, 0, 0 );
	poNextSchoolYearEvent = gpoEventQueue->NewEvent( this, 7, 0 );
	//Initalize derived states
	CountActor(3);
}

#pragma optimize( "", off )

void PrimarySchool::StartSpecial2()
{
	// initialization of expression and history states
	poStateFunctions->Set_time(TIME( CoarsenMantissa( time ) ), true);
	poStateFunctions->Set_age(TIME( CoarsenMantissa( age ) ), true);
	gdThreadCurrentTime = time;
	__time = time;
	UpdateLinkedStates();
	poPrev = NULL;
	poNext = gpoEventQueue->poHeadActor;
	if ( gpoEventQueue->poHeadActor != NULL ) {
		gpoEventQueue->poHeadActor->poPrev = this;
	}
	gpoEventQueue->poHeadActor = this;
	bUpdates = TRUE;
}

void PrimarySchool::FinishSpecial()
{
	CountFinishedActor(3);
	bUpdates = FALSE;
	gpoEventQueue->DestroyEvent( poAssignInitialPrimary );
	gpoEventQueue->DestroyEvent( poNextSchoolYearEvent );

	// Remove actor from master list of active actors
	if ( poPrev != NULL )
	{
		poPrev->poNext = poNext;
	}
	else
	{
		if (gpoEventQueue->poHeadActor != NULL && gpoEventQueue->poHeadActor == this)
		{
			gpoEventQueue->poHeadActor = poNext;
		}
	}
	if ( poNext != NULL )
	{
		poNext->poPrev = poPrev;
	}
	poPrev = NULL;

	// Add actor to head of empty list of actors of this type
	poNext = gpoEmptyPrimarySchool;
	gpoEmptyPrimarySchool = this;
}

#pragma optimize( "", on )

void PrimarySchool::WaitUntil( double target_time )
{
	//don't increase gdEventsForTabulation if special events because they are all treated as one event for tabulation purposes
	if (!gpoEventQueue->m_bSpecialEvents)
	{
		gdEventsForTabulation++;
	}
	__events = gdEventsForTabulation;
	if ( time < (TIME) target_time )
	{
		__time = (TIME) target_time;
		WaitSpecial( (TIME) target_time - time );
		poStateFunctions->Set_time( (TIME) target_time );
	}
}

PrimarySchool *PrimarySchool::UpdateTime(  )
{
	__events = gdEventsForTabulation;
	if ( time < (TIME) gdThreadCurrentTime ) {
		__time = (TIME) gdThreadCurrentTime;
		WaitSpecial( (TIME) gdThreadCurrentTime - time );
		poStateFunctions->Set_time( (TIME) gdThreadCurrentTime );
	}
	return this;
}

void PrimarySchool::WaitUntilThisActor( double target_time )
{
	WaitUntil(target_time);
}

void PrimarySchool::WaitSpecial( TIME wait_time )
{

	// increment the age
		poStateFunctions->Set_age( age + wait_time );
}

double PrimarySchool::EventTime( int nEventNum, int *pnEventInfo )
{
	double	event_time = double( TIME_INFINITE );

	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum ) {
		case 0:
			gszRuntimeEventName = _T("PrimarySchool.AssignInitialPrimary(time)");
			event_time = CoarsenMantissa( (TIME) timeAssignInitialPrimary() );
			TRCEVNT(case_seed, _T("PrimarySchool"), actor_id, _T("timeAssignInitialPrimary"), event_time);
			break;
		case 7:
			gszRuntimeEventName = _T("PrimarySchool.NextSchoolYearEvent(time)");
			event_time = CoarsenMantissa( (TIME) timeNextSchoolYearEvent() );
			TRCEVNT(case_seed, _T("PrimarySchool"), actor_id, _T("timeNextSchoolYearEvent"), event_time);
			break;
		case -1:
		default:;
	}
	return (TIME) event_time;
}

void PrimarySchool::Implement( int nEventNum, int nEventInfo )
{
	gdThreadEvents += 1;
	gdRuntimeTime = (double) time;
	gnRuntimeActorId = actor_id;
	switch( nEventNum )
	{
		case 0:
			CHKSUM(time, 0);
			TRCEVNT(case_seed, _T("PrimarySchool"), actor_id, _T("PrimarySchool.AssignInitialPrimary"), time);
			gszRuntimeEventName = _T("PrimarySchool.AssignInitialPrimary(implement)");
			AssignInitialPrimary();
			break;
		case 7:
			CHKSUM(time, 1);
			TRCEVNT(case_seed, _T("PrimarySchool"), actor_id, _T("PrimarySchool.NextSchoolYearEvent"), time);
			gszRuntimeEventName = _T("PrimarySchool.NextSchoolYearEvent(implement)");
			NextSchoolYearEvent();
			break;
		case -1:
		default:;
	}
}

double PrimarySchool::Set_actor_subsample_weight( double value )
{
	return poStateFunctions->Set_actor_subsample_weight( value);
}

double PrimarySchool::Set_actor_weight( double value )
{
	return poStateFunctions->Set_actor_weight( value);
}

TIME PrimarySchoolStateFunctions::Set_assign_primary_time( TIME value )
{

	if ( poParent->assign_primary_time != value )
	{


		TIME	OldValue;
		OldValue = poParent->assign_primary_time;
		poParent->assign_primary_time = value;

		if ( poParent->poAssignInitialPrimary->cRecalc == 0 ) {
			poParent->poAssignInitialPrimary->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poAssignInitialPrimary );
		};
	}
	return poParent->assign_primary_time = value;
}

TIME PrimarySchoolStateFunctions::Set_next_school_year( TIME value )
{

	if ( poParent->next_school_year != value )
	{


		TIME	OldValue;
		OldValue = poParent->next_school_year;
		poParent->next_school_year = value;

		if ( poParent->poNextSchoolYearEvent->cRecalc == 0 ) {
			poParent->poNextSchoolYearEvent->cRecalc = 1;
			gpoEventQueue->RecalculateEvent( poParent->poNextSchoolYearEvent );
		};
	}
	return poParent->next_school_year = value;
}
Thread ChildMortalityCalibrator	*gpoEmptyChildMortalityCalibrator = NULL;

void DeleteAllChildMortalityCalibratorActors()
{
}
Thread Observation	*gpoEmptyObservation = NULL;

void DeleteAllObservationActors()
{
}
Thread Person	*gpoEmptyPerson = NULL;

void DeleteAllPersonActors()
{
}
Thread PrimarySchool	*gpoEmptyPrimarySchool = NULL;

void DeleteAllPrimarySchoolActors()
{
}

void ChildMortalityCalibrator_mlBabies::Add( Person *prPerson )
{
	BOOL	bFound = FALSE;
	int	nIndex;
	int	nNewPosition = -1;
	int	nSize;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		if ( arData[nIndex] == prPerson ) {
			bFound = TRUE;
			break;
		}
		if ( arData[nIndex] == NULL && nNewPosition == -1 ) {
			nNewPosition = nIndex;
		}
	}
	if ( !bFound ) {
		prPerson->UpdateTime();
		if ( nNewPosition != -1 ) {
			arData[nNewPosition] = prPerson;
		}
		else {
			arData.Add( prPerson );
		}
		UpdateReferences();
		prPerson->Set_lChildMortalityCalibrator( prParentChildMortalityCalibrator );
	}
}

void ChildMortalityCalibrator_mlBabies::Remove( Person *prPerson )
{
	int	nIndex;
	int	nSize;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		if ( arData[nIndex] == prPerson ) {
			prPerson->UpdateTime();
			arData[nIndex] = NULL;
			UpdateReferences();
			if ( prPerson->lChildMortalityCalibrator == prParentChildMortalityCalibrator ) {
				prPerson->Set_lChildMortalityCalibrator( NULL );
			}
			break;
		}
	}
}

void ChildMortalityCalibrator_mlBabies::RemoveAll()
{
	int	nIndex;
	int	nSize;
	Person	*prPerson;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		prPerson = arData[nIndex];
		if ( prPerson != NULL ) {
			prPerson->UpdateTime();
			arData[nIndex] = NULL;
			if ( prPerson->lChildMortalityCalibrator == prParentChildMortalityCalibrator ) {
				prPerson->Set_lChildMortalityCalibrator( NULL );
			}
		}
	}
	UpdateReferences();
}

void ChildMortalityCalibrator_mlBabies::Finish()
{
	int	nSize = 0;
	nSize = arData.GetSize();
	RemoveAll();
	SetMaxActorLinked(0, 0, nSize);
}

void ChildMortalityCalibrator_mlBabies::FinishAll()
{
	int	nIndex;
	int	nSize;
	Person	*prPerson;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		prPerson = arData[nIndex];
		if ( prPerson != NULL ) {
			prPerson->UpdateTime();
			prPerson->Finish();
			arData[nIndex] = NULL;
		}
	}
	UpdateReferences();
}

Person *ChildMortalityCalibrator_mlBabies::GetNext( int nInitPos, int *pnPos )
{
	int	nIndex;
	int	nSize;
	Person	*prPerson;
	Person	*prFoundPerson = NULL;

	*pnPos = -1;
	nSize = arData.GetSize();
	for ( nIndex = nInitPos; nIndex < nSize; nIndex++ ) {
		prPerson = arData[nIndex];
		if ( prPerson != NULL ) {
			prFoundPerson = prPerson;
			*pnPos = nIndex;
			break;
		}
	}
	return prFoundPerson;
}

void ChildMortalityCalibrator_mlBabies::UpdateReferences()
{
}

void Person_mlChild::Add( Person *prPerson )
{
	BOOL	bFound = FALSE;
	int	nIndex;
	int	nNewPosition = -1;
	int	nSize;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		if ( arData[nIndex] == prPerson ) {
			bFound = TRUE;
			break;
		}
		if ( arData[nIndex] == NULL && nNewPosition == -1 ) {
			nNewPosition = nIndex;
		}
	}
	if ( !bFound ) {
		prPerson->UpdateTime();
		if ( nNewPosition != -1 ) {
			arData[nNewPosition] = prPerson;
		}
		else {
			arData.Add( prPerson );
		}
		UpdateReferences();
		prPerson->Set_lMother( prParentPerson );
	}
}

void Person_mlChild::Remove( Person *prPerson )
{
	int	nIndex;
	int	nSize;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		if ( arData[nIndex] == prPerson ) {
			prPerson->UpdateTime();
			arData[nIndex] = NULL;
			UpdateReferences();
			if ( prPerson->lMother == prParentPerson ) {
				prPerson->Set_lMother( NULL );
			}
			break;
		}
	}
}

void Person_mlChild::RemoveAll()
{
	int	nIndex;
	int	nSize;
	Person	*prPerson;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		prPerson = arData[nIndex];
		if ( prPerson != NULL ) {
			prPerson->UpdateTime();
			arData[nIndex] = NULL;
			if ( prPerson->lMother == prParentPerson ) {
				prPerson->Set_lMother( NULL );
			}
		}
	}
	UpdateReferences();
}

void Person_mlChild::Finish()
{
	int	nSize = 0;
	nSize = arData.GetSize();
	RemoveAll();
	SetMaxActorLinked(2, 2, nSize);
}

void Person_mlChild::FinishAll()
{
	int	nIndex;
	int	nSize;
	Person	*prPerson;

	nSize = arData.GetSize();
	for ( nIndex = 0; nIndex < nSize; nIndex++ ) {
		prPerson = arData[nIndex];
		if ( prPerson != NULL ) {
			prPerson->UpdateTime();
			prPerson->Finish();
			arData[nIndex] = NULL;
		}
	}
	UpdateReferences();
}

Person *Person_mlChild::GetNext( int nInitPos, int *pnPos )
{
	int	nIndex;
	int	nSize;
	Person	*prPerson;
	Person	*prFoundPerson = NULL;

	*pnPos = -1;
	nSize = arData.GetSize();
	for ( nIndex = nInitPos; nIndex < nSize; nIndex++ ) {
		prPerson = arData[nIndex];
		if ( prPerson != NULL ) {
			prFoundPerson = prPerson;
			*pnPos = nIndex;
			break;
		}
	}
	return prFoundPerson;
}

void Person_mlChild::UpdateReferences()
{
}

int	asAllObservationsActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Observation	*poObservation1 = NULL;
	Observation	*poObservation2 = NULL;

	poObservation1 = (Observation *) poActor1;
	poObservation2 = (Observation *) poActor2;
	return (poObservation1->actor_id == poObservation2->actor_id ? 0 :
			poObservation1->actor_id < poObservation2->actor_id ? -1 : 1);
}
Thread asAllObservationsActorSetTree	*asAllObservations;

int	asAllPersonsActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asAllPersonsActorSetTree	*asAllPersons;

int	asFemaleAgeEducActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asFemaleAgeEducActorSetTree	*asFemaleAgeEduc[rc_size_AGE_RANGE][rc_size_PRIMARY_LEVEL];

int	asForeignersAgeSexProvActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asForeignersAgeSexProvActorSetTree	*asForeignersAgeSexProv[rc_size_AGE_RANGE][rc_size_SEX][rc_size_PROVINCE_NAT];

int	asForeignersPrimAgeSexProvActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asForeignersPrimAgeSexProvActorSetTree	*asForeignersPrimAgeSexProv[rc_size_AGE_RANGE][rc_size_SEX][rc_size_PROVINCE_NAT];

int	asPotentialMothersActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asPotentialMothersActorSetTree	*asPotentialMothers;

int	asPotentialMothersByAgeActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asPotentialMothersByAgeActorSetTree	*asPotentialMothersByAge[rc_size_FERTILE_AGE_RANGE];

int	asPrimaryEntryCohortActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asPrimaryEntryCohortActorSetTree	*asPrimaryEntryCohort[rc_size_SEX][rc_size_PROVINCE_INT][rc_size_PRIMARY_LEVEL];

int	asPrimaryGraduationCohortActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asPrimaryGraduationCohortActorSetTree	*asPrimaryGraduationCohort[rc_size_SEX][rc_size_PROVINCE_INT][rc_size_PRIMARY_LEVEL];

int	asResidentsAgeSexActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asResidentsAgeSexActorSetTree	*asResidentsAgeSex[rc_size_AGE_RANGE][rc_size_SEX];

int	asResidentsPrimAgeSexActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	Person	*poPerson1 = NULL;
	Person	*poPerson2 = NULL;

	poPerson1 = (Person *) poActor1;
	poPerson2 = (Person *) poActor2;
	return (poPerson1->actor_id == poPerson2->actor_id ? 0 :
			poPerson1->actor_id < poPerson2->actor_id ? -1 : 1);
}
Thread asResidentsPrimAgeSexActorSetTree	*asResidentsPrimAgeSex[rc_size_AGE_RANGE][rc_size_SEX];

int	asTheChildMortalityCalibratorActorSetTree::ActorCompare(ActorClass *poActor1, ActorClass *poActor2)
	{
	ChildMortalityCalibrator	*poChildMortalityCalibrator1 = NULL;
	ChildMortalityCalibrator	*poChildMortalityCalibrator2 = NULL;

	poChildMortalityCalibrator1 = (ChildMortalityCalibrator *) poActor1;
	poChildMortalityCalibrator2 = (ChildMortalityCalibrator *) poActor2;
	return (poChildMortalityCalibrator1->actor_id == poChildMortalityCalibrator2->actor_id ? 0 :
			poChildMortalityCalibrator1->actor_id < poChildMortalityCalibrator2->actor_id ? -1 : 1);
}
Thread asTheChildMortalityCalibratorActorSetTree	*asTheChildMortalityCalibrator;

void PreSimulation() {
	PreSimulation0();
	PreSimulation1();
	PreSimulation2();
	PreSimulation3();
	PreSimulation4();
	PreSimulation5();
}

void PostSimulation() {
	PostSimulation0();
}

void UserTables() {
}

BOOL ValidateParameters( SCENARIO_EVENT eEvent ) {
	BOOL bProceed = TRUE;

	 return bProceed;
}

void InitActorSets()
{
	int	nIndex0;
	int	nIndex1;
	int	nIndex2;

	asAllObservations = new asAllObservationsActorSetTree();
	asAllPersons = new asAllPersonsActorSetTree();
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 3; nIndex1++)
		{
			asFemaleAgeEduc[nIndex0][nIndex1] = new asFemaleAgeEducActorSetTree();
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 13; nIndex2++)
			{
				asForeignersAgeSexProv[nIndex0][nIndex1][nIndex2] = new asForeignersAgeSexProvActorSetTree();
			}
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 13; nIndex2++)
			{
				asForeignersPrimAgeSexProv[nIndex0][nIndex1][nIndex2] = new asForeignersPrimAgeSexProvActorSetTree();
			}
		}
	}
	asPotentialMothers = new asPotentialMothersActorSetTree();
	for (nIndex0 = 0; nIndex0 < 40; nIndex0++)
	{
		asPotentialMothersByAge[nIndex0] = new asPotentialMothersByAgeActorSetTree();
	}
	for (nIndex0 = 0; nIndex0 < 2; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 14; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 3; nIndex2++)
			{
				asPrimaryEntryCohort[nIndex0][nIndex1][nIndex2] = new asPrimaryEntryCohortActorSetTree();
			}
		}
	}
	for (nIndex0 = 0; nIndex0 < 2; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 14; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 3; nIndex2++)
			{
				asPrimaryGraduationCohort[nIndex0][nIndex1][nIndex2] = new asPrimaryGraduationCohortActorSetTree();
			}
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			asResidentsAgeSex[nIndex0][nIndex1] = new asResidentsAgeSexActorSetTree();
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			asResidentsPrimAgeSex[nIndex0][nIndex1] = new asResidentsPrimAgeSexActorSetTree();
		}
	}
	asTheChildMortalityCalibrator = new asTheChildMortalityCalibratorActorSetTree();
}

void	DeleteActorSets()
{
	int	nIndex0;
	int	nIndex1;
	int	nIndex2;

	delete asAllObservations;
	asAllObservations = NULL;
	delete asAllPersons;
	asAllPersons = NULL;
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 3; nIndex1++)
		{
			delete asFemaleAgeEduc[nIndex0][nIndex1];
			asFemaleAgeEduc[nIndex0][nIndex1] = NULL;
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 13; nIndex2++)
			{
				delete asForeignersAgeSexProv[nIndex0][nIndex1][nIndex2];
				asForeignersAgeSexProv[nIndex0][nIndex1][nIndex2] = NULL;
			}
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 13; nIndex2++)
			{
				delete asForeignersPrimAgeSexProv[nIndex0][nIndex1][nIndex2];
				asForeignersPrimAgeSexProv[nIndex0][nIndex1][nIndex2] = NULL;
			}
		}
	}
	delete asPotentialMothers;
	asPotentialMothers = NULL;
	for (nIndex0 = 0; nIndex0 < 40; nIndex0++)
	{
		delete asPotentialMothersByAge[nIndex0];
		asPotentialMothersByAge[nIndex0] = NULL;
	}
	for (nIndex0 = 0; nIndex0 < 2; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 14; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 3; nIndex2++)
			{
				delete asPrimaryEntryCohort[nIndex0][nIndex1][nIndex2];
				asPrimaryEntryCohort[nIndex0][nIndex1][nIndex2] = NULL;
			}
		}
	}
	for (nIndex0 = 0; nIndex0 < 2; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 14; nIndex1++)
		{
			for (nIndex2 = 0; nIndex2 < 3; nIndex2++)
			{
				delete asPrimaryGraduationCohort[nIndex0][nIndex1][nIndex2];
				asPrimaryGraduationCohort[nIndex0][nIndex1][nIndex2] = NULL;
			}
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			delete asResidentsAgeSex[nIndex0][nIndex1];
			asResidentsAgeSex[nIndex0][nIndex1] = NULL;
		}
	}
	for (nIndex0 = 0; nIndex0 < 101; nIndex0++)
	{
		for (nIndex1 = 0; nIndex1 < 2; nIndex1++)
		{
			delete asResidentsPrimAgeSex[nIndex0][nIndex1];
			asResidentsPrimAgeSex[nIndex0][nIndex1] = NULL;
		}
	}
	delete asTheChildMortalityCalibrator;
	asTheChildMortalityCalibrator = NULL;
}

} // namespace 
